@@ -4,9 +4,11 @@ from . import partner
 from . import account
 from . import account_reconcile_model
 from . import account_payment
-from . import account_invoice
+from . import account_payment_term
 from . import account_bank_statement
 from . import account_move
+from . import account_payment_term
+from . import account_bank_statement
 from . import chart_template
 from . import account_analytic_line
 from . import account_journal_dashboard
@@ -18,5 +20,5 @@ from . import account_cash_rounding
 from . import reconciliation_widget
 from . import account_incoterms
 from . import digest
-from . import account_invoice_import_wizard
 from . import res_users
+from . import ir_actions_report
