@@ -2,33 +2,49 @@
 
 from datetime import timedelta, datetime
 import calendar
-import time
 from dateutil.relativedelta import relativedelta
 
 from odoo import fields, models, api, _
 from odoo.exceptions import ValidationError, UserError, RedirectWarning
-from odoo.tools.misc import DEFAULT_SERVER_DATE_FORMAT
+from odoo.tools.misc import DEFAULT_SERVER_DATE_FORMAT, format_date
 from odoo.tools.float_utils import float_round, float_is_zero
 from odoo.tools import date_utils
+from odoo.tests.common import Form
+
+
+MONTH_SELECTION = [
+    ('1', 'January'),
+    ('2', 'February'),
+    ('3', 'March'),
+    ('4', 'April'),
+    ('5', 'May'),
+    ('6', 'June'),
+    ('7', 'July'),
+    ('8', 'August'),
+    ('9', 'September'),
+    ('10', 'October'),
+    ('11', 'November'),
+    ('12', 'December'),
+]
 
 
 class ResCompany(models.Model):
     _inherit = "res.company"
 
-    def _get_invoice_reference_types(self):
-        return [('invoice_number', _('Based on Invoice Number')), ('partner', _('Based on Customer'))]
-
     #TODO check all the options/fields are in the views (settings + company form view)
     fiscalyear_last_day = fields.Integer(default=31, required=True)
-    fiscalyear_last_month = fields.Selection([(1, 'January'), (2, 'February'), (3, 'March'), (4, 'April'), (5, 'May'), (6, 'June'), (7, 'July'), (8, 'August'), (9, 'September'), (10, 'October'), (11, 'November'), (12, 'December')], default=12, required=True)
+    fiscalyear_last_month = fields.Selection(MONTH_SELECTION, default='12', required=True)
     period_lock_date = fields.Date(string="Lock Date for Non-Advisers", help="Only users with the 'Adviser' role can edit accounts prior to and inclusive of this date. Use it for period locking inside an open fiscal year, for example.")
     fiscalyear_lock_date = fields.Date(string="Lock Date", help="No users, including Advisers, can edit accounts prior to and inclusive of this date. Use it for fiscal year locking for example.")
+    tax_lock_date = fields.Date("Tax Lock Date", help="No users can edit journal entries related to a tax prior and inclusive of this date.")
     transfer_account_id = fields.Many2one('account.account',
         domain=lambda self: [('reconcile', '=', True), ('user_type_id.id', '=', self.env.ref('account.data_account_type_current_assets').id), ('deprecated', '=', False)], string="Inter-Banks Transfer Account", help="Intermediary account used when moving money from a liquidity account to another")
     expects_chart_of_accounts = fields.Boolean(string='Expects a Chart of Accounts', default=True)
     chart_template_id = fields.Many2one('account.chart.template', help='The chart template for the company (if any)')
-    bank_account_code_prefix = fields.Char(string='Prefix of the bank accounts', oldname="bank_account_code_char")
+    bank_account_code_prefix = fields.Char(string='Prefix of the bank accounts')
     cash_account_code_prefix = fields.Char(string='Prefix of the cash accounts')
+    default_cash_difference_income_account_id = fields.Many2one('account.account', string="Cash Difference Income Account")
+    default_cash_difference_expense_account_id = fields.Many2one('account.account', string="Cash Difference Expense Account")
     transfer_account_code_prefix = fields.Char(string='Prefix of the transfer accounts')
     account_sale_tax_id = fields.Many2one('account.tax', string="Default Sale Tax")
     account_purchase_tax_id = fields.Many2one('account.tax', string="Default Purchase Tax")
@@ -43,19 +59,10 @@ class ResCompany(models.Model):
     expense_currency_exchange_account_id = fields.Many2one('account.account', related='currency_exchange_journal_id.default_debit_account_id', readonly=False,
         string="Loss Exchange Rate Account", domain="[('internal_type', '=', 'other'), ('deprecated', '=', False), ('company_id', '=', id)]")
     anglo_saxon_accounting = fields.Boolean(string="Use anglo-saxon accounting")
-    property_stock_account_input_categ_id = fields.Many2one('account.account', string="Input Account for Stock Valuation", oldname="property_stock_account_input_categ")
-    property_stock_account_output_categ_id = fields.Many2one('account.account', string="Output Account for Stock Valuation", oldname="property_stock_account_output_categ")
+    property_stock_account_input_categ_id = fields.Many2one('account.account', string="Input Account for Stock Valuation")
+    property_stock_account_output_categ_id = fields.Many2one('account.account', string="Output Account for Stock Valuation")
     property_stock_valuation_account_id = fields.Many2one('account.account', string="Account Template for Stock Valuation")
     bank_journal_ids = fields.One2many('account.journal', 'company_id', domain=[('type', '=', 'bank')], string='Bank Journals')
-    overdue_msg = fields.Text(string='Overdue Payments Message', translate=True,
-        default=lambda s: _('''Dear Sir/Madam,
-
-Our records indicate that some payments on your account are still due. Please find details below.
-If the amount has already been paid, please disregard this notice. Otherwise, please forward us the total amount stated below.
-If you have any queries regarding your account, Please contact us.
-
-Thank you in advance for your cooperation.
-Best Regards,'''))
     tax_exigibility = fields.Boolean(string='Use Cash Basis')
     account_bank_reconciliation_start = fields.Date(string="Bank Reconciliation Threshold", help="""The bank reconciliation widget won't ask to reconcile payments older than this date.
                                                                                                        This is useful if you install accounting after having used invoicing for some time and
@@ -63,8 +70,6 @@ Best Regards,'''))
 
     incoterm_id = fields.Many2one('account.incoterms', string='Default incoterm',
         help='International Commercial Terms are a series of predefined commercial terms used in international transactions.')
-    invoice_reference_type = fields.Selection(string='Default Communication Type', selection='_get_invoice_reference_types',
-                                              default='invoice_number', help='You can set here the default communication that will appear on customer invoices, once validated, to help the customer to refer to that particular invoice when making the payment.')
 
     qr_code = fields.Boolean(string='Display SEPA QR code')
 
@@ -88,22 +93,36 @@ Best Regards,'''))
     # account dashboard onboarding
     account_invoice_onboarding_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done"), ('closed', "Closed")], string="State of the account invoice onboarding panel", default='not_done')
     account_dashboard_onboarding_state = fields.Selection([('not_done', "Not done"), ('just_done', "Just done"), ('done', "Done"), ('closed', "Closed")], string="State of the account dashboard onboarding panel", default='not_done')
+    invoice_terms = fields.Text(string='Default Terms and Conditions', translate=True)
 
-    @api.constrains('account_opening_date', 'fiscalyear_last_day', 'fiscalyear_last_month')
+    # Needed in the Point of Sale
+    account_default_pos_receivable_account_id = fields.Many2one('account.account', string="Default PoS Receivable Account")
+
+    # Accrual Accounting
+    expense_accrual_account_id = fields.Many2one('account.account',
+        help="Account used to move the period of an expense",
+        domain="[('internal_group', '=', 'liability'), ('internal_type', 'not in', ('receivable', 'payable')), ('reconcile', '=', True), ('company_id', '=', id)]")
+    revenue_accrual_account_id = fields.Many2one('account.account',
+        help="Account used to move the period of a revenue",
+        domain="[('internal_group', '=', 'asset'), ('internal_type', 'not in', ('receivable', 'payable')), ('reconcile', '=', True), ('company_id', '=', id)]")
+    accrual_default_journal_id = fields.Many2one('account.journal', help="Journal used by default for moving the period of an entry", domain="[('type', '=', 'general')]")
+
+    @api.constrains('account_opening_move_id', 'fiscalyear_last_day', 'fiscalyear_last_month')
     def _check_fiscalyear_last_day(self):
         # if the user explicitly chooses the 29th of February we allow it:
         # there is no "fiscalyear_last_year" so we do not know his intentions.
-        if self.fiscalyear_last_day == 29 and self.fiscalyear_last_month == 2:
-            return
+        for rec in self:
+            if rec.fiscalyear_last_day == 29 and rec.fiscalyear_last_month == '2':
+                continue
 
-        if self.account_opening_date:
-            year = self.account_opening_date.year
-        else:
-            year = datetime.now().year
+            if rec.account_opening_date:
+                year = rec.account_opening_date.year
+            else:
+                year = datetime.now().year
 
-        max_day = calendar.monthrange(year, self.fiscalyear_last_month)[1]
-        if self.fiscalyear_last_day > max_day:
-            raise ValidationError(_("Invalid fiscal year last day"))
+            max_day = calendar.monthrange(year, int(rec.fiscalyear_last_month))[1]
+            if rec.fiscalyear_last_day > max_day:
+                raise ValidationError(_("Invalid fiscal year last day"))
 
     def get_and_update_account_invoice_onboarding_state(self):
         """ This method is called on the controller rendering method and ensures that the animations
@@ -125,68 +144,20 @@ Best Regards,'''))
     def get_and_update_account_dashboard_onboarding_state(self):
         """ This method is called on the controller rendering method and ensures that the animations
             are displayed only one time. """
-        return self.get_and_update_onbarding_state('account_dashboard_onboarding_state', [
+        return self.get_and_update_onbarding_state(
+            'account_dashboard_onboarding_state',
+            self.get_account_dashboard_onboarding_steps_states_names()
+        )
+
+    def get_account_dashboard_onboarding_steps_states_names(self):
+        """ Necessary to add/edit steps from other modules (account_winbooks_import in this case). """
+        return [
             'base_onboarding_company_state',
             'account_setup_bank_data_state',
             'account_setup_fy_data_state',
             'account_setup_coa_state',
-        ])
+        ]
 
-    @api.multi
-    def _check_lock_dates(self, vals):
-        '''Check the lock dates for the current companies. This can't be done in a api.constrains because we need
-        to perform some comparison between new/old values. This method forces the lock dates to be irreversible.
-
-        * You cannot define stricter conditions on advisors than on users. Then, the lock date on advisor must be set
-        after the lock date for users.
-        * You cannot lock a period that is not finished yet. Then, the lock date for advisors must be set after the
-        last day of the previous month.
-        * The new lock date for advisors must be set after the previous lock date.
-
-        :param vals: The values passed to the write method.
-        '''
-        period_lock_date = vals.get('period_lock_date') and\
-            fields.Date.from_string(vals['period_lock_date'])
-        fiscalyear_lock_date = vals.get('fiscalyear_lock_date') and\
-            fields.Date.from_string(vals['fiscalyear_lock_date'])
-
-        previous_month = fields.Date.today() + relativedelta(months=-1)
-        days_previous_month = calendar.monthrange(previous_month.year, previous_month.month)
-        previous_month = previous_month.replace(day=days_previous_month[1])
-        for company in self:
-            old_fiscalyear_lock_date = company.fiscalyear_lock_date
-
-            # The user attempts to remove the lock date for advisors
-            if old_fiscalyear_lock_date and not fiscalyear_lock_date and 'fiscalyear_lock_date' in vals:
-                raise ValidationError(_('The lock date for advisors is irreversible and can\'t be removed.'))
-
-            # The user attempts to set a lock date for advisors prior to the previous one
-            if old_fiscalyear_lock_date and fiscalyear_lock_date and fiscalyear_lock_date < old_fiscalyear_lock_date:
-                raise ValidationError(_('The new lock date for advisors must be set after the previous lock date.'))
-
-            # In case of no new fiscal year in vals, fallback to the oldest
-            if not fiscalyear_lock_date:
-                if old_fiscalyear_lock_date:
-                    fiscalyear_lock_date = old_fiscalyear_lock_date
-                else:
-                    continue
-
-            # The user attempts to set a lock date for advisors prior to the last day of previous month
-            if fiscalyear_lock_date > previous_month:
-                raise ValidationError(_('You cannot lock a period that is not finished yet. Please make sure that the lock date for advisors is not set after the last day of the previous month.'))
-
-            # In case of no new period lock date in vals, fallback to the one defined in the company
-            if not period_lock_date:
-                if company.period_lock_date:
-                    period_lock_date = company.period_lock_date
-                else:
-                    continue
-
-            # The user attempts to set a lock date for advisors prior to the lock date for users
-            if period_lock_date < fiscalyear_lock_date:
-                raise ValidationError(_('You cannot define stricter conditions on advisors than on users. Please make sure that the lock date on advisor is set before the lock date for users.'))
-
-    @api.multi
     def compute_fiscalyear_dates(self, current_date):
         '''Computes the start and end dates of the fiscal year where the given 'date' belongs to.
 
@@ -214,7 +185,7 @@ Best Regards,'''))
             }
 
         date_from, date_to = date_utils.get_fiscal_year(
-            current_date, day=self.fiscalyear_last_day, month=self.fiscalyear_last_month)
+            current_date, day=self.fiscalyear_last_day, month=int(self.fiscalyear_last_month))
 
         date_from_str = date_from.strftime(DEFAULT_SERVER_DATE_FORMAT)
         date_to_str = date_to.strftime(DEFAULT_SERVER_DATE_FORMAT)
@@ -253,7 +224,6 @@ Best Regards,'''))
             if account.code.startswith(old_code):
                 account.write({'code': self.get_new_account_code(account.code, old_code, new_code)})
 
-    @api.multi
     def _validate_fiscalyear_lock(self, values):
         if values.get('fiscalyear_lock_date'):
             nb_draft_entries = self.env['account.move'].search([
@@ -263,7 +233,6 @@ Best Regards,'''))
             if nb_draft_entries:
                 raise ValidationError(_('There are still unposted entries in the period you want to lock. You should either post or delete them.'))
 
-    @api.multi
     def write(self, values):
         #restrict the closing of FY if there are still unposted entries
         self._validate_fiscalyear_lock(values)
@@ -293,21 +262,20 @@ Best Regards,'''))
                 'res_model': 'account.setup.bank.manual.config',
                 'target': 'new',
                 'view_mode': 'form',
-                'view_type': 'form',
                 'views': [[view_id, 'form']],
         }
 
     @api.model
     def setting_init_fiscal_year_action(self):
         """ Called by the 'Fiscal Year Opening' button of the setup bar."""
-        company = self.env.user.company_id
+        company = self.env.company
         company.create_op_move_if_non_existant()
         new_wizard = self.env['account.financial.year.op'].create({'company_id': company.id})
         view_id = self.env.ref('account.setup_financial_year_opening_form').id
 
         return {
             'type': 'ir.actions.act_window',
-            'name': _('Fiscal Year'),
+            'name': _('Accounting Periods'),
             'view_mode': 'form',
             'res_model': 'account.financial.year.op',
             'target': 'new',
@@ -318,8 +286,8 @@ Best Regards,'''))
     @api.model
     def setting_chart_of_accounts_action(self):
         """ Called by the 'Chart of Accounts' button of the setup bar."""
-        company = self.env.user.company_id
-        company.set_onboarding_step_done('account_setup_coa_state')
+        company = self.env.company
+        company.sudo().set_onboarding_step_done('account_setup_coa_state')
 
         # If an opening move has already been posted, we open the tree view showing all the accounts
         if company.opening_move_posted():
@@ -357,7 +325,7 @@ Best Regards,'''))
                 raise UserError(_("Please install a chart of accounts or create a miscellaneous journal before proceeding."))
 
             today = datetime.today().date()
-            opening_date = today.replace(month=self.fiscalyear_last_month, day=self.fiscalyear_last_day) + timedelta(days=1)
+            opening_date = today.replace(month=int(self.fiscalyear_last_month), day=self.fiscalyear_last_day) + timedelta(days=1)
             if opening_date > today:
                 opening_date = opening_date + relativedelta(years=-1)
 
@@ -415,22 +383,27 @@ Best Regards,'''))
         current year earnings account.
         """
         if self.account_opening_move_id and self.account_opening_move_id.state == 'draft':
-            debit_diff, credit_diff = self.get_opening_move_differences(self.account_opening_move_id.line_ids)
-
+            balancing_account = self.get_unaffected_earnings_account()
             currency = self.currency_id
-            balancing_move_line = self.account_opening_move_id.line_ids.filtered(lambda x: x.account_id == self.get_unaffected_earnings_account())
+
+            balancing_move_line = self.account_opening_move_id.line_ids.filtered(lambda x: x.account_id == balancing_account)
+            # There could be multiple lines if we imported the balance from unaffected earnings account too
+            if len(balancing_move_line) > 1:
+                self.with_context(check_move_validity=False).account_opening_move_id.line_ids -= balancing_move_line[1:]
+                balancing_move_line = balancing_move_line[0]
+
+            debit_diff, credit_diff = self.get_opening_move_differences(self.account_opening_move_id.line_ids)
 
             if float_is_zero(debit_diff + credit_diff, precision_rounding=currency.rounding):
                 if balancing_move_line:
                     # zero difference and existing line : delete the line
-                    balancing_move_line.unlink()
+                    self.account_opening_move_id.line_ids -= balancing_move_line
             else:
                 if balancing_move_line:
                     # Non-zero difference and existing line : edit the line
                     balancing_move_line.write({'debit': credit_diff, 'credit': debit_diff})
                 else:
                     # Non-zero difference and no existing line : create a new line
-                    balancing_account = self.get_unaffected_earnings_account()
                     self.env['account.move.line'].create({
                         'name': _('Automatic Balancing Line'),
                         'move_id': self.account_opening_move_id.id,
@@ -442,25 +415,18 @@ Best Regards,'''))
     @api.model
     def action_close_account_invoice_onboarding(self):
         """ Mark the invoice onboarding panel as closed. """
-        self.env.user.company_id.account_invoice_onboarding_state = 'closed'
+        self.env.company.account_invoice_onboarding_state = 'closed'
 
     @api.model
     def action_close_account_dashboard_onboarding(self):
         """ Mark the dashboard onboarding panel as closed. """
-        self.env.user.company_id.account_dashboard_onboarding_state = 'closed'
-
-    @api.model
-    def action_open_account_onboarding_invoice_layout(self):
-        """ Onboarding step for the invoice layout. """
-        action = self.env.ref('account.action_open_account_onboarding_invoice_layout').read()[0]
-        action['res_id'] = self.env.user.company_id.id
-        return action
+        self.env.company.account_dashboard_onboarding_state = 'closed'
 
     @api.model
     def action_open_account_onboarding_sale_tax(self):
         """ Onboarding step for the invoice layout. """
         action = self.env.ref('account.action_open_account_onboarding_sale_tax').read()[0]
-        action['res_id'] = self.env.user.company_id.id
+        action['res_id'] = self.env.company.id
         return action
 
     @api.model
@@ -469,15 +435,15 @@ Best Regards,'''))
         # use current user as partner
         partner = self.env.user.partner_id
 
-        company_id = self.env.user.company_id.id
+        company_id = self.env.company.id
         # try to find an existing sample invoice
-        sample_invoice = self.env['account.invoice'].search(
+        sample_invoice = self.env['account.move'].search(
             [('company_id', '=', company_id),
              ('partner_id', '=', partner.id)], limit=1)
 
         if len(sample_invoice) == 0:
             # If there are no existing accounts or no journal, fail
-            account = self.env.user.company_id.get_chart_of_accounts_or_fail()
+            account = self.env.company.get_chart_of_accounts_or_fail()
 
             journal = self.env['account.journal'].search([('company_id', '=', company_id)], limit=1)
             if len(journal) == 0:
@@ -486,25 +452,24 @@ Best Regards,'''))
                         "\nPlease go to Configuration > Journals.")
                 raise RedirectWarning(msg, action.id, _("Go to the journal configuration"))
 
-            sample_invoice = self.env['account.invoice'].create({
-                'name': _("Sample invoice"),
-                'journal_id': journal.id,
+            account = journal.default_credit_account_id or account
+            sample_invoice = self.env['account.move'].with_context(default_type='out_invoice', default_journal_id=journal.id).create({
+                'invoice_payment_ref': _('Sample invoice'),
                 'partner_id': partner.id,
-            })
-            # sample invoice lines
-            self.env['account.invoice.line'].create({
-                'name': _("Sample invoice line name"),
-                'invoice_id': sample_invoice.id,
-                'account_id': account.id,
-                'price_unit': 199.99,
-                'quantity': 2,
-            })
-            self.env['account.invoice.line'].create({
-                'name': _("Sample invoice line name 2"),
-                'invoice_id': sample_invoice.id,
-                'account_id': account.id,
-                'price_unit': 25,
-                'quantity': 1,
+                'invoice_line_ids': [
+                    (0, 0, {
+                        'name': _('Sample invoice line name'),
+                        'account_id': account.id,
+                        'quantity': 2,
+                        'price_unit': 199.99,
+                    }),
+                    (0, 0, {
+                        'name': _('Sample invoice line name 2'),
+                        'account_id': account.id,
+                        'quantity': 1,
+                        'price_unit': 25.0,
+                    }),
+                ],
             })
         return sample_invoice
 
@@ -519,7 +484,7 @@ Best Regards,'''))
             'default_res_id': sample_invoice.id,
             'default_use_template': bool(template),
             'default_template_id': template and template.id or False,
-            'default_model': 'account.invoice',
+            'default_model': 'account.move',
             'default_composition_mode': 'comment',
             'mark_invoice_as_sent': True,
             'custom_layout': 'mail.mail_notification_borders',
@@ -528,13 +493,11 @@ Best Regards,'''))
         }
         return action
 
-    @api.multi
     def action_save_onboarding_invoice_layout(self):
         """ Set the onboarding step as done """
-        if bool(self.logo) and self.logo != self._get_logo():
+        if bool(self.external_report_layout_id):
             self.set_onboarding_step_done('account_onboarding_invoice_layout_state')
 
-    @api.multi
     def action_save_onboarding_sale_tax(self):
         """ Set the onboarding step as done """
         self.set_onboarding_step_done('account_onboarding_sale_tax_state')
@@ -548,3 +511,95 @@ Best Regards,'''))
                 "Please go to Account Configuration and select or install a fiscal localization.")
             raise RedirectWarning(msg, action.id, _("Go to the configuration panel"))
         return account
+
+    @api.model
+    def _action_check_hash_integrity(self):
+        return self.env.ref('account.action_report_account_hash_integrity').report_action(self.id)
+
+    def _check_hash_integrity(self):
+        """Checks that all posted moves have still the same data as when they were posted
+        and raises an error with the result.
+        """
+        def build_move_info(move):
+            return(move.name, move.inalterable_hash, fields.Date.to_string(move.date))
+
+        journals = self.env['account.journal'].search([('company_id', '=', self.id)])
+        results_by_journal = {
+            'results': [],
+            'printing_date': format_date(self.env, fields.Date.to_string(fields.Date.today()))
+        }
+
+        for journal in journals:
+            rslt = {
+                'journal_name': journal.name,
+                'journal_code': journal.code,
+                'restricted_by_hash_table': journal.restrict_mode_hash_table and 'V' or 'X',
+                'msg_cover': '',
+                'first_hash': 'None',
+                'first_move_name': 'None',
+                'first_move_date': 'None',
+                'last_hash': 'None',
+                'last_move_name': 'None',
+                'last_move_date': 'None',
+            }
+            if not journal.restrict_mode_hash_table:
+                rslt.update({'msg_cover': _('This journal is not in strict mode.')})
+                results_by_journal['results'].append(rslt)
+                continue
+
+            all_moves_count = self.env['account.move'].search_count([('state', '=', 'posted'), ('journal_id', '=', journal.id)])
+            moves = self.env['account.move'].search([('state', '=', 'posted'), ('journal_id', '=', journal.id),
+                                            ('secure_sequence_number', '!=', 0)], order="secure_sequence_number ASC")
+            if not moves:
+                rslt.update({
+                    'msg_cover': _('There isn\'t any journal entry flagged for data inalterability yet for this journal.'),
+                })
+                results_by_journal['results'].append(rslt)
+                continue
+
+            previous_hash = u''
+            start_move_info = []
+            hash_corrupted = False
+            for move in moves:
+                if move.inalterable_hash != move._compute_hash(previous_hash=previous_hash):
+                    rslt.update({'msg_cover': _('Corrupted data on journal entry with id %s.') % move.id})
+                    results_by_journal['results'].append(rslt)
+                    hash_corrupted = True
+                    break
+                if not previous_hash:
+                    #save the date and sequence number of the first move hashed
+                    start_move_info = build_move_info(move)
+                previous_hash = move.inalterable_hash
+            end_move_info = build_move_info(move)
+
+            if hash_corrupted:
+                continue
+
+            rslt.update({
+                        'first_move_name': start_move_info[0],
+                        'first_hash': start_move_info[1],
+                        'first_move_date': format_date(self.env, start_move_info[2]),
+                        'last_move_name': end_move_info[0],
+                        'last_hash': end_move_info[1],
+                        'last_move_date': format_date(self.env, end_move_info[2]),
+                    })
+            if len(moves) == all_moves_count:
+                rslt.update({'msg_cover': _('All entries are hashed.')})
+            else:
+                rslt.update({'msg_cover': _('Entries are hashed from %s (%s)') % (start_move_info[0], format_date(self.env, start_move_info[2]))})
+            results_by_journal['results'].append(rslt)
+
+        return results_by_journal
+
+    def get_fiscal_country(self):
+        """ Returns the country to be used for this company's tax report.
+        By default, it'll be the one from the address; but a config parameter
+        may be used for each company to customize that behavior.
+        """
+        self.ensure_one()
+
+        fiscal_country_key = 'account_fiscal_country_%s' % self.id
+        forced_country_code = self.env['ir.config_parameter'].sudo().get_param(fiscal_country_key)
+        forced_country = forced_country_code and self.env['res.country'].search([('code', 'ilike', forced_country_code)], limit=1)
+
+        return forced_country or self.country_id
