@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 {
-    'name' : 'Import Vendor Bills From XML',
+    'name' : 'Import Bills/Invoices From XML',
     'version' : '1.0',
-    'category': 'Accounting',
+    'category': 'Accounting/Accounting',
     'depends' : ['account'],
     'data': [
         'data/facturx_templates.xml',
