@@ -1,4 +1,4 @@
 # -*- encoding: utf-8 -*-
 
-from . import account_invoice
+from . import account_move
 from . import ir_actions_report
