@@ -1,16 +1,16 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 {
-    'name' : 'Authentication via LDAP',
-    'depends' : ['base', 'base_setup'],
+    'name': 'Authentication via LDAP',
+    'depends': ['base', 'base_setup'],
     #'description': < auto-loaded from README file
-    'category' : 'Extra Tools',
-    'data' : [
+    'category': 'Tools',
+    'data': [
         'views/ldap_installer_views.xml',
         'security/ir.model.access.csv',
         'views/res_config_settings_views.xml',
     ],
-    'external_dependencies' : {
-        'python' : ['ldap'],
+    'external_dependencies': {
+        'python': ['ldap'],
     }
 }
