@@ -1,15 +1,14 @@
 {
     'name': 'Barcode',
     'version': '2.0',
-    'category': 'Extra Tools',
-    'summary': 'Barcodes Scanning and Parsing',
+    'category': 'Tools',
+    'summary': 'Scan and Parse Barcodes',
     'depends': ['web'],
     'data': [
         'data/barcodes_data.xml',
         'views/barcodes_view.xml',
         'security/ir.model.access.csv',
         'views/barcodes_templates.xml',
-        'views/res_company.xml',
     ],
     'installable': True,
     'auto_install': False,
