@@ -7,6 +7,7 @@ from odoo.tools.misc import find_in_path
 from odoo.tools import config
 from odoo.sql_db import TestCursor
 from odoo.http import request
+from odoo.osv.expression import NEGATIVE_TERM_OPERATORS, FALSE_DOMAIN
 
 import time
 import base64
@@ -17,6 +18,7 @@ import lxml.html
 import tempfile
 import subprocess
 import re
+import json
 
 from lxml import etree
 from contextlib import closing
@@ -24,6 +26,10 @@ from distutils.version import LooseVersion
 from reportlab.graphics.barcode import createBarcodeDrawing
 from PyPDF2 import PdfFileWriter, PdfFileReader
 from collections import OrderedDict
+from collections.abc import Iterable
+from PIL import Image, ImageFile
+# Allow truncated images
+ImageFile.LOAD_TRUNCATED_IMAGES = True
 
 
 _logger = logging.getLogger(__name__)
@@ -85,7 +91,8 @@ class IrActionsReport(models.Model):
     name = fields.Char(translate=True)
     type = fields.Char(default='ir.actions.report')
     binding_type = fields.Selection(default='report')
-    model = fields.Char(required=True)
+    model = fields.Char(required=True, string='Model Name')
+    model_id = fields.Many2one('ir.model', string='Model', compute='_compute_model_id', search='_search_model_id')
 
     report_type = fields.Selection([
         ('qweb-html', 'HTML'),
@@ -104,14 +111,39 @@ class IrActionsReport(models.Model):
     multi = fields.Boolean(string='On Multiple Doc.', help="If set to true, the action will not be displayed on the right toolbar of a form view.")
 
     paperformat_id = fields.Many2one('report.paperformat', 'Paper Format')
-    print_report_name = fields.Char('Printed Report Name',
+    print_report_name = fields.Char('Printed Report Name', translate=True,
                                     help="This is the filename of the report going to download. Keep empty to not change the report filename. You can use a python expression with the 'object' and 'time' variables.")
     attachment_use = fields.Boolean(string='Reload from Attachment',
                                     help='If you check this, then the second time the user prints with same attachment name, it returns the previous report.')
     attachment = fields.Char(string='Save as Attachment Prefix',
                              help='This is the filename of the attachment used to store the printing result. Keep empty to not save the printed reports. You can use a python expression with the object and time variables.')
 
-    @api.multi
+    @api.depends('model')
+    def _compute_model_id(self):
+        for action in self:
+            action.model_id = self.env['ir.model']._get(action.model).id
+
+    def _search_model_id(self, operator, value):
+        ir_model_ids = None
+        if isinstance(value, str):
+            names = self.env['ir.model'].name_search(value, operator=operator)
+            ir_model_ids = [n[0] for n in names]
+
+        elif isinstance(value, Iterable):
+            ir_model_ids = value
+
+        elif isinstance(value, int) and not isinstance(value, bool):
+            ir_model_ids = [value]
+
+        if ir_model_ids:
+            operator = 'not in' if operator in NEGATIVE_TERM_OPERATORS else 'in'
+            ir_model = self.env['ir.model'].browse(ir_model_ids)
+            return [('model', operator, ir_model.mapped('model'))]
+        elif isinstance(value, bool) or value is None:
+            return [('model', operator, value)]
+        else:
+            return FALSE_DOMAIN
+
     def associated_view(self):
         """Used in the ir.actions.report form view in order to search naively after the view(s)
         used in the rendering.
@@ -124,7 +156,6 @@ class IrActionsReport(models.Model):
         action_data['domain'] = [('name', 'ilike', self.report_name.split('.')[1]), ('type', '=', 'qweb')]
         return action_data
 
-    @api.multi
     def create_action(self):
         """ Create a contextual action for each report. """
         for report in self:
@@ -132,7 +163,6 @@ class IrActionsReport(models.Model):
             report.write({'binding_model_id': model.id, 'binding_type': 'report'})
         return True
 
-    @api.multi
     def unlink_action(self):
         """ Remove the contextual actions created for the reports. """
         self.check_access_rights('write', raise_exception=True)
@@ -142,7 +172,17 @@ class IrActionsReport(models.Model):
     #--------------------------------------------------------------------------
     # Main report methods
     #--------------------------------------------------------------------------
-    @api.multi
+    def _retrieve_stream_from_attachment(self, attachment):
+        #This import is needed to make sure a PDF stream can be saved in Image
+        from PIL import PdfImagePlugin
+        if attachment.mimetype.startswith('image'):
+            stream = io.BytesIO(base64.b64decode(attachment.datas))
+            img = Image.open(stream)
+            output_stream = io.BytesIO()
+            img.convert("RGB").save(output_stream, format="pdf")
+            return output_stream
+        return io.BytesIO(base64.decodestring(attachment.datas))
+
     def retrieve_attachment(self, record):
         '''Retrieve an attachment for a specific record.
 
@@ -150,16 +190,15 @@ class IrActionsReport(models.Model):
         :param attachment_name: The optional name of the attachment.
         :return: A recordset of length <=1 or None
         '''
-        attachment_name = safe_eval(self.attachment, {'object': record, 'time': time})
+        attachment_name = safe_eval(self.attachment, {'object': record, 'time': time}) if self.attachment else ''
         if not attachment_name:
             return None
         return self.env['ir.attachment'].search([
-                ('datas_fname', '=', attachment_name),
+                ('name', '=', attachment_name),
                 ('res_model', '=', self.model),
                 ('res_id', '=', record.id)
         ], limit=1)
 
-    @api.multi
     def postprocess_pdf_report(self, record, buffer):
         '''Hook to handle post processing during the pdf report generation.
         The basic behavior consists to create a new attachment containing the pdf
@@ -175,9 +214,9 @@ class IrActionsReport(models.Model):
         attachment_vals = {
             'name': attachment_name,
             'datas': base64.encodestring(buffer.getvalue()),
-            'datas_fname': attachment_name,
             'res_model': self.model,
             'res_id': record.id,
+            'type': 'binary',
         }
         try:
             self.env['ir.attachment'].create(attachment_vals)
@@ -202,7 +241,7 @@ class IrActionsReport(models.Model):
 
     @api.model
     def get_paperformat(self):
-        return self.paperformat_id or self.env.user.company_id.paperformat_id
+        return self.paperformat_id or self.env.company.paperformat_id
 
     @api.model
     def _build_wkhtmltopdf_args(
@@ -282,7 +321,6 @@ class IrActionsReport(models.Model):
 
         return command_args
 
-    @api.multi
     def _prepare_html(self, html):
         '''Divide and recreate the header/footer html by merging all found in html.
         The bodies are extracted and added to a list. Then, extract the specific_paperformat_args.
@@ -454,25 +492,31 @@ class IrActionsReport(models.Model):
         return report_obj.with_context(context).search(conditions, limit=1)
 
     @api.model
-    def barcode(self, barcode_type, value, width=600, height=100, humanreadable=0):
+    def barcode(self, barcode_type, value, width=600, height=100, humanreadable=0, quiet=1):
         if barcode_type == 'UPCA' and len(value) in (11, 12, 13):
             barcode_type = 'EAN13'
             if len(value) in (11, 12):
                 value = '0%s' % value
         try:
-            width, height, humanreadable = int(width), int(height), bool(int(humanreadable))
+            width, height, humanreadable, quiet = int(width), int(height), bool(int(humanreadable)), bool(int(quiet))
+            # for `QR` type, `quiet` is not supported. And is simply ignored.
+            # But we can use `barBorder` to get a similar behaviour.
+            bar_border = 4
+            if barcode_type == 'QR' and quiet:
+                bar_border = 0
+
             barcode = createBarcodeDrawing(
                 barcode_type, value=value, format='png', width=width, height=height,
-                humanReadable=humanreadable
+                humanReadable=humanreadable, quiet=quiet, barBorder=bar_border
             )
             return barcode.asString('png')
         except (ValueError, AttributeError):
             if barcode_type == 'Code128':
                 raise ValueError("Cannot convert into barcode.")
             else:
-                return self.barcode('Code128', value, width=width, height=height, humanreadable=humanreadable)
+                return self.barcode('Code128', value, width=width, height=height,
+                    humanreadable=humanreadable, quiet=quiet)
 
-    @api.multi
     def render_template(self, template, values=None):
         """Allow to render a QWeb template python-side. This function returns the 'ir.ui.view'
         render but embellish it with some variables/methods used in reports.
@@ -482,7 +526,7 @@ class IrActionsReport(models.Model):
         if values is None:
             values = {}
 
-        context = dict(self.env.context, inherit_branding=values.get('enable_editor'))
+        context = dict(self.env.context, inherit_branding=False)
 
         # Browse the user instead of using the sudo self.env.user
         user = self.env['res.users'].browse(self.env.uid)
@@ -496,7 +540,6 @@ class IrActionsReport(models.Model):
         values.update(
             time=time,
             context_timestamp=lambda t: fields.Datetime.context_timestamp(self.with_context(tz=user.tz), t),
-            editable=values.get('enable_editor'),
             user=user,
             res_company=user.company_id,
             website=website,
@@ -504,7 +547,6 @@ class IrActionsReport(models.Model):
         )
         return view_obj.render_template(template, values)
 
-    @api.multi
     def _post_pdf(self, save_in_attachment, pdf_content=None, res_ids=None):
         '''Merge the existing attachments by adding one by one the content of the attachments
         and then, we add the pdf_content if exists. Create the attachments for each record individually
@@ -524,8 +566,10 @@ class IrActionsReport(models.Model):
                     pass
 
         # Check special case having only one record with existing attachment.
+        # In that case, return directly the attachment content.
+        # In that way, we also ensure the embedded files are well preserved.
         if len(save_in_attachment) == 1 and not pdf_content:
-            return base64.decodestring(list(save_in_attachment.values())[0].datas)
+            return list(save_in_attachment.values())[0].getvalue()
 
         # Create a list of streams representing all sub-reports part of the final result
         # in order to append the existing attachments and the potentially modified sub-reports
@@ -553,15 +597,25 @@ class IrActionsReport(models.Model):
                     streams.append(pdf_content_stream)
                 else:
                     # In case of multiple docs, we need to split the pdf according the records.
-                    # To do so, we split the pdf based on outlines computed by wkhtmltopdf.
+                    # To do so, we split the pdf based on top outlines computed by wkhtmltopdf.
                     # An outline is a <h?> html tag found on the document. To retrieve this table,
-                    # we look on the pdf structure using pypdf to compute the outlines_pages that is
-                    # an array like [0, 3, 5] that means a new document start at page 0, 3 and 5.
+                    # we look on the pdf structure using pypdf to compute the outlines_pages from
+                    # the top level heading in /Outlines.
                     reader = PdfFileReader(pdf_content_stream)
-                    if reader.trailer['/Root'].get('/Dests'):
-                        outlines_pages = sorted(
-                            [outline.getObject()[0] for outline in reader.trailer['/Root']['/Dests'].values()])
+                    root = reader.trailer['/Root']
+                    if '/Outlines' in root and '/First' in root['/Outlines']:
+                        outlines_pages = []
+                        node = root['/Outlines']['/First']
+                        while True:
+                            outlines_pages.append(root['/Dests'][node['/Dest']][0])
+                            if '/Next' not in node:
+                                break
+                            node = node['/Next']
+                        outlines_pages = sorted(set(outlines_pages))
+                        # There should be only one top-level heading by document
                         assert len(outlines_pages) == len(res_ids)
+                        # There should be a top-level heading on first page
+                        assert outlines_pages[0] == 0
                         for i, num in enumerate(outlines_pages):
                             to = outlines_pages[i + 1] if i + 1 < len(outlines_pages) else reader.numPages
                             attachment_writer = PdfFileWriter()
@@ -584,37 +638,35 @@ class IrActionsReport(models.Model):
         # If attachment_use is checked, the records already having an existing attachment
         # are not been rendered by wkhtmltopdf. So, create a new stream for each of them.
         if self.attachment_use:
-            for attachment_id in save_in_attachment.values():
-                content = base64.decodestring(attachment_id.datas)
-                streams.append(io.BytesIO(content))
+            for stream in save_in_attachment.values():
+                streams.append(stream)
 
         # Build the final pdf.
         # If only one stream left, no need to merge them (and then, preserve embedded files).
         if len(streams) == 1:
             result = streams[0].getvalue()
         else:
-            writer = PdfFileWriter()
-            for stream in streams:
-                reader = PdfFileReader(stream)
-                writer.appendPagesFromReader(reader)
-            result_stream = io.BytesIO()
-            streams.append(result_stream)
-            writer.write(result_stream)
-            result = result_stream.getvalue()
+            result = self._merge_pdfs(streams)
 
         # We have to close the streams after PdfFileWriter's call to write()
         close_streams(streams)
         return result
 
-    @api.multi
+    def _merge_pdfs(self, streams):
+        writer = PdfFileWriter()
+        for stream in streams:
+            reader = PdfFileReader(stream)
+            writer.appendPagesFromReader(reader)
+        result_stream = io.BytesIO()
+        streams.append(result_stream)
+        writer.write(result_stream)
+        return result_stream.getvalue()
+
     def render_qweb_pdf(self, res_ids=None, data=None):
         if not data:
             data = {}
         data.setdefault('report_type', 'pdf')
 
-        # remove editor feature in pdf generation
-        data.update(enable_editor=False)
-
         # In case of test environment without enough workers to perform calls to wkhtmltopdf,
         # fallback to render_html.
         if (tools.config['test_enable'] or tools.config['test_file']) and not self.env.context.get('force_report_rendering'):
@@ -656,10 +708,10 @@ class IrActionsReport(models.Model):
             wk_record_ids = Model
             if self.attachment:
                 for record_id in record_ids:
-                    attachment_id = self.retrieve_attachment(record_id)
-                    if attachment_id:
-                        save_in_attachment[record_id.id] = attachment_id
-                    if not self.attachment_use or not attachment_id:
+                    attachment = self.retrieve_attachment(record_id)
+                    if attachment:
+                        save_in_attachment[record_id.id] = self._retrieve_stream_from_attachment(attachment)
+                    if not self.attachment_use or not attachment:
                         wk_record_ids += record_id
             else:
                 wk_record_ids = record_ids
@@ -699,7 +751,7 @@ class IrActionsReport(models.Model):
             set_viewport_size=context.get('set_viewport_size'),
         )
         if res_ids:
-            _logger.info('The PDF report has been generated for records %s.' % (str(res_ids)))
+            _logger.info('The PDF report has been generated for model: %s, records %s.' % (self.model, str(res_ids)))
             return self._post_pdf(save_in_attachment, pdf_content=pdf_content, res_ids=html_ids), 'pdf'
         return pdf_content, 'pdf'
 
@@ -745,7 +797,6 @@ class IrActionsReport(models.Model):
             })
         return data
 
-    @api.multi
     def render(self, res_ids, data=None):
         report_type = self.report_type.lower().replace('-', '_')
         render_func = getattr(self, 'render_' + report_type, None)
@@ -753,29 +804,12 @@ class IrActionsReport(models.Model):
             return None
         return render_func(res_ids, data=data)
 
-    @api.noguess
     def report_action(self, docids, data=None, config=True):
         """Return an action of type ir.actions.report.
 
         :param docids: id/ids/browserecord of the records to print (if not used, pass an empty list)
         :param report_name: Name of the template to generate an action for
         """
-        discard_logo_check = self.env.context.get('discard_logo_check')
-        if (self.env.user._is_admin()) and ((not self.env.user.company_id.external_report_layout_id) or (not discard_logo_check and not self.env.user.company_id.logo)) and config:
-            template = self.env.ref('base.view_company_report_form_with_print') if self.env.context.get('from_transient_model', False) else self.env.ref('base.view_company_report_form')
-            return {
-                'name': _('Choose Your Document Layout'),
-                'type': 'ir.actions.act_window',
-                'context': {'default_report_name': self.report_name, 'discard_logo_check': True},
-                'view_type': 'form',
-                'view_mode': 'form',
-                'res_id': self.env.user.company_id.id,
-                'res_model': 'res.company',
-                'views': [(template.id, 'form')],
-                'view_id': template.id,
-                'target': 'new',
-            }
-
         context = self.env.context
         if docids:
             if isinstance(docids, models.Model):
@@ -786,7 +820,7 @@ class IrActionsReport(models.Model):
                 active_ids = docids
             context = dict(self.env.context, active_ids=active_ids)
 
-        return {
+        report_action = {
             'context': context,
             'data': data,
             'type': 'ir.actions.report',
@@ -795,3 +829,15 @@ class IrActionsReport(models.Model):
             'report_file': self.report_file,
             'name': self.name,
         }
+
+        discard_logo_check = self.env.context.get('discard_logo_check')
+        if self.env.is_admin() and not self.env.company.external_report_layout_id and config and not discard_logo_check:
+            action = self.env.ref('base.action_base_document_layout_configurator').read()[0]
+            ctx = action.get('context')
+            py_ctx = json.loads(ctx) if ctx else {}
+            report_action['close_on_report_download'] = True
+            py_ctx['report_action'] = report_action
+            action['context'] = py_ctx
+            return action
+
+        return report_action
