@@ -8,7 +8,7 @@ import psycopg2
 import pytz
 
 from odoo import api, fields, models, _
-from odoo.tools import ustr, pycompat
+from odoo.tools import ustr
 
 REFERENCING_FIELDS = {None, 'id', '.id'}
 def only_ref_fields(record):
@@ -39,9 +39,9 @@ class IrFieldsConverter(models.AbstractModel):
     @api.model
     def _format_import_error(self, error_type, error_msg, error_params=(), error_args=None):
         # sanitize error params for later formatting by the import system
-        sanitize = lambda p: p.replace('%', '%%') if isinstance(p, pycompat.string_types) else p
+        sanitize = lambda p: p.replace('%', '%%') if isinstance(p, str) else p
         if error_params:
-            if isinstance(error_params, pycompat.string_types):
+            if isinstance(error_params, str):
                 error_params = sanitize(error_params)
             elif isinstance(error_params, dict):
                 error_params = {k: sanitize(v) for k, v in error_params.items()}
@@ -79,7 +79,7 @@ class IrFieldsConverter(models.AbstractModel):
                 try:
                     converted[field], ws = converters[field](value)
                     for w in ws:
-                        if isinstance(w, pycompat.string_types):
+                        if isinstance(w, str):
                             # wrap warning string in an ImportWarning for
                             # uniform handling
                             w = ImportWarning(w)
@@ -270,14 +270,14 @@ class IrFieldsConverter(models.AbstractModel):
         for item, label in selection:
             label = ustr(label)
             labels = [label] + self._get_translations(('selection', 'model', 'code'), label)
-            if value == pycompat.text_type(item) or value in labels:
+            if value == str(item) or value in labels:
                 return item, []
 
         raise self._format_import_error(
             ValueError,
             _(u"Value '%s' not found in selection field '%%(field)s'"),
             value,
-            {'moreinfo': [_label or pycompat.text_type(item) for item, _label in selection if _label or item]}
+            {'moreinfo': [_label or str(item) for item, _label in selection if _label or item]}
         )
 
     @api.model
@@ -305,10 +305,13 @@ class IrFieldsConverter(models.AbstractModel):
         id = None
         warnings = []
         error_msg = ''
-        action = {'type': 'ir.actions.act_window', 'target': 'new',
-                  'view_mode': 'tree,form', 'view_type': 'form',
-                  'views': [(False, 'tree'), (False, 'form')],
-                  'help': _(u"See all possible values")}
+        action = {
+            'name': 'Possible Values',
+            'type': 'ir.actions.act_window', 'target': 'new',
+            'view_mode': 'tree,form',
+            'views': [(False, 'list'), (False, 'form')],
+            'context': {'create': False},
+            'help': _(u"See all possible values")}
         if subfield is None:
             action['res_model'] = field.comodel_name
         elif subfield in ('id', '.id'):
@@ -318,6 +321,8 @@ class IrFieldsConverter(models.AbstractModel):
         RelatedModel = self.env[field.comodel_name]
         if subfield == '.id':
             field_type = _(u"database id")
+            if isinstance(value, str) and not self._str_to_boolean(model, field, value)[0]:
+                return False, field_type, warnings
             try: tentative_id = int(value)
             except ValueError: tentative_id = value
             try:
@@ -332,6 +337,8 @@ class IrFieldsConverter(models.AbstractModel):
                     {'moreinfo': action})
         elif subfield == 'id':
             field_type = _(u"external id")
+            if not self._str_to_boolean(model, field, value)[0]:
+                return False, field_type, warnings
             if '.' in value:
                 xmlid = value
             else:
@@ -340,6 +347,8 @@ class IrFieldsConverter(models.AbstractModel):
             id = self.env['ir.model.data'].xmlid_to_res_id(xmlid, raise_if_not_found=False) or None
         elif subfield is None:
             field_type = _(u"name")
+            if value == '':
+                return False, field_type, warnings
             flush()
             ids = RelatedModel.name_search(name=value, operator='=')
             if ids:
@@ -353,8 +362,8 @@ class IrFieldsConverter(models.AbstractModel):
                 if name_create_enabled_fields.get(field.name):
                     try:
                         id, _name = RelatedModel.name_create(name=value)
-                    except Exception as e:
-                        error_msg = repr(e)
+                    except (Exception, psycopg2.IntegrityError):
+                        error_msg = _(u"Cannot create new '%s' records from their name alone. Please create those records manually and try importing again.") % RelatedModel._description
         else:
             raise self._format_import_error(
                 Exception,
@@ -407,6 +416,10 @@ class IrFieldsConverter(models.AbstractModel):
         id, _, w2 = self.db_id_for(model, field, subfield, record[subfield])
         return id, w1 + w2
 
+    @api.model
+    def _str_to_many2one_reference(self, model, field, value):
+        return self._str_to_integer(model, field, value)
+
     @api.model
     def _str_to_many2many(self, model, field, value):
         [record] = value
@@ -426,6 +439,13 @@ class IrFieldsConverter(models.AbstractModel):
 
     @api.model
     def _str_to_one2many(self, model, field, records):
+        name_create_enabled_fields = self._context.get('name_create_enabled_fields') or {}
+        prefix = field.name + '/'
+        relative_name_create_enabled_fields = {
+            k[len(prefix):]: v
+            for k, v in name_create_enabled_fields.items()
+            if k.startswith(prefix)
+        }
         commands = []
         warnings = []
 
@@ -444,7 +464,7 @@ class IrFieldsConverter(models.AbstractModel):
                 raise e
             warnings.append(e)
 
-        convert = self.for_model(self.env[field.comodel_name])
+        convert = self.with_context(name_create_enabled_fields=relative_name_create_enabled_fields).for_model(self.env[field.comodel_name])
 
         for record in records:
             id = None
