@@ -3,13 +3,13 @@
 # ir_http modular http routing
 #----------------------------------------------------------
 import base64
-import datetime
 import hashlib
 import logging
 import mimetypes
 import os
 import re
 import sys
+import traceback
 
 import werkzeug
 import werkzeug.exceptions
@@ -20,12 +20,14 @@ import werkzeug.utils
 import odoo
 from odoo import api, http, models, tools, SUPERUSER_ID
 from odoo.exceptions import AccessDenied, AccessError
-from odoo.http import request, STATIC_CACHE, content_disposition
-from odoo.tools import pycompat, consteq
+from odoo.http import request, content_disposition
+from odoo.tools import consteq, pycompat
 from odoo.tools.mimetypes import guess_mimetype
-from ast import literal_eval
 from odoo.modules.module import get_resource_path, get_module_path
 
+from odoo.http import ALLOWED_DEBUG_MODES
+from odoo.tools.misc import str2bool
+
 _logger = logging.getLogger(__name__)
 
 
@@ -76,13 +78,17 @@ class IrHttp(models.AbstractModel):
     _name = 'ir.http'
     _description = "HTTP Routing"
 
+    #------------------------------------------------------
+    # Routing map
+    #------------------------------------------------------
+
     @classmethod
     def _get_converters(cls):
         return {'model': ModelConverter, 'models': ModelsConverter, 'int': SignedIntConverter}
 
     @classmethod
-    def _find_handler(cls, return_rule=False):
-        return cls.routing_map().bind_to_environ(request.httprequest.environ).match(return_rule=return_rule)
+    def _match(cls, path_info, key=None):
+        return cls.routing_map().bind_to_environ(request.httprequest.environ).match(path_info=path_info, return_rule=True)
 
     @classmethod
     def _auth_method_user(cls):
@@ -123,6 +129,21 @@ class IrHttp(models.AbstractModel):
             raise AccessDenied()
         return auth_method
 
+    @classmethod
+    def _handle_debug(cls):
+        # Store URL debug mode (might be empty) into session
+        if 'debug' in request.httprequest.args:
+            debug_mode = []
+            for debug in request.httprequest.args['debug'].split(','):
+                if debug not in ALLOWED_DEBUG_MODES:
+                    debug = '1' if str2bool(debug, debug) else ''
+                debug_mode.append(debug)
+            debug_mode = ','.join(debug_mode)
+
+            # Write on session only when needed
+            if debug_mode != request.session.debug:
+                request.session.debug = debug_mode
+
     @classmethod
     def _serve_attachment(cls):
         env = api.Environment(request.cr, SUPERUSER_ID, request.context)
@@ -160,6 +181,9 @@ class IrHttp(models.AbstractModel):
 
     @classmethod
     def _handle_exception(cls, exception):
+        # in case of Exception, e.g. 404, we don't step into _dispatch
+        cls._handle_debug()
+
         # If handle_exception returns something different than None, it will be used as a response
 
         # This is done first as the attachment path may
@@ -170,7 +194,12 @@ class IrHttp(models.AbstractModel):
                 return serve
 
         # Don't handle exception but use werkzeug debugger if server in --dev mode
-        if 'werkzeug' in tools.config['dev_mode'] and not isinstance(exception, werkzeug.exceptions.NotFound):
+        # Don't intercept JSON request to respect the JSON Spec and return exception as JSON
+        # "The Response is expressed as a single JSON Object, with the following members:
+        #   jsonrpc, result, error, id"
+        if ('werkzeug' in tools.config['dev_mode']
+                and not isinstance(exception, werkzeug.exceptions.NotFound)
+                and request._request_type != 'json'):
             raise exception
 
         try:
@@ -180,9 +209,11 @@ class IrHttp(models.AbstractModel):
 
     @classmethod
     def _dispatch(cls):
+        cls._handle_debug()
+
         # locate the controller method
         try:
-            rule, arguments = cls._find_handler(return_rule=True)
+            rule, arguments = cls._match(request.httprequest.path)
             func = rule.endpoint
         except werkzeug.exceptions.NotFound as e:
             return cls._handle_exception(e)
@@ -214,58 +245,188 @@ class IrHttp(models.AbstractModel):
         for key, val in list(arguments.items()):
             # Replace uid placeholder by the current request.uid
             if isinstance(val, models.BaseModel) and isinstance(val._uid, RequestUID):
-                arguments[key] = val.sudo(request.uid)
+                arguments[key] = val.with_user(request.uid)
                 if not val.exists():
                     return cls._handle_exception(werkzeug.exceptions.NotFound())
 
     @classmethod
-    def routing_map(cls):
+    def _generate_routing_rules(cls, modules, converters):
+        return http._generate_routing_rules(modules, False, converters)
+
+    @classmethod
+    def routing_map(cls, key=None):
         if not hasattr(cls, '_routing_map'):
-            _logger.info("Generating routing map")
-            installed = request.registry._init_modules - {'web'}
+            cls._routing_map = {}
+            cls._rewrite_len = {}
+        if key not in cls._routing_map:
+            _logger.info("Generating routing map for key %s" % str(key))
+            installed = request.registry._init_modules | set(odoo.conf.server_wide_modules)
             if tools.config['test_enable'] and odoo.modules.module.current_test:
                 installed.add(odoo.modules.module.current_test)
-            mods = [''] + odoo.conf.server_wide_modules + sorted(installed)
+            mods = sorted(installed)
             # Note : when routing map is generated, we put it on the class `cls`
             # to make it available for all instance. Since `env` create an new instance
             # of the model, each instance will regenared its own routing map and thus
             # regenerate its EndPoint. The routing map should be static.
-            cls._routing_map = http.routing_map(mods, False, converters=cls._get_converters())
-        return cls._routing_map
+            routing_map = werkzeug.routing.Map(strict_slashes=False, converters=cls._get_converters())
+            for url, endpoint, routing in cls._generate_routing_rules(mods, converters=cls._get_converters()):
+                xtra_keys = 'defaults subdomain build_only strict_slashes redirect_to alias host'.split()
+                kw = {k: routing[k] for k in xtra_keys if k in routing}
+                routing_map.add(werkzeug.routing.Rule(url, endpoint=endpoint, methods=routing['methods'], **kw))
+            cls._routing_map[key] = routing_map
+        return cls._routing_map[key]
 
     @classmethod
     def _clear_routing_map(cls):
         if hasattr(cls, '_routing_map'):
-            del cls._routing_map
+            cls._routing_map = {}
+            _logger.debug("Clear routing map")
 
-    @classmethod
-    def content_disposition(cls, filename):
-        return content_disposition(filename)
+    #------------------------------------------------------
+    # Binary server
+    #------------------------------------------------------
 
     @classmethod
     def _xmlid_to_obj(cls, env, xmlid):
         return env.ref(xmlid, False)
 
-    @classmethod
-    def _check_access_mode(cls, env, id, access_mode, model, access_token=None, related_id=None):
-        """
-        Implemented by each module to define an additional way to check access.
+    def _get_record_and_check(self, xmlid=None, model=None, id=None, field='datas', access_token=None):
+        # get object and content
+        record = None
+        if xmlid:
+            record = self._xmlid_to_obj(self.env, xmlid)
+        elif id and model in self.env:
+            record = self.env[model].browse(int(id))
 
-        :param env: the env of binary_content
-        :param id: id of the record from which to fetch the binary
-        :param access_mode: typically a string that describes the behaviour of the custom check
-        :param model: the model of the object for which binary_content was called
-        :param related_id: optional id to check security.
-        :return: True if the test passes, else False.
-        """
-        return False
+        # obj exists
+        if not record or not record.exists() or field not in record:
+            return None, 404
 
+        if model == 'ir.attachment':
+            record_sudo = record.sudo()
+            if access_token and not consteq(record_sudo.access_token or '', access_token):
+                return None, 403
+            elif (access_token and consteq(record_sudo.access_token or '', access_token)):
+                record = record_sudo
+            elif record_sudo.public:
+                record = record_sudo
+            elif self.env.user.has_group('base.group_portal'):
+                # Check the read access on the record linked to the attachment
+                # eg: Allow to download an attachment on a task from /my/task/task_id
+                record.check('read')
+                record = record_sudo
+
+        # check read access
+        try:
+            # We have prefetched some fields of record, among which the field
+            # 'write_date' used by '__last_update' below. In order to check
+            # access on record, we have to invalidate its cache first.
+            record._cache.clear()
+            record['__last_update']
+        except AccessError:
+            return None, 403
+
+        return record, 200
 
     @classmethod
-    def binary_content(cls, xmlid=None, model='ir.attachment', id=None, field='datas',
-                       unique=False, filename=None, filename_field='datas_fname', download=False,
+    def _binary_ir_attachment_redirect_content(cls, record, default_mimetype='application/octet-stream'):
+        # mainly used for theme images attachemnts
+        status = content = filename = filehash = None
+        mimetype = getattr(record, 'mimetype', False)
+        if record.type == 'url' and record.url:
+            # if url in in the form /somehint server locally
+            url_match = re.match("^/(\w+)/(.+)$", record.url)
+            if url_match:
+                module = url_match.group(1)
+                module_path = get_module_path(module)
+                module_resource_path = get_resource_path(module, url_match.group(2))
+
+                if module_path and module_resource_path:
+                    module_path = os.path.join(os.path.normpath(module_path), '')  # join ensures the path ends with '/'
+                    module_resource_path = os.path.normpath(module_resource_path)
+                    if module_resource_path.startswith(module_path):
+                        with open(module_resource_path, 'rb') as f:
+                            content = base64.b64encode(f.read())
+                        status = 200
+                        filename = os.path.basename(module_resource_path)
+                        mimetype = guess_mimetype(base64.b64decode(content), default=default_mimetype)
+                        filehash = '"%s"' % hashlib.md5(pycompat.to_text(content).encode('utf-8')).hexdigest()
+
+            if not content:
+                status = 301
+                content = record.url
+
+        return status, content, filename, mimetype, filehash
+
+    def _binary_record_content(
+            self, record, field='datas', filename=None,
+            filename_field='name', default_mimetype='application/octet-stream'):
+
+        model = record._name
+        mimetype = 'mimetype' in record and record.mimetype or False
+        content = None
+        filehash = 'checksum' in record and record['checksum'] or False
+
+        field_def = record._fields[field]
+        if field_def.type == 'binary' and field_def.attachment:
+            field_attachment = self.env['ir.attachment'].sudo().search_read(domain=[('res_model', '=', model), ('res_id', '=', record.id), ('res_field', '=', field)], fields=['datas', 'mimetype', 'checksum'], limit=1)
+            if field_attachment:
+                mimetype = field_attachment[0]['mimetype']
+                content = field_attachment[0]['datas']
+                filehash = field_attachment[0]['checksum']
+
+        if not content:
+            content = record[field] or ''
+
+        # filename
+        default_filename = False
+        if not filename:
+            if filename_field in record:
+                filename = record[filename_field]
+            if not filename:
+                default_filename = True
+                filename = "%s-%s-%s" % (record._name, record.id, field)
+
+        if not mimetype:
+            try:
+                decoded_content = base64.b64decode(content)
+            except base64.binascii.Error:  # if we could not decode it, no need to pass it down: it would crash elsewhere...
+                return (404, [], None)
+            mimetype = guess_mimetype(decoded_content, default=default_mimetype)
+
+        # extension
+        _, existing_extension = os.path.splitext(filename)
+        if not existing_extension or default_filename:
+            extension = mimetypes.guess_extension(mimetype)
+            if extension:
+                filename = "%s%s" % (filename, extension)
+
+        if not filehash:
+            filehash = '"%s"' % hashlib.md5(pycompat.to_text(content).encode('utf-8')).hexdigest()
+
+        status = 200 if content else 404
+        return status, content, filename, mimetype, filehash
+
+    def _binary_set_headers(self, status, content, filename, mimetype, unique, filehash=None, download=False):
+        headers = [('Content-Type', mimetype), ('X-Content-Type-Options', 'nosniff')]
+        # cache
+        etag = bool(request) and request.httprequest.headers.get('If-None-Match')
+        status = status or 200
+        if filehash:
+            headers.append(('ETag', filehash))
+            if etag == filehash and status == 200:
+                status = 304
+        headers.append(('Cache-Control', 'max-age=%s' % (http.STATIC_CACHE_LONG if unique else 0)))
+        # content-disposition default name
+        if download:
+            headers.append(('Content-Disposition', content_disposition(filename)))
+
+        return (status, headers, content)
+
+    def binary_content(self, xmlid=None, model='ir.attachment', id=None, field='datas',
+                       unique=False, filename=None, filename_field='name', download=False,
                        mimetype=None, default_mimetype='application/octet-stream',
-                       access_token=None, related_id=None, access_mode=None, env=None):
+                       access_token=None):
         """ Get file, attachment or downloadable content
 
         If the ``xmlid`` and ``id`` parameter is omitted, fetches the default value for the
@@ -281,125 +442,34 @@ class IrHttp(models.AbstractModel):
         :param str filename_field: if not create an filename with model-id-field
         :param bool download: apply headers to download the file
         :param str mimetype: mintype of the field (for headers)
-        :param related_id: the id of another record used for custom_check
-        :param  access_mode: if truthy, will call custom_check to fetch the object that contains the binary.
         :param str default_mimetype: default mintype if no mintype found
         :param str access_token: optional token for unauthenticated access
                                  only available  for ir.attachment
-        :param Environment env: by default use request.env
         :returns: (status, headers, content)
         """
-        env = env or request.env
-        # get object and content
-        obj = None
-        if xmlid:
-            obj = cls._xmlid_to_obj(env, xmlid)
-        elif id and model in env.registry:
-            obj = env[model].browse(int(id))
-        # obj exists
-        if not obj or not obj.exists() or field not in obj:
-            return (404, [], None)
+        record, status = self._get_record_and_check(xmlid=xmlid, model=model, id=id, field=field, access_token=access_token)
 
-        # access token grant access
-        if model == 'ir.attachment' and access_token:
-            obj = obj.sudo()
-            if access_mode:
-                if not cls._check_access_mode(env, id, access_mode, model, access_token=access_token,
-                                             related_id=related_id):
-                    return (403, [], None)
-            elif not consteq(obj.access_token or u'', access_token):
-                return (403, [], None)
+        if not record:
+            return (status or 404, [], None)
 
-        # check read access
-        try:
-            last_update = obj['__last_update']
-        except AccessError:
-            return (403, [], None)
+        content, headers, status = None, [], None
 
-        status, headers, content = None, [], None
+        if record._name == 'ir.attachment':
+            status, content, filename, mimetype, filehash = self._binary_ir_attachment_redirect_content(record, default_mimetype=default_mimetype)
+        if not content:
+            status, content, filename, mimetype, filehash = self._binary_record_content(
+                record, field=field, filename=filename, filename_field=filename_field,
+                default_mimetype='application/octet-stream')
 
-        # attachment by url check
-        module_resource_path = None
-        if model == 'ir.attachment' and obj.type == 'url' and obj.url:
-            url_match = re.match("^/(\w+)/(.+)$", obj.url)
-            if url_match:
-                module = url_match.group(1)
-                module_path = get_module_path(module)
-                module_resource_path = get_resource_path(module, url_match.group(2))
-                if module_path and module_resource_path:
-                    module_path = os.path.join(os.path.normpath(module_path), '')  # join ensures the path ends with '/'
-                    module_resource_path = os.path.normpath(module_resource_path)
-                    if module_resource_path.startswith(module_path):
-                        with open(module_resource_path, 'rb') as f:
-                            content = base64.b64encode(f.read())
-                        last_update = pycompat.text_type(os.path.getmtime(module_resource_path))
+        status, headers, content = self._binary_set_headers(
+            status, content, filename, mimetype, unique, filehash=filehash, download=download)
 
-            if not module_resource_path:
-                module_resource_path = obj.url
+        return status, headers, content
 
-            if not content:
-                status = 301
-                content = module_resource_path
-        else:
-            content = obj[field] or ''
-
-        # filename
-        if not filename:
-            if filename_field in obj:
-                filename = obj[filename_field]
-            elif module_resource_path:
-                filename = os.path.basename(module_resource_path)
-            else:
-                filename = "%s-%s-%s" % (obj._name, obj.id, field)
-
-        # mimetype
-        mimetype = 'mimetype' in obj and obj.mimetype or False
-        if not mimetype:
-            if filename:
-                mimetype = mimetypes.guess_type(filename)[0]
-            if not mimetype and getattr(env[model]._fields[field], 'attachment', False):
-                # for binary fields, fetch the ir_attachement for mimetype check
-                attach_mimetype = env['ir.attachment'].search_read(domain=[('res_model', '=', model), ('res_id', '=', id), ('res_field', '=', field)], fields=['mimetype'], limit=1)
-                mimetype = attach_mimetype and attach_mimetype[0]['mimetype']
-            if not mimetype:
-                mimetype = guess_mimetype(base64.b64decode(content), default=default_mimetype)
-
-        headers += [('Content-Type', mimetype), ('X-Content-Type-Options', 'nosniff')]
-
-        # cache
-        etag = bool(request) and request.httprequest.headers.get('If-None-Match')
-        retag = '"%s"' % hashlib.md5(pycompat.to_text(content).encode('utf-8')).hexdigest()
-        status = status or (304 if etag == retag else 200)
-        headers.append(('ETag', retag))
-        headers.append(('Cache-Control', 'max-age=%s' % (STATIC_CACHE if unique else 0)))
-
-        # content-disposition default name
-        if download:
-            headers.append(('Content-Disposition', cls.content_disposition(filename)))
-        return (status, headers, content)
-
-
-def convert_exception_to(to_type, with_message=False):
-    """ Should only be called from an exception handler. Fetches the current
-    exception data from sys.exc_info() and creates a new exception of type
-    ``to_type`` with the original traceback.
-
-    If ``with_message`` is ``True``, sets the new exception's message to be
-    the stringification of the original exception. If ``False``, does not
-    set the new exception's message. Otherwise, uses ``with_message`` as the
-    new exception's message.
-
-    :type with_message: str|bool
-    """
-    etype, original, tb = sys.exc_info()
-    try:
-        if with_message is False:
-            message = None
-        elif with_message is True:
-            message = str(original)
-        else:
-            message = str(with_message)
-
-        raise pycompat.reraise(to_type, to_type(message), tb)
-    except to_type as e:
-        return e
+    def _response_by_status(self, status, headers, content):
+        if status == 304:
+            return werkzeug.wrappers.Response(status=status, headers=headers)
+        elif status == 301:
+            return werkzeug.utils.redirect(content, code=301)
+        elif status != 200:
+            return request.not_found()
