@@ -8,14 +8,13 @@ import importlib
 import io
 import logging
 import os
+import pkg_resources
 import shutil
 import tempfile
 import zipfile
 
 import requests
 
-from odoo.tools import pycompat
-
 from docutils import nodes
 from docutils.core import publish_string
 from docutils.transforms import Transform, writer_aux
@@ -35,7 +34,6 @@ from odoo.http import request
 _logger = logging.getLogger(__name__)
 
 ACTION_DICT = {
-    'view_type': 'form',
     'view_mode': 'form',
     'res_model': 'base.module.upgrade',
     'target': 'new',
@@ -67,7 +65,7 @@ def assert_log_admin_access(method):
         user = self.env.user
         origin = request.httprequest.remote_addr if request else 'n/a'
         log_data = (method.__name__, self.sudo().mapped('name'), user.login, user.id, origin)
-        if not self.env.user._is_admin():
+        if not self.env.is_admin():
             _logger.warning('DENY access to module.%s on %s to user %s ID #%s via %s', *log_data)
             raise AccessDenied()
         _logger.info('ALLOW access to module.%s on %s to user %s #%s via %s', *log_data)
@@ -114,7 +112,6 @@ class ModuleCategory(models.Model):
         for cat in self:
             cat.xml_id = xml_ids.get(cat.id, [''])[0]
 
-
 class MyFilterMessages(Transform):
     """
     Custom docutils transform to remove `system message` for a document and
@@ -285,7 +282,7 @@ class Module(models.Model):
         ('GPL-3 or any later version', 'GPL-3 or later version'),
         ('AGPL-3', 'Affero GPL-3'),
         ('LGPL-3', 'LGPL Version 3'),
-        ('Other OSI approved licence', 'Other OSI Approved Licence'),
+        ('Other OSI approved licence', 'Other OSI Approved License'),
         ('OEEL-1', 'Odoo Enterprise Edition License v1.0'),
         ('OPL-1', 'Odoo Proprietary License v1.0'),
         ('Other proprietary', 'Other Proprietary')
@@ -302,7 +299,6 @@ class Module(models.Model):
         ('name_uniq', 'UNIQUE (name)', 'The name of the module must be unique!'),
     ]
 
-    @api.multi
     def unlink(self):
         if not self:
             return True
@@ -312,16 +308,33 @@ class Module(models.Model):
         self.clear_caches()
         return super(Module, self).unlink()
 
+    @staticmethod
+    def _check_python_external_dependency(pydep):
+        try:
+            pkg_resources.get_distribution(pydep)
+        except pkg_resources.DistributionNotFound as e:
+            try:
+                importlib.import_module(pydep)
+                _logger.info("python external dependency on '%s' does not appear to be a valid PyPI package. Using a PyPI package name is recommended.", pydep)
+            except ImportError:
+                # backward compatibility attempt failed
+                _logger.warning("DistributionNotFound: %s", e)
+                raise Exception('Python library not installed: %s' % (pydep,))
+        except pkg_resources.VersionConflict as e:
+            _logger.warning("VersionConflict: %s", e)
+            raise Exception('Python library version conflict: %s' % (pydep,))
+        except Exception as e:
+            _logger.warning("get_distribution(%s) failed: %s", pydep, e)
+            raise Exception('Error finding python library %s' % (pydep,))
+
+
     @staticmethod
     def _check_external_dependencies(terp):
         depends = terp.get('external_dependencies')
         if not depends:
             return
         for pydep in depends.get('python', []):
-            try:
-                importlib.import_module(pydep)
-            except ImportError:
-                raise ImportError('No module named %s' % (pydep,))
+            Module._check_python_external_dependency(pydep)
 
         for binary in depends.get('bin', []):
             try:
@@ -343,7 +356,6 @@ class Module(models.Model):
                 msg = _('Unable to process module "%s" because an external dependency is not met: %s')
             raise UserError(msg % (module_name, e.args[0]))
 
-    @api.multi
     def _state_update(self, newstate, states_to_update, level=100):
         if level < 1:
             raise UserError(_('Recursion error in modules dependencies !'))
@@ -375,7 +387,6 @@ class Module(models.Model):
         return demo
 
     @assert_log_admin_access
-    @api.multi
     def button_install(self):
         # domain to select auto-installable (but not yet installed) modules
         auto_domain = [('state', '=', 'uninstalled'), ('auto_install', '=', True)]
@@ -385,7 +396,7 @@ class Module(models.Model):
         #  - at least one dependency is 'to install'
         install_states = frozenset(('installed', 'to install', 'to upgrade'))
         def must_install(module):
-            states = set(dep.state for dep in module.dependencies_id)
+            states = {dep.state for dep in module.dependencies_id if dep.auto_install_required}
             return states <= install_states and 'to install' in states
 
         modules = self
@@ -412,7 +423,7 @@ class Module(models.Model):
             todo = result = module
             while todo:
                 result |= todo
-                todo = todo.mapped('dependencies_id.depend_id')
+                todo = todo.dependencies_id.depend_id
             return result
 
         exclusives = self.env['ir.module.category'].search([('exclusive', '=', True)])
@@ -433,7 +444,6 @@ class Module(models.Model):
         return dict(ACTION_DICT, name=_('Install'))
 
     @assert_log_admin_access
-    @api.multi
     def button_immediate_install(self):
         """ Installs the selected module(s) immediately and fully,
         returns the next res.config action to execute
@@ -442,16 +452,22 @@ class Module(models.Model):
         :rtype: dict[str, object]
         """
         _logger.info('User #%d triggered module installation', self.env.uid)
+        # We use here the request object (which is thread-local) as a kind of
+        # "global" env because the env is not usable in the following use case.
+        # When installing a Chart of Account, I would like to send the
+        # allowed companies to configure it on the correct company.
+        # Otherwise, the SUPERUSER won't be aware of that and will try to
+        # configure the CoA on his own company, which makes no sense.
+        if request:
+            request.allowed_company_ids = self.env.companies.ids
         return self._button_immediate_function(type(self).button_install)
 
     @assert_log_admin_access
-    @api.multi
     def button_install_cancel(self):
         self.write({'state': 'uninstalled', 'demo': False})
         return True
 
     @assert_log_admin_access
-    @api.multi
     def module_uninstall(self):
         """ Perform the various steps required to uninstall a module completely
         including the deletion of all database structures created by the module:
@@ -463,7 +479,6 @@ class Module(models.Model):
         self.with_context(prefetch_fields=False).write({'state': 'uninstalled', 'latest_version': False})
         return True
 
-    @api.multi
     def _remove_copied_views(self):
         """ Remove the copies of the views installed by the modules in `self`.
 
@@ -477,7 +492,6 @@ class Module(models.Model):
         orphans = self.env['ir.ui.view'].with_context(**{'active_test': False, MODULE_UNINSTALL_FLAG: True}).search(domain)
         orphans.unlink()
 
-    @api.multi
     @api.returns('self')
     def downstream_dependencies(self, known_deps=None,
                                 exclude_states=('uninstalled', 'uninstallable', 'to remove')):
@@ -502,7 +516,6 @@ class Module(models.Model):
             known_deps |= missing_mods.downstream_dependencies(known_deps, exclude_states)
         return known_deps
 
-    @api.multi
     @api.returns('self')
     def upstream_dependencies(self, known_deps=None,
                               exclude_states=('installed', 'uninstallable', 'to remove')):
@@ -544,7 +557,6 @@ class Module(models.Model):
             'url': '/web',
         }
 
-    @api.multi
     def _button_immediate_function(self, function):
         try:
             # This is done because the installation/uninstallation/upgrade can modify a currently
@@ -576,7 +588,6 @@ class Module(models.Model):
         }
 
     @assert_log_admin_access
-    @api.multi
     def button_immediate_uninstall(self):
         """
         Uninstall the selected module(s) immediately and fully,
@@ -586,16 +597,19 @@ class Module(models.Model):
         return self._button_immediate_function(type(self).button_uninstall)
 
     @assert_log_admin_access
-    @api.multi
     def button_uninstall(self):
         if 'base' in self.mapped('name'):
             raise UserError(_("The `base` module cannot be uninstalled"))
+        if not all(state in ('installed', 'to upgrade') for state in self.mapped('state')):
+            raise UserError(_(
+                "One or more of the selected modules have already been uninstalled, if you "
+                "believe this to be an error, you may try again later or contact support."
+            ))
         deps = self.downstream_dependencies()
         (self + deps).write({'state': 'to remove'})
         return dict(ACTION_DICT, name=_('Uninstall'))
 
     @assert_log_admin_access
-    @api.multi
     def button_uninstall_wizard(self):
         """ Launch the wizard to uninstall the given module. """
         return {
@@ -607,13 +621,11 @@ class Module(models.Model):
             'context': {'default_module_id': self.id},
         }
 
-    @api.multi
     def button_uninstall_cancel(self):
         self.write({'state': 'installed'})
         return True
 
     @assert_log_admin_access
-    @api.multi
     def button_immediate_upgrade(self):
         """
         Upgrade the selected module(s) immediately and fully,
@@ -622,7 +634,6 @@ class Module(models.Model):
         return self._button_immediate_function(type(self).button_upgrade)
 
     @assert_log_admin_access
-    @api.multi
     def button_upgrade(self):
         Dependency = self.env['ir.module.module.dependency']
         self.update_list()
@@ -653,7 +664,6 @@ class Module(models.Model):
         return dict(ACTION_DICT, name=_('Apply Schedule Upgrade'))
 
     @assert_log_admin_access
-    @api.multi
     def button_upgrade_cancel(self):
         self.write({'state': 'installed'})
         return True
@@ -670,7 +680,7 @@ class Module(models.Model):
             'license': terp.get('license', 'LGPL-3'),
             'sequence': terp.get('sequence', 100),
             'application': terp.get('application', False),
-            'auto_install': terp.get('auto_install', False),
+            'auto_install': terp.get('auto_install', False) is not False,
             'icon': terp.get('icon', False),
             'summary': terp.get('summary', ''),
             'url': terp.get('url') or terp.get('live_test_url', ''),
@@ -710,8 +720,7 @@ class Module(models.Model):
                 updated_values = {}
                 for key in values:
                     old = getattr(mod, key)
-                    updated = tools.ustr(values[key]) if isinstance(values[key], pycompat.string_types) else values[key]
-                    if (old or updated) and updated != old:
+                    if (old or values[key]) and values[key] != old:
                         updated_values[key] = values[key]
                 if terp.get('installable', True) and mod.state == 'uninstallable':
                     updated_values['state'] = 'uninstalled'
@@ -727,14 +736,13 @@ class Module(models.Model):
                 mod = self.create(dict(name=mod_name, state=state, **values))
                 res[1] += 1
 
-            mod._update_dependencies(terp.get('depends', []))
+            mod._update_dependencies(terp.get('depends', []), terp.get('auto_install'))
             mod._update_exclusions(terp.get('excludes', []))
             mod._update_category(terp.get('category', 'Uncategorized'))
 
         return res
 
     @assert_log_admin_access
-    @api.multi
     def download(self, download=True):
         return []
 
@@ -838,13 +846,15 @@ class Module(models.Model):
     def get_apps_server(self):
         return tools.config.get('apps_server', 'https://apps.odoo.com/apps')
 
-    def _update_dependencies(self, depends=None):
+    def _update_dependencies(self, depends=None, auto_install_requirements=()):
         existing = set(dep.name for dep in self.dependencies_id)
         needed = set(depends or [])
         for dep in (needed - existing):
             self._cr.execute('INSERT INTO ir_module_module_dependency (module_id, name) values (%s, %s)', (self.id, dep))
         for dep in (existing - needed):
             self._cr.execute('DELETE FROM ir_module_module_dependency WHERE module_id = %s and name = %s', (self.id, dep))
+        self._cr.execute('UPDATE ir_module_module_dependency SET auto_install_required = (name = any(%s)) WHERE module_id = %s',
+                         (list(auto_install_requirements or ()), self.id))
         self.invalidate_cache(['dependencies_id'], self.ids)
 
     def _update_exclusions(self, excludes=None):
@@ -868,11 +878,10 @@ class Module(models.Model):
             cat_id = modules.db.create_categories(self._cr, categs)
             self.write({'category_id': cat_id})
 
-    @api.multi
     def _update_translations(self, filter_lang=None):
         if not filter_lang:
-            langs = self.env['res.lang'].search([('translatable', '=', True)])
-            filter_lang = [lang.code for lang in langs]
+            langs = self.env['res.lang'].get_installed()
+            filter_lang = [code for code, _ in langs]
         elif not isinstance(filter_lang, (list, tuple)):
             filter_lang = [filter_lang]
 
@@ -882,9 +891,8 @@ class Module(models.Model):
             for mod in update_mods
         }
         mod_names = topological_sort(mod_dict)
-        self.env['ir.translation'].load_module_terms(mod_names, filter_lang)
+        self.env['ir.translation']._load_module_terms(mod_names, filter_lang)
 
-    @api.multi
     def _check(self):
         for module in self:
             if not module.description_html:
@@ -913,10 +921,15 @@ class ModuleDependency(models.Model):
     module_id = fields.Many2one('ir.module.module', 'Module', ondelete='cascade')
 
     # the module corresponding to the dependency, and its status
-    depend_id = fields.Many2one('ir.module.module', 'Dependency', compute='_compute_depend')
+    depend_id = fields.Many2one('ir.module.module', 'Dependency',
+                                compute='_compute_depend', search='_search_depend')
     state = fields.Selection(DEP_STATES, string='Status', compute='_compute_state')
 
-    @api.multi
+    auto_install_required = fields.Boolean(
+        default=True,
+        help="Whether this dependency blocks automatic installation "
+             "of the dependent")
+
     @api.depends('name')
     def _compute_depend(self):
         # retrieve all modules corresponding to the dependency names
@@ -928,10 +941,15 @@ class ModuleDependency(models.Model):
         for dep in self:
             dep.depend_id = name_mod.get(dep.name)
 
-    @api.one
+    def _search_depend(self, operator, value):
+        assert operator == 'in'
+        modules = self.env['ir.module.module'].browse(set(value))
+        return [('name', 'in', modules.mapped('name'))]
+
     @api.depends('depend_id.state')
     def _compute_state(self):
-        self.state = self.depend_id.state or 'unknown'
+        for dependency in self:
+            dependency.state = dependency.depend_id.state or 'unknown'
 
 
 class ModuleExclusion(models.Model):
@@ -945,10 +963,10 @@ class ModuleExclusion(models.Model):
     module_id = fields.Many2one('ir.module.module', 'Module', ondelete='cascade')
 
     # the module corresponding to the exclusion, and its status
-    exclusion_id = fields.Many2one('ir.module.module', 'Exclusion Module', compute='_compute_exclusion')
+    exclusion_id = fields.Many2one('ir.module.module', 'Exclusion Module',
+                                   compute='_compute_exclusion', search='_search_exclusion')
     state = fields.Selection(DEP_STATES, string='Status', compute='_compute_state')
 
-    @api.multi
     @api.depends('name')
     def _compute_exclusion(self):
         # retrieve all modules corresponding to the exclusion names
@@ -960,7 +978,12 @@ class ModuleExclusion(models.Model):
         for excl in self:
             excl.exclusion_id = name_mod.get(excl.name)
 
-    @api.one
+    def _search_exclusion(self, operator, value):
+        assert operator == 'in'
+        modules = self.env['ir.module.module'].browse(set(value))
+        return [('name', 'in', modules.mapped('name'))]
+
     @api.depends('exclusion_id.state')
     def _compute_state(self):
-        self.state = self.exclusion_id.state or 'unknown'
+        for exclusion in self:
+            exclusion.state = exclusion.exclusion_id.state or 'unknown'
