@@ -53,12 +53,10 @@ class ResConfigConfigurable(models.TransientModel):
     _name = 'res.config'
     _description = 'Config'
 
-    @api.multi
     def start(self):
         # pylint: disable=next-method-called
         return self.next()
 
-    @api.multi
     def next(self):
         """
         Reload the settings page
@@ -68,7 +66,6 @@ class ResConfigConfigurable(models.TransientModel):
             'tag': 'reload',
         }
 
-    @api.multi
     def execute(self):
         """ Method called when the user clicks on the ``Next`` button.
 
@@ -81,7 +78,6 @@ class ResConfigConfigurable(models.TransientModel):
         raise NotImplementedError(
             'Configuration items need to implement execute')
 
-    @api.multi
     def cancel(self):
         """ Method called when the user click on the ``Skip`` button.
 
@@ -96,7 +92,6 @@ class ResConfigConfigurable(models.TransientModel):
         """
         pass
 
-    @api.multi
     def action_next(self):
         """ Action handler for the ``next`` event.
 
@@ -108,7 +103,6 @@ class ResConfigConfigurable(models.TransientModel):
         # pylint: disable=next-method-called
         return self.execute() or self.next()
 
-    @api.multi
     def action_skip(self):
         """ Action handler for the ``skip`` event.
 
@@ -120,7 +114,6 @@ class ResConfigConfigurable(models.TransientModel):
         # pylint: disable=next-method-called
         return self.cancel() or self.next()
 
-    @api.multi
     def action_cancel(self):
         """ Action handler for the ``cancel`` event. That event isn't
         generated by the res.config.view.base inheritable view, the
@@ -316,7 +309,6 @@ class ResConfigInstaller(models.TransientModel, ResConfigModuleInstallationMixin
                      _('\n\nThis addon is already installed on your system'))
         return fields
 
-    @api.multi
     def execute(self):
         to_install = list(self.modules_to_install())
         _logger.info('Selecting addons %s to install', to_install)
@@ -355,7 +347,7 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
             By default 'group' is the group Employee.  Groups are given by their xml id.
             The attribute 'group' may contain several xml ids, separated by commas.
 
-        *   For a selection field like 'group_XXX' composed of 2 integers values ('0' and '1'),
+        *   For a selection field like 'group_XXX' composed of 2 string values ('0' and '1'),
             ``execute`` adds/removes 'implied_group' to/from the implied groups of 'group', 
             depending on the field's value.
             By default 'group' is the group Employee.  Groups are given by their xml id.
@@ -364,9 +356,9 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
         *   For a boolean field like 'module_XXX', ``execute`` triggers the immediate
             installation of the module named 'XXX' if the field has value ``True``.
 
-        *   For a selection field like 'module_XXX' composed of 2 integers values ('0' and '1'), 
+        *   For a selection field like 'module_XXX' composed of 2 string values ('0' and '1'), 
             ``execute`` triggers the immediate installation of the module named 'XXX' 
-            if the field has the integer value ``1``.
+            if the field has the value ``'1'``.
 
         *   For a field with no specific prefix BUT an attribute 'config_parameter',
             ``execute``` will save its value in an ir.config.parameter (global setting for the
@@ -383,7 +375,6 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
     _name = 'res.config.settings'
     _description = 'Config Settings'
 
-    @api.multi
     def copy(self, values):
         raise UserError(_("Cannot duplicate configuration!"), "")
 
@@ -412,14 +403,13 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
         ret_val['arch'] = etree.tostring(doc, encoding='unicode')
         return ret_val
 
-    @api.multi
     def onchange_module(self, field_value, module_name):
         ModuleSudo = self.env['ir.module.module'].sudo()
         modules = ModuleSudo.search(
             [('name', '=', module_name.replace("module_", '')),
             ('state', 'in', ['to install', 'installed', 'to upgrade'])])
 
-        if modules and not field_value:
+        if modules and not int(field_value):
             deps = modules.sudo().downstream_dependencies()
             dep_names = (deps | modules).mapped('shortdesc')
             message = '\n'.join(dep_names)
@@ -508,13 +498,13 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
         for name, groups, implied_group in classified['group']:
             res[name] = all(implied_group in group.implied_ids for group in groups)
             if self._fields[name].type == 'selection':
-                res[name] = int(res[name])
+                res[name] = str(int(res[name]))     # True, False -> '1', '0'
 
         # modules: which modules are installed/to install
         for name, module in classified['module']:
             res[name] = module.state in ('installed', 'to install', 'to upgrade')
             if self._fields[name].type == 'selection':
-                res[name] = int(res[name])
+                res[name] = str(int(res[name]))     # True, False -> '1', '0'
 
         # config: get & convert stored ir.config_parameter (or default)
         WARNING_MESSAGE = "Error when converting value %r of field %s for ir.config.parameter %r"
@@ -527,23 +517,23 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
                         # Special case when value is the id of a deleted record, we do not want to
                         # block the settings screen
                         value = self.env[field.comodel_name].browse(int(value)).exists().id
-                    except ValueError:
+                    except (ValueError, TypeError):
                         _logger.warning(WARNING_MESSAGE, value, field, icp)
                         value = False
                 elif field.type == 'integer':
                     try:
                         value = int(value)
-                    except ValueError:
+                    except (ValueError, TypeError):
                         _logger.warning(WARNING_MESSAGE, value, field, icp)
                         value = 0
                 elif field.type == 'float':
                     try:
                         value = float(value)
-                    except ValueError:
+                    except (ValueError, TypeError):
                         _logger.warning(WARNING_MESSAGE, value, field, icp)
                         value = 0.0
                 elif field.type == 'boolean':
-                    value = value.lower() == 'true'
+                    value = bool(value)
             res[name] = value
 
         # other fields: call the method 'get_values'
@@ -577,15 +567,16 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
         # group fields: modify group / implied groups
         current_settings = self.default_get(list(self.fields_get()))
         with self.env.norecompute():
-            for name, groups, implied_group in classified['group']:
+            for name, groups, implied_group in sorted(classified['group'], key=lambda k: self[k[0]]):
+                groups = groups.sudo()
+                implied_group = implied_group.sudo()
                 if self[name] == current_settings[name]:
                     continue
-                if self[name]:
+                if int(self[name]):
                     groups.write({'implied_ids': [(4, implied_group.id)]})
                 else:
                     groups.write({'implied_ids': [(3, implied_group.id)]})
-                    implied_group.write({'users': [(3, user.id) for user in groups.mapped('users')]})
-        self.recompute()
+                    implied_group.sudo().write({'users': [(5,)]})
 
         # config fields: store ir.config_parameters
         IrConfigParameter = self.env['ir.config_parameter'].sudo()
@@ -606,13 +597,12 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
         # other fields: execute method 'set_values'
         # Methods that start with `set_` are now deprecated
         for method in dir(self):
-            if method.startswith('set_') and method is not 'set_values':
+            if method.startswith('set_') and method != 'set_values':
                 _logger.warning(_('Methods that start with `set_` are deprecated. Override `set_values` instead (Method %s)') % method)
 
-    @api.multi
     def execute(self):
         self.ensure_one()
-        if not self.env.user._is_admin() and not self.env.user.has_group('base.group_system'):
+        if not self.env.is_admin():
             raise AccessError(_("Only administrators can change the settings"))
 
         self = self.with_context(active_test=False)
@@ -625,18 +615,21 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
         to_uninstall_modules = self.env['ir.module.module']
         lm = len('module_')
         for name, module in classified['module']:
-            if self[name]:
+            if int(self[name]):
                 to_install.append((name[lm:], module))
             else:
                 if module and module.state in ('installed', 'to upgrade'):
                     to_uninstall_modules += module
 
+        if to_install or to_uninstall_modules:
+            self.flush()
+
         if to_uninstall_modules:
             to_uninstall_modules.button_immediate_uninstall()
 
-        self._install_modules(to_install)
+        result = self._install_modules(to_install)
 
-        if to_install or to_uninstall_modules:
+        if result or to_uninstall_modules:
             # After the uninstall/install calls, the registry and environments
             # are no longer valid. So we reset the environment.
             self.env.reset()
@@ -653,7 +646,6 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
             'tag': 'reload',
         }
 
-    @api.multi
     def cancel(self):
         # ignore the current record, and send the action to reopen the view
         actions = self.env['ir.actions.act_window'].search([('res_model', '=', self._name)], limit=1)
@@ -661,7 +653,6 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
             return actions.read()[0]
         return {}
 
-    @api.multi
     def name_get(self):
         """ Override name_get method to return an appropriate configuration wizard
         name, and not the generated name."""
@@ -695,7 +686,7 @@ class ResConfigSettings(models.TransientModel, ResConfigModuleInstallationMixin)
         model_name, field_name = full_field_name.rsplit('.', 1)
         return self.env[model_name].fields_get([field_name])[field_name]['string']
 
-    @api.model_cr_context
+    @api.model
     def get_config_warning(self, msg):
         """
         Helper: return a Warning exception with the given message where the %(field:xxx)s
