@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from . import test_acl
 from . import test_api
 from . import test_base
@@ -7,8 +8,10 @@ from . import test_db_cursor
 from . import test_expression
 from . import test_float
 from . import test_func
+from . import test_image
 from . import test_ir_actions
 from . import test_ir_attachment
+from . import test_ir_cron
 from . import test_ir_http
 from . import test_ir_filters
 from . import test_ir_model
@@ -34,6 +37,9 @@ from . import test_views
 from . import test_xmlrpc
 from . import test_res_partner
 from . import test_res_partner_bank
+from . import test_res_users
 from . import test_reports
 from . import test_tests_tags
-from . import test_non_regression
+from . import test_base_document_layout
+from . import test_form_create
+from . import test_cloc
