@@ -1,15 +1,20 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-import unittest
 
-from odoo.tests.common import tagged
-from odoo.tools import html_sanitize, append_content_to_html, plaintext2html, email_split, misc
+from unittest.mock import patch
+
+from odoo.tests.common import BaseCase
+from odoo.tests.common import SavepointCase
+from odoo.tools import (
+    html_sanitize, append_content_to_html, plaintext2html, email_split,
+    misc, formataddr, decode_smtp_header
+)
+
 from . import test_mail_examples
 
 
-@tagged('standard', 'at_install')
-class TestSanitizer(unittest.TestCase):
+class TestSanitizer(BaseCase):
     """ Test the html sanitizer that filters html to remove unwanted attributes """
 
     def test_basic_sanitizer(self):
@@ -92,16 +97,6 @@ class TestSanitizer(unittest.TestCase):
         for attr in ['javascript']:
             self.assertNotIn(attr, sanitized_html, 'html_sanitize did not remove enough unwanted attributes')
 
-    def test_sanitize_escape_emails(self):
-        emails = [
-            "Charles <charles.bidule@truc.fr>",
-            "Dupuis <'tr/-: ${dupuis#$'@truc.baz.fr>",
-            "Technical <service/technical+2@open.com>",
-            "Div nico <div-nico@open.com>"
-        ]
-        for email in emails:
-            self.assertIn(misc.html_escape(email), html_sanitize(email), 'html_sanitize stripped emails of original html')
-
     def test_sanitize_unescape_emails(self):
         not_emails = [
             '<blockquote cite="mid:CAEJSRZvWvud8c6Qp=wfNG6O1+wK3i_jb33qVrF7XyrgPNjnyUA@mail.gmail.com" type="cite">cat</blockquote>',
@@ -295,8 +290,7 @@ class TestSanitizer(unittest.TestCase):
     #         self.assertNotIn(ext, new_html)
 
 
-@tagged('standard', 'at_install')
-class TestHtmlTools(unittest.TestCase):
+class TestHtmlTools(BaseCase):
     """ Test some of our generic utility functions about html """
 
     def test_plaintext2html(self):
@@ -316,6 +310,8 @@ class TestHtmlTools(unittest.TestCase):
              '<!DOCTYPE...><html encoding="blah">some <b>content</b>\n<pre>--\nYours truly</pre>\n</html>'),
             ('<!DOCTYPE...><HTML encoding="blah">some <b>content</b></HtMl>', '--\nYours truly', True, False, False,
              '<!DOCTYPE...><html encoding="blah">some <b>content</b>\n<p>--<br/>Yours truly</p>\n</html>'),
+            ('<html><body>some <b>content</b></body></html>', '--\nYours & <truly>', True, True, False,
+             '<html><body>some <b>content</b>\n<pre>--\nYours &amp; &lt;truly&gt;</pre>\n</body></html>'),
             ('<html><body>some <b>content</b></body></html>', '<!DOCTYPE...>\n<html><body>\n<p>--</p>\n<p>Yours truly</p>\n</body>\n</html>', False, False, False,
              '<html><body>some <b>content</b>\n\n\n<p>--</p>\n<p>Yours truly</p>\n\n\n</body></html>'),
         ]
@@ -323,8 +319,7 @@ class TestHtmlTools(unittest.TestCase):
             self.assertEqual(append_content_to_html(html, content, plaintext_flag, preserve_flag, container_tag), expected, 'append_content_to_html is broken')
 
 
-@tagged('standard', 'at_install')
-class TestEmailTools(unittest.TestCase):
+class TestEmailTools(BaseCase):
     """ Test some of our generic utility functions for emails """
 
     def test_email_split(self):
@@ -337,3 +332,80 @@ class TestEmailTools(unittest.TestCase):
         ]
         for text, expected in cases:
             self.assertEqual(email_split(text), expected, 'email_split is broken')
+
+    def test_email_formataddr(self):
+        email = 'joe@example.com'
+        cases = [
+            # (name, address),          charsets            expected
+            (('', email),               ['ascii', 'utf-8'], 'joe@example.com'),
+            (('joe', email),            ['ascii', 'utf-8'], '"joe" <joe@example.com>'),
+            (('joe doe', email),        ['ascii', 'utf-8'], '"joe doe" <joe@example.com>'),
+            (('joe"doe', email),        ['ascii', 'utf-8'], '"joe\\"doe" <joe@example.com>'),
+            (('joé', email),            ['ascii'],          '=?utf-8?b?am/DqQ==?= <joe@example.com>'),
+            (('joé', email),            ['utf-8'],          '"joé" <joe@example.com>'),
+            (('', 'joé@example.com'),   ['ascii', 'utf-8'], UnicodeEncodeError),  # need SMTPUTF8 support
+            (('', 'joe@examplé.com'),   ['ascii', 'utf-8'], UnicodeEncodeError),  # need IDNA support
+        ]
+
+        for pair, charsets, expected in cases:
+            for charset in charsets:
+                with self.subTest(pair=pair, charset=charset):
+                    if isinstance(expected, str):
+                        self.assertEqual(formataddr(pair, charset), expected)
+                    else:
+                        self.assertRaises(expected, formataddr, pair, charset)
+
+
+    def test_decode_smtp_header_email(self):
+        cases = [
+            # In == Out for trivial ASCII cases
+            ('Joe Doe <joe@ex.com>', 'Joe Doe <joe@ex.com>'),
+            ('Joe <joe@ex.com>, Mike <mike@ex.com>', 'Joe <joe@ex.com>, Mike <mike@ex.com>'),
+
+            # Same thing, but RFC822 quoted-strings must be preserved
+            ('"Doe, Joe" <joe@ex.com>', '"Doe, Joe" <joe@ex.com>'),
+            ('"Doe, Joe" <joe@ex.com>, "Foo, Mike" <mike@ex.com>',
+                    '"Doe, Joe" <joe@ex.com>, "Foo, Mike" <mike@ex.com>'),
+
+            # RFC2047-encoded words have to be quoted after decoding, because
+            # they are considered RFC822 `words` i.e. atom or quoted-string only!
+            # It's ok to quote a single word even if unnecessary.
+            # Example values produced by `formataddr((name, address), 'ascii')`
+            ("=?utf-8?b?Sm/DqQ==?= <joe@ex.com>", '"Joé" <joe@ex.com>'),
+            ("=?utf-8?b?Sm/DqQ==?= <joe@ex.com>, =?utf-8?b?RsO2w7YsIE1pa2U=?= <mike@ex.com>",
+                    '"Joé" <joe@ex.com>, "Föö, Mike" <mike@ex.com>'),
+            ('=?utf-8?b?RG/DqSwg?= =?US-ASCII?Q?Joe?= <joe@ex.com>',
+                '"Doé, ""Joe" <joe@ex.com>'),
+
+            # Double-quotes may appear in the encoded form and /must/ be turned
+            # into a RFC2822 quoted-pair (i.e. escaped)
+            #   "Trevor \"Banana\" Dumoulin" <tbd@ex.com>
+            ('=?utf-8?b?VHLDqXZvciAiQmFuYW5hIiBEdW1vdWxpbg==?= <tbd@ex.com>',
+                    '"Trévor \\"Banana\\" Dumoulin" <tbd@ex.com>'),
+        ]
+
+        for test, truth in cases:
+            self.assertEqual(decode_smtp_header(test, quoted=True), truth)
+
+
+
+class EmailConfigCase(SavepointCase):
+    @patch.dict("odoo.tools.config.options", {"email_from": "settings@example.com"})
+    def test_default_email_from(self, *args):
+        """Email from setting is respected."""
+        # ICP setting is more important
+        ICP = self.env["ir.config_parameter"].sudo()
+        ICP.set_param("mail.catchall.domain", "example.org")
+        ICP.set_param("mail.default.from", "icp")
+        message = self.env["ir.mail_server"].build_email(
+            False, "recipient@example.com", "Subject",
+            "The body of an email",
+        )
+        self.assertEqual(message["From"], "icp@example.org")
+        # Without ICP, the config file/CLI setting is used
+        ICP.set_param("mail.default.from", False)
+        message = self.env["ir.mail_server"].build_email(
+            False, "recipient@example.com", "Subject",
+            "The body of an email",
+        )
+        self.assertEqual(message["From"], "settings@example.com")
