@@ -4,7 +4,7 @@
 {
     'name': 'Automated Action Rules',
     'version': '1.0',
-    'category': 'Sales',
+    'category': 'Sales/Sales',
     'description': """
 This module allows to implement action rules for any object.
 ============================================================
