@@ -2,7 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 {
     'name': 'IBAN Bank Accounts',
-    'category': 'Extra Tools',
+    'category': 'Tools',
     'description': """
 This module installs the base for IBAN (International Bank Account Number) bank accounts and checks for it's validity.
 ======================================================================================================================
@@ -10,6 +10,10 @@ This module installs the base for IBAN (International Bank Account Number) bank
 The ability to extract the correctly represented local accounts from IBAN accounts
 with a single statement.
     """,
-    'depends': ['account'],
+    'depends': ['account', 'web'],
+    'data': [
+        'views/templates.xml',
+        'views/partner_view.xml'
+    ],
     'demo': ['data/res_partner_bank_demo.xml'],
 }
