@@ -5,7 +5,6 @@ var BoardView = require('board.BoardView');
 
 var ListController = require('web.ListController');
 var testUtils = require('web.test_utils');
-var testUtilsDom = require('web.test_utils_dom');
 var ListRenderer = require('web.ListRenderer');
 var pyUtils = require('web.py_utils');
 
@@ -26,29 +25,33 @@ QUnit.module('Dashboard', {
                     display_name: {string: "Displayed name", type: "char", searchable: true},
                     foo: {string: "Foo", type: "char", default: "My little Foo Value", searchable: true},
                     bar: {string: "Bar", type: "boolean"},
+                    int_field: {string: "Integer field", type: "integer", group_operator: 'sum'},
                 },
                 records: [{
                     id: 1,
                     display_name: "first record",
                     foo: "yop",
+                    int_field: 3,
                 }, {
                     id: 2,
                     display_name: "second record",
                     foo: "lalala",
+                    int_field: 5,
                 }, {
                     id: 4,
                     display_name: "aaa",
                     foo: "abc",
+                    int_field: 2,
                 }],
             },
         };
-    }
+    },
 });
 
-QUnit.test('dashboard basic rendering', function (assert) {
+QUnit.test('dashboard basic rendering', async function (assert) {
     assert.expect(4);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -56,12 +59,12 @@ QUnit.test('dashboard basic rendering', function (assert) {
             '</form>',
     });
 
-    assert.notOk(form.renderer.$el.hasClass('o_dashboard'),
+    assert.doesNotHaveClass(form.renderer.$el, 'o_dashboard',
         "should not have the o_dashboard css class");
 
     form.destroy();
 
-    form = createView({
+    form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -72,19 +75,19 @@ QUnit.test('dashboard basic rendering', function (assert) {
             '</form>',
     });
 
-    assert.ok(form.renderer.$el.hasClass('o_dashboard'),
+    assert.hasClass(form.renderer.$el,'o_dashboard',
         "with a dashboard, the renderer should have the proper css class");
-    assert.strictEqual(form.$('.o_dashboard .o_view_nocontent').length, 1,
+    assert.containsOnce(form, '.o_dashboard .o_view_nocontent',
         "should have a no content helper");
-    assert.strictEqual(form.get('title'), "My Dashboard",
+    assert.strictEqual(form.$('.o_control_panel .breadcrumb-item').text(), "My Dashboard",
         "should have the correct title");
     form.destroy();
 });
 
-QUnit.test('display the no content helper', function (assert) {
+QUnit.test('display the no content helper', async function (assert) {
     assert.expect(1);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -100,39 +103,45 @@ QUnit.test('display the no content helper', function (assert) {
         },
     });
 
-    assert.strictEqual(form.$('.o_dashboard .o_view_nocontent').length, 1,
+    assert.containsOnce(form, '.o_dashboard .o_view_nocontent',
         "should have a no content helper with action help");
     form.destroy();
 });
 
-QUnit.test('basic functionality, with one sub action', function (assert) {
-    assert.expect(25);
+QUnit.test('basic functionality, with one sub action', async function (assert) {
+    assert.expect(26);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
         arch: '<form string="My Dashboard">' +
                 '<board style="2-1">' +
                     '<column>' +
-                        '<action context="{}" view_mode="list" string="ABC" name="51" domain="[[\'foo\', \'!=\', \'False\']]"></action>' +
+                        '<action context="{&quot;orderedBy&quot;: [{&quot;name&quot;: &quot;foo&quot;, &quot;asc&quot;: True}]}" view_mode="list" string="ABC" name="51" domain="[[\'foo\', \'!=\', \'False\']]"></action>' +
                     '</column>' +
                 '</board>' +
             '</form>',
         mockRPC: function (route, args) {
             if (route === '/web/action/load') {
                 assert.step('load action');
-                return $.when({
+                return Promise.resolve({
                     res_model: 'partner',
                     views: [[4, 'list']],
                 });
             }
             if (route === '/web/dataset/search_read') {
                 assert.deepEqual(args.domain, [['foo', '!=', 'False']], "the domain should be passed");
+                assert.deepEqual(args.context.orderedBy, [{
+                        'name': 'foo',
+                        'asc': true,
+                    }],
+                    'orderedBy is present in the search read when specified on the custom action'
+                );
             }
             if (route === '/web/view/edit_custom') {
                 assert.step('edit custom');
-                return $.when(true);
+                return Promise.resolve(true);
             }
             return this._super.apply(this, arguments);
         },
@@ -142,59 +151,94 @@ QUnit.test('basic functionality, with one sub action', function (assert) {
         },
     });
 
-    assert.strictEqual(form.$('.oe_dashboard_links').length, 1,
+    assert.containsOnce(form, '.oe_dashboard_links',
         "should have rendered a link div");
-    assert.strictEqual(form.$('table.oe_dashboard[data-layout="2-1"]').length, 1,
+    assert.containsOnce(form, 'table.oe_dashboard[data-layout="2-1"]',
         "should have rendered a table");
-    assert.strictEqual(form.$('td.o_list_record_selector').length, 0,
+    assert.containsNone(form, 'td.o_list_record_selector',
         "td should not have a list selector");
     assert.strictEqual(form.$('h2 span.oe_header_txt:contains(ABC)').length, 1,
         "should have rendered a header with action string");
-    assert.strictEqual(form.$('tr.o_data_row').length, 3,
+    assert.containsN(form, 'tr.o_data_row', 3,
         "should have rendered 3 data rows");
 
     assert.ok(form.$('.oe_content').is(':visible'), "content is visible");
 
-    form.$('.oe_fold').click();
+    await testUtils.dom.click(form.$('.oe_fold'));
 
     assert.notOk(form.$('.oe_content').is(':visible'), "content is no longer visible");
 
-    form.$('.oe_fold').click();
+    await testUtils.dom.click(form.$('.oe_fold'));
 
     assert.ok(form.$('.oe_content').is(':visible'), "content is visible again");
     assert.verifySteps(['load action', 'edit custom', 'edit custom']);
 
     assert.strictEqual($('.modal').length, 0, "should have no modal open");
 
-    form.$('button.oe_dashboard_link_change_layout').click();
+    await testUtils.dom.click(form.$('button.oe_dashboard_link_change_layout'));
 
     assert.strictEqual($('.modal').length, 1, "should have opened a modal");
     assert.strictEqual($('.modal li[data-layout="2-1"] i.oe_dashboard_selected_layout').length, 1,
         "should mark currently selected layout");
 
-    $('.modal .oe_dashboard_layout_selector li[data-layout="1-1"]').click();
+    await testUtils.dom.click($('.modal .oe_dashboard_layout_selector li[data-layout="1-1"]'));
 
     assert.strictEqual($('.modal').length, 0, "should have no modal open");
-    assert.strictEqual(form.$('table.oe_dashboard[data-layout="1-1"]').length, 1,
+    assert.containsOnce(form, 'table.oe_dashboard[data-layout="1-1"]',
         "should have rendered a table with correct layout");
 
 
-    assert.strictEqual(form.$('.oe_action').length, 1, "should have one displayed action");
-    form.$('span.oe_close').click();
+    assert.containsOnce(form, '.oe_action', "should have one displayed action");
+    await testUtils.dom.click(form.$('span.oe_close'));
 
     assert.strictEqual($('.modal').length, 1, "should have opened a modal");
 
     // confirm the close operation
-    $('.modal button.btn-primary').click();
+    await testUtils.dom.click($('.modal button.btn-primary'));
 
     assert.strictEqual($('.modal').length, 0, "should have no modal open");
-    assert.strictEqual(form.$('.oe_action').length, 0, "should have no displayed action");
+    assert.containsNone(form, '.oe_action', "should have no displayed action");
 
-    assert.verifySteps(['load action', 'edit custom', 'edit custom', 'edit custom', 'edit custom']);
+    assert.verifySteps(['edit custom', 'edit custom']);
     form.destroy();
 });
 
-QUnit.test('can render an action without view_mode attribute', function (assert) {
+QUnit.test('views in the dashboard do not have a control panel', async function (assert) {
+    assert.expect(2);
+
+    var form = await createView({
+        View: BoardView,
+        model: 'board',
+        data: this.data,
+        arch: '<form>' +
+                '<board style="2-1">' +
+                    '<column>' +
+                        '<action context="{}" view_mode="list" string="ABC" name="51" domain="[]"></action>' +
+                    '</column>' +
+                '</board>' +
+            '</form>',
+        mockRPC: function (route) {
+            if (route === '/web/action/load') {
+                return Promise.resolve({
+                    res_model: 'partner',
+                    views: [[4, 'list'], [5, 'form']],
+                });
+            }
+            return this._super.apply(this, arguments);
+        },
+        archs: {
+            'partner,4,list':
+                '<tree string="Partner"><field name="foo"/></tree>',
+        },
+    });
+
+    assert.containsOnce(form, '.o_action .o_list_view');
+    assert.containsNone(form, '.o_action .o_control_panel');
+
+    form.destroy();
+});
+
+QUnit.test('can render an action without view_mode attribute', async function (assert) {
     // The view_mode attribute is automatically set to the 'action' nodes when
     // the action is added to the dashboard using the 'Add to dashboard' button
     // in the searchview. However, other dashboard views can be written by hand
@@ -204,7 +248,7 @@ QUnit.test('can render an action without view_mode attribute', function (assert)
     // first view of the action can be used, by default.
     assert.expect(3);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -221,10 +265,10 @@ QUnit.test('can render an action without view_mode attribute', function (assert)
         },
         mockRPC: function (route, args) {
             if (route === '/board/static/src/img/layout_1-1-1.png') {
-                return $.when();
+                return Promise.resolve();
             }
             if (route === '/web/action/load') {
-                return $.when({
+                return Promise.resolve({
                     context: '{"b": 2}',
                     domain: '[["foo", "=", "yop"]]',
                     res_model: 'partner',
@@ -249,12 +293,12 @@ QUnit.test('can render an action without view_mode attribute', function (assert)
     form.destroy();
 });
 
-QUnit.test('can sort a sub list', function (assert) {
+QUnit.test('can sort a sub list', async function (assert) {
     assert.expect(2);
 
     this.data.partner.fields.foo.sortable = true;
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -267,7 +311,7 @@ QUnit.test('can sort a sub list', function (assert) {
             '</form>',
         mockRPC: function (route) {
             if (route === '/web/action/load') {
-                return $.when({
+                return Promise.resolve({
                     res_model: 'partner',
                     views: [[4, 'list']],
                 });
@@ -283,17 +327,17 @@ QUnit.test('can sort a sub list', function (assert) {
     assert.strictEqual($('tr.o_data_row').text(), 'yoplalalaabc',
         "should have correct initial data");
 
-    form.$('th.o_column_sortable:contains(Foo)').click();
+    await testUtils.dom.click(form.$('th.o_column_sortable:contains(Foo)'));
 
     assert.strictEqual($('tr.o_data_row').text(), 'abclalalayop',
         "data should have been sorted");
     form.destroy();
 });
 
-QUnit.test('can open a record', function (assert) {
+QUnit.test('can open a record', async function (assert) {
     assert.expect(1);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -306,7 +350,7 @@ QUnit.test('can open a record', function (assert) {
             '</form>',
         mockRPC: function (route) {
             if (route === '/web/action/load') {
-                return $.when({
+                return Promise.resolve({
                     res_model: 'partner',
                     views: [[4, 'list']],
                 });
@@ -329,14 +373,14 @@ QUnit.test('can open a record', function (assert) {
         },
     });
 
-    form.$('tr.o_data_row td:contains(yop)').click();
+    await testUtils.dom.click(form.$('tr.o_data_row td:contains(yop)'));
     form.destroy();
 });
 
-QUnit.test('can open record using action form view', function (assert) {
+QUnit.test('can open record using action form view', async function (assert) {
     assert.expect(1);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -349,7 +393,7 @@ QUnit.test('can open record using action form view', function (assert) {
             '</form>',
         mockRPC: function (route) {
             if (route === '/web/action/load') {
-                return $.when({
+                return Promise.resolve({
                     res_model: 'partner',
                     views: [[4, 'list'], [5, 'form']],
                 });
@@ -374,14 +418,14 @@ QUnit.test('can open record using action form view', function (assert) {
         },
     });
 
-    form.$('tr.o_data_row td:contains(yop)').click();
+    await testUtils.dom.click(form.$('tr.o_data_row td:contains(yop)'));
     form.destroy();
 });
 
-QUnit.test('can drag and drop a view', function (assert) {
-    assert.expect(4);
+QUnit.test('can drag and drop a view', async function (assert) {
+    assert.expect(5);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -394,14 +438,14 @@ QUnit.test('can drag and drop a view', function (assert) {
             '</form>',
         mockRPC: function (route) {
             if (route === '/web/action/load') {
-                return $.when({
+                return Promise.resolve({
                     res_model: 'partner',
                     views: [[4, 'list']],
                 });
             }
             if (route === '/web/view/edit_custom') {
                 assert.step('edit custom');
-                return $.when(true);
+                return Promise.resolve(true);
             }
             return this._super.apply(this, arguments);
         },
@@ -411,23 +455,24 @@ QUnit.test('can drag and drop a view', function (assert) {
         },
     });
 
-    assert.strictEqual(form.$('td.index_0 .oe_action').length, 1,
+    assert.containsOnce(form, 'td.index_0 .oe_action',
         "initial action is in column 0");
 
-    testUtils.dragAndDrop(form.$('.oe_dashboard_column.index_0 .oe_header'),
+    await testUtils.dom.dragAndDrop(form.$('.oe_dashboard_column.index_0 .oe_header'),
         form.$('.oe_dashboard_column.index_1'));
-    assert.strictEqual(form.$('td.index_0 .oe_action').length, 0,
+    assert.containsNone(form, 'td.index_0 .oe_action',
         "initial action is not in column 0");
-    assert.strictEqual(form.$('td.index_1 .oe_action').length, 1,
+    assert.containsOnce(form, 'td.index_1 .oe_action',
         "initial action is in in column 1");
+    assert.verifySteps(['edit custom']);
 
     form.destroy();
 });
 
-QUnit.test('twice the same action in a dashboard', function (assert) {
+QUnit.test('twice the same action in a dashboard', async function (assert) {
     assert.expect(2);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -441,14 +486,14 @@ QUnit.test('twice the same action in a dashboard', function (assert) {
             '</form>',
         mockRPC: function (route) {
             if (route === '/web/action/load') {
-                return $.when({
+                return Promise.resolve({
                     res_model: 'partner',
                     views: [[4, 'list'],[5, 'kanban']],
                 });
             }
             if (route === '/web/view/edit_custom') {
                 assert.step('edit custom');
-                return $.when(true);
+                return Promise.resolve(true);
             }
             return this._super.apply(this, arguments);
         },
@@ -472,10 +517,10 @@ QUnit.test('twice the same action in a dashboard', function (assert) {
     form.destroy();
 });
 
-QUnit.test('non-existing action in a dashboard', function (assert) {
+QUnit.test('non-existing action in a dashboard', async function (assert) {
     assert.expect(1);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -493,11 +538,11 @@ QUnit.test('non-existing action in a dashboard', function (assert) {
         },
         mockRPC: function (route) {
             if (route === '/board/static/src/img/layout_1-1-1.png') {
-                return $.when();
+                return Promise.resolve();
             }
             if (route === '/web/action/load') {
                 // server answer if the action doesn't exist anymore
-                return $.when(false);
+                return Promise.resolve(false);
             }
             return this._super.apply(this, arguments);
         },
@@ -509,10 +554,10 @@ QUnit.test('non-existing action in a dashboard', function (assert) {
     form.destroy();
 });
 
-QUnit.test('clicking on a kanban\'s button should trigger the action', function (assert) {
+QUnit.test('clicking on a kanban\'s button should trigger the action', async function (assert) {
     assert.expect(2);
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -544,34 +589,34 @@ QUnit.test('clicking on a kanban\'s button should trigger the action', function
 
         mockRPC: function (route) {
             if (route === '/board/static/src/img/layout_1-1-1.png') {
-                return $.when();
+                return Promise.resolve();
             }
             if (route === '/web/action/load') {
-                return $.when({res_model: 'partner', view_mode: 'kanban', views: [[false, 'kanban']]});
+                return Promise.resolve({res_model: 'partner', view_mode: 'kanban', views: [[false, 'kanban']]});
             }
             if (route === '/web/dataset/search_read') {
-                return $.when({records: [{foo: 'aqualung'}]});
+                return Promise.resolve({records: [{foo: 'aqualung'}]});
             }
             return this._super.apply(this, arguments);
         }
     });
 
-    form.$('.o_kanban_test').find('button:first').click();
+    await testUtils.dom.click(form.$('.o_kanban_test').find('button:first'));
 
     form.destroy();
 });
 
-QUnit.test('subviews are aware of attach in or detach from the DOM', function (assert) {
+QUnit.test('subviews are aware of attach in or detach from the DOM', async function (assert) {
     assert.expect(2);
 
     // patch list renderer `on_attach_callback` for the test only
-    testUtils.patch(ListRenderer, {
+    testUtils.mock.patch(ListRenderer, {
         on_attach_callback: function () {
             assert.step('subview on_attach_callback');
         }
     });
 
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -584,7 +629,7 @@ QUnit.test('subviews are aware of attach in or detach from the DOM', function (a
             '</form>',
         mockRPC: function (route) {
             if (route === '/web/action/load') {
-                return $.when({
+                return Promise.resolve({
                     res_model: 'partner',
                     views: [[4, 'list']],
                 });
@@ -600,25 +645,24 @@ QUnit.test('subviews are aware of attach in or detach from the DOM', function (a
     assert.verifySteps(['subview on_attach_callback']);
 
     // restore on_attach_callback of ListRenderer
-    testUtils.unpatch(ListRenderer);
+    testUtils.mock.unpatch(ListRenderer);
 
     form.destroy();
 });
 
-QUnit.test('dashboard intercepts custom events triggered by sub controllers', function (assert) {
-    assert.expect(4);
+QUnit.test('dashboard intercepts custom events triggered by sub controllers', async function (assert) {
+    assert.expect(1);
 
     // we patch the ListController to force it to trigger the custom events that
     // we want the dashboard to intercept (to stop them or to tweak their data)
-    testUtils.patch(ListController, {
+    testUtils.mock.patch(ListController, {
         start: function () {
             this.trigger_up('update_filters');
-            this.trigger_up('env_updated');
-            this.do_action({}, {keepSearchView: true});
+            return this._super.apply(this, arguments);
         },
     });
 
-    var board = createView({
+    var board = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -631,7 +675,7 @@ QUnit.test('dashboard intercepts custom events triggered by sub controllers', fu
             '</form>',
         mockRPC: function (route) {
             if (route === '/web/action/load') {
-                return $.when({res_model: 'partner', views: [[false, 'list']]});
+                return Promise.resolve({res_model: 'partner', views: [[false, 'list']]});
             }
             return this._super.apply(this, arguments);
         },
@@ -639,31 +683,32 @@ QUnit.test('dashboard intercepts custom events triggered by sub controllers', fu
             'partner,false,list': '<tree string="Partner"/>',
         },
         intercepts: {
-            do_action: function (ev) {
-                assert.strictEqual(ev.data.options.keepSearchView, false,
-                    "the 'keepSearchView' options should have been set to false");
-            },
-            env_updated: function (ev) {
-                assert.strictEqual(ev.target.modelName, 'board',
-                    "env_updated event should be triggered by the dashboard itself");
-                assert.step('env_updated');
-            },
             update_filters: assert.step.bind(assert, 'update_filters'),
         },
     });
 
-    assert.verifySteps([
-        'env_updated', // triggered by the dashboard itself
-    ]);
+    assert.verifySteps([]);
 
-    testUtils.unpatch(ListController);
+    testUtils.mock.unpatch(ListController);
     board.destroy();
 });
 
-QUnit.test('save actions to dashboard', function (assert) {
-    assert.expect(4);
+QUnit.test('save actions to dashboard', async function (assert) {
+    assert.expect(6);
 
-    var actionManager = createActionManager({
+    testUtils.patch(ListController, {
+        getOwnedQueryParams: function () {
+            var result = this._super.apply(this, arguments);
+            result.context = {
+                'fire': 'on the bayou',
+            }
+            return result;
+        }
+    });
+
+    this.data['partner'].fields.foo.sortable = true;
+
+    var actionManager = await createActionManager({
         data: this.data,
         archs: {
             'partner,false,list': '<list><field name="foo"/></list>',
@@ -671,39 +716,60 @@ QUnit.test('save actions to dashboard', function (assert) {
         },
         mockRPC: function (route, args) {
             if (route === '/board/add_to_dashboard') {
+                assert.deepEqual(args.context_to_save.group_by, ['foo'],
+                    'The group_by should have been saved');
+                assert.deepEqual(args.context_to_save.orderedBy,
+                    [{
+                        name: 'foo',
+                        asc: true,
+                    }],
+                    'The orderedBy should have been saved');
+                assert.strictEqual(args.context_to_save.fire, 'on the bayou',
+                    'The context of a controller should be passed and flattened');
                 assert.strictEqual(args.action_id, 1,
                     "should save the correct action");
                 assert.strictEqual(args.view_mode, 'list',
                     "should save the correct view type");
-                return $.when(true);
+                return Promise.resolve(true);
             }
             return this._super.apply(this, arguments);
         },
     });
 
-    actionManager.doAction({
+    await actionManager.doAction({
         id: 1,
         res_model: 'partner',
         type: 'ir.actions.act_window',
         views: [[false, 'list']],
     });
 
-    assert.strictEqual(actionManager.$('.o_list_view').length, 1,
+    assert.containsOnce(actionManager, '.o_list_view',
         "should display the list view");
-    assert.strictEqual($('.o_add_to_dashboard_link').length, 1,
-        "should allow the 'Add to dashboard' feature");
+
+    // Sort the list
+    await testUtils.dom.click($('.o_column_sortable'));
+
+    // Group It
+    await testUtils.dom.click($('.o_search_options .o_dropdown button:contains(Group By)'));
+    await testUtils.dom.click($('.o_search_options .o_group_by_menu button'));
+    await testUtils.dom.click($('.o_search_options .o_group_by_menu button.o_apply_group'));
 
     // add this action to dashboard
-    $('.o_add_to_dashboard_button').click();
+    await testUtils.dom.click($('.o_search_options .o_dropdown button:contains(Favorites)'));
+    await testUtils.dom.click($('.o_add_to_board.o_menu_header'));
+    testUtils.fields.editInput($('input.o_add_to_board_input'), 'a name');
+    await testUtils.dom.click($('.o_add_to_board_confirm_button'));
+
+    testUtils.unpatch(ListController);
 
     actionManager.destroy();
 });
 
-QUnit.test('save two searches to dashboard', function (assert) {
+QUnit.test('save two searches to dashboard', async function (assert) {
     // the second search saved should not be influenced by the first
     assert.expect(2);
 
-    var actionManager = createActionManager({
+    var actionManager = await createActionManager({
         data: this.data,
         archs: {
             'partner,false,list': '<list><field name="foo"/></list>',
@@ -721,13 +787,13 @@ QUnit.test('save two searches to dashboard', function (assert) {
                 }
 
                 filter_count += 1;
-                return $.when(true);
+                return Promise.resolve(true);
             }
             return this._super.apply(this, arguments);
         },
     });
 
-    actionManager.doAction({
+    await actionManager.doAction({
         id: 1,
         res_model: 'partner',
         type: 'ir.actions.act_window',
@@ -736,50 +802,53 @@ QUnit.test('save two searches to dashboard', function (assert) {
 
     var filter_count = 0;
     // Add a first filter
-    $('span.fa-filter').click();
-    $('.o_add_custom_filter:visible').click();
-    $('.o_searchview_extended_prop_value .o_input').val('a')
-    $('.o_apply_filter').click();
+    await testUtils.dom.click(actionManager.$('.o_filters_menu_button'));
+    await testUtils.dom.click(actionManager.$('.o_add_custom_filter'));
+    actionManager.$('.o_searchview_extended_prop_value .o_input').val('a');
+    await testUtils.dom.click(actionManager.$('.o_apply_filter'));
     // Add it to dashboard
-    testUtilsDom.click($('.o_search_options button:contains(Favorites)'));
+    await testUtils.dom.click(actionManager.$('.o_favorites_menu_button'));
     $('.o_search_options .dropdown-menu.o_favorites_menu').one('click', function (ev) {
         // This handler is on the webClient
         // But since the test suite doesn't have one
         // We manually set it here
         ev.stopPropagation();
     });
-    testUtilsDom.click($('.o_search_options .o_add_to_dashboard_link'));
-    testUtilsDom.click($('.o_add_to_dashboard_button'));
+    await testUtils.dom.click(actionManager.$('.o_add_to_board'));
+    await testUtils.dom.click(actionManager.$('.o_add_to_board_confirm_button'));
     // Remove it
-    $('.o_facet_remove').click();
+    await testUtils.dom.click(actionManager.$('.o_facet_remove'));
 
     // Add the second filter
-    $('span.fa-filter').click();
-    $('.o_add_custom_filter:visible').click();
-    $('.o_searchview_extended_prop_value .o_input').val('b')
-    $('.o_apply_filter').click();
+    await testUtils.dom.click(actionManager.$('.o_filters_menu_button'));
+    await testUtils.dom.click(actionManager.$('.o_add_custom_filter'));
+    actionManager.$('.o_searchview_extended_prop_value .o_input').val('b');
+    await testUtils.dom.click(actionManager.$('.o_apply_filter'));
     // Add it to dashboard
-    testUtilsDom.click($('.o_search_options button:contains(Favorites)'));
+    await testUtils.dom.click(actionManager.$('.o_favorites_menu_button'));
     $('.o_search_options .dropdown-menu.o_favorites_menu').one('click', function (ev) {
         // This handler is on the webClient
         // But since the test suite doesn't have one
         // We manually set it here
         ev.stopPropagation();
     });
-    testUtilsDom.click($('.o_search_options .o_add_to_dashboard_link'));
-    testUtilsDom.click($('.o_add_to_dashboard_button'));
+    await testUtils.dom.click(actionManager.$('.o_add_to_board'));
+    await testUtils.dom.click(actionManager.$('.o_add_to_board_confirm_button'));
 
     actionManager.destroy();
 });
 
-QUnit.test('save a action domain to dashboard', function (assert) {
+QUnit.test('save a action domain to dashboard', async function (assert) {
     // View domains are to be added to the dashboard domain
     assert.expect(1);
 
     var view_domain = ["display_name", "ilike", "a"];
     var filter_domain = ["display_name", "ilike", "b"];
 
-    var actionManager = createActionManager({
+    // The filter domain already contains the view domain, but is always added by dashboard..,
+    var expected_domain = ['&', '&', view_domain, view_domain, filter_domain]
+
+    var actionManager = await createActionManager({
         data: this.data,
         archs: {
             'partner,false,list': '<list><field name="foo"/></list>',
@@ -787,15 +856,15 @@ QUnit.test('save a action domain to dashboard', function (assert) {
         },
         mockRPC: function (route, args) {
             if (route === '/board/add_to_dashboard') {
-                assert.deepEqual(args.domain, [view_domain, filter_domain],
+                assert.deepEqual(args.domain, expected_domain,
                     "the correct domain should be sent");
-                return $.when(true);
+                return Promise.resolve(true);
             }
             return this._super.apply(this, arguments);
         },
     });
 
-    actionManager.doAction({
+    await actionManager.doAction({
         id: 1,
         res_model: 'partner',
         type: 'ir.actions.act_window',
@@ -804,69 +873,21 @@ QUnit.test('save a action domain to dashboard', function (assert) {
     });
 
     // Add a filter
-    $('span.fa-filter').click();
-    $('.o_add_custom_filter:visible').click();
-    $('.o_searchview_extended_prop_value .o_input').val('b')
-    $('.o_apply_filter').click();
+    await testUtils.dom.click(actionManager.$('.o_filters_menu_button'));
+    await testUtils.dom.click(actionManager.$('.o_add_custom_filter'));
+    actionManager.$('.o_searchview_extended_prop_value .o_input').val('b');
+    await testUtils.dom.click(actionManager.$('.o_apply_filter'));
     // Add it to dashboard
-    $('.o_add_to_dashboard_button').click();
+    await testUtils.dom.click(actionManager.$('.o_favorites_menu_button'));
+    await testUtils.dom.click(actionManager.$('.o_add_to_board'));
+    await testUtils.dom.click(actionManager.$('.o_add_to_board_confirm_button'));
 
     actionManager.destroy();
 });
 
-QUnit.test('save to dashboard actions with flag keepSearchView', function (assert) {
-    assert.expect(4);
-
-    var actionManager = createActionManager({
-        data: this.data,
-        archs: {
-            'partner,false,graph': '<graph><field name="foo"/></graph>',
-            'partner,false,list': '<list><field name="foo"/></list>',
-            'partner,false,search': '<search></search>',
-        },
-        mockRPC: function (route, args) {
-            if (route === '/board/add_to_dashboard') {
-                assert.strictEqual(args.action_id, 2,
-                    "should save the correct action");
-                assert.strictEqual(args.view_mode, 'graph',
-                    "should save the correct view type");
-                return $.when(true);
-            }
-            return this._super.apply(this, arguments);
-        },
-    });
-
-    // execute a first action
-    actionManager.doAction({
-        id: 1,
-        res_model: 'partner',
-        type: 'ir.actions.act_window',
-        views: [[false, 'list']],
-    });
-
-    // execute another action with flag 'keepSearchView' and add it to dashboard
-    var options = {keepSearchView: true};
-    actionManager.doAction({
-        id: 2,
-        res_model: 'partner',
-        type: 'ir.actions.act_window',
-        views: [[false, 'graph']],
-    }, options);
-
-    assert.strictEqual(actionManager.$('.o_graph').length, 1,
-        "should display the graph view");
-    assert.strictEqual($('.o_add_to_dashboard_link').length, 1,
-        "should allow the 'Add to dashboard' feature (this is the same searchview)");
-
-    // add this action to dashboard
-    $('.o_add_to_dashboard_button').click();
-
-    actionManager.destroy();
-});
-
-QUnit.test("Views should be loaded in the user's language", function (assert) {
+QUnit.test("Views should be loaded in the user's language", async function (assert) {
     assert.expect(2);
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -884,11 +905,11 @@ QUnit.test("Views should be loaded in the user's language", function (assert) {
                     'The views should be loaded with the correct context');
             }
             if (route === "/web/dataset/search_read") {
-                assert.deepEqual(args.context, {lang: 'fr_FR'},
+                assert.equal(args.context.lang, 'fr_FR',
                     'The data should be loaded with the correct context');
             }
             if (route === '/web/action/load') {
-                return $.when({
+                return Promise.resolve({
                     res_model: 'partner',
                     views: [[4, 'list']],
                 });
@@ -904,9 +925,9 @@ QUnit.test("Views should be loaded in the user's language", function (assert) {
     form.destroy();
 });
 
-QUnit.test("Dashboard should use correct groupby", function (assert) {
+QUnit.test("Dashboard should use correct groupby", async function (assert) {
     assert.expect(1);
-    var form = createView({
+    var form = await createView({
         View: BoardView,
         model: 'board',
         data: this.data,
@@ -918,12 +939,12 @@ QUnit.test("Dashboard should use correct groupby", function (assert) {
                 '</board>' +
             '</form>',
         mockRPC: function (route, args) {
-            if (args.method === 'read_group') {
+            if (args.method === 'web_read_group') {
                 assert.deepEqual(args.kwargs.groupby, ['bar'],
                     'user defined groupby should have precedence on action groupby');
             }
             if (route === '/web/action/load') {
-                return $.when({
+                return Promise.resolve({
                     res_model: 'partner',
                     context: {
                         group_by: 'some_field',
@@ -941,4 +962,85 @@ QUnit.test("Dashboard should use correct groupby", function (assert) {
 
     form.destroy();
 });
+
+QUnit.test("Dashboard should use correct groupby when defined as a string of one field", async function (assert) {
+    assert.expect(1);
+    var form = await createView({
+        View: BoardView,
+        model: 'board',
+        data: this.data,
+        arch: '<form string="My Dashboard">' +
+                '<board style="2-1">' +
+                    '<column>' +
+                        '<action context="{\'group_by\': \'bar\'}" string="ABC" name="51"></action>' +
+                    '</column>' +
+                '</board>' +
+            '</form>',
+        mockRPC: function (route, args) {
+            if (args.method === 'web_read_group') {
+                assert.deepEqual(args.kwargs.groupby, ['bar'],
+                    'user defined groupby should have precedence on action groupby');
+            }
+            if (route === '/web/action/load') {
+                return Promise.resolve({
+                    res_model: 'partner',
+                    context: {
+                        group_by: 'some_field',
+                    },
+                    views: [[4, 'list']],
+                });
+            }
+            return this._super.apply(this, arguments);
+        },
+        archs: {
+            'partner,4,list':
+                '<list string="Partner"><field name="foo"/></list>',
+        },
+    });
+
+    form.destroy();
+});
+
+QUnit.test('click on a cell of pivot view inside dashboard', async function (assert) {
+    assert.expect(3);
+
+    var form = await createView({
+        View: BoardView,
+        model: 'board',
+        data: this.data,
+        arch: '<form>' +
+                '<board style="2-1">' +
+                    '<column>' +
+                        '<action view_mode="pivot" string="ABC" name="51"></action>' +
+                    '</column>' +
+                '</board>' +
+            '</form>',
+        mockRPC: function (route) {
+            if (route === '/web/action/load') {
+                return Promise.resolve({
+                    res_model: 'partner',
+                    views: [[4, 'pivot']],
+                });
+            }
+            return this._super.apply(this, arguments);
+        },
+        archs: {
+            'partner,4,pivot': '<pivot><field name="int_field" type="measure"/></pivot>',
+        },
+        intercepts: {
+            do_action: function () {
+                assert.step('do action');
+            },
+        },
+    });
+
+    assert.verifySteps([]);
+
+    await testUtils.dom.click(form.$('.o_pivot .o_pivot_cell_value'));
+
+    assert.verifySteps(['do action']);
+
+    form.destroy();
+});
+
 });
