@@ -7,6 +7,7 @@ from datetime import datetime, timedelta, time
 from odoo import fields
 from odoo.tests.common import TransactionCase
 import pytz
+import re
 
 
 class TestCalendar(TransactionCase):
@@ -94,7 +95,7 @@ class TestCalendar(TransactionCase):
             'name': '1 Day before event starts',
             'duration': 1,
             'interval': 'days',
-            'type': 'notification'
+            'alarm_type': 'notification'
         })
 
         # Now I will assign this reminder to all day event|
@@ -225,10 +226,11 @@ class TestCalendar(TransactionCase):
         })
         now = datetime.now()
         test_user = self.env.ref('base.user_demo')
-        test_name, test_description, test_description2 = 'Test-Meeting', '<p>Test-Description</p>', '<p>NotTest</p>'
+        test_name, test_description, test_description2 = 'Test-Meeting', 'Test-Description', 'NotTest'
+        test_note, test_note2 = '<p>Test-Description</p>', '<p>NotTest</p>'
 
         # create using default_* keys
-        test_event = self.env['calendar.event'].sudo(test_user).with_context(
+        test_event = self.env['calendar.event'].with_user(test_user).with_context(
             default_res_model=test_record._name,
             default_res_id=test_record.id,
         ).create({
@@ -242,7 +244,7 @@ class TestCalendar(TransactionCase):
         self.assertEqual(test_event.res_id, test_record.id)
         self.assertEqual(len(test_record.activity_ids), 1)
         self.assertEqual(test_record.activity_ids.summary, test_name)
-        self.assertEqual(test_record.activity_ids.note, test_description)
+        self.assertEqual(test_record.activity_ids.note, test_note)
         self.assertEqual(test_record.activity_ids.user_id, self.env.user)
         self.assertEqual(test_record.activity_ids.date_deadline, (now + timedelta(days=-1)).date())
 
@@ -254,16 +256,25 @@ class TestCalendar(TransactionCase):
             'user_id': test_user.id,
         })
         self.assertEqual(test_record.activity_ids.summary, '%s2' % test_name)
-        self.assertEqual(test_record.activity_ids.note, test_description2)
+        self.assertEqual(test_record.activity_ids.note, test_note2)
         self.assertEqual(test_record.activity_ids.user_id, test_user)
         self.assertEqual(test_record.activity_ids.date_deadline, (now + timedelta(days=-2)).date())
 
+        # update event with a description that have a special character and a new line
+        test_description3 = 'Test & \n Description'
+        test_note3 = '<p>Test &amp; <br> Description</p>'
+        test_event.write({
+            'description': test_description3,
+        })
+
+        self.assertEqual(test_record.activity_ids.note, test_note3)
+
         # deleting meeting should delete its activity
         test_record.activity_ids.unlink()
         self.assertEqual(self.env['calendar.event'], self.env['calendar.event'].search([('name', '=', test_name)]))
 
         # create using active_model keys
-        test_event = self.env['calendar.event'].sudo(self.env.ref('base.user_demo')).with_context(
+        test_event = self.env['calendar.event'].with_user(self.env.ref('base.user_demo')).with_context(
             active_model=test_record._name,
             active_id=test_record.id,
         ).create({
@@ -384,3 +395,60 @@ class TestCalendar(TransactionCase):
         })
 
         self.assertEqual(str(activity_id.date_deadline), '2018-10-16')
+
+    def test_event_creation_mail(self):
+        """
+        Check that mail are sent to the attendees on event creation
+        Check that mail are sent to the added attendees on event edit
+        Check that mail are NOT sent to the attendees when detaching a recurring event
+        """
+
+        def _test_one_mail_per_attendee(self, m, partners):
+            # check that every attendee receive a (single) mail for the event
+            for partner in partners:
+                mail = self.env['mail.mail'].search([
+                    ('recipient_ids', 'in', partner.id),
+                    ('subject', 'like', m.name),
+                    ])
+                self.assertEqual(len(mail), 1)
+
+        partners = [
+           self.env['res.partner'].create({'name':'testuser0','email': u'bob@example.com'}),
+           self.env['res.partner'].create({'name':'testuser1','email': u'alice@example.com'}),
+           ]
+        partner_ids = [(6, False, [p.id for p in partners]),]
+        now = fields.Datetime.now()
+        m = self.CalendarEvent.create({
+            'name': "mailTest1",
+            'allday': False,
+            'rrule': u'FREQ=DAILY;INTERVAL=1;COUNT=5',
+            'duration': 0.5,
+            'partner_ids': partner_ids,
+            'start': fields.Datetime.to_string(now + timedelta(days=10)),
+            'stop': fields.Datetime.to_string(now + timedelta(days=15)),
+            })
+
+        # every partner should have 1 mail sent
+        _test_one_mail_per_attendee(self, m, partners)
+
+        # adding more partners to the event
+        partners.extend([
+            self.env['res.partner'].create({'name':'testuser2','email': u'marc@example.com'}),
+            self.env['res.partner'].create({'name':'testuser3','email': u'carl@example.com'}),
+            self.env['res.partner'].create({'name':'testuser4','email': u'alain@example.com'}),
+            ])
+        partner_ids = [(6, False, [p.id for p in partners]),]
+        m.write({'partner_ids': partner_ids})
+
+        # more email should be sent
+        _test_one_mail_per_attendee(self, m, partners)
+
+        # calculate virtualid to detach one event
+        virtid = str(m.id) + '-' + ''.join(re.split('[\D]', fields.Datetime.to_string(now + timedelta(days=12))))
+
+        # detaching a virtual event in the chain
+        self.env['calendar.event'].browse(virtid).detach_recurring_event(values={'active':False})
+
+        # since the detach actually create an event in the backend
+        # we check that no mail notifications are sent to the attendees
+        _test_one_mail_per_attendee(self, m, partners)
