@@ -9,3 +9,5 @@ from . import crm_team
 from . import res_config_settings
 from . import res_partner
 from . import digest
+from . import crm_lead_scoring_frequency
+from . import utm
