@@ -3,7 +3,7 @@
 {
     'name': 'Events Sales',
     'version': '1.1',
-    'category': 'Marketing',
+    'category': 'Marketing/Events',
     'website': 'https://www.odoo.com/page/events',
     'description': """
 Creating registration with sales orders.
@@ -20,8 +20,8 @@ this event.
 """,
     'depends': ['event', 'sale_management'],
     'data': [
+        'views/assets.xml',
         'views/event_views.xml',
-        'views/event_views_template.xml',
         'views/product_views.xml',
         'views/sale_order_views.xml',
         'data/event_sale_data.xml',
@@ -29,6 +29,7 @@ this event.
         'security/ir.model.access.csv',
         'security/event_security.xml',
         'wizard/event_edit_registration.xml',
+        'wizard/event_configurator_views.xml',
     ],
     'demo': ['data/event_demo.xml'],
     'installable': True,
