@@ -5,7 +5,7 @@
 {
     'name': 'Attendances',
     'version': '2.0',
-    'category': 'Human Resources',
+    'category': 'Human Resources/Attendances',
     'sequence': 81,
     'summary': 'Track employee attendance',
     'description': """
@@ -22,7 +22,6 @@ actions(Check in/Check out) performed by them.
         'security/ir.model.access.csv',
         'views/web_asset_backend_template.xml',
         'views/hr_attendance_view.xml',
-        'report/hr_employee_badge.xml',
         'views/hr_department_view.xml',
         'views/hr_employee_view.xml',
         'views/res_config_settings_views.xml',
