@@ -4,7 +4,7 @@
 {
     'name': 'Recruitment',
     'version': '1.0',
-    'category': 'Human Resources',
+    'category': 'Human Resources/Recruitment',
     'sequence': 90,
     'summary': 'Track your recruitment pipeline',
     'description': "",
@@ -14,7 +14,7 @@
         'calendar',
         'fetchmail',
         'utm',
-        'document',
+        'attachment_indexation',
         'web_tour',
         'digest',
     ],
@@ -23,6 +23,7 @@
         'security/ir.model.access.csv',
         'data/hr_recruitment_data.xml',
         'data/digest_data.xml',
+        'data/hr_recruitment_templates.xml',
         'views/hr_recruitment_views.xml',
         'views/res_config_settings_views.xml',
         'views/hr_recruitment_templates.xml',
