@@ -2,7 +2,7 @@
 {
     'name': "Hr Recruitment Interview Forms",
     'version': '1.0',
-    'category': 'Human Resources Survey',
+    'category': 'Human Resources',
     'summary': 'Surveys',
     'description': """
         Use interview forms during recruitment process.
@@ -12,13 +12,13 @@
     'depends': ['survey', 'hr_recruitment'],
     'data': [
         'security/hr_recruitment_survey_security.xml',
-        'security/ir.model.access.csv',
-        'data/survey_survey_data.xml',
         'views/hr_job_views.xml',
         'views/hr_applicant_views.xml',
+        'views/survey_survey_views.xml',
         'views/res_config_setting_views.xml',
     ],
     'demo': [
+        'data/survey_demo.xml',
         'data/hr_job_demo.xml',
     ],
     'auto_install': False,
