@@ -8,8 +8,8 @@ class Job(models.Model):
 
     survey_id = fields.Many2one(
         'survey.survey', "Interview Form",
+        domain=[('category', '=', 'hr_recruitment')],
         help="Choose an interview form for this job position and you will be able to print/answer this interview from all applicants who apply for this job")
 
-    @api.multi
     def action_print_survey(self):
         return self.survey_id.action_print_survey()
