@@ -5,7 +5,7 @@
 {
     'name': 'Task Logs',
     'version': '1.0',
-    'category': 'Human Resources',
+    'category': 'Operations/Timesheets',
     'sequence': 23,
     'summary': 'Track employee time on tasks',
     'description': """
@@ -29,6 +29,7 @@ up a management by affair.
         'views/res_config_settings_views.xml',
         'views/project_views.xml',
         'views/project_portal_templates.xml',
+        'views/hr_timesheet_portal_templates.xml',
         'report/hr_timesheet_report_view.xml',
         'report/report_timesheet_templates.xml',
         'views/hr_views.xml',
