@@ -3,7 +3,7 @@
 
 {
     'name': 'ESC/POS Hardware Driver',
-    'category': 'Point of Sale',
+    'category': 'Sales/Point Of Sale',
     'sequence': 6,
     'website': 'https://www.odoo.com/page/point-of-sale-hardware',
     'summary': 'Hardware Driver for ESC/POS Printers and Cashdrawers',
@@ -18,7 +18,7 @@ that would need such functionality.
 """,
     'depends': ['hw_proxy'],
     'external_dependencies': {
-        'python' : ['usb.core','serial','qrcode'],
+        'python' : ['pyusb','pyserial','qrcode'],
     },
     'installable': False,
 }
