@@ -3,7 +3,7 @@
 
 {
     'name': 'IoT Box Homepage',
-    'category': 'Point of Sale',
+    'category': 'Sales/Point Of Sale',
     'sequence': 6,
     'website': 'https://www.odoo.com/page/point-of-sale-hardware',
     'summary': 'A homepage for the IoT Box',
@@ -12,7 +12,7 @@ IoT Box Homepage
 ================
 
 This module overrides Odoo web interface to display a simple
-Homepage that explains what's the iotbox and show the status,
+Homepage that explains what's the iotbox and shows the status,
 and where to find documentation.
 
 If you activate this module, you won't be able to access the 
