@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*
+from . import res_users
+from . import res_partner
 from . import im_livechat_channel
 from . import ir_autovacuum
 from . import mail_channel
