@@ -26,10 +26,9 @@ This is the latest Ethiopian Odoo localization and consists of:
         'data/l10n_et_chart_data.xml',
         'data/account.account.template.csv',
         'data/account_chart_template_data.xml',
-        'data/account_account_tag_data.xml',
         'data/account.tax.group.csv',
-        'data/account.tax.template.csv',
+        'data/account_tax_report_data.xml',
+        'data/account_tax_data.xml',
         'data/account_chart_template_configure_data.xml',
-        'data/res.country.state.csv',
     ],
 }
