@@ -15,16 +15,14 @@ Install some generic chart of accounts.
         'account',
     ],
     'data': [
-        'data/account_data.xml',
-        'data/l10n_generic_coa_chart_data.xml',
+        'data/l10n_generic_coa.xml',
         'data/account.account.template.csv',
-        'data/l10n_generic_coa_chart_post_data.xml',
-        'data/account_tax_template_data.xml',
-        'data/account_chart_template_data.xml',
+        'data/l10n_generic_coa_post.xml',
     ],
     'demo': [
-        'data/account_bank_statement_demo.xml',
-        'data/account_invoice_demo.xml',
+        'demo/account_bank_statement_demo.xml',
+        'demo/account_invoice_demo.xml',
+        'demo/account_reconcile_model.xml',
     ],
     'uninstall_hook': 'uninstall_hook',
 }
