@@ -12,7 +12,7 @@
 Base module for Hungarian localization
 ==========================================
 
-This module consists :
+This module consists of:
 
  - Generic Hungarian chart of accounts
  - Hungarian taxes
@@ -24,9 +24,9 @@ This module consists :
     'data': [
         'data/l10n_hu_chart_data.xml',
         'data/account.account.template.csv',
-        'data/account.account.tag.csv',
         'data/account.tax.group.csv',
-        'data/account.tax.template.csv',
+        'data/account_tax_report_data.xml',
+        'data/account_tax_template_data.xml',
         'data/account.fiscal.position.template.csv',
         'data/account.fiscal.position.tax.template.csv',
         'data/res.bank.csv',
