@@ -5,7 +5,7 @@
     'name': 'Indian - Stock Report(GST)',
     'version': '1.0',
     'description': """GST Stock Report""",
-    'category': 'Accounting',
+    'category': 'Accounting/Accounting',
     'depends': [
         'l10n_in',
         'stock',
@@ -13,6 +13,9 @@
     'data': [
         'views/report_stockpicking_operations.xml',
     ],
+    'demo': [
+        'data/product_demo.xml',
+    ],
     'installable': True,
     'application': False,
     'auto_install': True,
