@@ -22,7 +22,7 @@ Italian accounting chart and localization.
         'data/l10n_it_chart_data.xml',
         'data/account.account.template.csv',
         'data/account.tax.group.csv',
-        'data/account.tax.template.csv',
+        'data/account_tax_template.xml',
         'data/account.fiscal.position.template.csv',
         'data/account.chart.template.csv',
         'data/account_chart_template_data.xml',
