@@ -7,13 +7,10 @@ class L10nItDdt(models.Model):
     _name = 'l10n_it.ddt'
     _description = 'Transport Document'
 
-    invoice_id = fields.One2many('account.invoice', 'l10n_it_ddt_id', string='Invoice Reference',
-                                 ondelete='cascade')
-
+    invoice_id = fields.One2many('account.move', 'l10n_it_ddt_id', string='Invoice Reference', ondelete='cascade')
     name = fields.Char(string="Numero DDT", size=20, help="Transport document number", required=True)
     date = fields.Date(string="Data DDT", help="Transport document date", required=True)
 
-    @api.multi
     def name_get(self):
         res = []
         for ddt in self:
