@@ -19,8 +19,7 @@ from lxml import etree
 from datetime import datetime
 
 from odoo import api, fields, models, tools, _
-from odoo.exceptions import ValidationError
-from odoo.tools import pycompat
+from odoo.exceptions import ValidationError, UserError
 
 
 _logger = logging.getLogger(__name__)
@@ -32,19 +31,18 @@ class FetchmailServer(models.Model):
     l10n_it_is_pec = fields.Boolean('PEC server', help="If PEC Server, only mail from '...@pec.fatturapa.it' will be processed.")
     l10n_it_last_uid = fields.Integer(string='Last message UID', default=1)
 
-    @api.constrains('l10n_it_is_pec', 'type')
+    @api.constrains('l10n_it_is_pec', 'server_type')
     def _check_pec(self):
         for record in self:
-            if record.l10n_it_is_pec and record.type != 'imap':
-                raise ValidationError("PEC mail server must be of type IMAP.")
+            if record.l10n_it_is_pec and record.server_type != 'imap':
+                raise ValidationError(_("PEC mail server must be of type IMAP."))
 
-    @api.multi
     def fetch_mail(self):
         """ WARNING: meant for cron usage only - will commit() after each email! """
 
         MailThread = self.env['mail.thread']
         for server in self.filtered(lambda s: s.l10n_it_is_pec):
-            _logger.info('start checking for new emails on %s PEC server %s', server.type, server.name)
+            _logger.info('start checking for new emails on %s PEC server %s', server.server_type, server.name)
 
             count, failed = 0, 0
             imap_server = None
@@ -74,7 +72,7 @@ class FetchmailServer(models.Model):
                     # See details in message_process() in mail_thread.py
                     if isinstance(message, xmlrpclib.Binary):
                         message = bytes(message.data)
-                    if isinstance(message, pycompat.text_type):
+                    if isinstance(message, str):
                         message = message.encode('utf-8')
                     msg_txt = email.message_from_bytes(message)
 
@@ -82,14 +80,14 @@ class FetchmailServer(models.Model):
                         self._attachment_invoice(msg_txt)
                         new_max_uid = max(new_max_uid, int(uid))
                     except Exception:
-                        _logger.info('Failed to process mail from %s server %s.', server.type, server.name, exc_info=True)
+                        _logger.info('Failed to process mail from %s server %s.', server.server_type, server.name, exc_info=True)
                         failed += 1
                     self._cr.commit()
                     count += 1
                 server.write({'l10n_it_last_uid': new_max_uid})
-                _logger.info("Fetched %d email(s) on %s server %s; %d succeeded, %d failed.", count, server.type, server.name, (count - failed), failed)
+                _logger.info("Fetched %d email(s) on %s server %s; %d succeeded, %d failed.", count, server.server_type, server.name, (count - failed), failed)
             except Exception:
-                _logger.info("General failure when trying to fetch mail from %s server %s.", server.type, server.name, exc_info=True)
+                _logger.info("General failure when trying to fetch mail from %s server %s.", server.server_type, server.name, exc_info=True)
             finally:
                 if imap_server:
                     imap_server.close()
@@ -98,7 +96,8 @@ class FetchmailServer(models.Model):
         return super(FetchmailServer, self.filtered(lambda s: not s.l10n_it_is_pec)).fetch_mail()
 
     def _attachment_invoice(self, msg_txt):
-        body, attachments = self.env['mail.thread']._message_extract_payload(msg_txt)
+        parsed_values = self.env['mail.thread']._message_parse_extract_payload(msg_txt)
+        body, attachments = parsed_values['body'], parsed_values['attachments']
         from_address = tools.decode_smtp_header(msg_txt.get('from'))
         for attachment in attachments:
             split_attachment = attachment.fname.rpartition('.')
@@ -128,7 +127,7 @@ class FetchmailServer(models.Model):
                     self._create_invoice_from_mail_with_zip(attachment, from_address)
 
     def _create_invoice_from_mail(self, att_content, att_name, from_address):
-        if self.env['account.invoice'].search([('l10n_it_einvoice_name', '=', att_name)], limit=1):
+        if self.env['account.move'].search([('l10n_it_einvoice_name', '=', att_name)], limit=1):
             # invoice already exist
             _logger.info('E-invoice already exist: %s', att_name)
             return
@@ -136,11 +135,15 @@ class FetchmailServer(models.Model):
         invoice_attachment = self.env['ir.attachment'].create({
                 'name': att_name,
                 'datas': base64.encodestring(att_content),
-                'datas_fname': att_name,
                 'type': 'binary',
                 })
 
-        invoice = self.env['account.invoice']._import_xml_invoice(att_content, invoice_attachment)
+        try:
+            tree = etree.fromstring(att_content)
+        except Exception:
+            raise UserError(_('The xml file is badly formatted : {}').format(att_name))
+
+        invoice = self.env['account.move']._import_xml_invoice(tree)
         invoice.l10n_it_send_state = "new"
         invoice.source_email = from_address
         self._cr.commit()
@@ -151,7 +154,7 @@ class FetchmailServer(models.Model):
     def _create_invoice_from_mail_with_zip(self, attachment_zip, from_address):
         with zipfile.ZipFile(io.BytesIO(attachment_zip.content)) as z:
             for att_name in z.namelist():
-                if self.env['account.invoice'].search([('l10n_it_einvoice_name', '=', att_name)], limit=1):
+                if self.env['account.move'].search([('l10n_it_einvoice_name', '=', att_name)], limit=1):
                     # invoice already exist
                     _logger.info('E-invoice in zip file (%s) already exist: %s', attachment_zip.fname, att_name)
                     continue
@@ -177,13 +180,13 @@ class FetchmailServer(models.Model):
                         _logger.info('Error in decoding new receipt file: %s', attachment_name)
                         return
 
-                    elements = tree.xpath('//NomeFile', namespaces=tree.nsmap)
+                    elements = tree.xpath('//NomeFile')
                     if elements and elements[0].text:
                         filename = elements[0].text
                     else:
                         return
 
-                    related_invoice = self.env['account.invoice'].search([
+                    related_invoice = self.env['account.move'].search([
                         ('l10n_it_einvoice_name', '=', filename)])
                     if not related_invoice:
                         _logger.info('Error: invoice not found for receipt file: %s', filename)
@@ -203,7 +206,7 @@ class FetchmailServer(models.Model):
             _logger.info('Error in decoding new receipt file: %s', attachment.fname)
             return {}
 
-        elements = tree.xpath('//NomeFile', namespaces=tree.nsmap)
+        elements = tree.xpath('//NomeFile')
         if elements and elements[0].text:
             filename = elements[0].text
         else:
@@ -213,7 +216,7 @@ class FetchmailServer(models.Model):
             # Delivery receipt
             # This is the receipt sent by the ES to the transmitting subject to communicate
             # delivery of the file to the addressee
-            related_invoice = self.env['account.invoice'].search([
+            related_invoice = self.env['account.move'].search([
                 ('l10n_it_einvoice_name', '=', filename),
                 ('l10n_it_send_state', '=', 'sent')])
             if not related_invoice:
@@ -229,7 +232,7 @@ class FetchmailServer(models.Model):
             # Rejection notice
             # This is the receipt sent by the ES to the transmitting subject if one or more of
             # the checks carried out by the ES on the file received do not have a successful result.
-            related_invoice = self.env['account.invoice'].search([
+            related_invoice = self.env['account.move'].search([
                 ('l10n_it_einvoice_name', '=', filename),
                 ('l10n_it_send_state', '=', 'sent')])
             if not related_invoice:
@@ -242,7 +245,7 @@ class FetchmailServer(models.Model):
             )
             activity_vals = {
                 'activity_type_id': self.env.ref('mail.mail_activity_data_todo').id,
-                'user_id': related_invoice.user_id.id if related_invoice.user_id else self.env.user.id
+                'invoice_user_id': related_invoice.invoice_user_id.id if related_invoice.invoice_user_id else self.env.user.id
             }
             related_invoice.activity_schedule(summary='Rejection notice', **activity_vals)
 
@@ -250,7 +253,7 @@ class FetchmailServer(models.Model):
             # Failed delivery notice
             # This is the receipt sent by the ES to the transmitting subject if the file is not
             # delivered to the addressee.
-            related_invoice = self.env['account.invoice'].search([
+            related_invoice = self.env['account.move'].search([
                 ('l10n_it_einvoice_name', '=', filename),
                 ('l10n_it_send_state', '=', 'sent')])
             if not related_invoice:
@@ -275,13 +278,13 @@ class FetchmailServer(models.Model):
             # This is the receipt sent by the ES to the invoice sender to communicate the result
             # (acceptance or refusal of the invoice) of the checks carried out on the document by
             # the addressee.
-            related_invoice = self.env['account.invoice'].search([
+            related_invoice = self.env['account.move'].search([
                 ('l10n_it_einvoice_name', '=', filename),
                 ('l10n_it_send_state', '=', 'delivered')])
             if not related_invoice:
                 _logger.info('Error: invoice not found for receipt file: %s', attachment.fname)
                 return
-            elements = tree.xpath('//Esito', namespaces=tree.nsmap)
+            elements = tree.xpath('//Esito')
             if elements and elements[0].text:
                 if elements[0].text == 'EC01':
                     related_invoice.l10n_it_send_state = 'delivered_accepted'
@@ -302,7 +305,7 @@ class FetchmailServer(models.Model):
             if related_invoice.l10n_it_send_state == 'delivered_refused':
                 activity_vals = {
                     'activity_type_id': self.env.ref('mail.mail_activity_data_todo').id,
-                    'user_id': related_invoice.user_id.id if related_invoice.user_id else self.env.user.id
+                    'invoice_user_id': related_invoice.invoice_user_id.id if related_invoice.invoice_user_id else self.env.user.id
                 }
                 related_invoice.activity_schedule(summary='Outcome notice: Refused', **activity_vals)
 
@@ -318,7 +321,7 @@ class FetchmailServer(models.Model):
             # This is the receipt sent by the ES to both the invoice sender and the invoice
             # addressee to communicate the expiry of the maximum term for communication of
             # acceptance/refusal.
-            related_invoice = self.env['account.invoice'].search([
+            related_invoice = self.env['account.move'].search([
                 ('l10n_it_einvoice_name', '=', filename), ('l10n_it_send_state', '=', 'delivered')])
             if not related_invoice:
                 _logger.info('Error: invoice not found for receipt file: %s', attachment.fname)
@@ -337,19 +340,19 @@ class FetchmailServer(models.Model):
         output_str = "<ul>"
 
         for element_tag in element_tags:
-            elements = tree.xpath(element_tag, namespaces=tree.nsmap)
+            elements = tree.xpath(element_tag)
             if not elements:
                 continue
             for element in elements:
-                text = " ".join(element.text.split())
-                if text:
+                if element.text:
+                    text = " ".join(element.text.split())
                     output_str += "<li>%s: %s</li>" % (element.tag, text)
         return output_str + "</ul>"
 
     def _return_error_xml(self, tree):
         output_str = "<ul>"
 
-        elements = tree.xpath('//Errore', namespaces=tree.nsmap)
+        elements = tree.xpath('//Errore')
         if not elements:
             return
         for element in elements:
@@ -357,3 +360,17 @@ class FetchmailServer(models.Model):
             if descrizione:
                 output_str += "<li>Errore %s: %s</li>" % (element[0].text, descrizione)
         return output_str + "</ul>"
+
+class IrMailServer(models.Model):
+    _name = "ir.mail_server"
+    _inherit = "ir.mail_server"
+
+    def build_email(self, email_from, email_to, subject, body, email_cc=None, email_bcc=None, reply_to=False,
+                attachments=None, message_id=None, references=None, object_id=False, subtype='plain', headers=None,
+                body_alternative=None, subtype_alternative='plain'):
+
+        if self.env.context.get('wo_bounce_return_path') and headers:
+            headers['Return-Path'] = email_from
+        return super(IrMailServer, self).build_email(email_from, email_to, subject, body, email_cc=email_cc, email_bcc=email_bcc, reply_to=reply_to,
+                attachments=attachments, message_id=message_id, references=references, object_id=object_id, subtype=subtype, headers=headers,
+                body_alternative=body_alternative, subtype_alternative=subtype_alternative)
