@@ -17,8 +17,11 @@ Updated for Odoo 9 by Bringsvor Consulting AS <www.bringsvor.com>
     ],
     "data": ['data/l10n_no_chart_data.xml',
              'data/account_data.xml',
+             'data/account_tax_report_data.xml',
+             'data/account.account.template.csv',
              'data/account_tax_data.xml',
-             'data/account_chart_template_data.xml'],
+             'data/account_chart_template_data.xml',
+             ],
     "active": False,
     'post_init_hook': '_preserve_tag_on_taxes',
 }
