@@ -30,9 +30,9 @@ Wewnętrzny numer wersji OpenGLOBE 1.02
               'data/account.account.template.csv',
               'data/l10n_pl_chart_post_data.xml',
               'data/account_data.xml',
+              'data/account_tax_report_data.xml',
               'data/account_tax_data.xml',
               'data/account_fiscal_position_data.xml',
-              'data/res_country_state_data.xml',
               'data/account_chart_template_data.xml'
     ],
     'post_init_hook': '_preserve_tag_on_taxes',
