@@ -28,10 +28,10 @@ Romanian accounting chart and localization.
              'data/account.account.template.csv',
              'data/l10n_ro_chart_post_data.xml',
              'data/account_data.xml',
+             'data/account_tax_report_data.xml',
              'data/account_tax_data.xml',
              'data/account_fiscal_position_data.xml',
              'data/account_chart_template_data.xml',
-             'data/res.country.state.csv',
              'data/res.bank.csv',
              ],
 }
