@@ -3,6 +3,6 @@
 
 # Copyright (C) 2014 Tech Receptives (<http://techreceptives.com>).
 
-from . import account_invoice
+from . import account_move
 from . import res_company
 from . import res_partner
