@@ -5,7 +5,7 @@
 
 {
     "name": "Slovenian - Accounting",
-    "versiom": "1.1",
+    "version": "1.1",
     "author": "Mentis d.o.o.",
     "website": "http://www.mentis.si",
     'category': 'Localization',
@@ -13,15 +13,14 @@
     "depends": [
         "account",
         "base_iban",
-        "account_cancel",
     ],
     "data": [
         "data/l10n_si_chart_data.xml",
         "data/account.account.template.csv",
         "data/account.chart.template.csv",
-        'data/account.account.tag.csv',
         "data/account.tax.group.csv",
-        "data/account.tax.template.csv",
+        "data/account_tax_report_data.xml",
+        "data/account_tax_data.xml",
         "data/account.fiscal.position.template.csv",
         "data/account.fiscal.position.account.template.csv",
         "data/account.fiscal.position.tax.template.csv",
