@@ -15,7 +15,7 @@ This is the latest UK Odoo localisation necessary to run Odoo accounting for UK
     - InfoLogic UK counties listing
     - a few other adaptations""",
     'author': 'SmartMode LTD',
-    'website': 'http://www.smartmode.co.uk',
+    'website': 'https://www.odoo.com/page/accounting',
     'depends': [
         'account',
         'base_iban',
@@ -25,10 +25,9 @@ This is the latest UK Odoo localisation necessary to run Odoo accounting for UK
         'data/l10n_uk_chart_data.xml',
         'data/account.account.template.csv',
         'data/account.chart.template.csv',
-        'data/account.account.tag.csv',
         'data/account.tax.group.csv',
-        'data/account.tax.template.csv',
-        'data/res.country.state.csv',
+        'data/account_tax_report_data.xml',
+        'data/account_tax_data.xml',
         'data/account_chart_template_data.xml',
     ],
     'demo' : ['demo/l10n_uk_demo.xml'],
