@@ -10,10 +10,13 @@ from . import mail_message
 from . import mail_activity
 from . import mail_mail
 from . import mail_thread
+from . import mail_cc_mixin
+from . import mail_address_mixin
 from . import mail_blacklist
 from . import mail_channel
 from . import mail_template
 from . import mail_shortcode
+from . import models
 from . import res_partner
 from . import res_users
 from . import res_company
@@ -23,5 +26,7 @@ from . import ir_action_act_window
 from . import ir_actions
 from . import ir_attachment
 from . import ir_autovacuum
+from . import ir_http
 from . import ir_model
+from . import ir_model_fields
 from . import ir_ui_view
