@@ -5,8 +5,11 @@ var mailUtils = require('mail.utils');
 
 var AbstractField = require('web.AbstractField');
 var BasicModel = require('web.BasicModel');
+var config = require('web.config');
 var core = require('web.core');
 var field_registry = require('web.field_registry');
+var session = require('web.session');
+var framework = require('web.framework');
 var time = require('web.time');
 
 var QWeb = core.qweb;
@@ -23,11 +26,11 @@ var _t = core._t;
  *
  * @param {Widget} self a widget instance that can perform RPCs
  * @param {Array} ids the ids of activities to read
- * @return {Deferred<Array>} resolved with the activities
+ * @return {Promise<Array>} resolved with the activities
  */
 function _readActivities(self, ids) {
     if (!ids.length) {
-        return $.when([]);
+        return Promise.resolve([]);
     }
     var context = self.getSession().user_context;
     if (self.record && !_.isEmpty(self.record.getContext())) {
@@ -62,7 +65,7 @@ BasicModel.include({
      * @private
      * @param {Object} record - an element from the localData
      * @param {string} fieldName
-     * @return {Deferred<Array>} resolved with the activities
+     * @return {Promise<Array>} resolved with the activities
      */
     _fetchSpecialActivity: function (record, fieldName) {
         var localID = (record._changes && fieldName in record._changes) ?
@@ -105,10 +108,27 @@ var setDelayLabel = function (activities){
     return activities;
 };
 
+/**
+ * Set the file upload identifier for 'upload_file' type activities
+ *
+ * @param {Array} activities list of activity Object
+ * @return {Array} : list of modified activity Object
+ */
+var setFileUploadID = function (activities) {
+    _.each(activities, function (activity) {
+        if (activity.activity_category === 'upload_file') {
+            activity.fileuploadID = _.uniqueId('o_fileupload');
+        }
+    });
+    return activities;
+};
+
 var BasicActivity = AbstractField.extend({
     events: {
         'click .o_edit_activity': '_onEditActivity',
+        'change input.o_input_file': '_onFileChanged',
         'click .o_mark_as_done': '_onMarkActivityDone',
+        'click .o_mark_as_done_upload_file': '_onMarkActivityDoneUploadFile',
         'click .o_activity_template_preview': '_onPreviewMailTemplate',
         'click .o_schedule_activity': '_onScheduleActivity',
         'click .o_activity_template_send': '_onSendMailTemplate',
@@ -125,7 +145,7 @@ var BasicActivity = AbstractField.extend({
 
     /**
      * @param {integer} previousActivityTypeID
-     * @return {$.Promise}
+     * @return {Promise}
      */
     scheduleActivity: function () {
         var callback = this._reload.bind(this, { activity: true, thread: true });
@@ -142,13 +162,17 @@ var BasicActivity = AbstractField.extend({
      * @private
      * @param {Object} params
      * @param {integer} params.activityID
+     * @param {integer[]} params.attachmentIds
      * @param {string} params.feedback
+     *
+     * @return {$.Promise}
      */
     _markActivityDone: function (params) {
         var activityID = params.activityID;
-        var feedback = params.feedback;
+        var feedback = params.feedback || false;
+        var attachmentIds = params.attachmentIds || [];
 
-        this._sendActivityFeedback(activityID, feedback)
+        return this._sendActivityFeedback(activityID, feedback, attachmentIds)
             .then(this._reload.bind(this, { activity: true, thread: true }));
     },
     /**
@@ -190,14 +214,14 @@ var BasicActivity = AbstractField.extend({
      * @param {integer} id
      * @param {integer} previousActivityTypeID
      * @param {function} callback
-     * @return {$.Deferred}
+     * @return {Promise}
      */
     _openActivityForm: function (id, callback) {
         var action = {
             type: 'ir.actions.act_window',
+            name: _t("Schedule Activity"),
             res_model: 'mail.activity',
             view_mode: 'form',
-            view_type: 'form',
             views: [[false, 'form']],
             target: 'new',
             context: {
@@ -212,14 +236,18 @@ var BasicActivity = AbstractField.extend({
      * @private
      * @param {integer} activityID
      * @param {string} feedback
-     * @return {$.Promise}
+     * @param {integer[]} attachmentIds
+     * @return {Promise}
      */
-    _sendActivityFeedback: function (activityID, feedback) {
+    _sendActivityFeedback: function (activityID, feedback, attachmentIds) {
         return this._rpc({
                 model: 'mail.activity',
                 method: 'action_feedback',
                 args: [[activityID]],
-                kwargs: {feedback: feedback},
+                kwargs: {
+                    feedback: feedback,
+                    attachment_ids: attachmentIds || [],
+                },
                 context: this.record.getContext(),
             });
     },
@@ -228,6 +256,28 @@ var BasicActivity = AbstractField.extend({
     // Handlers
     //------------------------------------------------------------
 
+    /**
+     * @private
+     * @param {Object[]} activities
+     */
+    _bindOnUploadAction: function (activities) {
+        var self = this;
+        _.each(activities, function (activity) {
+            if (activity.fileuploadID) {
+                $(window).on(activity.fileuploadID, function() {
+                    framework.unblockUI();
+                    // find the button clicked and display the feedback popup on it
+                    var files = Array.prototype.slice.call(arguments, 1);
+                    self._markActivityDone({
+                        activityID: activity.id,
+                        attachmentIds: _.pluck(files, 'id')
+                    }).then(function () {
+                        self.trigger_up('reload');
+                    });
+                });
+            }
+        });
+    },
     /** Binds a focusout handler on a bootstrap popover
      *  Useful to do some operations on the popover's HTML,
      *  like keeping the user's input for the feedback
@@ -253,14 +303,25 @@ var BasicActivity = AbstractField.extend({
     /**
      * @private
      * @param {MouseEvent} ev
-     * @returns {$.Promise}
+     * @returns {Promise}
      */
     _onEditActivity: function (ev) {
         ev.preventDefault();
         var activityID = $(ev.currentTarget).data('activity-id');
         return this._openActivityForm(activityID, this._reload.bind(this, { activity: true, thread: true }));
     },
-     /**
+    /**
+     * @private
+     * @param {FormEvent} ev
+     */
+    _onFileChanged: function (ev) {
+        ev.stopPropagation();
+        ev.preventDefault();
+        var $form = $(ev.currentTarget).closest('form');
+        $form.submit();
+        framework.blockUI();
+    },
+    /**
      * Called when marking an activity as done
      *
      * It lets the current user write a feedback in a popup menu.
@@ -285,7 +346,10 @@ var BasicActivity = AbstractField.extend({
             var $panel = self.$('#o_activity_form_' + activityID);
 
             if (!$panel.data('bs.collapse')) {
-                var $form = $(QWeb.render('mail.activity_feedback_form', { previous_activity_type_id: previousActivityTypeID, force_next: forceNextActivity}));
+                var $form = $(QWeb.render('mail.activity_feedback_form', {
+                    previous_activity_type_id: previousActivityTypeID,
+                    force_next: forceNextActivity
+                }));
                 $panel.append($form);
                 self._onMarkActivityDoneActions($markDoneBtn, $form, activityID);
 
@@ -319,10 +383,13 @@ var BasicActivity = AbstractField.extend({
                 container: $markDoneBtn,
                 title : _t("Feedback"),
                 html: true,
-                trigger:'click',
+                trigger: 'manual',
                 placement: 'right', // FIXME: this should work, maybe a bug in the popper lib
                 content : function () {
-                    var $popover = $(QWeb.render('mail.activity_feedback_form', { previous_activity_type_id: previousActivityTypeID, force_next: forceNextActivity}));
+                    var $popover = $(QWeb.render('mail.activity_feedback_form', {
+                        previous_activity_type_id: previousActivityTypeID,
+                        force_next: forceNextActivity
+                    }));
                     self._onMarkActivityDoneActions($markDoneBtn, $popover, activityID);
                     return $popover;
                 },
@@ -333,6 +400,11 @@ var BasicActivity = AbstractField.extend({
                 $popover.find('#activity_feedback').focus();
                 self._bindPopoverFocusout($(this));
             }).popover('show');
+        } else {
+            var popover = $markDoneBtn.data('bs.popover');
+            if ($('#' + popover.tip.id).length === 0) {
+               popover.show();
+            }
         }
     },
     /**
@@ -349,14 +421,14 @@ var BasicActivity = AbstractField.extend({
             ev.stopPropagation();
             self._markActivityDone({
                 activityID: activityID,
-                feedback: _.escape($form.find('#activity_feedback').val()),
+                feedback: $form.find('#activity_feedback').val(),
             });
         });
         $form.on('click', '.o_activity_popover_done_next', function (ev) {
             ev.stopPropagation();
             self._markActivityDoneAndScheduleNext({
                 activityID: activityID,
-                feedback: _.escape($form.find('#activity_feedback').val()),
+                feedback: $form.find('#activity_feedback').val(),
             });
         });
         $form.on('click', '.o_activity_popover_discard', function (ev) {
@@ -371,7 +443,18 @@ var BasicActivity = AbstractField.extend({
     /**
      * @private
      * @param {MouseEvent} ev
-     * @returns {$.Deferred}
+     */
+    _onMarkActivityDoneUploadFile: function (ev) {
+        ev.preventDefault();
+        ev.stopPropagation();
+        var fileuploadID = $(ev.currentTarget).data('fileupload-id');
+        var $input = this.$("[target='" + fileuploadID + "'] > input.o_input_file");
+        $input.click();
+    },
+    /**
+     * @private
+     * @param {MouseEvent} ev
+     * @returns {Promise}
      */
     _onPreviewMailTemplate: function (ev) {
         ev.stopPropagation();
@@ -399,7 +482,7 @@ var BasicActivity = AbstractField.extend({
     /**
      * @private
      * @param {MouseEvent} ev
-     * @returns {$.Promise}
+     * @returns {Promise}
      */
     _onSendMailTemplate: function (ev) {
         ev.stopPropagation();
@@ -415,7 +498,7 @@ var BasicActivity = AbstractField.extend({
     /**
      * @private
      * @param {MouseEvent} ev
-     * @returns {$.Deferred}
+     * @returns {Promise}
      */
     _onScheduleActivity: function (ev) {
         ev.preventDefault();
@@ -426,7 +509,7 @@ var BasicActivity = AbstractField.extend({
      * @private
      * @param {MouseEvent} ev
      * @param {Object} options
-     * @returns {$.Promise}
+     * @returns {Promise}
      */
     _onUnlinkActivity: function (ev, options) {
         ev.preventDefault();
@@ -442,6 +525,19 @@ var BasicActivity = AbstractField.extend({
             })
             .then(this._reload.bind(this, {activity: true}));
     },
+    /**
+     * Unbind event triggered when a file is uploaded.
+     *
+     * @private
+     * @param {Array} activities: list of activity to unbind
+     */
+    _unbindOnUploadAction: function (activities) {
+        _.each(activities, function (activity) {
+            if (activity.fileuploadID) {
+                $(window).off(activity.fileuploadID);
+            }
+        });
+    },
 });
 
 // -----------------------------------------------------------------------------
@@ -460,11 +556,17 @@ var Activity = BasicActivity.extend({
         this._super.apply(this, arguments);
         this._activities = this.record.specialData[this.name];
     },
+    /**
+     * @override
+     */
+    destroy: function () {
+        this._unbindOnUploadAction();
+        return this._super.apply(this, arguments);
+    },
 
     //------------------------------------------------------------
     // Private
     //------------------------------------------------------------
-
     /**
      * @private
      * @param {Object} fieldsToReload
@@ -486,18 +588,23 @@ var Activity = BasicActivity.extend({
                 activity.note = '';
             }
         });
-        var activities = setDelayLabel(this._activities);
+        var activities = setFileUploadID(setDelayLabel(this._activities));
         if (activities.length) {
             var nbActivities = _.countBy(activities, 'state');
             this.$el.html(QWeb.render('mail.activity_items', {
+                uid: session.uid,
                 activities: activities,
                 nbPlannedActivities: nbActivities.planned,
                 nbTodayActivities: nbActivities.today,
                 nbOverdueActivities: nbActivities.overdue,
                 dateFormat: time.getLangDateFormat(),
                 datetimeFormat: time.getLangDatetimeFormat(),
+                session: session,
+                widget: this,
             }));
+            this._bindOnUploadAction(this._activities);
         } else {
+            this._unbindOnUploadAction(this._activities);
             this.$el.empty();
         }
     },
@@ -544,6 +651,10 @@ var KanbanActivity = BasicActivity.extend({
     events:_.extend({}, BasicActivity.prototype.events, {
         'show.bs.dropdown': '_onDropdownShow',
     }),
+    fieldDependencies: _.extend({}, BasicActivity.prototype.fieldDependencies, {
+        activity_exception_decoration: {type: 'selection'},
+        activity_exception_icon: {type: 'char'}
+    }),
 
     /**
      * @override
@@ -557,7 +668,13 @@ var KanbanActivity = BasicActivity.extend({
         this.selection = selection;
         this._setState(record);
     },
-
+    /**
+     * @override
+     */
+    destroy: function () {
+        this._unbindOnUploadAction();
+        return this._super.apply(this, arguments);
+    },
     //------------------------------------------------------------
     // Private
     //------------------------------------------------------------
@@ -573,13 +690,18 @@ var KanbanActivity = BasicActivity.extend({
      * @private
      */
     _render: function () {
-        var $span = this.$('.o_activity_btn > span');
-        $span.removeClass(function (index, classNames) {
-            return classNames.split(/\s+/).filter(function (className) {
-                return _.str.startsWith(className, 'o_activity_color_');
-            }).join(' ');
-        });
-        $span.addClass('o_activity_color_' + (this.activityState || 'default'));
+        // span classes need to be updated manually because the template cannot
+        // be re-rendered eaasily (because of the dropdown state)
+        const spanClasses = ['fa', 'fa-lg', 'fa-fw'];
+        spanClasses.push('o_activity_color_' + (this.activityState || 'default'));
+        if (this.recordData.activity_exception_decoration) {
+            spanClasses.push('text-' + this.recordData.activity_exception_decoration);
+            spanClasses.push(this.recordData.activity_exception_icon);
+        } else {
+            spanClasses.push('fa-clock-o');
+        }
+        this.$('.o_activity_btn > span').removeClass().addClass(spanClasses.join(' '));
+
         if (this.$el.hasClass('show')) {
             // note: this part of the rendering might be asynchronous
             this._renderDropdown();
@@ -590,13 +712,18 @@ var KanbanActivity = BasicActivity.extend({
      */
     _renderDropdown: function () {
         var self = this;
-        this.$('.o_activity').html(QWeb.render('mail.KanbanActivityLoading'));
+        this.$('.o_activity')
+            .toggleClass('dropdown-menu-right', config.device.isMobile)
+            .html(QWeb.render('mail.KanbanActivityLoading'));
         return _readActivities(this, this.value.res_ids).then(function (activities) {
+            activities = setFileUploadID(activities);
             self.$('.o_activity').html(QWeb.render('mail.KanbanActivityDropdown', {
                 selection: self.selection,
                 records: _.groupBy(setDelayLabel(activities), 'state'),
-                uid: self.getSession().uid,
+                session: session,
+                widget: self,
             }));
+            self._bindOnUploadAction(activities);
         });
     },
     /**
@@ -629,9 +756,51 @@ var KanbanActivity = BasicActivity.extend({
     },
 });
 
+// -----------------------------------------------------------------------------
+// Activity Exception Widget to display Exception icon ('activity_exception' widget)
+// -----------------------------------------------------------------------------
+
+var ActivityException = AbstractField.extend({
+    noLabel: true,
+    fieldDependencies: _.extend({}, AbstractField.prototype.fieldDependencies, {
+        activity_exception_icon: {type: 'char'}
+    }),
+
+    //------------------------------------------------------------
+    // Private
+    //------------------------------------------------------------
+
+    /**
+     * There is no edit mode for this widget, the icon is always readonly.
+     *
+     * @override
+     * @private
+     */
+    _renderEdit: function () {
+        return this._renderReadonly();
+    },
+
+    /**
+     * Displays the exception icon if there is one.
+     *
+     * @override
+     * @private
+     */
+    _renderReadonly: function () {
+        this.$el.empty();
+        if (this.value) {
+            this.$el.attr({
+                'title': _t('This record has an exception activity.'),
+                'class': "pull-right mt-1 text-" + this.value + " fa " + this.recordData.activity_exception_icon
+            });
+        }
+    }
+});
+
 field_registry
     .add('mail_activity', Activity)
-    .add('kanban_activity', KanbanActivity);
+    .add('kanban_activity', KanbanActivity)
+    .add('activity_exception', ActivityException);
 
 return Activity;
 
