@@ -4,23 +4,36 @@ odoo.define('mail.ActivityView', function (require) {
 var ActivityController = require('mail.ActivityController');
 var ActivityModel = require('mail.ActivityModel');
 var ActivityRenderer = require('mail.ActivityRenderer');
-var AbstractView = require('web.AbstractView');
+var BasicView = require('web.BasicView');
 var core = require('web.core');
 var view_registry = require('web.view_registry');
 
 var _lt = core._lt;
 
-var ActivityView = AbstractView.extend({
+var ActivityView = BasicView.extend({
     accesskey: "a",
     display_name: _lt('Activity'),
-    icon: 'fa-th',
-    config: _.extend({}, AbstractView.prototype.config, {
+    icon: 'fa-clock-o',
+    config: _.extend({}, BasicView.prototype.config, {
         Controller: ActivityController,
         Model: ActivityModel,
         Renderer: ActivityRenderer,
     }),
     viewType: 'activity',
-    groupable: false,
+    searchMenuTypes: ['filter', 'favorite'],
+
+    /**
+     * @override
+     */
+    init: function () {
+        this._super.apply(this, arguments);
+
+        this.loadParams.type = 'list';
+        // limit makes no sense in this view as we display all records having activities
+        this.loadParams.limit = false;
+
+        this.rendererParams.templates = _.findWhere(this.arch.children, { 'tag': 'templates' });
+    },
 });
 
 view_registry.add('activity', ActivityView);
