@@ -16,6 +16,29 @@ QUnit.module('Basic', {
 
         // define channel to link to chat window
         this.data = {
+            'mail.channel': {
+                fields: {
+                    name: {
+                        string: "Name",
+                        type: "char",
+                        required: true,
+                    },
+                    channel_type: {
+                        string: "Channel Type",
+                        type: "selection",
+                    },
+                    channel_message_ids: {
+                        string: "Messages",
+                        type: "many2many",
+                        relation: 'mail.message'
+                    },
+                    message_unread_counter: {
+                        string: "Amount of Unread Messages",
+                        type: "integer"
+                    },
+                },
+                records: [],
+            },
             'mail.message': {
                 fields: {},
                 records: [],
@@ -45,7 +68,7 @@ QUnit.module('Basic', {
                 self.services.mail_service.prototype.THREAD_WINDOW_APPENDTO = '#qunit-fixture';
             }
 
-            testUtils.addMockEnvironment(widget, params);
+            testUtils.mock.addMockEnvironment(widget, params);
             return widget;
         };
     },
@@ -55,7 +78,7 @@ QUnit.module('Basic', {
     },
 });
 
-QUnit.test('basic rendering thread window', function (assert) {
+QUnit.test('basic rendering thread window', async function (assert) {
     assert.expect(10);
 
     var parent = this.createParent({
@@ -63,9 +86,13 @@ QUnit.test('basic rendering thread window', function (assert) {
         services: this.services,
     });
 
+    await testUtils.nextTick();
     // detach channel 1, so that it opens corresponding thread window.
-    parent.call('mail_service', 'getChannel', 1).detach();
+    var result = parent.call('mail_service', 'getChannel', 1);
+    await testUtils.nextTick();
+    result.detach();
 
+    await testUtils.nextTick();
     var $threadWindow = $('.o_thread_window');
     assert.strictEqual($threadWindow.length, 1,
         "thread window should be open");
@@ -98,7 +125,7 @@ QUnit.test('basic rendering thread window', function (assert) {
     parent.destroy();
 });
 
-QUnit.test('close thread window using ESCAPE key', function (assert) {
+QUnit.test('close thread window using ESCAPE key', async function (assert) {
     assert.expect(5);
 
     var parent = this.createParent({
@@ -107,28 +134,31 @@ QUnit.test('close thread window using ESCAPE key', function (assert) {
         mockRPC: function (route, args) {
             if (args.method === 'channel_fold') {
                 assert.ok(true, "should call channel_fold");
-                return $.when();
+                return Promise.resolve();
             }
             return this._super.apply(this, arguments);
         },
     });
+    await testUtils.nextTick();
 
     // get channel instance to link to thread window
     var channel = parent.call('mail_service', 'getChannel', 1);
+    await testUtils.nextTick();
     assert.ok(channel, "there should exist a channel locally with ID 1");
 
     channel.detach();
+    await testUtils.nextTick();
     assert.strictEqual($('.o_thread_window').length, 1,
         "there should be a thread window that is opened");
 
     // focus on the thread window and press ESCAPE
-    $('.o_thread_window .o_composer_text_field').click();
+    await testUtils.dom.click($('.o_thread_window .o_composer_text_field'));
     assert.strictEqual(document.activeElement,
         $('.o_thread_window .o_composer_text_field')[0],
         "thread window's input should now be focused");
 
     var upKeyEvent = $.Event( "keyup", { which: 27 });
-    $('.o_thread_window .o_composer_text_field').trigger(upKeyEvent);
+    await testUtils.dom.triggerEvents($('.o_thread_window .o_composer_text_field'), [upKeyEvent]);
 
     assert.strictEqual($('.o_thread_window').length, 0,
         "the thread window should be closed");
@@ -136,7 +166,7 @@ QUnit.test('close thread window using ESCAPE key', function (assert) {
     parent.destroy();
 });
 
-QUnit.test('thread window\'s input can still be focused when the UI is blocked', function (assert) {
+QUnit.test('thread window\'s input can still be focused when the UI is blocked', async function (assert) {
     assert.expect(2);
 
     var parent = this.createParent({
@@ -147,11 +177,13 @@ QUnit.test('thread window\'s input can still be focused when the UI is blocked',
     var $dom = $('#qunit-fixture');
 
     // get channel instance to link to thread window
+    await testUtils.nextTick();
     var channel = parent.call('mail_service', 'getChannel', 1);
+    await testUtils.nextTick();
     channel.detach();
-
+    await testUtils.nextTick();
     var $input = $('<input/>', {type: 'text'}).appendTo($dom);
-    $input.focus().click();
+    await testUtils.dom.click($input.focus());
     assert.strictEqual(document.activeElement, $input[0],
         "fake input should be focused");
 
@@ -160,16 +192,17 @@ QUnit.test('thread window\'s input can still be focused when the UI is blocked',
     // makes no sense, this test is just about
     // making sure that the code which forces the
     // focus on click is not removed
-    $('.o_thread_window .o_composer_text_field').click();
+    await testUtils.dom.click($('.o_thread_window .o_composer_text_field'));
     assert.strictEqual(document.activeElement,
         $('.o_thread_window .o_composer_text_field')[0],
         "thread window's input should now be focused");
 
     framework.unblockUI();
     parent.destroy();
+    $input.remove();
 });
 
-QUnit.test('emoji popover should open correctly in thread windows', function (assert) {
+QUnit.test('emoji popover should open correctly in thread windows', async function (assert) {
     assert.expect(1);
 
     var parent = this.createParent({
@@ -178,23 +211,20 @@ QUnit.test('emoji popover should open correctly in thread windows', function (as
     });
 
     // get channel instance to link to thread window
+    await testUtils.nextTick();
     var channel = parent.call('mail_service', 'getChannel', 1);
+    await testUtils.nextTick();
     channel.detach();
-
+    await testUtils.nextTick();
     var $emojiButton = $('.o_composer_button_emoji');
-    $emojiButton.trigger('focusin').focus().click();
+    await testUtils.dom.click($emojiButton.trigger('focusin').focus());
     var $popover = $('.o_mail_emoji_container');
 
-    var done = assert.async();
-    // Async is needed as the popover focusout hiding is deferred
-    setTimeout(function () {
-        assert.ok($popover.is(':visible'), "emoji popover should have stayed opened");
-        parent.destroy();
-        done();
-    }, 0);
+    assert.ok($popover.is(':visible'), "emoji popover should have stayed opened");
+    parent.destroy();
 });
 
-QUnit.test('do not increase unread counter when receiving message with myself as author', function (assert) {
+QUnit.test('do not increase unread counter when receiving message with myself as author', async function (assert) {
     assert.expect(4);
 
     var parent = this.createParent({
@@ -204,9 +234,11 @@ QUnit.test('do not increase unread counter when receiving message with myself as
     });
 
     // get channel instance to link to thread window
+    await testUtils.nextTick();
     var channel = parent.call('mail_service', 'getChannel', 1);
+    await testUtils.nextTick();
     channel.detach();
-
+    await testUtils.nextTick();
     var threadWindowHeaderText = $('.o_thread_window_header').text().replace(/\s/g, "");
 
     assert.strictEqual(threadWindowHeaderText, "#general",
@@ -226,6 +258,7 @@ QUnit.test('do not increase unread counter when receiving message with myself as
     var notification = [[false, 'mail.channel', 1], messageData];
     parent.call('bus_service', 'trigger', 'notification', [notification]);
 
+    await testUtils.nextTick();
     threadWindowHeaderText = $('.o_thread_window_header').text().replace(/\s/g, "");
 
     assert.strictEqual(threadWindowHeaderText, "#general",
@@ -236,7 +269,7 @@ QUnit.test('do not increase unread counter when receiving message with myself as
     parent.destroy();
 });
 
-QUnit.test('do not increment unread counter with focus on thread window', function (assert) {
+QUnit.test('do not increment unread counter with focus on thread window', async function (assert) {
     // 'hard' focus means that the user has clicked on the thread window in
     // order to set the focus on it.
     assert.expect(2);
@@ -248,14 +281,17 @@ QUnit.test('do not increment unread counter with focus on thread window', functi
     });
 
     // get channel instance to link to thread window
+    await testUtils.nextTick();
     var channel = parent.call('mail_service', 'getChannel', 1);
+    await testUtils.nextTick();
     channel.detach();
+    await testUtils.nextTick();
 
     assert.strictEqual(channel.getUnreadCounter(), 0,
         "thread should have unread counter to 0 initially");
 
     // hard focus on thread window composer
-    $('.o_composer_text_field').click();
+    await testUtils.dom.click($('.o_composer_text_field'));
 
     // simulate receiving message from someone else
     var messageData = {
@@ -268,6 +304,7 @@ QUnit.test('do not increment unread counter with focus on thread window', functi
     };
     var notification = [[false, 'mail.channel', 1], messageData];
     parent.call('bus_service', 'trigger', 'notification', [notification]);
+    await testUtils.nextTick();
 
     assert.strictEqual(channel.getUnreadCounter(), 0,
         "thread should not have incremented its unread counter after receiving the message");
@@ -275,37 +312,15 @@ QUnit.test('do not increment unread counter with focus on thread window', functi
     parent.destroy();
 });
 
-QUnit.test('do not mark as read the newly open thread window from received message', function (assert) {
+QUnit.test('do not mark as read the newly open thread window from received message', async function (assert) {
     assert.expect(5);
 
-    this.data['mail.channel'] = {
-        fields: {
-            name: {
-                string: "Name",
-                type: "char",
-                required: true,
-            },
-            channel_type: {
-                string: "Channel Type",
-                type: "selection",
-            },
-            channel_message_ids: {
-                string: "Messages",
-                type: "many2many",
-                relation: 'mail.message'
-            },
-            message_unread_counter: {
-                string: "Amount of Unread Messages",
-                type: "integer"
-            },
-        },
-        records: [{
-            id: 2,
-            name: "DM",
-            channel_type: "chat",
-            message_unread_counter: 0,
-        }],
-    };
+    this.data['mail.channel'].records = [{
+        id: 2,
+        name: "DM",
+        channel_type: "chat",
+        message_unread_counter: 0,
+    }];
 
     var parent = this.createParent({
         data: this.data,
@@ -315,7 +330,7 @@ QUnit.test('do not mark as read the newly open thread window from received messa
             if (args.method === 'channel_join_and_get_info') {
                 this.data['mail.channel'].records[0].state = 'open';
                 this.data['mail.channel'].records[0].is_minimized = true;
-                return $.when(this.data['mail.channel'].records[0]);
+                return Promise.resolve(this.data['mail.channel'].records[0]);
             }
             return this._super.apply(this, arguments);
         },
@@ -338,7 +353,7 @@ QUnit.test('do not mark as read the newly open thread window from received messa
     // a new message from a new channel.
     this.data['mail.channel'].records[0].message_unread_counter++;
     parent.call('bus_service', 'trigger', 'notification', [notification]);
-
+    await testUtils.nextTick();
     var $threadWindow = $('.o_thread_window');
     assert.strictEqual($threadWindow.length, 1,
         "a thread window should be open after receiving a new message on a new DM chat");
@@ -347,43 +362,21 @@ QUnit.test('do not mark as read the newly open thread window from received messa
     assert.strictEqual($threadWindow.find('.o_thread_window_title').text().replace(/\s/g, ''), '#DM(1)',
         "open DM chat window should have one unread message");
 
-    $threadWindow.find('.o_thread_composer').click();
+    await testUtils.dom.click($threadWindow.find('.o_thread_composer'));
     assert.strictEqual($threadWindow.find('.o_thread_window_title').text().replace(/\s/g, ''), '#DM',
         "open DM chat window should have message marked as read on composer focus");
 
     parent.destroy();
 });
 
-QUnit.test('show document link of message linked to a document', function (assert) {
+QUnit.test('show document link of message linked to a document', async function (assert) {
     assert.expect(6);
 
-    this.data['mail.channel'] = {
-        fields: {
-            name: {
-                string: "Name",
-                type: "char",
-                required: true,
-            },
-            channel_type: {
-                string: "Channel Type",
-                type: "selection",
-            },
-            channel_message_ids: {
-                string: "Messages",
-                type: "many2many",
-                relation: 'mail.message'
-            },
-            message_unread_counter: {
-                string: "Amount of Unread Messages",
-                type: "integer"
-            },
-        },
-        records: [{
-            id: 2,
-            name: "R&D Tasks",
-            channel_type: "channel",
-        }],
-    };
+    this.data['mail.channel'].records = [{
+        id: 2,
+        name: "R&D Tasks",
+        channel_type: "channel",
+    }];
     this.data['mail.message'].records.push({
         author_id: [5, "Someone else"],
         body: "<p>Test message</p>",
@@ -410,8 +403,11 @@ QUnit.test('show document link of message linked to a document', function (asser
         "no thread window should be open initially");
 
     // get channel instance to link to thread window
+    await testUtils.nextTick();
     var channel = parent.call('mail_service', 'getChannel', 2);
+    await testUtils.nextTick();
     channel.detach();
+    await testUtils.nextTick();
 
     var $threadWindow = $('.o_thread_window');
     assert.strictEqual($threadWindow.length, 1,
@@ -429,7 +425,7 @@ QUnit.test('show document link of message linked to a document', function (asser
     parent.destroy();
 });
 
-QUnit.test('do not autofocus chat window on receiving new direct message', function (assert) {
+QUnit.test('do not autofocus chat window on receiving new direct message', async function (assert) {
     // Receiving a message doesn't make other input loose focus
     assert.expect(3);
 
@@ -443,37 +439,15 @@ QUnit.test('do not autofocus chat window on receiving new direct message', funct
         }],
     };
 
-    this.data['mail.channel'] = {
-        fields: {
-            name: {
-                string: "Name",
-                type: "char",
-                required: true,
-            },
-            channel_type: {
-                string: "Channel Type",
-                type: "selection",
-            },
-            channel_message_ids: {
-                string: "Messages",
-                type: "many2many",
-                relation: 'mail.message'
-            },
-            message_unread_counter: {
-                string: "Amount of Unread Messages",
-                type: "integer"
-            },
-        },
-        records: [{
-            id: 2,
-            name: "DM",
-            channel_type: "chat",
-            message_unread_counter: 0,
-            direct_partner: [{ id: 666, name: 'DemoUser1', im_status: '' }],
-        }],
-    };
+    this.data['mail.channel'].records = [{
+        id: 2,
+        name: "DM",
+        channel_type: "chat",
+        message_unread_counter: 0,
+        direct_partner: [{ id: 666, name: 'DemoUser1', im_status: '' }],
+    }];
 
-    var form = testUtils.createView({
+    var form = await testUtils.createView({
         View: FormView,
         model: 'partner',
         data: this.data,
@@ -487,13 +461,13 @@ QUnit.test('do not autofocus chat window on receiving new direct message', funct
         },
         mockRPC: function (route, args) {
             if (args.method === 'channel_join_and_get_info') {
-                return $.when(this.data['mail.channel'].records[0]);
+                return Promise.resolve(this.data['mail.channel'].records[0]);
             }
             return this._super.apply(this, arguments);
         }
     });
 
-    var $formInput = form.$('input');
+    var $formInput = form.$('input[name=display_name]');
     $formInput.focus();
 
     assert.equal(document.activeElement, $formInput[0],
@@ -512,7 +486,7 @@ QUnit.test('do not autofocus chat window on receiving new direct message', funct
     this.data['mail.message'].records.push(messageData);
     var notification = [[false, 'mail.channel', 2], messageData];
     form.call('bus_service', 'trigger', 'notification', [notification]);
-
+    await testUtils.nextTick();
     assert.ok($('.o_thread_window.o_in_home_menu').length,
         'Chat window is opened');
 
@@ -522,41 +496,19 @@ QUnit.test('do not autofocus chat window on receiving new direct message', funct
     form.destroy();
 });
 
-QUnit.test('do not auto-focus chat window on receiving new message from new DM', function (assert) {
+QUnit.test('do not auto-focus chat window on receiving new message from new DM', async function (assert) {
     assert.expect(10);
 
     var self = this;
-    this.data['mail.channel'] = {
-        fields: {
-            name: {
-                string: "Name",
-                type: "char",
-                required: true,
-            },
-            channel_type: {
-                string: "Channel Type",
-                type: "selection",
-            },
-            channel_message_ids: {
-                string: "Messages",
-                type: "many2many",
-                relation: 'mail.message'
-            },
-            message_unread_counter: {
-                string: "Amount of Unread Messages",
-                type: "integer"
-            },
-        },
-        records: [{
-            id: 2,
-            name: "DM",
-            channel_type: "chat",
-            message_unread_counter: 1,
-            direct_partner: [{ id: 666, name: 'DemoUser1', im_status: '' }],
-            is_minimized: false,
-            state: 'open',
-        }],
-    };
+    this.data['mail.channel'].records = [{
+        id: 2,
+        name: "DM",
+        channel_type: "chat",
+        message_unread_counter: 1,
+        direct_partner: [{ id: 666, name: 'DemoUser1', im_status: '' }],
+        is_minimized: false,
+        state: 'open',
+    }];
 
     var parent = this.createParent({
         data: this.data,
@@ -564,7 +516,7 @@ QUnit.test('do not auto-focus chat window on receiving new message from new DM',
         session: { partner_id: 3 },
         mockRPC: function (route, args) {
             if (args.method === 'channel_join_and_get_info') {
-                return $.when(_.extend({}, self.data['mail.channel'].records[0], { info: 'join' }));
+                return Promise.resolve(_.extend({}, self.data['mail.channel'].records[0], { info: 'join' }));
             }
             if (args.method === 'channel_minimize') {
                 _.extend(self.data['mail.channel'].records[0], {
@@ -594,6 +546,7 @@ QUnit.test('do not auto-focus chat window on receiving new message from new DM',
     this.data['mail.message'].records.push(messageData);
     var notification = [[false, 'mail.channel', 2], messageData];
     parent.call('bus_service', 'trigger', 'notification', [notification]);
+    await testUtils.nextTick();
 
     assert.strictEqual($('.o_thread_window').length, 1,
         "should have DM window open");
@@ -610,6 +563,7 @@ QUnit.test('do not auto-focus chat window on receiving new message from new DM',
     });
     notification = [[false, 'res.partner', 3], dmInfo];
     parent.call('bus_service', 'trigger', 'notification', [notification]);
+    await testUtils.nextTick();
 
     assert.strictEqual($('.o_thread_window').length, 1,
         "should still have DM window open after receiving DM info from polling");
@@ -621,6 +575,7 @@ QUnit.test('do not auto-focus chat window on receiving new message from new DM',
     // simulate receiving detached DM notification (cross-tab synchronization)
     notification = [[false, 'res.partner', 3], self.data['mail.channel'].records[0]];
     parent.call('bus_service', 'trigger', 'notification', [notification]);
+    await testUtils.nextTick();
 
     assert.strictEqual($('.o_thread_window').length, 1,
         "should still have DM open after receiving detached info from polling");
@@ -632,6 +587,408 @@ QUnit.test('do not auto-focus chat window on receiving new message from new DM',
     parent.destroy();
 });
 
+QUnit.test('out-of-office status in thread window', async function (assert) {
+    assert.expect(1);
+    this.data.initMessaging = {
+        channel_slots: {
+            channel_channel: [{
+                id: 1,
+                name: "DM",
+                channel_type: "chat",
+                message_unread_counter: 0,
+                direct_partner: [{ id: 666, name: 'DemoUser1', im_status: 'online', out_of_office_message: 'Please don\'t disturb'}],
+            }],
+        },
+    };
+    var parent = this.createParent({
+        data: this.data,
+        services: this.services,
+    });
+    await testUtils.nextTick();
+
+    // detach channel 1, so that it opens corresponding thread window.
+    var channel = parent.call('mail_service', 'getChannel', 1);
+    await testUtils.nextTick();
+    channel.detach();
+    await testUtils.nextTick();
+
+    var $threadWindow = $('.o_thread_window');
+    assert.containsOnce($threadWindow, '.o_out_of_office_text');
+
+    parent.destroy();
+});
+
+QUnit.test('no out-of-office status in thread window', async function (assert) {
+    assert.expect(1);
+    this.data.initMessaging = {
+        channel_slots: {
+            channel_channel: [{
+                id: 1,
+                name: "DM",
+                channel_type: "chat",
+                message_unread_counter: 0,
+                direct_partner: [{ id: 666, name: 'DemoUser1', im_status: 'online'}],
+            }],
+        },
+    };
+    var parent = this.createParent({
+        data: this.data,
+        services: this.services,
+    });
+    await testUtils.nextTick();
+
+    // detach channel 1, so that it opens corresponding thread window.
+    var channel = parent.call('mail_service', 'getChannel', 1);
+    await testUtils.nextTick();
+    channel.detach();
+    await testUtils.nextTick();
+
+    var $threadWindow = $('.o_thread_window');
+    assert.containsNone($threadWindow, '.o_out_of_office_text');
+
+    parent.destroy();
+});
+
+QUnit.test('auto-update out-of-office info on im_status change', async function (assert) {
+    assert.expect(5);
+
+    var imStatusDefs = [testUtils.makeTestPromise(), testUtils.makeTestPromise()];
+    var channelInfoDefs = [testUtils.makeTestPromise(), testUtils.makeTestPromise()];
+    var timeoutMock = mailTestUtils.patchMailTimeouts();
+    var step = 0;
+
+    this.data.initMessaging = {
+        channel_slots: {
+            channel_channel: [{
+                id: 1,
+                name: "DM",
+                channel_type: "chat",
+                message_unread_counter: 0,
+                direct_partner: [{
+                    id: 666,
+                    name: 'DemoUser1',
+                    im_status: 'online',
+                }],
+            }],
+        },
+    };
+
+    var parent = this.createParent({
+        data: this.data,
+        services: this.services,
+        mockRPC: function (route, args) {
+            if (route === '/longpolling/im_status') {
+                step++;
+                if (step === 1) {
+                    imStatusDefs[0].resolve();
+                    return Promise.resolve([
+                        {
+                            id: 666,
+                            im_status: 'leave',
+                        },
+                    ]);
+                } else if (step === 2) {
+                    imStatusDefs[1].resolve();
+                    return Promise.resolve([
+                        {
+                            id: 666,
+                            im_status: 'online',
+                        },
+                    ]);
+                }
+            }
+            if (args.method === 'channel_info') {
+                if (step === 1) {
+                    channelInfoDefs[0].resolve();
+                    return Promise.resolve([{
+                        id: 1,
+                        direct_partner: [{
+                            out_of_office_message: "Leave me alone",
+                            out_of_office_date_end: false,
+                        }],
+                    }]);
+                }
+                else if (step === 2) {
+                    channelInfoDefs[1].resolve();
+                    return Promise.resolve([{
+                        id: 1,
+                        direct_partner: [{
+                            out_of_office_message: false,
+                            out_of_office_date_end: false,
+                        }],
+                    }]);
+                }
+            }
+            return this._super.apply(this, arguments);
+        },
+    });
+    await testUtils.nextTick();
+
+    // detach channel 1, so that it opens corresponding thread window.
+    parent.call('mail_service', 'getChannel', 1).detach();
+    await testUtils.nextTick();
+
+    assert.containsNone($, '.o_out_of_office',
+        "should contain no out of office section on chat window initially");
+
+    timeoutMock.addTime(51*1000); // wait for next im status fetch
+
+    await Promise.all([imStatusDefs[0], channelInfoDefs[0]]);
+    await testUtils.nextTick();
+    assert.containsOnce($, '.o_out_of_office',
+        "should contain out of office section on chat window");
+    assert.containsOnce($, '.o_out_of_office > .o_out_of_office_text',
+        "should contain out of office text on chat window");
+    assert.ok(
+        $('.o_out_of_office > .o_out_of_office_text')
+            .text()
+            .replace(/\s/g, "")
+            .indexOf("Leavemealone") !== -1,
+        "should contain out of office text on chat window");
+    timeoutMock.addTime(51*1000); // wait for next im status fetch
+
+    await Promise.all([imStatusDefs[1], channelInfoDefs[1]]);
+    await testUtils.nextTick();
+    assert.containsNone($, '.o_out_of_office',
+        "should no longer contain out of office section on chat window");
+
+    timeoutMock.runPendingTimeouts();
+    parent.destroy();
+});
+
+QUnit.test('receive 2 new DM messages in quick succession (no chat window initially)', async function (assert) {
+    assert.expect(3);
+
+    const self = this;
+    this.data['mail.channel'].records = [{
+        channel_type: "chat",
+        direct_partner: [{
+            id: 5,
+            name: 'Someone else',
+            im_status: 'online',
+        }],
+        id: 10,
+        is_minimized: false,
+        message_unread_counter: 1,
+        name: "DM",
+        state: 'open',
+    }];
+
+    this.data.initMessaging.channel_slots = {
+        channel_direct_message: [{
+            channel_type: 'chat',
+            direct_partner: [{
+                id: 5,
+                name: 'Someone else',
+                im_status: 'online',
+            }],
+            id: 10,
+            message_unread_counter: 0,
+            name: "DM",
+        }],
+    };
+
+    const parent = this.createParent({
+        data: this.data,
+        mockRPC(route, args) {
+            if (args.method === 'channel_minimize') {
+                Object.assign(self.data['mail.channel'].records[0], {
+                    is_minimized: true,
+                    state: 'open',
+                });
+            }
+            return this._super(...arguments);
+        },
+        services: this.services,
+        session: {
+            partner_id: 3,
+        },
+    });
+
+    await testUtils.nextTick();
+    assert.containsNone(
+        $,
+        '.o_thread_window',
+        "should not have any DM window open");
+
+    // simulate receiving 2 new messages from someone else in quick succession
+    const messageData1 = {
+        author_id: [5, "Someone else"],
+        body: "<p>Test message1</p>",
+        channel_ids: [10],
+        id: 2,
+        model: 'mail.channel',
+        res_id: 10,
+    };
+    this.data['mail.message'].records.push(messageData1);
+    const notification1 = [[false, 'mail.channel', 2], messageData1];
+    parent.call('bus_service', 'trigger', 'notification', [notification1]);
+    // simulate short delay for receiving new message
+    await testUtils.nextMicrotaskTick();
+    const messageData2 = {
+        author_id: [5, "Someone else"],
+        body: "<p>Test message2</p>",
+        channel_ids: [10],
+        id: 3,
+        model: 'mail.channel',
+        res_id: 10,
+    };
+    this.data['mail.message'].records.push(messageData2);
+    const notification2 = [[false, 'mail.channel', 2], messageData2];
+    parent.call('bus_service', 'trigger', 'notification', [notification2]);
+    await testUtils.nextTick();
+    assert.containsOnce(
+        $,
+        '.o_thread_window',
+        "should have DM window open");
+    assert.containsN(
+        $('.o_thread_window'),
+        '.o_thread_message',
+        2,
+        "should have 2 messages in chat window");
+
+    parent.destroy();
+});
+
+QUnit.test('non-deletable message attachments', async function (assert) {
+    assert.expect(3);
+
+    this.data['mail.channel'].records = [{
+        id: 1,
+        name: "General",
+        channel_type: 'channel',
+    }];
+    this.data['mail.message'].records = [{
+        attachment_ids: [{
+            filename: "text.txt",
+            id: 250,
+            mimetype: 'text/plain',
+            name: "text.txt",
+        }, {
+            filename: "image.png",
+            id: 251,
+            mimetype: 'image/png',
+            name: "image.png",
+        }],
+        author_id: [5, "Demo User"],
+        body: "<p>test</p>",
+        channel_ids: [1],
+        id: 100,
+        model: 'mail.channel',
+        record_name: "general",
+        res_id: 1,
+    }];
+    const parent = this.createParent({
+        data: this.data,
+        services: this.services,
+    });
+    await testUtils.nextTick();
+    const channel = parent.call('mail_service', 'getChannel', 1);
+    channel.detach();
+    await testUtils.nextTick();
+    assert.containsOnce(
+        $,
+        '.o_thread_window',
+        "a thread window should be open");
+    assert.containsN(
+        $('.o_thread_window'),
+        '.o_attachment',
+        2,
+        "thread window should have 2 attachments");
+    assert.containsNone(
+        $('.o_thread_window .o_attachment'),
+        'o_attachment_delete_cross',
+        "attachments should not be deletable");
+
+    parent.destroy();
+});
+
+QUnit.test('join channel from click channel mention', async function (assert) {
+    assert.expect(10);
+
+    this.data['mail.channel'].records = [{
+        id: 2,
+        name: "R&D Tasks",
+        channel_type: "channel",
+    }];
+    this.data['mail.message'].records.push({
+        author_id: [5, "Someone else"],
+        body: `<p><a href="#" class="o_mail_redirect" data-oe-id="2" data-oe-model="mail.channel">#R&D Tasks</a></p>`,
+        id: 10,
+        res_id: 1,
+        channel_ids: [1],
+    });
+
+    const parent = this.createParent({
+        data: this.data,
+        services: this.services,
+        session: { partner_id: 3 },
+        async mockRPC(route, args) {
+            if (args.method === 'channel_join_and_get_info') {
+                const channelID = args.args[0][0];
+                assert.step(`channel_join_and_get_info:${channelID}`);
+                return this.data['mail.channel'].records.find(channel => channel.id === channelID);
+            }
+            return this._super(...arguments);
+        },
+    });
+    await testUtils.nextTick();
+    assert.containsNone(
+        $,
+        '.o_thread_window',
+        "no thread window should be open initially"
+    );
+
+    // get channel instance to link to thread window
+    const channel = parent.call('mail_service', 'getChannel', 1);
+    channel.detach();
+    await testUtils.nextTick();
+    assert.containsOnce(
+        $,
+        '.o_thread_window',
+        "should have one thread open in chat window"
+    );
+    assert.strictEqual(
+        $('.o_thread_window .o_thread_window_title').text().trim(),
+        "#general",
+        "should have thread '#general' as open in chat window"
+    );
+    assert.containsOnce(
+        $,
+        '.o_thread_window .o_thread_message',
+        "should contain a single message in chat window"
+    );
+    assert.containsOnce(
+        $,
+        '.o_thread_window .o_thread_message a.o_mail_redirect',
+        "message should have a mention"
+    );
+    assert.containsOnce(
+        $,
+        '.o_thread_window .o_thread_message a.o_mail_redirect[data-oe-id="2"][data-oe-model="mail.channel"]',
+        "message should have a channel mention to R&D Task"
+    );
+
+    await testUtils.dom.click($('.o_thread_window .o_thread_message a.o_mail_redirect[data-oe-id="2"][data-oe-model="mail.channel"]'));
+    assert.verifySteps(
+        ['channel_join_and_get_info:2'],
+        "should have joined channel #R&D Task"
+    );
+    assert.containsN(
+        $,
+        '.o_thread_window',
+        2,
+        "should now have 2 threads open in chat windows after cliking on channel mention"
+    );
+    assert.strictEqual(
+        $('.o_thread_window .o_thread_window_title').text().replace(/\s/g, ""),
+        "#general#R&DTasks",
+        "should have threads '#general' and '#R&D Tasks' as open in chat windows"
+    );
+
+    parent.destroy();
+});
+
 });
 });
 });
