@@ -8,7 +8,7 @@
     'version': '2.0',
     'sequence': 110,
     'website': 'https://www.odoo.com/page/mailing',
-    'category': 'Marketing',
+    'category': 'Marketing/Email Marketing',
     'depends': [
         'contacts',
         'mail',
@@ -20,28 +20,32 @@
     ],
     'data': [
         'security/mass_mailing_security.xml',
+        'security/ir.model.access.csv',
+        'data/mail_data.xml',
         'data/mass_mailing_data.xml',
         'wizard/mail_compose_message_views.xml',
-        'wizard/mass_mailing_list_merge_views.xml',
-        'wizard/test_mailing_views.xml',
-        'wizard/mass_mailing_schedule_date_views.xml',
-        'views/mass_mailing_report_views.xml',
-        'views/mass_mailing_stats_views.xml',
+        'wizard/mailing_list_merge_views.xml',
+        'wizard/mailing_mailing_schedule_date_views.xml',
+        'wizard/mailing_mailing_test_views.xml',
+        'views/mailing_mailing_views_menus.xml',
+        'views/mailing_trace_views.xml',
         'views/link_tracker_views.xml',
-        'views/mass_mailing_template.xml',
-        'views/mass_mailing_views.xml',
+        'views/mailing_contact_views.xml',
+        'views/mailing_list_views.xml',
+        'views/mailing_mailing_views.xml',
         'views/res_config_settings_views.xml',
-        'security/ir.model.access.csv',
-        'views/mass_mailing_layout.xml',
-        'views/editor_field_html.xml',
+        'views/utm_campaign_views.xml',
+        'report/mailing_trace_report_views.xml',
+        'views/assets.xml',
+        'views/mass_mailing_templates_portal.xml',
         'views/themes_templates.xml',
         'views/snippets_themes.xml',
-        'views/snippets_themes_options.xml',
-        'views/mass_mailing_blacklist_views.xml',
-        'views/unsubscribe_templates.xml',
     ],
     'demo': [
         'data/mass_mailing_demo.xml',
     ],
+    'qweb': [
+        'static/src/xml/*.xml',
+    ],
     'application': True,
 }
