@@ -1,11 +1,13 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import mass_mailing
-from . import mass_mailing_stats
+from . import link_tracker
+from . import mailing_contact
+from . import mailing_list
+from . import mailing_trace
+from . import mailing
 from . import mail_mail
 from . import mail_template
 from . import mail_thread
 from . import res_config_settings
-from . import mass_mailing_report
-from . import link_tracker
+from . import utm
