@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import test_mailing
 from . import mail_compose_message
-from . import mass_mailing_list_merge
-from . import mass_mailing_schedule_date
+from . import mailing_list_merge
+from . import mailing_mailing_schedule_date
+from . import mailing_mailing_test
