@@ -2,4 +2,4 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import event
-from . import mass_mailing
+from . import mailing_mailing
