@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 
 from . import test_bom
+from . import test_byproduct
+from . import test_cancel_mo
 from . import test_order
 from . import test_stock
 from . import test_warehouse_multistep_manufacturing
@@ -9,3 +11,4 @@ from . import test_procurement
 from . import test_unbuild
 from . import test_oee
 from . import test_traceability
+from . import test_multicompany
