@@ -4,7 +4,7 @@
 {
     'name': 'Collaborative Pads',
     'version': '2.0',
-    'category': 'Extra Tools',
+    'category': 'Tools',
     'description': """
 Adds enhanced support for (Ether)Pad attachments in the web client.
 ===================================================================
