@@ -20,7 +20,7 @@ class ResCompany(models.Model):
     def action_open_payment_onboarding_payment_acquirer(self):
         """ Called by onboarding panel above the customer invoice list."""
         # Fail if there are no existing accounts
-        self.env.user.company_id.get_chart_of_accounts_or_fail()
+        self.env.company.get_chart_of_accounts_or_fail()
 
         action = self.env.ref('payment.action_open_payment_onboarding_payment_acquirer_wizard').read()[0]
         return action
