@@ -4,6 +4,7 @@
 
 import json
 import logging
+import pprint
 import werkzeug
 
 from odoo import http
@@ -27,17 +28,25 @@ class SipsController(http.Controller):
 
     @http.route([
         '/payment/sips/ipn/'],
-        type='http', auth='none', methods=['POST'], csrf=False)
+        type='http', auth='public', methods=['POST'], csrf=False)
     def sips_ipn(self, **post):
         """ Sips IPN. """
-        self.sips_validate_data(**post)
+        _logger.info('Beginning Sips IPN form_feedback with post data %s', pprint.pformat(post))  # debug
+        if not post:
+            # SIPS sometimes send empty notification, the reason why is
+            # unclear but they tend to pollute logs and do not provide any
+            # meaningful information; log as a warning instead of a traceback
+            _logger.warning('Sips: received empty notification; skip.')
+        else:
+            self.sips_validate_data(**post)
         return ''
 
     @http.route([
-        '/payment/sips/dpn'], type='http', auth="none", methods=['POST'], csrf=False)
+        '/payment/sips/dpn'], type='http', auth="public", methods=['POST'], csrf=False)
     def sips_dpn(self, **post):
         """ Sips DPN """
         try:
+            _logger.info('Beginning Sips DPN form_feedback with post data %s', pprint.pformat(post))  # debug
             self.sips_validate_data(**post)
         except:
             pass
