@@ -7,7 +7,6 @@ from odoo.exceptions import UserError
 class AccountTax(models.Model):
     _inherit = 'account.tax'
 
-    @api.multi
     def write(self, vals):
         forbidden_fields = set([
             'amount_type', 'amount', 'type_tax_use', 'tax_group_id', 'price_include',
@@ -25,3 +24,12 @@ class AccountTax(models.Model):
                     'You must close the POS sessions before modifying the tax.'
                 ))
         return super(AccountTax, self).write(vals)
+
+    def get_real_tax_amount(self):
+        tax_list = []
+        for tax in self:
+            tax_repartition_lines = tax.invoice_repartition_line_ids.filtered(lambda x: x.repartition_type == 'tax')
+            total_factor = sum(tax_repartition_lines.mapped('factor'))
+            real_amount = tax.amount * total_factor
+            tax_list.append({'id': tax.id, 'amount': real_amount})
+        return tax_list
