@@ -2,6 +2,7 @@ odoo.define('point_of_sale.DB', function (require) {
 "use strict";
 
 var core = require('web.core');
+var utils = require('web.utils');
 /* The PosDB holds reference to data that is either
  * - static: does not change between pos reloads
  * - persistent : must stay between reloads ( orders )
@@ -41,9 +42,12 @@ var PosDB = core.Class.extend({
         this.category_search_string = {};
     },
 
-    /* 
-     * sets an uuid to prevent conflict in locally stored data between multiple databases running
-     * in the same browser at the same origin (Doing this is not advised !)
+    /** 
+     * sets an uuid to prevent conflict in locally stored data between multiple PoS Configs. By
+     * using the uuid of the config the local storage from other configs will not get effected nor
+     * loaded in sessions that don't belong to them.
+     *
+     * @param {string} uuid Unique identifier of the PoS Config linked to the current session.
      */
     set_uuid: function(uuid){
         this.name = this.name + '_' + uuid;
@@ -94,19 +98,20 @@ var PosDB = core.Class.extend({
                 name : 'Root',
             };
         }
-        for(var i=0, len = categories.length; i < len; i++){
-            this.category_by_id[categories[i].id] = categories[i];
-        }
-        len = categories.length;
-        for(i=0; i < len; i++){
-            var cat = categories[i];
-            var parent_id = cat.parent_id[0] || this.root_category_id;
-            this.category_parent[cat.id] = cat.parent_id[0];
-            if(!this.category_childs[parent_id]){
-                this.category_childs[parent_id] = [];
+        categories.forEach(function(cat){
+            self.category_by_id[cat.id] = cat;
+        });
+        categories.forEach(function(cat){
+            var parent_id = cat.parent_id[0];
+            if(!(parent_id && self.category_by_id[parent_id])){
+                parent_id = self.root_category_id;
             }
-            this.category_childs[parent_id].push(cat.id);
-        }
+            self.category_parent[cat.id] = parent_id;
+            if(!self.category_childs[parent_id]){
+                self.category_childs[parent_id] = [];
+            }
+            self.category_childs[parent_id].push(cat.id);
+        });
         function make_ancestors(cat_id, ancestors){
             self.category_ancestors[cat_id] = ancestors;
 
@@ -175,7 +180,7 @@ var PosDB = core.Class.extend({
         }
         for(var i = 0, len = products.length; i < len; i++){
             var product = products[i];
-            var search_string = this._product_search_string(product);
+            var search_string = utils.unaccent(this._product_search_string(product));
             var categ_id = product.pos_categ_id ? product.pos_categ_id[0] : this.root_category_id;
             product.product_tmpl_id = product.product_tmpl_id[0];
             if(!stored_categories[categ_id]){
@@ -209,7 +214,7 @@ var PosDB = core.Class.extend({
         }
     },
     _partner_search_string: function(partner){
-        var str =  partner.name;
+        var str =  partner.name || '';
         if(partner.barcode){
             str += '|' + partner.barcode;
         }
@@ -275,12 +280,15 @@ var PosDB = core.Class.extend({
                 if(partner.barcode){
                     this.partner_by_barcode[partner.barcode] = partner;
                 }
-                partner.address = (partner.street || '') +', '+ 
-                                  (partner.zip || '')    +' '+
-                                  (partner.city || '')   +', '+ 
-                                  (partner.country_id[1] || '');
+                partner.address = (partner.street ? partner.street + ', ': '') +
+                                  (partner.zip ? partner.zip + ', ': '') +
+                                  (partner.city ? partner.city + ', ': '') +
+                                  (partner.state_id ? partner.state_id[1] + ', ': '') +
+                                  (partner.country_id ? partner.country_id[1]: '');
                 this.partner_search_string += this._partner_search_string(partner);
             }
+
+            this.partner_search_string = utils.unaccent(this.partner_search_string);
         }
         return updated_count;
     },
@@ -305,7 +313,7 @@ var PosDB = core.Class.extend({
         try {
             query = query.replace(/[\[\]\(\)\+\*\?\.\-\!\&\^\$\|\~\_\{\}\:\,\\\/]/g,'.');
             query = query.replace(/ /g,'.+');
-            var re = RegExp("([0-9]+):.*?"+query,"gi");
+            var re = RegExp("([0-9]+):.*?"+utils.unaccent(query),"gi");
         }catch(e){
             return [];
         }
@@ -365,7 +373,7 @@ var PosDB = core.Class.extend({
         try {
             query = query.replace(/[\[\]\(\)\+\*\?\.\-\!\&\^\$\|\~\_\{\}\:\,\\\/]/g,'.');
             query = query.replace(/ /g,'.+');
-            var re = RegExp("([0-9]+):.*?"+query,"gi");
+            var re = RegExp("([0-9]+):.*?"+utils.unaccent(query),"gi");
         }catch(e){
             return [];
         }
@@ -482,6 +490,54 @@ var PosDB = core.Class.extend({
         }
         return orders;
     },
+    /**
+     * Return the orders with requested ids if they are unpaid.
+     * @param {array<number>} ids order_ids.
+     * @return {array<object>} list of orders.
+     */
+    get_unpaid_orders_to_sync: function(ids){
+        var saved = this.load('unpaid_orders',[]);
+        var orders = [];
+        saved.forEach(function(o) {
+            if (ids.includes(o.id) && (o.data.server_id || o.data.lines.length)){
+                orders.push(o);
+            }
+        });
+        return orders;
+    },
+    /**
+     * Add a given order to the orders to be removed from the server.
+     *
+     * If an order is removed from a table it also has to be removed from the server to prevent it from reapearing 
+     * after syncing. This function will add the server_id of the order to a list of orders still to be removed.
+     * @param {object} order object.
+     */
+    set_order_to_remove_from_server: function(order){
+        if (order.server_id !== undefined) {
+            var to_remove = this.load('unpaid_orders_to_remove',[]);
+            to_remove.push(order.server_id);
+            this.save('unpaid_orders_to_remove', to_remove);
+        }
+    },
+    /**
+     * Get a list of server_ids of orders to be removed.
+     * @return {array<number>} list of server_ids.
+     */
+    get_ids_to_remove_from_server: function(){
+        return this.load('unpaid_orders_to_remove',[]);
+    },
+    /**
+     * Remove server_ids from the list of orders to be removed.
+     * @param {array<number>} ids
+     */
+    set_ids_removed_from_server: function(ids){
+        var to_remove = this.load('unpaid_orders_to_remove',[]);
+        
+        to_remove = _.filter(to_remove, function(id){
+            return !ids.includes(id);
+        });
+        this.save('unpaid_orders_to_remove', to_remove);
+    },
     set_cashier: function(cashier) {
         // Always update if the user is the same as before
         this.save('cashier', cashier || null);
