@@ -3,24 +3,29 @@ import time
 
 import odoo
 from odoo import fields
-from odoo.tools import float_compare, mute_logger, test_reports, pycompat
+from odoo.tools import float_compare, mute_logger, test_reports
+from odoo.tests.common import Form
 from odoo.addons.point_of_sale.tests.common import TestPointOfSaleCommon
 
 
 @odoo.tests.tagged('post_install', '-at_install')
 class TestPointOfSaleFlow(TestPointOfSaleCommon):
 
-    def test_register_open(self):
-        """
-            In order to test the Point of Sale module, I will open all cash registers through the wizard
-            """
-        # open all statements/cash registers
-        self.env['pos.open.statement'].create({}).open_statement()
+    def compute_tax(self, product, price, qty=1, taxes=None):
+        if not taxes:
+            taxes = product.taxes_id.filtered(lambda t: t.company_id.id == self.env.user.id)
+        currency = self.pos_config.pricelist_id.currency_id
+        res = taxes.compute_all(price, currency, qty, product=product)
+        untax = res['total_excluded']
+        return untax, sum(tax.get('amount', 0.0) for tax in res['taxes'])
 
     def test_order_refund(self):
+        self.pos_config.open_session_cb()
+        current_session = self.pos_config.current_session_id
         # I create a new PoS order with 2 lines
         order = self.PosOrder.create({
             'company_id': self.company_id,
+            'session_id': current_session.id,
             'partner_id': self.partner1.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
             'lines': [(0, 0, {
@@ -44,10 +49,18 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             })],
             'amount_total': 1710.0,
             'amount_tax': 0.0,
-            'amount_paid': 1710.0,
+            'amount_paid': 0.0,
             'amount_return': 0.0,
         })
 
+        payment_context = {"active_ids": order.ids, "active_id": order.id}
+        order_payment = self.PosMakePayment.with_context(**payment_context).create({
+            'amount': order.amount_total,
+            'payment_method_id': self.cash_payment_method.id
+        })
+        order_payment.with_context(**payment_context).check()
+        self.assertAlmostEqual(order.amount_total, order.amount_paid, msg='Order should be fully paid.')
+
         # I create a refund
         refund_action = order.refund()
         refund = self.PosOrder.browse(refund_action['res_id'])
@@ -57,13 +70,128 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
 
         payment_context = {"active_ids": refund.ids, "active_id": refund.id}
         refund_payment = self.PosMakePayment.with_context(**payment_context).create({
-            'amount': refund.amount_total
+            'amount': refund.amount_total,
+            'payment_method_id': self.cash_payment_method.id,
         })
 
         # I click on the validate button to register the payment.
         refund_payment.with_context(**payment_context).check()
 
         self.assertEqual(refund.state, 'paid', "The refund is not marked as paid")
+        self.assertTrue(refund.payment_ids.payment_method_id.is_cash_count, msg='There should only be one payment and paid in cash.')
+
+        current_session.action_pos_session_closing_control()
+        self.assertEqual(current_session.state, 'closed', msg='State of current session should be closed.')
+
+    def test_order_refund_lots(self):
+        # open pos session
+        self.pos_config.open_session_cb()
+        current_session = self.pos_config.current_session_id
+
+        # set up product iwith SN tracing and create two lots (1001, 1002)
+        self.stock_location = self.env.ref('stock.stock_location_stock')
+        self.product2 = self.env['product.product'].create({
+            'name': 'Product A',
+            'type': 'product',
+            'tracking': 'serial',
+            'categ_id': self.env.ref('product.product_category_all').id,
+        })
+
+        inventory = self.env['stock.inventory'].create({
+            'name': 'add product2',
+            'location_ids': [(4, self.stock_location.id)],
+            'product_ids': [(4, self.product2.id)],
+        })
+        inventory.action_start()
+
+        lot1 = self.env['stock.production.lot'].create({
+            'name': '1001',
+            'product_id': self.product2.id,
+            'company_id': self.env.company.id,
+        })
+        lot2 = self.env['stock.production.lot'].create({
+            'name': '1002',
+            'product_id': self.product2.id,
+            'company_id': self.env.company.id,
+        })
+
+        self.env['stock.inventory.line'].create([
+            {
+            'inventory_id': inventory.id,
+            'location_id': self.stock_location.id,
+            'product_id': self.product2.id,
+            'prod_lot_id': lot1.id,
+            'product_qty': 1
+            },
+            {
+            'inventory_id': inventory.id,
+            'location_id': self.stock_location.id,
+            'product_id': self.product2.id,
+            'prod_lot_id': lot2.id,
+            'product_qty': 1
+            },
+        ])
+
+        inventory.action_validate()
+
+        # create pos order with the two SN created before
+
+        order = self.PosOrder.create({
+            'company_id': self.company_id,
+            'session_id': current_session.id,
+            'partner_id': self.partner1.id,
+            'lines': [(0, 0, {
+                'name': "OL/0001",
+                'id': 1,
+                'product_id': self.product2.id,
+                'price_unit': 6,
+                'discount': 0,
+                'qty': 2,
+                'tax_ids': [[6, False, []]],
+                'price_subtotal': 12,
+                'price_subtotal_incl': 12,
+                'pack_lot_ids': [
+                    [0, 0, {'lot_name': '1001'}],
+                    [0, 0, {'lot_name': '1002'}],
+                ]
+            })],
+            'pricelist_id': 1,
+            'amount_paid': 12.0,
+            'amount_total': 12.0,
+            'amount_tax': 0.0,
+            'amount_return': 0.0,
+            'to_invoice': False,
+            })
+
+        payment_context = {"active_ids": order.ids, "active_id": order.id}
+        order_payment = self.PosMakePayment.with_context(**payment_context).create({
+            'amount': order.amount_total,
+            'payment_method_id': self.cash_payment_method.id
+        })
+        order_payment.with_context(**payment_context).check()
+
+        # I create a refund
+        refund_action = order.refund()
+        refund = self.PosOrder.browse(refund_action['res_id'])
+
+        order_lot_id = [lot_id.lot_name for lot_id in order.lines.pack_lot_ids]
+        refund_lot_id = [lot_id.lot_name for lot_id in refund.lines.pack_lot_ids]
+        self.assertEqual(
+            order_lot_id,
+            refund_lot_id,
+            "In the refund we should find the same lot as in the original order")
+
+        payment_context = {"active_ids": refund.ids, "active_id": refund.id}
+        refund_payment = self.PosMakePayment.with_context(**payment_context).create({
+            'amount': refund.amount_total,
+            'payment_method_id': self.cash_payment_method.id,
+        })
+
+        # I click on the validate button to register the payment.
+        refund_payment.with_context(**payment_context).check()
+
+        self.assertEqual(refund.state, 'paid', "The refund is not marked as paid")
+        current_session.action_pos_session_closing_control()
 
     def test_order_to_picking(self):
         """
@@ -74,23 +202,17 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             check the quantity, the locations and return picking logic
         """
 
-        def compute_tax(product, price, qty=1, taxes=None):
-            if taxes is None:
-                taxes = product.taxes_id.filtered(lambda t: t.company_id.id == self.env.user.id)
-            currency = self.pos_config.pricelist_id.currency_id
-            res = taxes.compute_all(price, currency, qty, product=product)
-            untax = res['total_excluded']
-            return untax, sum(tax.get('amount', 0.0) for tax in res['taxes'])
-
         # I click on create a new session button
         self.pos_config.open_session_cb()
+        current_session = self.pos_config.current_session_id
 
         # I create a PoS order with 2 units of PCSC234 at 450 EUR
         # and 3 units of PCSC349 at 300 EUR.
-        untax1, atax1 = compute_tax(self.product3, 450, 2)
-        untax2, atax2 = compute_tax(self.product4, 300, 3)
+        untax1, atax1 = self.compute_tax(self.product3, 450, 2)
+        untax2, atax2 = self.compute_tax(self.product4, 300, 3)
         self.pos_order_pos1 = self.PosOrder.create({
             'company_id': self.company_id,
+            'session_id': current_session.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
             'partner_id': self.partner1.id,
             'lines': [(0, 0, {
@@ -151,10 +273,11 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         )
 
         # I create a second order
-        untax1, atax1 = compute_tax(self.product3, 450, -2)
-        untax2, atax2 = compute_tax(self.product4, 300, -3)
+        untax1, atax1 = self.compute_tax(self.product3, 450, -2)
+        untax2, atax2 = self.compute_tax(self.product4, 300, -3)
         self.pos_order_pos2 = self.PosOrder.create({
             'company_id': self.company_id,
+            'session_id': current_session.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
             'partner_id': self.partner1.id,
             'lines': [(0, 0, {
@@ -215,10 +338,11 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             'Move Lines should be in done state.'
         )
 
-        untax1, atax1 = compute_tax(self.product3, 450, -2)
-        untax2, atax2 = compute_tax(self.product4, 300, 3)
+        untax1, atax1 = self.compute_tax(self.product3, 450, -2)
+        untax2, atax2 = self.compute_tax(self.product4, 300, 3)
         self.pos_order_pos3 = self.PosOrder.create({
             'company_id': self.company_id,
+            'session_id': current_session.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
             'partner_id': self.partner1.id,
             'lines': [(0, 0, {
@@ -280,22 +404,17 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         # I close the session to generate the journal entries
         self.pos_config.current_session_id.action_pos_session_closing_control()
 
-
     def test_order_to_invoice(self):
 
-        def compute_tax(product, price, qty=1, taxes=None):
-            if taxes is None:
-                taxes = product.taxes_id.filtered(lambda t: t.company_id.id == self.env.user.id)
-            currency = self.pos_config.pricelist_id.currency_id
-            res = taxes.compute_all(price, currency, qty, product=product)
-            untax = res['total_excluded']
-            return untax, sum(tax.get('amount', 0.0) for tax in res['taxes'])
+        self.pos_config.open_session_cb()
+        current_session = self.pos_config.current_session_id
 
-        untax1, atax1 = compute_tax(self.product3, 450*0.95, 2)
-        untax2, atax2 = compute_tax(self.product4, 300*0.95, 3)
+        untax1, atax1 = self.compute_tax(self.product3, 450*0.95, 2)
+        untax2, atax2 = self.compute_tax(self.product4, 300*0.95, 3)
         # I create a new PoS order with 2 units of PC1 at 450 EUR (Tax Incl) and 3 units of PCSC349 at 300 EUR. (Tax Excl)
         self.pos_order_pos1 = self.PosOrder.create({
             'company_id': self.company_id,
+            'session_id': current_session.id,
             'partner_id': self.partner1.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
             'lines': [(0, 0, {
@@ -304,7 +423,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
                 'price_unit': 450,
                 'discount': 5.0,
                 'qty': 2.0,
-                'tax_ids': [(6, 0, self.product3.taxes_id.ids)],
+                'tax_ids': [(6, 0, self.product3.taxes_id.filtered(lambda t: t.company_id.id == self.company_id).ids)],
                 'price_subtotal': untax1,
                 'price_subtotal_incl': untax1 + atax1,
             }), (0, 0, {
@@ -313,7 +432,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
                 'price_unit': 300,
                 'discount': 5.0,
                 'qty': 3.0,
-                'tax_ids': [(6, 0, self.product4.taxes_id.ids)],
+                'tax_ids': [(6, 0, self.product4.taxes_id.filtered(lambda t: t.company_id.id == self.company_id).ids)],
                 'price_subtotal': untax2,
                 'price_subtotal_incl': untax2 + atax2,
             })],
@@ -335,17 +454,20 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         # I check that the order is marked as paid and there is no invoice
         # attached to it
         self.assertEqual(self.pos_order_pos1.state, 'paid', "Order should be in paid state.")
-        self.assertFalse(self.pos_order_pos1.invoice_id, 'Invoice should not be attached to order.')
+        self.assertFalse(self.pos_order_pos1.account_move, 'Invoice should not be attached to order.')
 
         # I generate an invoice from the order
         res = self.pos_order_pos1.action_pos_order_invoice()
-        self.assertIn('res_id', res, "No invoice created")
+        self.assertIn('res_id', res, "Invoice should be created")
 
         # I test that the total of the attached invoice is correct
-        invoice = self.env['account.invoice'].browse(res['res_id'])
+        invoice = self.env['account.move'].browse(res['res_id'])
         self.assertAlmostEqual(
             invoice.amount_total, self.pos_order_pos1.amount_total, places=2, msg="Invoice not correct")
 
+        # I close the session to generate the journal entries
+        current_session.action_pos_session_closing_control()
+
         """In order to test the reports on Bank Statement defined in point_of_sale module, I create a bank statement line, confirm it and print the reports"""
 
         # I select the period and journal for the bank statement
@@ -399,21 +521,13 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         Simulation of sales coming from the interface, even after closing the session
         """
 
-        def compute_tax(product, price, qty=1, taxes=None):
-            if not taxes:
-                taxes = product.taxes_id.filtered(lambda t: t.company_id.id == self.env.user.id)
-            currency = self.pos_config.pricelist_id.currency_id
-            res = taxes.compute_all(price, currency, qty, product=product)
-            untax = res['total_excluded']
-            return untax, sum(tax.get('amount', 0.0) for tax in res['taxes'])
-
         # I click on create a new session button
         self.pos_config.open_session_cb()
 
         current_session = self.pos_config.current_session_id
         num_starting_orders = len(current_session.order_ids)
 
-        untax, atax = compute_tax(self.led_lamp, 0.9)
+        untax, atax = self.compute_tax(self.led_lamp, 0.9)
         carrot_order = {'data':
           {'amount_paid': untax + atax,
            'amount_return': 0,
@@ -439,17 +553,15 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
            'sequence_number': 2,
            'statement_ids': [[0,
              0,
-             {'account_id': self.env.user.partner_id.property_account_receivable_id.id,
-              'amount': untax + atax,
-              'journal_id': self.pos_config.journal_ids[0].id,
+             {'amount': untax + atax,
               'name': fields.Datetime.now(),
-              'statement_id': current_session.statement_ids[0].id}]],
+              'payment_method_id': self.cash_payment_method.id}]],
            'uid': '00042-003-0014',
            'user_id': self.env.uid},
           'id': '00042-003-0014',
           'to_invoice': False}
 
-        untax, atax = compute_tax(self.whiteboard_pen, 1.2)
+        untax, atax = self.compute_tax(self.whiteboard_pen, 1.2)
         zucchini_order = {'data':
           {'amount_paid': untax + atax,
            'amount_return': 0,
@@ -475,17 +587,15 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
            'sequence_number': self.pos_config.journal_id.id,
            'statement_ids': [[0,
              0,
-             {'account_id': self.env.user.partner_id.property_account_receivable_id.id,
-              'amount': untax + atax,
-              'journal_id': self.pos_config.journal_ids[0].id,
+             {'amount': untax + atax,
               'name': fields.Datetime.now(),
-              'statement_id': current_session.statement_ids[0].id}]],
+              'payment_method_id': self.credit_payment_method.id}]],
            'uid': '00043-003-0014',
            'user_id': self.env.uid},
           'id': '00043-003-0014',
           'to_invoice': False}
 
-        untax, atax = compute_tax(self.newspaper_rack, 1.28)
+        untax, atax = self.compute_tax(self.newspaper_rack, 1.28)
         newspaper_rack_order = {'data':
           {'amount_paid': untax + atax,
            'amount_return': 0,
@@ -511,11 +621,9 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
            'sequence_number': self.pos_config.journal_id.id,
            'statement_ids': [[0,
              0,
-             {'account_id': self.env.user.partner_id.property_account_receivable_id.id,
-              'amount': untax + atax,
-              'journal_id': self.pos_config.journal_ids[0].id,
+             {'amount': untax + atax,
               'name': fields.Datetime.now(),
-              'statement_id': current_session.statement_ids[0].id}]],
+              'payment_method_id': self.bank_payment_method.id}]],
            'uid': '00044-003-0014',
            'user_id': self.env.uid},
           'id': '00044-003-0014',
@@ -525,10 +633,6 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         self.PosOrder.create_from_ui([carrot_order])
         self.assertEqual(num_starting_orders + 1, len(current_session.order_ids), "Submitted order not encoded")
 
-        # I resubmit the same order
-        self.PosOrder.create_from_ui([carrot_order])
-        self.assertEqual(num_starting_orders + 1, len(current_session.order_ids), "Resubmitted order was not skipped")
-
         # I close the session
         current_session.action_pos_session_closing_control()
         self.assertEqual(current_session.state, 'closed', "Session was not properly closed")
@@ -559,7 +663,7 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         # Make sure the company is in USD
         self.env.cr.execute(
             "UPDATE res_company SET currency_id = %s WHERE id = %s",
-            [self.env.ref('base.USD').id, self.env.user.company_id.id])
+            [self.env.ref('base.USD').id, self.env.company.id])
 
         # Demo data are crappy, clean-up the rates
         self.env['res.currency.rate'].search([]).unlink()
@@ -569,25 +673,39 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
             'currency_id': self.env.ref('base.EUR').id,
         })
 
-        def compute_tax(product, price, qty=1, taxes=None):
-            if not taxes:
-                taxes = product.taxes_id.filtered(lambda t: t.company_id.id == self.env.user.id)
-            currency = self.pos_config.pricelist_id.currency_id
-            res = taxes.compute_all(price, currency, qty, product=product)
-            untax = res['total_excluded']
-            return untax, sum(tax.get('amount', 0.0) for tax in res['taxes'])
+        # make a config that has currency different from the company
+        eur_pricelist = self.partner1.property_product_pricelist.copy(default={'currency_id': self.env.ref('base.EUR').id})
+        sale_journal = self.env['account.journal'].create({
+            'name': 'PoS Sale EUR',
+            'type': 'sale',
+            'code': 'POSE',
+            'company_id': self.company.id,
+            'sequence': 12,
+            'currency_id': self.env.ref('base.EUR').id
+        })
+        eur_config = self.pos_config.create({
+            'name': 'Shop EUR Test',
+            'module_account': False,
+            'journal_id': sale_journal.id,
+            'use_pricelist': True,
+            'available_pricelist_ids': [(6, 0, eur_pricelist.ids)],
+            'pricelist_id': eur_pricelist.id,
+            'payment_method_ids': [(6, 0, self.bank_payment_method.ids)]
+        })
 
         # I click on create a new session button
-        self.pos_config.open_session_cb()
+        eur_config.open_session_cb()
+        current_session = eur_config.current_session_id
 
         # I create a PoS order with 2 units of PCSC234 at 450 EUR (Tax Incl)
         # and 3 units of PCSC349 at 300 EUR. (Tax Excl)
 
-        untax1, atax1 = compute_tax(self.product3, 450*0.95, 2)
-        untax2, atax2 = compute_tax(self.product4, 300*0.95, 3)
+        untax1, atax1 = self.compute_tax(self.product3, 450, 2)
+        untax2, atax2 = self.compute_tax(self.product4, 300, 3)
         self.pos_order_pos0 = self.PosOrder.create({
             'company_id': self.company_id,
-            'pricelist_id': self.partner1.property_product_pricelist.copy(default={'currency_id': self.env.ref('base.EUR').id}).id,
+            'session_id': current_session.id,
+            'pricelist_id': eur_pricelist.id,
             'partner_id': self.partner1.id,
             'lines': [(0, 0, {
                 'name': "OL/0001",
@@ -617,14 +735,15 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         # I check that the total of the order is now equal to (450*2 +
         # 300*3*1.05)*0.95
         self.assertLess(
-            abs(self.pos_order_pos0.amount_total - (450 * 2 + 300 * 3 * 1.05) * 0.95),
+            abs(self.pos_order_pos0.amount_total - (450 * 2 + 300 * 3 * 1.05)),
             0.01, 'The order has a wrong total including tax and discounts')
 
         # I click on the "Make Payment" wizard to pay the PoS order with a
         # partial amount of 100.0 EUR
         context_make_payment = {"active_ids": [self.pos_order_pos0.id], "active_id": self.pos_order_pos0.id}
         self.pos_make_payment_0 = self.PosMakePayment.with_context(context_make_payment).create({
-            'amount': 100.0
+            'amount': 100.0,
+            'payment_method_id': self.bank_payment_method.id,
         })
 
         # I click on the validate button to register the payment.
@@ -639,14 +758,15 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         defs = self.pos_make_payment_0.with_context({'active_id': self.pos_order_pos0.id}).default_get(['amount'])
 
         self.assertLess(
-            abs(defs['amount'] - ((450 * 2 + 300 * 3 * 1.05) * 0.95 - 100.0)), 0.01, "The remaining balance is incorrect.")
+            abs(defs['amount'] - ((450 * 2 + 300 * 3 * 1.05) - 100.0)), 0.01, "The remaining balance is incorrect.")
 
         #'I pay the remaining balance.
         context_make_payment = {
             "active_ids": [self.pos_order_pos0.id], "active_id": self.pos_order_pos0.id}
 
         self.pos_make_payment_1 = self.PosMakePayment.with_context(context_make_payment).create({
-            'amount': (450 * 2 + 300 * 3 * 1.05) * 0.95 - 100.0
+            'amount': (450 * 2 + 300 * 3 * 1.05) - 100.0,
+            'payment_method_id': self.bank_payment_method.id,
         })
 
         # I click on the validate button to register the payment.
@@ -656,26 +776,31 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         self.assertEqual(self.pos_order_pos0.state, 'paid', 'Order should be in paid state.')
 
         # I generate the journal entries
-        self.pos_order_pos0._create_account_move_line()
+        current_session.action_pos_session_validate()
 
         # I test that the generated journal entry is attached to the PoS order
-        self.assertTrue(self.pos_order_pos0.account_move, "Journal entry has not been attached to Pos order.")
+        self.assertTrue(current_session.move_id, "Journal entry should have been attached to the session.")
 
         # Check the amounts
-        debit_lines = self.pos_order_pos0.account_move.mapped('line_ids.debit')
-        credit_lines = self.pos_order_pos0.account_move.mapped('line_ids.credit')
-        amount_currency_lines = self.pos_order_pos0.account_move.mapped('line_ids.amount_currency')
-        for a, b in pycompat.izip(sorted(debit_lines), [0.0, 0.0, 0.0, 0.0, 879.55]):
+        debit_lines = current_session.move_id.mapped('line_ids.debit')
+        credit_lines = current_session.move_id.mapped('line_ids.credit')
+        amount_currency_lines = current_session.move_id.mapped('line_ids.amount_currency')
+        for a, b in zip(sorted(debit_lines), [0.0, 0.0, 0.0, 0.0, 922.5]):
             self.assertAlmostEqual(a, b)
-        for a, b in pycompat.izip(sorted(credit_lines), [0.0, 22.5, 40.91, 388.64, 427.5]):
+        for a, b in zip(sorted(credit_lines), [0.0, 22.5, 40.91, 409.09, 450]):
             self.assertAlmostEqual(a, b)
-        for a, b in pycompat.izip(sorted(amount_currency_lines), [-855.0, -777.27, -81.82, -45.0, 1752.75]):
+        for a, b in zip(sorted(amount_currency_lines), [-900, -818.18, -81.82, -45, 1845]):
             self.assertAlmostEqual(a, b)
 
     def test_order_to_invoice_no_tax(self):
+
+        self.pos_config.open_session_cb()
+        current_session = self.pos_config.current_session_id
+
         # I create a new PoS order with 2 units of PC1 at 450 EUR (Tax Incl) and 3 units of PCSC349 at 300 EUR. (Tax Excl)
         self.pos_order_pos1 = self.PosOrder.create({
             'company_id': self.company_id,
+            'session_id': current_session.id,
             'partner_id': self.partner1.id,
             'pricelist_id': self.partner1.property_product_pricelist.id,
             'lines': [(0, 0, {
@@ -713,16 +838,83 @@ class TestPointOfSaleFlow(TestPointOfSaleCommon):
         # I check that the order is marked as paid and there is no invoice
         # attached to it
         self.assertEqual(self.pos_order_pos1.state, 'paid', "Order should be in paid state.")
-        self.assertFalse(self.pos_order_pos1.invoice_id, 'Invoice should not be attached to order.')
+        self.assertFalse(self.pos_order_pos1.account_move, 'Invoice should not be attached to order yet.')
 
         # I generate an invoice from the order
         res = self.pos_order_pos1.action_pos_order_invoice()
         self.assertIn('res_id', res, "No invoice created")
 
         # I test that the total of the attached invoice is correct
-        invoice = self.env['account.invoice'].browse(res['res_id'])
+        invoice = self.env['account.move'].browse(res['res_id'])
         self.assertAlmostEqual(
             invoice.amount_total, self.pos_order_pos1.amount_total, places=2, msg="Invoice not correct")
 
         for iline in invoice.invoice_line_ids:
-            self.assertFalse(iline.invoice_line_tax_ids)
+            self.assertFalse(iline.tax_ids)
+
+        self.pos_config.current_session_id.action_pos_session_closing_control()
+
+    def test_order_with_deleted_tax(self):
+        # create tax
+        dummy_50_perc_tax = self.env['account.tax'].create({
+            'name': 'Tax 50%',
+            'amount_type': 'percent',
+            'amount': 50.0,
+            'price_include': 0
+        })
+
+        # set tax to product
+        product5 = self.env['product.product'].create({
+            'name': 'product5',
+            'type': 'product',
+            'categ_id': self.env.ref('product.product_category_all').id,
+            'taxes_id': dummy_50_perc_tax.ids
+        })
+
+        # sell product thru pos
+        self.pos_config.open_session_cb()
+        pos_session = self.pos_config.current_session_id
+        untax, atax = self.compute_tax(product5, 10.0)
+        product5_order = {'data':
+          {'amount_paid': untax + atax,
+           'amount_return': 0,
+           'amount_tax': atax,
+           'amount_total': untax + atax,
+           'creation_date': fields.Datetime.to_string(fields.Datetime.now()),
+           'fiscal_position_id': False,
+           'pricelist_id': self.pos_config.available_pricelist_ids[0].id,
+           'lines': [[0,
+             0,
+             {'discount': 0,
+              'id': 42,
+              'pack_lot_ids': [],
+              'price_unit': 10.0,
+              'product_id': product5.id,
+              'price_subtotal': 10.0,
+              'price_subtotal_incl': 15.0,
+              'qty': 1,
+              'tax_ids': [(6, 0, product5.taxes_id.ids)]}]],
+           'name': 'Order 12345-123-1234',
+           'partner_id': False,
+           'pos_session_id': pos_session.id,
+           'sequence_number': 2,
+           'statement_ids': [[0,
+             0,
+             {'amount': untax + atax,
+              'name': fields.Datetime.now(),
+              'payment_method_id': self.cash_payment_method.id}]],
+           'uid': '12345-123-1234',
+           'user_id': self.env.uid},
+          'id': '12345-123-1234',
+          'to_invoice': False}
+        self.PosOrder.create_from_ui([product5_order])
+
+        # delete tax
+        dummy_50_perc_tax.unlink()
+
+        # close session (should not fail here)
+        pos_session.action_pos_session_closing_control()
+
+        # check the difference line
+        diff_line = pos_session.move_id.line_ids.filtered(lambda line: line.name == 'Difference at closing PoS session')
+        self.assertAlmostEqual(diff_line.credit, 5.0, msg="Missing amount of 5.0")
