@@ -7,4 +7,4 @@ import odoo.tests
 class TestUi(odoo.tests.HttpCase):
 
     def test_01_point_of_sale_tour(self):
-        self.phantom_js("/web", "odoo.__DEBUG__.services['web_tour.tour'].run('point_of_sale_tour')", "odoo.__DEBUG__.services['web_tour.tour'].tours.point_of_sale_tour.ready", login="admin")
+        self.start_tour("/web", 'point_of_sale_tour', login="admin")
