@@ -15,7 +15,7 @@ portal.
 This module contains most code coming from odoo v10 website_portal. Purpose
 of this module is to allow the display of a customer portal without having
 a dependency towards website edition and customization capabilities.""",
-    'depends': ['http_routing', 'mail'],
+    'depends': ['web', 'web_editor', 'http_routing', 'mail', 'auth_signup'],
     'data': [
         'data/portal_data.xml',
         'views/assets.xml',
