@@ -8,6 +8,7 @@ var models = require('point_of_sale.models');
 var screens = require('point_of_sale.screens');
 var core = require('web.core');
 var rpc = require('web.rpc');
+var session = require('web.session');
 
 var QWeb = core.qweb;
 var _t = core._t;
@@ -60,8 +61,6 @@ var TableWidget = PosBaseWidget.extend({
         this.selected = false;
         this.moved    = false;
         this.dragpos  = {x:0, y:0};
-        this.handle_dragging = false;
-        this.handle   = null;
     },
     // computes the absolute position of a DOM mouse event, used
     // when resizing tables
@@ -94,81 +93,25 @@ var TableWidget = PosBaseWidget.extend({
         }
     },
     // drag and drop for moving the table, at drag start
-    dragstart_handler: function(event,$el,drag){
-        if (this.selected && !this.handle_dragging) {
-            this.dragging = true;
-            this.dragpos  = { x: drag.offsetX, y: drag.offsetY };
-        }
+    dragstart_handler:   function(event, ui){
+        this.dragging = true;
     },
     // drag and drop for moving the table, at drag end
-    dragend_handler:   function(){
+    dragend_handler:   function(event, ui){
         this.dragging = false;
-    },
-    // drag and drop for moving the table, at every drop movement.
-    dragmove_handler: function(event,$el,drag){
-        if (this.dragging) {
-            var dx   = drag.offsetX - this.dragpos.x;
-            var dy   = drag.offsetY - this.dragpos.y;
-
-            this.dragpos = { x: drag.offsetX, y: drag.offsetY };
-            this.moved   = true;
-
-            this.table.position_v += dy;
-            this.table.position_h += dx;
-
-            $el.css(this.table_style());
-        }
+        this.moved = true;
+        this.table.position_h = ui.position.left - this.table.width/2;
+        this.table.position_v = ui.position.top - this.table.height/2;
+        this.$el.css(this.table_style());
     },
     // drag and dropping the resizing handles
-    handle_dragstart_handler: function(event, $el, drag) {
-        if (this.selected && !this.dragging) {
-            this.handle_dragging = true;
-            this.handle_dragpos  = this.event_position(event);
-            this.handle          = drag.target;
-        }
-    },
-    handle_dragend_handler: function() {
-        this.handle_dragging = false;
-    },
-    handle_dragmove_handler: function(event) {
-        if (this.handle_dragging) {
-            var pos  = this.event_position(event);
-            var dx   = pos.x - this.handle_dragpos.x;
-            var dy   = pos.y - this.handle_dragpos.y;
-
-            this.handle_dragpos = pos;
-            this.moved   = true;
-
-            var cl     = this.handle.classList;
-
-            var MIN_SIZE = 40;  // smaller than this, and it becomes impossible to edit.
-
-            var tw = Math.max(MIN_SIZE, this.table.width);
-            var th = Math.max(MIN_SIZE, this.table.height);
-            var tx = this.table.position_h;
-            var ty = this.table.position_v;
-
-            if (cl.contains('left') && tw - dx >= MIN_SIZE) {
-                tw -= dx;
-                tx += dx;
-            } else if (cl.contains('right') && tw + dx >= MIN_SIZE) {
-                tw += dx;
-            }
-
-            if (cl.contains('top') && th - dy >= MIN_SIZE) {
-                th -= dy;
-                ty += dy;
-            } else if (cl.contains('bottom') && th + dy >= MIN_SIZE) {
-                th += dy;
-            }
-
-            this.table.width  = tw;
-            this.table.height = th;
-            this.table.position_h = tx;
-            this.table.position_v = ty;
-
-            this.$el.css(this.table_style());
-        }
+    handle_dragmove_handler: function(event, ui) {
+        this.moved = true;
+        this.table.width  = ui.size.width;
+        this.table.height = ui.size.height;
+        this.table.position_h = ui.position.left - ui.originalSize.width/2;
+        this.table.position_v = ui.position.top - ui.originalSize.height/2;
+        this.$el.css(this.table_style());
     },
     set_table_color: function(color){
         this.table.color = _.escape(color);
@@ -225,6 +168,15 @@ var TableWidget = PosBaseWidget.extend({
     select: function() {
         this.selected = true;
         this.renderElement();
+
+        this.$el.resizable({
+            handles: 'all',
+            resize: this.handle_dragmove_handler.bind(this),
+        });
+
+        this.$el.draggable({
+            stop: this.dragend_handler.bind(this),
+        });
     },
     // deselect the table (should be called via the floorplan)
     deselect: function() {
@@ -264,13 +216,16 @@ var TableWidget = PosBaseWidget.extend({
                         for (var field in table) {
                             self.table[field] = table[field];
                         }
-                        self.renderElement();
+                        self.pos.tables_by_id[table.id] = self.table;
+                        // If selected, render with drag and resize event handlers.
+                        if (!self.selected) {
+                            self.renderElement();
+                        } else {
+                            self.select();
+                        }
                     });
             }, function(type,err) {
-                self.gui.show_popup('error',{
-                    'title':_t('Changes could not be saved'),
-                    'body': _t('You must be connected to the internet to save your changes.'),
-                });
+                    self.gui.show_sync_error_popup();
             });
     },
     // destroy the table.  We do not really destroy it, we set it
@@ -306,10 +261,7 @@ var TableWidget = PosBaseWidget.extend({
                 floorplan.update_toolbar();
                 self.destroy();
             }, function(type, err) {
-                self.gui.show_popup('error', {
-                    'title':_t('Changes could not be saved'),
-                    'body': _t('You must be connected to the internet to save your changes.'),
-                });
+                self.gui.show_sync_error_popup();
             });
     },
     get_notifications: function(){  //FIXME : Make this faster
@@ -325,34 +277,28 @@ var TableWidget = PosBaseWidget.extend({
         }
         return notifications;
     },
-        update_click_handlers: function(editing){
-            var self = this;
-            this.$el.off('mouseup touchend touchcancel click dragend');
+    update_click_handlers: function(editing){
+        var self = this;
+        this.$el.off('mouseup touchend touchcancel click dragend');
 
-            if (editing) {
-                this.$el.on('mouseup touchend touchcancel', function(event){ self.click_handler(event,$(this)); });
-            } else {
-                this.$el.on('click dragend', function(event){ self.click_handler(event,$(this)); });
-            }
-        },
+        if (editing) {
+            this.$el.on('mouseup touchend touchcancel', function(event){ self.click_handler(event,$(this)); });
+        } else {
+            this.$el.on('click dragend', function(event){ self.click_handler(event,$(this)); });
+        }
+    },
     renderElement: function(){
         var self = this;
-        this.order_count    = this.pos.get_table_orders(this.table).length;
+        this.order_count    = this.table.order_count !== undefined ?
+            this.table.order_count :
+            this.pos.get_table_orders(this.table)
+                .filter(o => o.orderlines.length !== 0 || o.paymentlines.length !==0).length;
         this.customer_count = this.pos.get_customer_count(this.table);
         this.fill           = Math.min(1,Math.max(0,this.customer_count / this.table.seats));
         this.notifications  = this.get_notifications();
         this._super();
 
         this.update_click_handlers();
-
-        this.$el.on('dragstart', function(event,drag){ self.dragstart_handler(event,$(this),drag); });
-        this.$el.on('drag',      function(event,drag){ self.dragmove_handler(event,$(this),drag); });
-        this.$el.on('dragend',   function(event,drag){ self.dragend_handler(event,$(this),drag); });
-
-        var handles = this.$el.find('.table-handle');
-        handles.on('dragstart',  function(event,drag){ self.handle_dragstart_handler(event,$(this),drag); });
-        handles.on('drag',       function(event,drag){ self.handle_dragmove_handler(event,$(this),drag); });
-        handles.on('dragend',    function(event,drag){ self.handle_dragend_handler(event,$(this),drag); });
     },
 });
 
@@ -373,12 +319,44 @@ var FloorScreenWidget = screens.ScreenWidget.extend({
         this.selected_table = null;
         this.editing = false;
     },
+    _table_longpolling: function(){
+        if (this.editing) {
+            return;
+        }
+        var self = this;
+        rpc.query({
+            model: 'pos.config',
+            method: 'get_tables_order_count',
+            args: [this.pos.config.id],
+        })
+        .then(function (result){
+            result.forEach(function(table){
+                var table_obj = self.pos.tables_by_id[table.id];
+                var unsynced_orders = self.pos.get_table_orders(table_obj)
+                    .filter(o => o.server_id === undefined &&
+                            (o.orderlines.length !== 0 || o.paymentlines.length !== 0)).length
+                table_obj.order_count = table.orders + unsynced_orders;
+            });
+            self.table_widgets.forEach(
+                    function(tw){
+                        tw.renderElement();
+                    });
+        }, function(type,err) {
+            self.table_widgets.forEach(
+                    function(tw){
+                        tw.table.order_count = self.pos.get_table_orders(tw.table)
+                            .filter(o => o.orderlines.length !== 0 || o.paymentlines.length !== 0).length
+                        tw.renderElement();
+                    });
+        });
+    },
     hide: function(){
         this._super();
         if (this.editing) {
             this.toggle_editing();
         }
         this.chrome.widget.order_selector.show();
+        clearInterval(this.table_longpolling);
     },
     show: function(){
         this._super();
@@ -387,6 +365,9 @@ var FloorScreenWidget = screens.ScreenWidget.extend({
             this.table_widgets[i].renderElement();
         }
         this.check_empty_floor();
+
+        this._table_longpolling();
+        this.table_longpolling = setInterval(this._table_longpolling.bind(this), 5000);
     },
     click_floor_button: function(event,$el){
         var floor = this.pos.floors_by_id[$el.data('id')];
@@ -421,11 +402,8 @@ var FloorScreenWidget = screens.ScreenWidget.extend({
                 method: 'write',
                 args: [[this.floor.id], {'background_color': background}],
             })
-            .fail(function (type, err){
-                self.gui.show_popup('error',{
-                    'title':_t('Changes could not be saved'),
-                    'body': _t('You must be connected to the internet to save your changes.'),
-                });
+            .guardedCatch(function (){
+                self.gui.show_sync_error_popup();
             });
         this.$('.floor-map').css({"background-color": _.escape(background)});
     },
@@ -712,6 +690,38 @@ var FloorScreenWidget = screens.ScreenWidget.extend({
     },
 });
 
+screens.ProductScreenWidget.include({
+    show: function () {
+        var self = this;
+        this._super();
+        if (this.pos.config.iface_floorplan) {
+            this.$el.bind("mousemove mousedown touchstart click scroll keypress", function () {
+                self.set_idle_timer();
+            });
+            this.set_idle_timer();
+        }
+    },
+    hide: function () {
+        this._super();
+        if (this.pos.config.iface_floorplan) {
+            this.$el.unbind("mousemove mousedown touchstart click scroll keypress");
+            clearTimeout(this.idle_timer);
+        }
+    },
+    /**
+     * Set a timeout to go back to the floorplan and clear the previous timeout
+     *
+     * @param {number} timeout, optional timeout in miliseconds, default one minute.
+     */
+    set_idle_timer: function(timeout=60000) {
+        var self = this;
+        clearTimeout(this.idle_timer);
+        this.idle_timer = setTimeout(function () {
+            self.pos.set_table(null);
+        }, timeout);
+    },
+});
+
 gui.define_screen({
     'name': 'floors',
     'widget': FloorScreenWidget,
@@ -720,6 +730,19 @@ gui.define_screen({
     },
 });
 
+gui.Gui.include({
+    show_sync_error_popup: function() {
+        if (this.show_sync_errors) {
+            this.show_popup('error-sync',{
+                'title':_t('Changes could not be saved'),
+                'body': _t('You must be connected to the internet to save your changes.\n\n' +
+                        'Changes made to previously synced orders will get lost at the next sync.\n' +
+                        'Orders that where not synced before will be synced next time you open and close the same table.'),
+            });
+        }
+    },
+});
+
 // Add the FloorScreen to the GUI, and set it as the default screen
 chrome.Chrome.include({
     build_widgets: function(){
@@ -811,9 +834,12 @@ chrome.OrderSelectorWidget.include({
 // if there is none.
 var _super_posmodel = models.PosModel.prototype;
 models.PosModel = models.PosModel.extend({
-    initialize: function(session, attributes) {
-        this.table = null;
-        return _super_posmodel.initialize.call(this,session,attributes);
+    after_load_server_data: function() {
+        var res = _super_posmodel.after_load_server_data.call(this);
+        if (this.config.iface_floorplan) {
+            this.table = null;
+        }
+        return res;
     },
 
     transfer_order_to_different_table: function () {
@@ -825,26 +851,171 @@ models.PosModel = models.PosModel.extend({
         this.set_table(null);
     },
 
-    // changes the current table.
+    remove_from_server_and_set_sync_state: function(ids_to_remove){
+        var self = this;
+        this.set_synch('connecting', ids_to_remove.length);
+        self._remove_from_server(ids_to_remove)
+            .then(function(server_ids) {
+                self.set_synch('connected');
+            }).catch(function(reason){
+                self.set_synch('error');
+            });
+    },
+
+    /**
+     * Request the orders of the table with given id.
+     * @param {number} table_id.
+     * @param {dict} options.
+     * @param {number} options.timeout optional timeout parameter for the rpc call.
+     * @return {Promise}
+     */
+    _get_from_server: function (table_id, options) {
+        options = options || {};
+        var self = this;
+        var timeout = typeof options.timeout === 'number' ? options.timeout : 7500;
+        return new Promise( function(resolve, reject) {
+            rpc.query({
+                model: 'pos.order',
+                method: 'get_table_draft_orders',
+                args: [table_id],
+                kwargs: {context: session.user_context},
+            }, {
+                timeout: timeout,
+                shadow: false,
+            }).then(function(orders){
+                orders.forEach(function(order) {
+                    order.multiprint_resume = JSON.parse(order.multiprint_resume? order.multiprint_resume: false);
+                });
+                resolve(orders);
+            }).catch(function(err) {
+                reject(err);
+            });
+        });
+    },
+
+    transfer_order_to_table: function(table) {
+        this.order_to_transfer_to_different_table.table = table;
+        this.order_to_transfer_to_different_table.save_to_db();
+    },
+
+    push_order_for_transfer: function(order_ids, table_orders) {
+        order_ids.push(this.order_to_transfer_to_different_table.uid);
+        table_orders.push(this.order_to_transfer_to_different_table);
+    },
+
+    clean_table_transfer: function(table) {
+        if (this.order_to_transfer_to_different_table && table) {
+            this.order_to_transfer_to_different_table = null;
+            this.set_table(table);
+        }
+    },
+
+    sync_from_server: function(table, table_orders, order_ids) {
+        var self = this;
+        var ids_to_remove = this.db.get_ids_to_remove_from_server();
+        var orders_to_sync = this.db.get_unpaid_orders_to_sync(order_ids);
+        if (orders_to_sync.length) {
+            this.set_synch('connecting', orders_to_sync.length);
+            this._save_to_server(orders_to_sync, {'draft': true}).then(function (server_ids) {
+                server_ids.forEach(function(server_id){
+                    table_orders.some(function(o){
+                        if (o.name === server_id.pos_reference) {
+                            o.server_id = server_id.id;
+                            o.save_to_db();
+                        }
+                    });
+                });
+                if (!ids_to_remove.length) {
+                    self.set_synch('connected');
+                } else {
+                    self.remove_from_server_and_set_sync_state(ids_to_remove);
+                }
+            }).catch(function(reason){
+                self.set_synch('error');
+            }).finally(function(){
+                self.clean_table_transfer(table);
+            });
+        } else {
+            if (ids_to_remove.length) {
+                self.remove_from_server_and_set_sync_state(ids_to_remove);
+            }
+            self.clean_table_transfer(table);
+        }
+    },
+
+    set_order_on_table: function() {
+        var orders = this.get_order_list();
+        if (orders.length) {
+            this.set_order(orders[0]); // and go to the first one ...
+        } else {
+            this.add_new_order();  // or create a new order with the current table
+        }
+    },
+
+    sync_to_server: function(table) {
+        var self = this;
+        var ids_to_remove = this.db.get_ids_to_remove_from_server();
+
+        clearInterval(this.table_longpolling);
+
+        this.set_synch('connecting', 1);
+        this._get_from_server(table.id).then(function (server_orders) {
+            var orders = self.get_order_list();
+            orders.forEach(function(order){
+                if (order.server_id){
+                    self.get("orders").remove(order);
+                    order.destroy();
+                }
+            });
+            server_orders.forEach(function(server_order){
+                if (server_order.lines.length){
+                    var new_order = new models.Order({},{pos: self, json: server_order});
+                    self.get("orders").add(new_order);
+                    new_order.save_to_db();
+                }
+            })
+            if (!ids_to_remove.length) {
+                self.set_synch('connected');
+            } else {
+                self.remove_from_server_and_set_sync_state(ids_to_remove);
+            }
+        }).catch(function(reason){
+            self.set_synch('error');
+        }).finally(function(){
+            self.set_order_on_table();
+        });
+    },
+
+    get_order_with_uid: function() {
+        var order_ids = [];
+        this.get_order_list().forEach(function(o){
+            order_ids.push(o.uid);
+        });
+
+        return order_ids;
+    },
+
+    /**
+     * Changes the current table.
+     *
+     * Switch table and make sure all nececery syncing tasks are done.
+     * @param {object} table.
+     */
     set_table: function(table) {
-        if (!table) { // no table ? go back to the floor plan, see ScreenSelector
+        if(!table){
+            this.sync_from_server(table, this.get_order_list(), this.get_order_with_uid());
             this.set_order(null);
         } else if (this.order_to_transfer_to_different_table) {
-            this.order_to_transfer_to_different_table.table = table;
-            this.order_to_transfer_to_different_table.save_to_db();
-            this.order_to_transfer_to_different_table = null;
+            var order_ids = this.get_order_with_uid();
 
-            // set this table
-            this.set_table(table);
+            this.transfer_order_to_table(table);
+            this.push_order_for_transfer(order_ids, this.get_order_list());
 
+            this.sync_from_server(table, this.get_order_list(), order_ids);
+            this.set_order(null);
         } else {
             this.table = table;
-            var orders = this.get_order_list();
-            if (orders.length) {
-                this.set_order(orders[0]); // and go to the first one ...
-            } else {
-                this.add_new_order();  // or create a new order with the current table
-            }
+            this.sync_to_server(table);
         }
     },
 
@@ -918,6 +1089,9 @@ models.PosModel = models.PosModel.extend({
     on_removed_order: function(removed_order,index,reason){
         if (this.config.iface_floorplan) {
             var order_list = this.get_order_list();
+            if (reason === 'abandon') {
+                this.db.set_order_to_remove_from_server(removed_order);
+            }
             if( (reason === 'abandon' || removed_order.temporary) && order_list.length > 0){
                 this.set_order(order_list[index] || order_list[order_list.length -1]);
             }else{
