@@ -5,7 +5,7 @@
 {
     'name': 'Just In Time Scheduling',
     'version': '1.0',
-    'category': 'Warehouse',
+    'category': 'Operations/Inventory',
     'description': """
 This module will automatically reserve the picking from stock when a sales order is confirmed
 =============================================================================================
