@@ -5,10 +5,11 @@
     'name': 'Project',
     'version': '1.1',
     'website': 'https://www.odoo.com/page/project-management',
-    'category': 'Project',
+    'category': 'Operations/Project',
     'sequence': 10,
     'summary': 'Organize and schedule your projects ',
     'depends': [
+        'analytic',
         'base_setup',
         'mail',
         'portal',
@@ -23,6 +24,7 @@
         'security/project_security.xml',
         'security/ir.model.access.csv',
         'report/project_report_views.xml',
+        'views/analytic_views.xml',
         'views/digest_views.xml',
         'views/rating_views.xml',
         'views/project_views.xml',
@@ -36,7 +38,6 @@
         'data/project_mail_template_data.xml',
         'data/project_data.xml',
     ],
-    'qweb': ['static/src/xml/project.xml'],
     'demo': ['data/project_demo.xml'],
     'test': [
     ],
