@@ -5,7 +5,7 @@
 {
     'name': 'Purchase and MRP Management',
     'version': '1.0',
-    'category': 'Hidden',
+    'category': 'Operations/Purchase',
     'description': """
 This module provides facility to the user to install mrp and purchase modules at a time.
 ========================================================================================
@@ -13,7 +13,7 @@ This module provides facility to the user to install mrp and purchase modules at
 It is basically used when we want to keep track of production orders generated
 from purchase order.
     """,
-    'depends': ['mrp', 'purchase'],
+    'depends': ['mrp', 'purchase_stock'],
     'installable': True,
     'auto_install': True,
 }
