@@ -13,7 +13,7 @@ class PurchaseReport(models.Model):
         return super(PurchaseReport, self)._select() + ", spt.warehouse_id as picking_type_id"
 
     def _from(self):
-        return super(PurchaseReport, self)._from() + " left join stock_picking_type spt on (spt.id=s.picking_type_id)"
+        return super(PurchaseReport, self)._from() + " left join stock_picking_type spt on (spt.id=po.picking_type_id)"
 
     def _group_by(self):
         return super(PurchaseReport, self)._group_by() + ", spt.warehouse_id"
