@@ -14,3 +14,5 @@ from . import test_purchase_order_process
 from . import test_stockvaluation
 from . import test_replenish_wizard
 from . import test_reordering_rule
+from . import test_move_cancel_propagation
+from . import test_product_template
