@@ -8,4 +8,6 @@ from . import test_sale_pricelist
 from . import test_onchange
 from . import test_reinvoice
 from . import test_access_rights
+from . import test_sale_refund
+from . import test_sale_signature
 from . import test_sale_transaction
