@@ -25,7 +25,7 @@ class TestSaleProductAttributeValueSetup(TestProductAttributeValueSetup):
         non-reliant on demo data."""
         name = fields.Date.today()
         currency_id = currency.id
-        company_id = self.env.user.company_id.id
+        company_id = self.env.company.id
 
         CurrencyRate = self.env['res.currency.rate']
 
@@ -71,11 +71,10 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
         pricelist_item = self.env['product.pricelist.item'].create({
             'min_quantity': 2,
             'compute_price': 'percentage',
-            'percent_price': discount
+            'percent_price': discount,
+            'pricelist_id': pricelist.id,
         })
 
-        pricelist.item_ids += pricelist_item
-
         return (pricelist, pricelist_item, currency_ratio, 1 - discount / 100)
 
     def test_01_is_combination_possible_archived(self):
@@ -85,7 +84,7 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
         model which had product_id as required and without cascade on delete.
         Here we have the sales order line in this situation.
 
-        This is a necessary condition for `create_variant_ids` to archive
+        This is a necessary condition for `_create_variant_ids` to archive
         instead of delete the variants.
         """
         def do_test(self):
@@ -97,8 +96,11 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
             variant = self.computer._get_variant_for_combination(computer_ssd_256 + computer_ram_8 + computer_hdd_1)
             variant2 = self.computer._get_variant_for_combination(computer_ssd_256 + computer_ram_8 + computer_hdd_2)
 
+            self.assertTrue(variant)
+            self.assertTrue(variant2)
+
             # Create a dummy SO to prevent the variant from being deleted by
-            # create_variant_ids() because the variant is a related field that
+            # _create_variant_ids() because the variant is a related field that
             # is required on the SO line
             so = self.env['sale.order'].create({'partner_id': 1})
             self.env['sale.order.line'].create({
@@ -115,7 +117,6 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
 
             variant2.active = False
             # CASE: 1 not archived, 2 archived
-            # Test invalidate_cache on product.product write
             self.assertTrue(self.computer._is_combination_possible(computer_ssd_256 + computer_ram_8 + computer_hdd_1))
             self.assertFalse(self.computer._is_combination_possible(computer_ssd_256 + computer_ram_8 + computer_hdd_2))
             # CASE: both archived combination (without no_variant)
@@ -123,13 +124,13 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
             self.assertFalse(self.computer._is_combination_possible(computer_ssd_256 + computer_ram_8 + computer_hdd_2))
             self.assertFalse(self.computer._is_combination_possible(computer_ssd_256 + computer_ram_8 + computer_hdd_1))
 
-            # CASE: not archived (with no_variant)
+            # CASE: OK after attribute line removed
             self.computer_hdd_attribute_lines.unlink()
+            self.assertTrue(self.computer._is_combination_possible(computer_ssd_256 + computer_ram_8))
+
+            # CASE: not archived (with no_variant)
             self.hdd_attribute.create_variant = 'no_variant'
             self._add_hdd_attribute_line()
-            self.computer.create_variant_ids()
-            computer_ssd_256 = self._get_product_template_attribute_value(self.ssd_256)
-            computer_ram_8 = self._get_product_template_attribute_value(self.ram_8)
             computer_hdd_1 = self._get_product_template_attribute_value(self.hdd_1)
             computer_hdd_2 = self._get_product_template_attribute_value(self.hdd_2)
 
@@ -142,7 +143,6 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
 
             # CASE: archived combination has different attributes (including no_variant)
             self.computer_ssd_attribute_lines.unlink()
-            self.computer.create_variant_ids()
 
             variant4 = self.computer._get_variant_for_combination(computer_ram_8 + computer_hdd_1)
             self.env['sale.order.line'].create({
@@ -156,7 +156,6 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
             self.computer_hdd_attribute_lines.unlink()
             self.hdd_attribute.create_variant = 'always'
             self._add_hdd_attribute_line()
-            self.computer.create_variant_ids()
             computer_ssd_256 = self._get_product_template_attribute_value(self.ssd_256)
             computer_ram_8 = self._get_product_template_attribute_value(self.ram_8)
             computer_hdd_1 = self._get_product_template_attribute_value(self.hdd_1)
@@ -178,16 +177,15 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
         do_test(self)
 
         # CASE: add back the removed attribute and try everything again
-        # It will be the same attribute but the ptal and ptav will be different!
         self.computer_ssd_attribute_lines = self.env['product.template.attribute.line'].create({
             'product_tmpl_id': self.computer.id,
             'attribute_id': self.ssd_attribute.id,
             'value_ids': [(6, 0, [self.ssd_256.id, self.ssd_512.id])],
         })
-        self.computer.create_variant_ids()
 
         computer_ssd_256_after = self._get_product_template_attribute_value(self.ssd_256)
-        self.assertTrue(computer_ssd_256_after != computer_ssd_256_before)
+        self.assertEqual(computer_ssd_256_after, computer_ssd_256_before)
+        self.assertEqual(computer_ssd_256_after.attribute_line_id, computer_ssd_256_before.attribute_line_id)
         do_test(self)
 
     def test_02_get_combination_info(self):
@@ -215,8 +213,7 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
         res = self.computer._get_combination_info(self.env['product.template.attribute.value'], computer_variant.id)
         self.assertEqual(res['product_template_id'], self.computer.id)
         self.assertEqual(res['product_id'], computer_variant.id)
-        # the variant has the same name as the template
-        self.assertEqual(res['display_name'], "Super Computer")
+        self.assertEqual(res['display_name'], "Super Computer (256 GB, 8 GB, 1 To)")
         self.assertEqual(res['price'], 2222)
         self.assertEqual(res['list_price'], 2222)
 
@@ -235,13 +232,14 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
         self.computer_ssd_attribute_lines.unlink()
         self.ssd_attribute.create_variant = 'no_variant'
         self._add_ssd_attribute_line()
-        self.computer.create_variant_ids()
         computer_ssd_256 = self._get_product_template_attribute_value(self.ssd_256)
         computer_ram_8 = self._get_product_template_attribute_value(self.ram_8)
         computer_hdd_1 = self._get_product_template_attribute_value(self.hdd_1)
         combination = computer_ssd_256 + computer_ram_8 + computer_hdd_1
 
         computer_variant_new = self.computer._get_variant_for_combination(combination)
+        self.assertTrue(computer_variant_new)
+
         res = self.computer._get_combination_info(combination, add_qty=2, pricelist=pricelist)
         self.assertEqual(res['product_template_id'], self.computer.id)
         self.assertEqual(res['product_id'], computer_variant_new.id)
@@ -253,12 +251,14 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
         self.computer_hdd_attribute_lines.unlink()
         self.hdd_attribute.create_variant = 'dynamic'
         self._add_hdd_attribute_line()
-        self.computer.create_variant_ids()
         computer_ssd_256 = self._get_product_template_attribute_value(self.ssd_256)
         computer_ram_8 = self._get_product_template_attribute_value(self.ram_8)
         computer_hdd_1 = self._get_product_template_attribute_value(self.hdd_1)
         combination = computer_ssd_256 + computer_ram_8 + computer_hdd_1
 
+        computer_variant_new = self.computer._create_product_variant(combination)
+        self.assertTrue(computer_variant_new)
+
         res = self.computer._get_combination_info(combination, add_qty=2, pricelist=pricelist)
         self.assertEqual(res['product_template_id'], self.computer.id)
         self.assertEqual(res['product_id'], computer_variant_new.id)
@@ -267,9 +267,8 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
         self.assertEqual(res['list_price'], 2222 * currency_ratio)
 
         # CASE: dynamic combination, no variant existing
-        # Test invalidate_cache on product.template create_variant_ids
+        # Test invalidate_cache on product.template _create_variant_ids
         self._add_keyboard_attribute()
-        self.computer.create_variant_ids()
         combination += self._get_product_template_attribute_value(self.keyboard_excluded)
         res = self.computer._get_combination_info(combination, add_qty=2, pricelist=pricelist)
         self.assertEqual(res['product_template_id'], self.computer.id)
@@ -349,10 +348,8 @@ class TestSaleProductAttributeValueConfig(TestSaleProductAttributeValueSetup):
         work with dynamic. If the combination is possible, it should create it.
         If it's not possible, it should not create it."""
         self.computer_hdd_attribute_lines.unlink()
-        self.computer.create_variant_ids()
         self.hdd_attribute.create_variant = 'dynamic'
         self._add_hdd_attribute_line()
-        self.computer.create_variant_ids()
 
         computer_ssd_256 = self._get_product_template_attribute_value(self.ssd_256)
         computer_ram_8 = self._get_product_template_attribute_value(self.ram_8)
