@@ -3,4 +3,4 @@
 
 from . import payment_acquirer_onboarding_wizard
 from . import sale_make_invoice_advance
-from . import sale_product_configurator
\ No newline at end of file
+from . import sale_payment_link
