@@ -4,7 +4,7 @@
 {
     'name': 'Sales Expense',
     'version': '1.0',
-    'category': 'Sales',
+    'category': 'Sales/Sales',
     'summary': 'Quotation, Sales Orders, Delivery & Invoicing Control',
     'description': """
 Reinvoice Employee Expense
@@ -16,9 +16,7 @@ This module allow to reinvoice employee expense, by setting the SO directly on t
     'depends': ['sale_management', 'hr_expense'],
     'data': [
         'data/digest_data.xml',
-        'security/ir.model.access.csv',
-        'security/sale_expense_security.xml',
-        'data/sale_expense_data.xml',
+        'views/assets.xml',
         'views/product_view.xml',
         'views/hr_expense_views.xml',
         'views/sale_order_views.xml',
