@@ -26,15 +26,16 @@ Preferences
         'views/sale_order_views.xml',
         'views/stock_views.xml',
         'views/res_config_settings_views.xml',
-        'views/account_invoice_views.xml',
         'views/sale_stock_portal_template.xml',
         'views/stock_production_lot_views.xml',
+        'views/report_invoice.xml',
         'report/sale_order_report_templates.xml',
         'report/stock_report_deliveryslip.xml',
         'data/sale_stock_data.xml',
         'wizard/stock_rules_report_views.xml',
     ],
     'demo': ['data/sale_order_demo.xml'],
+    'qweb': ['static/src/xml/qty_at_date.xml'],
     'installable': True,
     'auto_install': True,
 }
