@@ -24,18 +24,15 @@ have real delivered quantities in sales orders.
         'views/hr_timesheet_views.xml',
         'views/res_config_settings_views.xml',
         'views/hr_timesheet_templates.xml',
+        'views/sale_timesheet_portal_templates.xml',
         'report/project_profitability_report_analysis_views.xml',
         'data/sale_timesheet_filters.xml',
-        'wizard/project_task_assign_so_line_views.xml',
         'wizard/project_create_sale_order_views.xml',
         'wizard/project_create_invoice_views.xml',
     ],
     'demo': [
         'data/sale_service_demo.xml',
     ],
-    'qweb': [
-        'static/src/xml/timesheet_plan.xml',
-    ],
     'auto_install': True,
     'uninstall_hook': 'uninstall_hook',
 }
