@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
-# Part of Odoo. See LICENSE file for full copyright and licensing details.
+# # -*- coding: utf-8 -*-
+# # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import common
 from . import test_sale_timesheet
@@ -7,3 +7,4 @@ from . import test_sale_service
 from . import test_project_billing
 from . import test_reinvoice
 from . import test_reporting
+from . import test_project_billing_multicompany
