@@ -58,7 +58,7 @@ class TestSaleService(TestCommonSaleTimesheetNoChart):
             'employee_id': self.employee_manager.id,
         })
         self.assertEqual(self.sale_order.invoice_status, 'to invoice', 'Sale Service: there should be sale_ordermething to invoice after registering timesheets')
-        self.sale_order.action_invoice_create()
+        self.sale_order._create_invoices()
 
         self.assertTrue(sale_order_line.product_uom_qty == sale_order_line.qty_delivered == sale_order_line.qty_invoiced, 'Sale Service: line should be invoiced completely')
         self.assertEqual(self.sale_order.invoice_status, 'invoiced', 'Sale Service: SO should be invoiced')
@@ -126,7 +126,7 @@ class TestSaleService(TestCommonSaleTimesheetNoChart):
             'unit_amount': 24,
             'employee_id': self.employee_user.id,
         })
-        self.sale_order.action_invoice_create()
+        self.sale_order._create_invoices()
         self.assertEqual(self.sale_order.invoice_status, 'invoiced', 'Sale Timesheet: "invoice on delivery" timesheets should not modify the invoice_status of the so')
 
     def test_task_so_line_assignation(self):
@@ -168,9 +168,8 @@ class TestSaleService(TestCommonSaleTimesheetNoChart):
         self.assertEqual(task_serv2.timesheet_ids.mapped('so_line'), so_line_deliver_global_project, "Old timesheet are not modified when changing the task SO line")
 
         # invoice SO, and validate invoice
-        invoice_id = self.sale_order.action_invoice_create()[0]
-        invoice = self.env['account.invoice'].browse(invoice_id)
-        invoice.action_invoice_open()
+        invoice = self.sale_order._create_invoices()[0]
+        invoice.post()
 
         # try to update timesheets, catch error 'You cannot modify invoiced timesheet'
         with self.assertRaises(UserError):
@@ -363,8 +362,8 @@ class TestSaleService(TestCommonSaleTimesheetNoChart):
         # check each line has or no generate something
         self.assertTrue(so_line1.project_id, "Line1 should have create a project based on template A")
         self.assertTrue(so_line2.project_id, "Line2 should have create an empty project")
-        self.assertFalse(so_line3.project_id, "Line3 should not have create a project, since line1 already create a project based on template A")
-        self.assertFalse(so_line4.project_id, "Line4 should not have create a project, since line1 already create an empty project")
+        self.assertEqual(so_line3.project_id, so_line1.project_id, "Line3 should reuse project of line1")
+        self.assertEqual(so_line4.project_id, so_line2.project_id, "Line4 should reuse project of line2")
         self.assertTrue(so_line4.task_id, "Line4 should have create a new task, even if no project created.")
         self.assertTrue(so_line5.project_id, "Line5 should have create a project based on template B")
 
@@ -390,6 +389,106 @@ class TestSaleService(TestCommonSaleTimesheetNoChart):
         self.assertEqual(so_line2.project_id.sale_line_id, so_line2, "SO line of project should be the one that create it.")
         self.assertEqual(so_line5.project_id.sale_line_id, so_line5, "SO line of project with template B should be the one that create it.")
 
+    def test_sale_task_in_project_with_project(self):
+        """ This will test the new 'task_in_project' service tracking correctly creates tasks and projects
+            when a project_id is configured on the parent sale_order (ref task #1915660).
+
+            Setup:
+            - Configure a project_id on the SO
+            - SO line 1: a product with its delivery tracking set to 'task_in_project'
+            - SO line 2: the same product as SO line 1
+            - SO line 3: a product with its delivery tracking set to 'project_only'
+            - Confirm sale_order
+
+            Expected result:
+            - 2 tasks created on the project_id configured on the SO
+            - 1 project created with the correct template for the 'project_only' product
+        """
+
+        self.sale_order.write({'project_id': self.project_global.id})
+        self.sale_order._onchange_project_id()
+        self.assertEqual(self.sale_order.analytic_account_id, self.analytic_account_sale, "Changing the project on the SO should set the analytic account accordingly.")
+
+        so_line1 = self.env['sale.order.line'].create({
+            'name': self.product_order_timesheet3.name,
+            'product_id': self.product_order_timesheet3.id,
+            'product_uom_qty': 11,
+            'product_uom': self.product_order_timesheet3.uom_id.id,
+            'price_unit': self.product_order_timesheet3.list_price,
+            'order_id': self.sale_order.id,
+        })
+        so_line2 = self.env['sale.order.line'].create({
+            'name': self.product_order_timesheet3.name,
+            'product_id': self.product_order_timesheet3.id,
+            'product_uom_qty': 10,
+            'product_uom': self.product_order_timesheet3.uom_id.id,
+            'price_unit': self.product_order_timesheet3.list_price,
+            'order_id': self.sale_order.id,
+        })
+        so_line3 = self.env['sale.order.line'].create({
+            'name': self.product_order_timesheet4.name,
+            'product_id': self.product_order_timesheet4.id,
+            'product_uom_qty': 5,
+            'product_uom': self.product_order_timesheet4.uom_id.id,
+            'price_unit': self.product_order_timesheet4.list_price,
+            'order_id': self.sale_order.id,
+        })
+
+        # temporary project_template_id for our checks
+        self.product_order_timesheet4.write({
+            'project_template_id': self.project_template.id
+        })
+        self.sale_order.action_confirm()
+        # remove it after the confirm because other tests don't like it
+        self.product_order_timesheet4.write({
+            'project_template_id': False
+        })
+
+        self.assertTrue(so_line1.task_id, "so_line1 should create a task as its product's service_tracking is set as 'task_in_project'")
+        self.assertEqual(so_line1.task_id.project_id, self.project_global, "The project on so_line1's task should be project_global as configured on its parent sale_order")
+        self.assertTrue(so_line2.task_id, "so_line2 should create a task as its product's service_tracking is set as 'task_in_project'")
+        self.assertEqual(so_line2.task_id.project_id, self.project_global, "The project on so_line2's task should be project_global as configured on its parent sale_order")
+        self.assertFalse(so_line3.task_id.name, "so_line3 should not create a task as its product's service_tracking is set as 'project_only'")
+        self.assertNotEqual(so_line3.project_id, self.project_template, "so_line3 should create a new project and not directly use the configured template")
+        self.assertIn(self.project_template.name, so_line3.project_id.name, "The created project for so_line3 should use the configured template")
+
+    def test_sale_task_in_project_without_project(self):
+        """ This will test the new 'task_in_project' service tracking correctly creates tasks and projects
+            when the parent sale_order does NOT have a configured project_id (ref task #1915660).
+
+            Setup:
+            - SO line 1: a product with its delivery tracking set to 'task_in_project'
+            - Confirm sale_order
+
+            Expected result:
+            - 1 project created with the correct template for the 'task_in_project' because the SO
+              does not have a configured project_id
+            - 1 task created from this new project
+        """
+
+        so_line1 = self.env['sale.order.line'].create({
+            'name': self.product_order_timesheet3.name,
+            'product_id': self.product_order_timesheet3.id,
+            'product_uom_qty': 10,
+            'product_uom': self.product_order_timesheet3.uom_id.id,
+            'price_unit': self.product_order_timesheet3.list_price,
+            'order_id': self.sale_order.id,
+        })
+
+        # temporary project_template_id for our checks
+        self.product_order_timesheet3.write({
+            'project_template_id': self.project_template.id
+        })
+        self.sale_order.action_confirm()
+        # remove it after the confirm because other tests don't like it
+        self.product_order_timesheet3.write({
+            'project_template_id': False
+        })
+
+        self.assertTrue(so_line1.task_id, "so_line1 should create a task as its product's service_tracking is set as 'task_in_project'")
+        self.assertNotEqual(so_line1.project_id, self.project_template, "so_line1 should create a new project and not directly use the configured template")
+        self.assertIn(self.project_template.name, so_line1.project_id.name, "The created project for so_line1 should use the configured template")
+
     def test_billable_task_and_subtask(self):
         """ Test if subtasks and tasks are billed on the correct SO line """
         # create SO line and confirm it
