@@ -3,7 +3,7 @@
 {
     'name': 'Sales Teams',
     'version': '1.0',
-    'category': 'Sales',
+    'category': 'Sales/Sales',
     'summary': 'Sales Teams',
     'description': """
 Using this application you can manage Sales Teams  with CRM and/or Sales
@@ -15,6 +15,7 @@ Using this application you can manage Sales Teams  with CRM and/or Sales
              'security/ir.model.access.csv',
              'data/sales_team_data.xml',
              'views/crm_team_views.xml',
+             'views/res_partner_views.xml',
              'views/sales_team_dashboard.xml',
              ],
     'demo': ['data/sales_team_demo.xml'],
