@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import report_stock_forecast
+from . import report_stock_quantity
 from . import report_stock_rule
+from . import stock_traceability
