@@ -1,23 +1,28 @@
 # -*- coding: utf-8 -*-
 
 from odoo.addons.stock.tests.common2 import TestStockCommon
+from odoo.tests import Form
+from odoo.exceptions import AccessError
+from odoo.tools import mute_logger
 
 
 class TestWarehouse(TestStockCommon):
 
+    def setUp(self):
+        super(TestWarehouse, self).setUp()
+        self.partner = self.env['res.partner'].create({'name': 'Deco Addict'})
+
     def test_inventory_product(self):
         self.product_1.type = 'product'
-        inventory_wizard = self.env['stock.change.product.qty'].create({
+        product_1_quant = self.env['stock.quant'].with_context(inventory_mode=True).create({
             'product_id': self.product_1.id,
-            'new_quantity': 50.0,
+            'inventory_quantity': 50.0,
             'location_id': self.warehouse_1.lot_stock_id.id,
         })
-        inventory_wizard.change_product_qty()
-        inventory = self.env['stock.inventory'].sudo(self.user_stock_manager).create({
+        inventory = self.env['stock.inventory'].with_user(self.user_stock_manager).create({
             'name': 'Starting for product_1',
-            'filter': 'product',
-            'location_id': self.warehouse_1.lot_stock_id.id,
-            'product_id': self.product_1.id,
+            'location_ids': [(4, self.warehouse_1.lot_stock_id.id)],
+            'product_ids': [(4, self.product_1.id)],
         })
         inventory.action_start()
         # As done in common.py, there is already an inventory line existing
@@ -34,9 +39,9 @@ class TestWarehouse(TestStockCommon):
         self.assertIn(inventory.name, inventory.move_ids.name)
         self.assertEqual(inventory.move_ids.product_qty, 15.0)
         self.assertEqual(inventory.move_ids.location_id, self.warehouse_1.lot_stock_id)
-        self.assertEqual(inventory.move_ids.location_dest_id, self.env.ref('stock.location_inventory'))  # Inventory loss
+        self.assertEqual(inventory.move_ids.location_dest_id, self.product_1.property_stock_inventory)  # Inventory loss
         self.assertEqual(inventory.move_ids.state, 'done')
-        quants = self.env['stock.quant']._gather(self.product_1, self.env.ref('stock.location_inventory'))
+        quants = self.env['stock.quant']._gather(self.product_1, self.product_1.property_stock_inventory)
         self.assertEqual(len(quants), 1)  # One quant created for inventory loss
 
         # Check quantity of product in various locations: current, its parent, brother and other
@@ -47,37 +52,51 @@ class TestWarehouse(TestStockCommon):
         self.assertEqual(self.env['stock.quant']._gather(self.product_1, self.warehouse_1.wh_input_stock_loc_id).quantity, 0.0)
         self.assertEqual(self.env['stock.quant']._gather(self.product_1, self.env.ref('stock.stock_location_stock')).quantity, 0.0)
 
-    def test_inventory_wizard(self):
+    def test_inventory_wizard_as_manager(self):
+        """ Using the "Update Quantity" wizard as stock manager.
+        """
         self.product_1.type = 'product'
-        inventory_wizard = self.env['stock.change.product.qty'].create({
+        InventoryWizard = self.env['stock.change.product.qty'].with_user(self.user_stock_manager)
+        inventory_wizard = InventoryWizard.create({
             'product_id': self.product_1.id,
+            'product_tmpl_id': self.product_1.product_tmpl_id.id,
             'new_quantity': 50.0,
-            'location_id': self.warehouse_1.lot_stock_id.id,
         })
         inventory_wizard.change_product_qty()
-        # Check inventory performed in setup was effectivley performed
+        # Check quantity was updated
         self.assertEqual(self.product_1.virtual_available, 50.0)
         self.assertEqual(self.product_1.qty_available, 50.0)
 
-        # Check inventory obj details (1 inventory with 1 line, because 1 product change)
-        inventory = self.env['stock.inventory'].search([('id', 'not in', self.existing_inventories.ids)])
-        self.assertEqual(len(inventory), 1)
-        self.assertIn('INV: %s' % self.product_1.display_name, inventory.name)
-        self.assertEqual(len(inventory.line_ids), 1)
-        self.assertEqual(inventory.line_ids.product_id, self.product_1)
-        self.assertEqual(inventory.line_ids.product_qty, 50.0)
-
         # Check associated quants: 2 quants for the product and the quantity (1 in stock, 1 in inventory adjustment)
         quant = self.env['stock.quant'].search([('id', 'not in', self.existing_quants.ids)])
         self.assertEqual(len(quant), 2)
-        # print quant.name, quant.product_id, quant.location_id
-        # TDE TODO: expand this test
+
+    def test_inventory_wizard_as_user(self):
+        """ Using the "Update Quantity" wizard as stock user.
+        """
+        self.product_1.type = 'product'
+        InventoryWizard = self.env['stock.change.product.qty'].with_user(self.user_stock_user)
+        inventory_wizard = InventoryWizard.create({
+            'product_id': self.product_1.id,
+            'product_tmpl_id': self.product_1.product_tmpl_id.id,
+            'new_quantity': 50.0,
+        })
+        # User has no right on quant, must raise an AccessError
+        with self.assertRaises(AccessError):
+            inventory_wizard.change_product_qty()
+        # Check quantity wasn't updated
+        self.assertEqual(self.product_1.virtual_available, 0.0)
+        self.assertEqual(self.product_1.qty_available, 0.0)
+
+        # Check associated quants: 0 quant expected
+        quant = self.env['stock.quant'].search([('id', 'not in', self.existing_quants.ids)])
+        self.assertEqual(len(quant), 0)
 
     def test_basic_move(self):
-        product = self.product_3.sudo(self.user_stock_manager)
+        product = self.product_3.with_user(self.user_stock_manager)
         product.type = 'product'
         picking_out = self.env['stock.picking'].create({
-            'partner_id': self.env.ref('base.res_partner_2').id,
+            'partner_id': self.partner.id,
             'picking_type_id': self.env.ref('stock.picking_type_out').id,
             'location_id': self.warehouse_1.lot_stock_id.id,
             'location_dest_id': self.env.ref('stock.stock_location_customers').id,
@@ -138,7 +157,7 @@ class TestWarehouse(TestStockCommon):
 
         # Create a picking out and force availability
         picking_out = self.env['stock.picking'].create({
-            'partner_id': self.env.ref('base.res_partner_2').id,
+            'partner_id': self.partner.id,
             'picking_type_id': self.env.ref('stock.picking_type_out').id,
             'location_id': stock_location.id,
             'location_dest_id': customer_location.id,
@@ -158,9 +177,10 @@ class TestWarehouse(TestStockCommon):
 
         quant = self.env['stock.quant'].search([('product_id', '=', productA.id), ('location_id', '=', stock_location.id)])
         self.assertEqual(len(quant), 1)
-        stock_return_picking = self.env['stock.return.picking']\
-            .with_context(active_ids=picking_out.ids, active_id=picking_out.ids[0])\
-            .create({})
+        stock_return_picking_form = Form(self.env['stock.return.picking']
+            .with_context(active_ids=picking_out.ids, active_id=picking_out.ids[0],
+            active_model='stock.picking'))
+        stock_return_picking = stock_return_picking_form.save()
         stock_return_picking.product_return_moves.quantity = 1.0
         stock_return_picking_action = stock_return_picking.create_returns()
         return_pick = self.env['stock.picking'].browse(stock_return_picking_action['res_id'])
@@ -176,11 +196,11 @@ class TestWarehouse(TestStockCommon):
         productA = self.env['product.product'].create({'name': 'Product A', 'type': 'product'})
         stock_location = self.env.ref('stock.stock_location_stock')
         customer_location = self.env.ref('stock.stock_location_customers')
-        location_loss = self.env.ref('stock.location_inventory')
+        location_loss = productA.property_stock_inventory
 
         # Create a picking out and force availability
         picking_out = self.env['stock.picking'].create({
-            'partner_id': self.env.ref('base.res_partner_2').id,
+            'partner_id': self.partner.id,
             'picking_type_id': self.env.ref('stock.picking_type_out').id,
             'location_id': stock_location.id,
             'location_dest_id': customer_location.id,
@@ -201,9 +221,8 @@ class TestWarehouse(TestStockCommon):
         # Make an inventory adjustment to set the quantity to 0
         inventory = self.env['stock.inventory'].create({
             'name': 'Starting for product_1',
-            'filter': 'product',
-            'location_id': stock_location.id,
-            'product_id': productA.id,
+            'location_ids': [(4, stock_location.id)],
+            'product_ids': [(4, productA.id)],
         })
         inventory.action_start()
         self.assertEqual(len(inventory.line_ids), 1, "Wrong inventory lines generated.")
@@ -266,7 +285,7 @@ class TestWarehouse(TestStockCommon):
         })
 
         picking_out = self.env['stock.picking'].create({
-            'partner_id': self.env.ref('base.res_partner_2').id,
+            'partner_id': self.partner.id,
             'picking_type_id': self.env.ref('stock.picking_type_out').id,
             'location_id': warehouse_shop.lot_stock_id.id,
             'location_dest_id': self.env.ref('stock.stock_location_customers').id,
@@ -346,7 +365,7 @@ class TestWarehouse(TestStockCommon):
         # Create the move for the shop Namur. Should create a resupply from
         # distribution warehouse Namur.
         picking_out_namur = self.env['stock.picking'].create({
-            'partner_id': self.env.ref('base.res_partner_2').id,
+            'partner_id': self.partner.id,
             'picking_type_id': self.env.ref('stock.picking_type_out').id,
             'location_id': warehouse_shop_namur.lot_stock_id.id,
             'location_dest_id': customer_location.id,
@@ -390,7 +409,7 @@ class TestWarehouse(TestStockCommon):
         # Create the move for the shop Wavre. Should create a resupply from
         # distribution warehouse Wavre.
         picking_out_wavre = self.env['stock.picking'].create({
-            'partner_id': self.env.ref('base.res_partner_2').id,
+            'partner_id': self.partner.id,
             'picking_type_id': self.env.ref('stock.picking_type_out').id,
             'location_id': warehouse_shop_wavre.lot_stock_id.id,
             'location_dest_id': customer_location.id,
@@ -431,22 +450,157 @@ class TestWarehouse(TestStockCommon):
         # Ensure there still no quants in distribution warehouse
         self.assertEqual(sum(self.env['stock.quant']._gather(product, warehouse_distribution_wavre.lot_stock_id).mapped('quantity')), 0)
 
-class TestResupply(TestStockCommon):
-    def setUp(self):
-        super(TestResupply, self).setUp()
+    def test_noleak(self):
+        # non-regression test to avoid company_id leaking to other warehouses (see blame)
+        wh = self.env["stock.warehouse"].search([])
 
-        self.warehouse_2 = self.env['stock.warehouse'].create({
-            'name': 'Small Warehouse',
-            'code': 'SWH',
-            'resupply_wh_ids': [(6, 0, [self.warehouse_1.id])]
+        assert len(set(wh.mapped("company_id.id"))) > 1
+
+        companies_before = wh.mapped(lambda w: (w.id, w.company_id))
+        # writing on any field should change the company of warehouses
+        wh.write({"name": "whatever"})
+        companies_after = wh.mapped(lambda w: (w.id, w.company_id))
+
+        self.assertEqual(companies_after, companies_before)
+
+    def test_toggle_active_warehouse_1(self):
+        """ Basic test that create a warehouse with classic configuration.
+        Archive it and check that locations, picking types, routes, rules are
+        correclty active or archive.
+        """
+        wh = Form(self.env['stock.warehouse'])
+        wh.name = "The attic of Willy"
+        wh.code = "WIL"
+        warehouse = wh.save()
+
+        custom_location = Form(self.env['stock.location'])
+        custom_location.name = "A Trunk"
+        custom_location.location_id = warehouse.lot_stock_id
+        custom_location = custom_location.save()
+
+        # Archive warehouse
+        warehouse.toggle_active()
+        # Global rule
+        self.assertFalse(warehouse.mto_pull_id.active)
+
+        # Route
+        self.assertFalse(warehouse.reception_route_id.active)
+        self.assertFalse(warehouse.delivery_route_id.active)
+
+        # Location
+        self.assertFalse(warehouse.lot_stock_id.active)
+        self.assertFalse(warehouse.wh_input_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_qc_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_output_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_pack_stock_loc_id.active)
+        self.assertFalse(custom_location.active)
+
+        # Picking Type
+        self.assertFalse(warehouse.in_type_id.active)
+        self.assertFalse(warehouse.out_type_id.active)
+        self.assertFalse(warehouse.int_type_id.active)
+        self.assertFalse(warehouse.pick_type_id.active)
+        self.assertFalse(warehouse.pack_type_id.active)
+
+        # Active warehouse
+        warehouse.toggle_active()
+        # Global rule
+        self.assertTrue(warehouse.mto_pull_id.active)
+
+        # Route
+        self.assertTrue(warehouse.reception_route_id.active)
+        self.assertTrue(warehouse.delivery_route_id.active)
+
+        # Location
+        self.assertTrue(warehouse.lot_stock_id.active)
+        self.assertFalse(warehouse.wh_input_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_qc_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_output_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_pack_stock_loc_id.active)
+        self.assertTrue(custom_location.active)
+
+        # Picking Type
+        self.assertTrue(warehouse.in_type_id.active)
+        self.assertTrue(warehouse.out_type_id.active)
+        self.assertTrue(warehouse.int_type_id.active)
+        self.assertFalse(warehouse.pick_type_id.active)
+        self.assertFalse(warehouse.pack_type_id.active)
+
+    def test_toggle_active_warehouse_2(self):
+        wh = Form(self.env['stock.warehouse'])
+        wh.name = "The attic of Willy"
+        wh.code = "WIL"
+        wh.reception_steps = "two_steps"
+        wh.delivery_steps = "pick_pack_ship"
+        warehouse = wh.save()
+
+        warehouse.resupply_wh_ids = [(6, 0, [self.warehouse_1.id])]
+
+        custom_location = Form(self.env['stock.location'])
+        custom_location.name = "A Trunk"
+        custom_location.location_id = warehouse.lot_stock_id
+        custom_location = custom_location.save()
+
+        # Add a warehouse on the route.
+        warehouse.reception_route_id.write({
+            'warehouse_ids': [(4, self.warehouse_1.id)]
         })
 
-        # minimum stock rule for test product on this warehouse
-        self.env['stock.warehouse.orderpoint'].create({
-            'warehouse_id': self.warehouse_2.id,
-            'location_id': self.warehouse_2.lot_stock_id.id,
-            'product_id': self.product_1.id,
-            'product_min_qty': 10,
-            'product_max_qty': 100,
-            'product_uom': self.uom_unit.id,
-        })
+        route = Form(self.env['stock.location.route'])
+        route.name = "Stair"
+        route = route.save()
+
+        route.warehouse_ids = [(6, 0, [warehouse.id, self.warehouse_1.id])]
+
+        # Pre archive a location and a route
+        warehouse.delivery_route_id.toggle_active()
+        warehouse.wh_pack_stock_loc_id.toggle_active()
+
+        # Archive warehouse
+        warehouse.toggle_active()
+        # Global rule
+        self.assertFalse(warehouse.mto_pull_id.active)
+
+        # Route
+        self.assertTrue(warehouse.reception_route_id.active)
+        self.assertFalse(warehouse.delivery_route_id.active)
+        self.assertTrue(route.active)
+
+        # Location
+        self.assertFalse(warehouse.lot_stock_id.active)
+        self.assertFalse(warehouse.wh_input_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_qc_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_output_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_pack_stock_loc_id.active)
+        self.assertFalse(custom_location.active)
+
+        # Picking Type
+        self.assertFalse(warehouse.in_type_id.active)
+        self.assertFalse(warehouse.out_type_id.active)
+        self.assertFalse(warehouse.int_type_id.active)
+        self.assertFalse(warehouse.pick_type_id.active)
+        self.assertFalse(warehouse.pack_type_id.active)
+
+        # Active warehouse
+        warehouse.toggle_active()
+        # Global rule
+        self.assertTrue(warehouse.mto_pull_id.active)
+
+        # Route
+        self.assertTrue(warehouse.reception_route_id.active)
+        self.assertTrue(warehouse.delivery_route_id.active)
+
+        # Location
+        self.assertTrue(warehouse.lot_stock_id.active)
+        self.assertTrue(warehouse.wh_input_stock_loc_id.active)
+        self.assertFalse(warehouse.wh_qc_stock_loc_id.active)
+        self.assertTrue(warehouse.wh_output_stock_loc_id.active)
+        self.assertTrue(warehouse.wh_pack_stock_loc_id.active)
+        self.assertTrue(custom_location.active)
+
+        # Picking Type
+        self.assertTrue(warehouse.in_type_id.active)
+        self.assertTrue(warehouse.out_type_id.active)
+        self.assertTrue(warehouse.int_type_id.active)
+        self.assertTrue(warehouse.pick_type_id.active)
+        self.assertTrue(warehouse.pack_type_id.active)
