@@ -13,13 +13,13 @@ class StockOverProcessedTransfer(models.TransientModel):
     overprocessed_product_name = fields.Char(compute='_compute_overprocessed_product_name',
                                              readonly=True)
 
-    @api.multi
     def _compute_overprocessed_product_name(self):
         for wizard in self:
-            moves = wizard.picking_id._get_overprocessed_stock_moves()
-            wizard.overprocessed_product_name = moves[0].product_id.display_name
+            wizard.overprocessed_product_name = False
+            if wizard.picking_id:
+                moves = wizard.picking_id._get_overprocessed_stock_moves()
+                wizard.overprocessed_product_name = moves[0].product_id.display_name
 
-    @api.multi
     def action_confirm(self):
         self.ensure_one()
         return self.picking_id.with_context(skip_overprocessed_check=True).button_validate()
