@@ -11,9 +11,9 @@ class StockTrackConfirmation(models.TransientModel):
     tracking_line_ids = fields.One2many('stock.track.line', 'wizard_id')
     inventory_id = fields.Many2one('stock.inventory', 'Inventory')
 
-    @api.one
     def action_confirm(self):
-        return self.inventory_id._action_done()
+        for confirmation in self:
+            confirmation.inventory_id._action_done()
 
 class StockTrackingLines(models.TransientModel):
     _name = 'stock.track.line'
