@@ -31,6 +31,8 @@ Dashboard / Reports for Warehouse Management includes:
         'views/res_config_settings_views.xml',
         'data/product_data.xml',
         'views/product_views.xml',
+        'views/stock_quant_views.xml',
+        'views/stock_valuation_layer_views.xml',
     ],
     'test': [
     ],
