@@ -8,7 +8,7 @@ class AccountReconciliation(models.AbstractModel):
     _inherit = "account.reconciliation.widget"
 
     @api.model
-    def _domain_move_lines_for_reconciliation(self, st_line, aml_accounts, partner_id, excluded_ids=None, search_str=False):
+    def _domain_move_lines_for_reconciliation(self, st_line, aml_accounts, partner_id, excluded_ids=None, search_str=False, mode='rp'):
         def to_int(val):
             try:
                 return int(val)
@@ -16,7 +16,7 @@ class AccountReconciliation(models.AbstractModel):
                 return None
 
         domain = super()._domain_move_lines_for_reconciliation(
-            st_line, aml_accounts, partner_id, excluded_ids=excluded_ids, search_str=search_str
+            st_line, aml_accounts, partner_id, excluded_ids=excluded_ids, search_str=search_str, mode=mode
         )
         acc_props = (
             "property_stock_account_input",
