@@ -4,7 +4,7 @@
 {
     'name': 'Drop Shipping',
     'version': '1.0',
-    'category': 'Warehouse',
+    'category': 'Operations/Inventory',
     'summary': 'Drop Shipping',
     'description': """
 Manage drop shipping orders
@@ -20,8 +20,8 @@ going through the retailer's warehouse. In this case no
 internal transfer document is needed.
 
 """,
-    'depends': ['sale_purchase', 'sale_stock'],
-    'data': ['data/stock_data.xml'],
+    'depends': ['sale_purchase', 'sale_stock', 'purchase_stock'],
+    'data': ['data/stock_data.xml', 'views/sale_order_views.xml'],
     'installable': True,
     'auto_install': False,
 }
