@@ -4,31 +4,48 @@
 from collections import defaultdict
 
 from odoo import api, fields, models, tools, _
-from odoo.addons import decimal_precision as dp
-from odoo.addons.stock_landed_costs.models import product
 from odoo.exceptions import UserError
+from odoo.tools.float_utils import float_is_zero
 
 
-class StockMove(models.Model):
-    _inherit = 'stock.move'
-
-    landed_cost_value = fields.Float('Landed Cost')
+SPLIT_METHOD = [
+    ('equal', 'Equal'),
+    ('by_quantity', 'By Quantity'),
+    ('by_current_cost_price', 'By Current Cost'),
+    ('by_weight', 'By Weight'),
+    ('by_volume', 'By Volume'),
+]
 
 
 class LandedCost(models.Model):
     _name = 'stock.landed.cost'
     _description = 'Stock Landed Cost'
-    _inherit = 'mail.thread'
+    _inherit = ['mail.thread', 'mail.activity.mixin']
+
+    def _default_account_journal_id(self):
+        """Take the journal configured in the company, else fallback on the stock journal."""
+        lc_journal = self.env['account.journal']
+        if self.env.company.lc_journal_id:
+            lc_journal = self.env.company.lc_journal_id
+        else:
+            ir_property = self.env['ir.property'].search([
+                ('name', '=', 'property_stock_journal'),
+                ('company_id', '=', self.env.company.id)
+            ], limit=1)
+            if ir_property:
+                lc_journal = ir_property.get_by_record()
+        return lc_journal
 
     name = fields.Char(
         'Name', default=lambda self: _('New'),
-        copy=False, readonly=True, track_visibility='always')
+        copy=False, readonly=True, tracking=True)
     date = fields.Date(
         'Date', default=fields.Date.context_today,
-        copy=False, required=True, states={'done': [('readonly', True)]}, track_visibility='onchange')
+        copy=False, required=True, states={'done': [('readonly', True)]}, tracking=True)
     picking_ids = fields.Many2many(
         'stock.picking', string='Transfers',
         copy=False, states={'done': [('readonly', True)]})
+    allowed_picking_ids = fields.Many2many('stock.picking', compute='_compute_allowed_picking_ids')
     cost_lines = fields.One2many(
         'stock.landed.cost.lines', 'cost_id', 'Cost Lines',
         copy=True, states={'done': [('readonly', True)]})
@@ -39,25 +56,41 @@ class LandedCost(models.Model):
         'Item Description', states={'done': [('readonly', True)]})
     amount_total = fields.Float(
         'Total', compute='_compute_total_amount',
-        digits=0, store=True, track_visibility='always')
+        digits=0, store=True, tracking=True)
     state = fields.Selection([
         ('draft', 'Draft'),
         ('done', 'Posted'),
         ('cancel', 'Cancelled')], 'State', default='draft',
-        copy=False, readonly=True, track_visibility='onchange')
+        copy=False, readonly=True, tracking=True)
     account_move_id = fields.Many2one(
         'account.move', 'Journal Entry',
         copy=False, readonly=True)
     account_journal_id = fields.Many2one(
         'account.journal', 'Account Journal',
-        required=True, states={'done': [('readonly', True)]})
+        required=True, states={'done': [('readonly', True)]}, default=lambda self: self._default_account_journal_id())
     company_id = fields.Many2one('res.company', string="Company",
-        related='account_journal_id.company_id', readonly=False)
+        related='account_journal_id.company_id')
+    stock_valuation_layer_ids = fields.One2many('stock.valuation.layer', 'stock_landed_cost_id')
+    vendor_bill_id = fields.Many2one(
+        'account.move', 'Vendor Bill', copy=False, domain=[('type', '=', 'in_invoice')])
+    currency_id = fields.Many2one('res.currency', related='company_id.currency_id')
 
-    @api.one
     @api.depends('cost_lines.price_unit')
     def _compute_total_amount(self):
-        self.amount_total = sum(line.price_unit for line in self.cost_lines)
+        for cost in self:
+            cost.amount_total = sum(line.price_unit for line in cost.cost_lines)
+
+    @api.depends('company_id')
+    def _compute_allowed_picking_ids(self):
+        self.env.cr.execute("""SELECT sm.picking_id, sm.company_id
+                                 FROM stock_move AS sm
+                           INNER JOIN stock_valuation_layer AS svl ON svl.stock_move_id = sm.id
+                                WHERE sm.picking_id IS NOT NULL""")
+        valued_picking_ids_per_company = defaultdict(list)
+        for res in self.env.cr.fetchall():
+            valued_picking_ids_per_company[res[1]].append(res[0])
+        for cost in self:
+            cost.allowed_picking_ids = valued_picking_ids_per_company[cost.company_id.id]
 
     @api.model
     def create(self, vals):
@@ -65,30 +98,29 @@ class LandedCost(models.Model):
             vals['name'] = self.env['ir.sequence'].next_by_code('stock.landed.cost')
         return super(LandedCost, self).create(vals)
 
-    @api.multi
     def unlink(self):
         self.button_cancel()
         return super(LandedCost, self).unlink()
 
-    @api.multi
     def _track_subtype(self, init_values):
         if 'state' in init_values and self.state == 'done':
-            return 'stock_landed_costs.mt_stock_landed_cost_open'
+            return self.env.ref('stock_landed_costs.mt_stock_landed_cost_open')
         return super(LandedCost, self)._track_subtype(init_values)
 
-    @api.multi
     def button_cancel(self):
         if any(cost.state == 'done' for cost in self):
             raise UserError(
                 _('Validated landed costs cannot be cancelled, but you could create negative landed costs to reverse them'))
         return self.write({'state': 'cancel'})
 
-    @api.multi
     def button_validate(self):
         if any(cost.state != 'draft' for cost in self):
             raise UserError(_('Only draft landed costs can be validated'))
-        if any(not cost.valuation_adjustment_lines for cost in self):
-            raise UserError(_('No valuation adjustments lines. You should maybe recompute the landed costs.'))
+        if not all(cost.picking_ids for cost in self):
+            raise UserError(_('Please define the transfers on which those additional costs should apply.'))
+        cost_without_adjusment_lines = self.filtered(lambda c: not c.valuation_adjustment_lines)
+        if cost_without_adjusment_lines:
+            cost_without_adjusment_lines.compute_landed_cost()
         if not self._check_sum():
             raise UserError(_('Cost and adjustments lines do not match. You should maybe recompute the landed costs.'))
 
@@ -99,45 +131,69 @@ class LandedCost(models.Model):
                 'date': cost.date,
                 'ref': cost.name,
                 'line_ids': [],
+                'type': 'entry',
             }
+            valuation_layer_ids = []
             for line in cost.valuation_adjustment_lines.filtered(lambda line: line.move_id):
-                # Prorate the value at what's still in stock
-                cost_to_add = (line.move_id.remaining_qty / line.move_id.product_qty) * line.additional_landed_cost
+                remaining_qty = sum(line.move_id.stock_valuation_layer_ids.mapped('remaining_qty'))
+                linked_layer = line.move_id.stock_valuation_layer_ids[:1]
 
-                new_landed_cost_value = line.move_id.landed_cost_value + line.additional_landed_cost
-                line.move_id.write({
-                    'landed_cost_value': new_landed_cost_value,
-                    'value': line.move_id.value + line.additional_landed_cost,
-                    'remaining_value': line.move_id.remaining_value + cost_to_add,
-                    'price_unit': (line.move_id.value + line.additional_landed_cost) / line.move_id.product_qty,
-                })
+                # Prorate the value at what's still in stock
+                cost_to_add = (remaining_qty / line.move_id.product_qty) * line.additional_landed_cost
+                if not cost.company_id.currency_id.is_zero(cost_to_add):
+                    valuation_layer = self.env['stock.valuation.layer'].create({
+                        'value': cost_to_add,
+                        'unit_cost': 0,
+                        'quantity': 0,
+                        'remaining_qty': 0,
+                        'stock_valuation_layer_id': linked_layer.id,
+                        'description': cost.name,
+                        'stock_move_id': line.move_id.id,
+                        'product_id': line.move_id.product_id.id,
+                        'stock_landed_cost_id': cost.id,
+                        'company_id': cost.company_id.id,
+                    })
+                    linked_layer.remaining_value += cost_to_add
+                    valuation_layer_ids.append(valuation_layer.id)
+                # Update the AVCO
+                product = line.move_id.product_id
+                if product.cost_method == 'average' and not float_is_zero(product.quantity_svl, precision_rounding=product.uom_id.rounding):
+                    product.with_context(force_company=self.company_id.id).sudo().standard_price += cost_to_add / product.quantity_svl
                 # `remaining_qty` is negative if the move is out and delivered proudcts that were not
                 # in stock.
                 qty_out = 0
                 if line.move_id._is_in():
-                    qty_out = line.move_id.product_qty - line.move_id.remaining_qty
+                    qty_out = line.move_id.product_qty - remaining_qty
                 elif line.move_id._is_out():
                     qty_out = line.move_id.product_qty
                 move_vals['line_ids'] += line._create_accounting_entries(move, qty_out)
 
+            move_vals['stock_valuation_layer_ids'] = [(6, None, valuation_layer_ids)]
             move = move.create(move_vals)
             cost.write({'state': 'done', 'account_move_id': move.id})
             move.post()
+
+            if cost.vendor_bill_id and cost.vendor_bill_id.state == 'posted' and cost.company_id.anglo_saxon_accounting:
+                all_amls = cost.vendor_bill_id.line_ids | cost.account_move_id.line_ids
+                for product in cost.cost_lines.product_id:
+                    accounts = product.product_tmpl_id.get_product_accounts()
+                    input_account = accounts['stock_input']
+                    all_amls.filtered(lambda aml: aml.account_id == input_account and not aml.full_reconcile_id).reconcile()
         return True
 
     def _check_sum(self):
         """ Check if each cost line its valuation lines sum to the correct amount
         and if the overall total amount is correct also """
-        prec_digits = self.env.user.company_id.currency_id.decimal_places
+        prec_digits = self.env.company.currency_id.decimal_places
         for landed_cost in self:
             total_amount = sum(landed_cost.valuation_adjustment_lines.mapped('additional_landed_cost'))
-            if not tools.float_compare(total_amount, landed_cost.amount_total, precision_digits=prec_digits) == 0:
+            if not tools.float_is_zero(total_amount - landed_cost.amount_total, precision_digits=prec_digits):
                 return False
 
             val_to_cost_lines = defaultdict(lambda: 0.0)
             for val_line in landed_cost.valuation_adjustment_lines:
                 val_to_cost_lines[val_line.cost_line_id] += val_line.additional_landed_cost
-            if any(tools.float_compare(cost_line.price_unit, val_amount, precision_digits=prec_digits) != 0
+            if any(not tools.float_is_zero(cost_line.price_unit - val_amount, precision_digits=prec_digits)
                    for cost_line, val_amount in val_to_cost_lines.items()):
                 return False
         return True
@@ -147,28 +203,27 @@ class LandedCost(models.Model):
 
         for move in self.mapped('picking_ids').mapped('move_lines'):
             # it doesn't make sense to make a landed cost for a product that isn't set as being valuated in real time at real cost
-            if move.product_id.valuation != 'real_time' or move.product_id.cost_method != 'fifo':
+            if move.product_id.valuation != 'real_time' or move.product_id.cost_method not in ('fifo', 'average') or move.state == 'cancel':
                 continue
             vals = {
                 'product_id': move.product_id.id,
                 'move_id': move.id,
                 'quantity': move.product_qty,
-                'former_cost': move.value,
+                'former_cost': sum(move.stock_valuation_layer_ids.mapped('value')),
                 'weight': move.product_id.weight * move.product_qty,
                 'volume': move.product_id.volume * move.product_qty
             }
             lines.append(vals)
 
         if not lines and self.mapped('picking_ids'):
-            raise UserError(_("You cannot apply landed costs on the chosen transfer(s). Landed costs can only be applied for products with automated inventory valuation and FIFO costing method."))
+            raise UserError(_("You cannot apply landed costs on the chosen transfer(s). Landed costs can only be applied for products with automated inventory valuation and FIFO or average costing method."))
         return lines
 
-    @api.multi
     def compute_landed_cost(self):
         AdjustementLines = self.env['stock.valuation.adjustment.lines']
         AdjustementLines.search([('cost_id', 'in', self.ids)]).unlink()
 
-        digits = dp.get_precision('Product Price')(self._cr)
+        digits = self.env['decimal.precision'].precision_get('Product Price')
         towrite_dict = {}
         for cost in self.filtered(lambda cost: cost.picking_ids):
             total_qty = 0.0
@@ -187,7 +242,7 @@ class LandedCost(models.Model):
 
                 former_cost = val_line_values.get('former_cost', 0.0)
                 # round this because former_cost on the valuation lines is also rounded
-                total_cost += tools.float_round(former_cost, precision_digits=digits[1]) if digits else former_cost
+                total_cost += tools.float_round(former_cost, precision_digits=digits) if digits else former_cost
 
                 total_line += 1
 
@@ -214,7 +269,7 @@ class LandedCost(models.Model):
                             value = (line.price_unit / total_line)
 
                         if digits:
-                            value = tools.float_round(value, precision_digits=digits[1], rounding_method='UP')
+                            value = tools.float_round(value, precision_digits=digits, rounding_method='UP')
                             fnc = min if line.price_unit > 0 else max
                             value = fnc(value, line.price_unit - value_split)
                             value_split += value
@@ -227,6 +282,12 @@ class LandedCost(models.Model):
             AdjustementLines.browse(key).write({'additional_landed_cost': value})
         return True
 
+    def action_view_stock_valuation_layers(self):
+        self.ensure_one()
+        domain = [('id', 'in', self.stock_valuation_layer_ids.ids)]
+        action = self.env.ref('stock_account.stock_valuation_layer_action').read()[0]
+        return dict(action, domain=domain)
+
 
 class LandedCostLine(models.Model):
     _name = 'stock.landed.cost.lines'
@@ -237,8 +298,8 @@ class LandedCostLine(models.Model):
         'stock.landed.cost', 'Landed Cost',
         required=True, ondelete='cascade')
     product_id = fields.Many2one('product.product', 'Product', required=True)
-    price_unit = fields.Float('Cost', digits=dp.get_precision('Product Price'), required=True)
-    split_method = fields.Selection(product.SPLIT_METHOD, string='Split Method', required=True)
+    price_unit = fields.Float('Cost', digits='Product Price', required=True)
+    split_method = fields.Selection(SPLIT_METHOD, string='Split Method', required=True)
     account_id = fields.Many2one('account.account', 'Account', domain=[('deprecated', '=', False)])
 
     @api.onchange('product_id')
@@ -246,9 +307,10 @@ class LandedCostLine(models.Model):
         if not self.product_id:
             self.quantity = 0.0
         self.name = self.product_id.name or ''
-        self.split_method = self.product_id.split_method or 'equal'
+        self.split_method = self.split_method or 'equal'
         self.price_unit = self.product_id.standard_price or 0.0
-        self.account_id = self.product_id.property_account_expense_id.id or self.product_id.categ_id.property_account_expense_categ_id.id
+        accounts_data = self.product_id.product_tmpl_id.get_product_accounts()
+        self.account_id = accounts_data['stock_input']
 
 
 class AdjustmentLines(models.Model):
@@ -269,36 +331,29 @@ class AdjustmentLines(models.Model):
         digits=0, required=True)
     weight = fields.Float(
         'Weight', default=1.0,
-        digits=dp.get_precision('Stock Weight'))
+        digits='Stock Weight')
     volume = fields.Float(
-        'Volume', default=1.0)
+        'Volume', default=1.0, digits='Volume')
     former_cost = fields.Float(
-        'Former Cost', digits=dp.get_precision('Product Price'))
-    former_cost_per_unit = fields.Float(
-        'Former Cost(Per Unit)', compute='_compute_former_cost_per_unit',
-        digits=0, store=True)
+        'Original Value', digits='Product Price')
     additional_landed_cost = fields.Float(
         'Additional Landed Cost',
-        digits=dp.get_precision('Product Price'))
+        digits='Product Price')
     final_cost = fields.Float(
-        'Final Cost', compute='_compute_final_cost',
+        'New Value', compute='_compute_final_cost',
         digits=0, store=True)
+    currency_id = fields.Many2one('res.currency', related='cost_id.company_id.currency_id')
 
-    @api.one
     @api.depends('cost_line_id.name', 'product_id.code', 'product_id.name')
     def _compute_name(self):
-        name = '%s - ' % (self.cost_line_id.name if self.cost_line_id else '')
-        self.name = name + (self.product_id.code or self.product_id.name or '')
+        for line in self:
+            name = '%s - ' % (line.cost_line_id.name if line.cost_line_id else '')
+            line.name = name + (line.product_id.code or line.product_id.name or '')
 
-    @api.one
-    @api.depends('former_cost', 'quantity')
-    def _compute_former_cost_per_unit(self):
-        self.former_cost_per_unit = self.former_cost / (self.quantity or 1.0)
-
-    @api.one
     @api.depends('former_cost', 'additional_landed_cost')
     def _compute_final_cost(self):
-        self.final_cost = self.former_cost + self.additional_landed_cost
+        for line in self:
+            line.final_cost = line.former_cost + line.additional_landed_cost
 
     def _create_accounting_entries(self, move, qty_out):
         # TDE CLEANME: product chosen for computation ?
@@ -311,7 +366,7 @@ class AdjustmentLines(models.Model):
         if self.move_id._is_dropshipped():
             debit_account_id = accounts.get('expense') and accounts['expense'].id or False
         already_out_account_id = accounts['stock_output'].id
-        credit_account_id = self.cost_line_id.account_id.id or cost_product.property_account_expense_id.id or cost_product.categ_id.property_account_expense_categ_id.id
+        credit_account_id = self.cost_line_id.account_id.id or cost_product.categ_id.property_stock_account_input_categ_id.id
 
         if not credit_account_id:
             raise UserError(_('Please configure Stock Expense Account for product: %s.') % (cost_product.name))
@@ -364,12 +419,12 @@ class AdjustmentLines(models.Model):
             AccountMoveLine.append([0, 0, debit_line])
             AccountMoveLine.append([0, 0, credit_line])
 
-            # TDE FIXME: oh dear
-            if self.env.user.company_id.anglo_saxon_accounting:
+            if self.env.company.anglo_saxon_accounting:
+                expense_account_id = self.product_id.product_tmpl_id.get_product_accounts()['expense'].id
                 debit_line = dict(base_line,
                                   name=(self.name + ": " + str(qty_out) + _(' already out')),
                                   quantity=0,
-                                  account_id=credit_account_id)
+                                  account_id=expense_account_id)
                 credit_line = dict(base_line,
                                    name=(self.name + ": " + str(qty_out) + _(' already out')),
                                    quantity=0,
