@@ -6,11 +6,10 @@ from odoo import api, fields, models
 
 class StockPickingToBatch(models.TransientModel):
     _name = 'stock.picking.to.batch'
-    _description = 'Batch Picking Lines'
+    _description = 'Batch Transfer Lines'
 
-    batch_id = fields.Many2one('stock.picking.batch', string='Batch Picking', oldname="wave_id")
+    batch_id = fields.Many2one('stock.picking.batch', string='Batch Transfer')
 
-    @api.multi
     def attach_pickings(self):
         # use active_ids to add picking line to the selected batch
         self.ensure_one()
