@@ -3,7 +3,7 @@
 {
     'name': 'Surveys',
     'version': '3.0',
-    'category': 'Marketing',
+    'category': 'Marketing/Survey',
     'description': """
 Create beautiful surveys and visualize answers
 ==============================================
@@ -16,21 +16,35 @@ sent mails with personal token for the invitation of the survey.
     """,
     'summary': 'Create surveys and analyze answers',
     'website': 'https://www.odoo.com/page/survey',
-    'depends': ['http_routing', 'mail'],
+    'depends': [
+        'auth_signup',
+        'http_routing',
+        'mail',
+        'web_tour',
+        'gamification'],
     'data': [
+        'views/survey_report_templates.xml',
+        'views/survey_reports.xml',
+        'data/mail_template_data.xml',
+        'data/ir_actions_data.xml',
         'security/survey_security.xml',
         'security/ir.model.access.csv',
-        'views/survey_views.xml',
+        'views/assets.xml',
+        'views/survey_menus.xml',
+        'views/survey_survey_views.xml',
+        'views/survey_user_views.xml',
+        'views/survey_question_views.xml',
         'views/survey_templates.xml',
-        'views/survey_result.xml',
-        'data/mail_template_data.xml',
-        'wizard/survey_email_compose_message.xml',
-        'data/survey_stages.xml',
+        'views/gamification_badge_views.xml',
+        'wizard/survey_invite_views.xml',
+        'views/res_partner_views.xml',
+    ],
+    'demo': [
+        'data/survey_demo_user.xml',
+        'data/survey_demo_feedback.xml',
+        'data/survey_demo_certification.xml',
+        'data/survey.user_input_line.csv'
     ],
-    'demo': ['data/survey_demo_user.xml',
-             'data/survey_demo_feedback.xml',
-             'data/survey.user_input.csv',
-             'data/survey.user_input_line.csv'],
     'installable': True,
     'auto_install': False,
     'application': True,
