@@ -4,38 +4,38 @@
 from collections import Counter
 import datetime
 import errno
-from os import utime
+import os
 import time
+from unittest.mock import patch
 
 from odoo import api
 from odoo.addons.base.models.assetsbundle import AssetsBundle
+from odoo.addons.base.models.ir_attachment import IrAttachment
 from odoo.modules.module import get_resource_path
 from odoo.tests import HttpCase
 from odoo.tests.common import TransactionCase
 
-
-def _touch(filepath, asset, t=None):
-    try:
-        utime(filepath, (t, t) if t else None)
-    except OSError as e:
-        if e.errno in [errno.EPERM, errno.EACCES, errno.EROFS]:
-            # Permission denied when touching the asset file, possibly read-only filesystem.
-            # We alter the asset last modified time to simulate a change in the file
-            now = datetime.datetime.now()
-            asset.last_modified = now
-            for sheet in asset.stylesheets:
-                sheet.last_modified = now
-        else:
-            raise
+GETMTINE = os.path.getmtime
 
 
-class TestJavascriptAssetsBundle(TransactionCase):
+class FileTouchable(TransactionCase):
+    def setUp(self):
+        super(FileTouchable, self).setUp()
+        self.touches = {}
+
+    def _touch(self, filepath, touch_time=None):
+        self.touches[filepath] = touch_time or time.time()
+        return patch('os.path.getmtime', lambda filename: self.touches.get(filename) or GETMTINE(filename))
+
+
+class TestJavascriptAssetsBundle(FileTouchable):
     def setUp(self):
         super(TestJavascriptAssetsBundle, self).setUp()
         self.jsbundle_xmlid = 'test_assetsbundle.bundle1'
         self.cssbundle_xmlid = 'test_assetsbundle.bundle2'
         self.env['res.lang'].load_lang('ar_SY')
 
+
     def _get_asset(self, xmlid, env=None):
         env = (env or self.env)
         files, remains = env['ir.qweb']._get_asset_content(xmlid, env.context)
@@ -44,11 +44,18 @@ class TestJavascriptAssetsBundle(TransactionCase):
     def _any_ira_for_bundle(self, type, lang=None):
         """ Returns all ir.attachments associated to a bundle, regardless of the verion.
         """
-        user_direction = self.env['res.lang'].search([('code', '=', (lang or self.env.user.lang))]).direction
+        user_direction = self.env['res.lang']._lang_get(lang or self.env.user.lang).direction
         bundle = self.jsbundle_xmlid if type == 'js' else self.cssbundle_xmlid
-        return self.env['ir.attachment'].search([
-            ('url', '=like', '/web/content/%-%/{0}{1}%.{2}'.format(('rtl/' if type == 'css' and user_direction == 'rtl' else ''), bundle, type))
-        ])
+        url = '/web/content/%-%/{0}{1}.{2}'.format(('rtl/' if type == 'css' and user_direction == 'rtl' else ''), bundle, type)
+        domain = [('url', '=like', url)]
+        return self.env['ir.attachment'].search(domain)
+
+    def _node_to_list(self, nodes):
+        res = []
+        for index, (tagName, t_attrs, content) in enumerate(nodes):
+            for name, value in t_attrs.items():
+                res.append(value)
+        return res
 
     def test_01_generation(self):
         """ Checks that a bundle creates an ir.attachment record when its `js` method is called
@@ -102,16 +109,16 @@ class TestJavascriptAssetsBundle(TransactionCase):
 
         path = get_resource_path('test_assetsbundle', 'static', 'src', 'js', 'test_jsfile1.js')
         bundle1 = self._get_asset(self.jsbundle_xmlid)
-        _touch(path, bundle1)
 
-        bundle1.js()
-        last_modified1 = bundle1.last_modified
-        version1 = bundle1.version
-        self.assertNotEquals(last_modified0, last_modified1)
-        self.assertNotEquals(version0, version1)
+        with self._touch(path):
+            bundle1.js()
+            last_modified1 = bundle1.last_modified
+            version1 = bundle1.version
+            self.assertNotEquals(last_modified0, last_modified1)
+            self.assertNotEquals(version0, version1)
 
-        # check if the previous attachment is correctly cleaned
-        self.assertEquals(len(self._any_ira_for_bundle('js')), 1)
+            # check if the previous attachment is correctly cleaned
+            self.assertEquals(len(self._any_ira_for_bundle('js')), 1)
 
     def test_04_content_invalidation(self):
         """ Checks that a bundle is invalidated when its content is modified by adding a file to
@@ -154,101 +161,56 @@ class TestJavascriptAssetsBundle(TransactionCase):
         """ Checks that a bundle rendered in debug mode outputs non-minified assets.
         """
         debug_bundle = self._get_asset(self.jsbundle_xmlid)
-        content = debug_bundle.to_html(debug='assets')
+        nodes = debug_bundle.to_node(debug='assets')
+        content = self._node_to_list(nodes)
         # find back one of the original asset file
         self.assertIn('/test_assetsbundle/static/src/js/test_jsfile1.js', content)
 
         # there shouldn't be any assets created in debug mode
         self.assertEquals(len(self._any_ira_for_bundle('js')), 0)
 
-    def test_06_paginated_css_generation1(self):
-        """ Checks that a bundle creates enough ir.attachment records when its `css` method is called
-        for the first time while the number of css rules exceed the limit.
-        """
-        # note: changing the max_css_rules of a bundle does not invalidate its attachments
+    def test_08_css_generation3(self):
         # self.cssbundle_xlmid contains 3 rules
-        self.bundle = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 1}))
-        self.bundle.css()
-        self.assertEquals(len(self._any_ira_for_bundle('css')), 3)
-        self.assertEquals(len(self.bundle.get_attachments('css')), 3)
-
-    def test_07_paginated_css_generation2(self):
-        # self.cssbundle_xlmid contains 3 rules
-        self.bundle = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 2}))
-        self.bundle.css()
-        self.assertEquals(len(self._any_ira_for_bundle('css')), 2)
-        self.assertEquals(len(self.bundle.get_attachments('css')), 2)
-
-    def test_08_paginated_css_generation3(self):
-        # self.cssbundle_xlmid contains 3 rules
-        self.bundle = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 3}))
+        self.bundle = self._get_asset(self.cssbundle_xmlid)
         self.bundle.css()
         self.assertEquals(len(self._any_ira_for_bundle('css')), 1)
         self.assertEquals(len(self.bundle.get_attachments('css')), 1)
 
-    def test_09_paginated_css_access(self):
+    def test_09_css_access(self):
         """ Checks that the bundle's cache is working, i.e. that a bundle creates only enough
         ir.attachment records when rendered multiple times.
         """
-        bundle0 = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 1}))
+        bundle0 = self._get_asset(self.cssbundle_xmlid)
         bundle0.css()
 
-        self.assertEquals(len(self._any_ira_for_bundle('css')), 3)
+        self.assertEquals(len(self._any_ira_for_bundle('css')), 1)
 
         version0 = bundle0.version
-        ira0, ira1, ira2 = self._any_ira_for_bundle('css')
+        ira0 = self._any_ira_for_bundle('css')
         date0 = ira0.create_date
-        date1 = ira1.create_date
-        date2 = ira2.create_date
 
-        bundle1 = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 1}))
+        bundle1 = self._get_asset(self.cssbundle_xmlid)
         bundle1.css()
 
-        self.assertEquals(len(self._any_ira_for_bundle('css')), 3)
+        self.assertEquals(len(self._any_ira_for_bundle('css')), 1)
 
         version1 = bundle1.version
-        ira3, ira4, ira5 = self._any_ira_for_bundle('css')
-        date3 = ira1.create_date
-        date4 = ira1.create_date
-        date5 = ira1.create_date
+        ira1 = self._any_ira_for_bundle('css')
+        date1 = ira1.create_date
 
         self.assertEquals(version0, version1)
-        self.assertEquals(date0, date3)
-        self.assertEquals(date1, date4)
-        self.assertEquals(date2, date5)
+        self.assertEquals(date0, date1)
 
-    def test_10_paginated_css_date_invalidation(self):
-        """ Checks that a bundle is invalidated when one of its assets' modification date is changed.
-        """
-        bundle0 = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 1}))
-        bundle0.css()
-        last_modified0 = bundle0.last_modified
-        version0 = bundle0.version
-
-        path = get_resource_path('test_assetsbundle', 'static', 'src', 'css', 'test_cssfile1.css')
-        bundle1 = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 1}))
-        _touch(path, bundle1)
-
-        bundle1.css()
-        last_modified1 = bundle1.last_modified
-        version1 = bundle1.version
-
-        self.assertNotEquals(last_modified0, last_modified1)
-        self.assertNotEquals(version0, version1)
-
-        # check if the previous attachment is correctly cleaned
-        self.assertEquals(len(self._any_ira_for_bundle('css')), 3)
-
-    def test_11_paginated_css_content_invalidation(self):
+    def test_11_css_content_invalidation(self):
         """ Checks that a bundle is invalidated when its content is modified by adding a file to
         source.
         """
-        bundle0 = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 1}))
+        bundle0 = self._get_asset(self.cssbundle_xmlid)
         bundle0.css()
         files0 = bundle0.files
         version0 = bundle0.version
 
-        self.assertEquals(len(self._any_ira_for_bundle('css')), 3)
+        self.assertEquals(len(self._any_ira_for_bundle('css')), 1)
 
         view_arch = """
         <data>
@@ -265,7 +227,7 @@ class TestJavascriptAssetsBundle(TransactionCase):
             'inherit_id': bundle.id,
         })
 
-        bundle1 = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'check_view_ids': view.ids, 'max_css_rules': 1}))
+        bundle1 = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'check_view_ids': view.ids}))
         bundle1.css()
         files1 = bundle1.files
         version1 = bundle1.version
@@ -274,28 +236,20 @@ class TestJavascriptAssetsBundle(TransactionCase):
         self.assertNotEquals(version0, version1)
 
         # check if the previous attachment are correctly cleaned
-        self.assertEquals(len(self._any_ira_for_bundle('css')), 4)
+        self.assertEquals(len(self._any_ira_for_bundle('css')), 1)
 
-    def test_12_paginated_css_debug(self):
+    def test_12_css_debug(self):
         """ Check that a bundle in debug mode outputs non-minified assets.
         """
-        debug_bundle = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 1}))
-        content = debug_bundle.to_html(debug='assets')
+        debug_bundle = self._get_asset(self.cssbundle_xmlid)
+        nodes = debug_bundle.to_node(debug='assets')
+        content = self._node_to_list(nodes)
         # find back one of the original asset file
         self.assertIn('/test_assetsbundle/static/src/css/test_cssfile1.css', content)
 
         # there shouldn't be any assets created in debug mode
         self.assertEquals(len(self._any_ira_for_bundle('css')), 0)
 
-    def test_13_paginated_css_order(self):
-        # self.cssbundle_xlmid contains 3 rules
-        self.bundle = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'max_css_rules': 1}))
-        stylesheets = self.bundle.css()
-
-        self.assertTrue(stylesheets[0].url.endswith('.0.css'))
-        self.assertTrue(stylesheets[1].url.endswith('.1.css'))
-        self.assertTrue(stylesheets[2].url.endswith('.2.css'))
-
     def test_14_duplicated_css_assets(self):
         """ Checks that if the bundle's ir.attachment record is duplicated, the bundle is only sourced once. This could
         happen if multiple transactions try to render the bundle simultaneously.
@@ -311,8 +265,9 @@ class TestJavascriptAssetsBundle(TransactionCase):
         self.assertEquals(ira0.store_fname, ira1.store_fname)
 
         # the ir.attachment records should be deduplicated in the bundle's content
-        content = bundle0.to_html()
-        self.assertEqual(content.count('test_assetsbundle.bundle2.0.css'), 1)
+        nodes = bundle0.to_node()
+        content = self._node_to_list(nodes)
+        self.assertEqual(content[2].count('test_assetsbundle.bundle2.css'), 1)
 
     # Language direction specific tests
 
@@ -410,32 +365,32 @@ class TestJavascriptAssetsBundle(TransactionCase):
         # Note: No lang specific context given while calling _get_asset so it will load assets for en_US
         path = get_resource_path('test_assetsbundle', 'static', 'src', 'css', 'test_cssfile1.css')
         ltr_bundle1 = self._get_asset(self.cssbundle_xmlid)
-        _touch(path, ltr_bundle1)
 
-        ltr_bundle1.css()
-        ltr_last_modified1 = ltr_bundle1.last_modified
-        ltr_version1 = ltr_bundle1.version
-        ltr_ira1 = self._any_ira_for_bundle('css')
-        self.assertNotEquals(ltr_last_modified0, ltr_last_modified1)
-        self.assertNotEquals(ltr_version0, ltr_version1)
+        with self._touch(path):
+            ltr_bundle1.css()
+            ltr_last_modified1 = ltr_bundle1.last_modified
+            ltr_version1 = ltr_bundle1.version
+            ltr_ira1 = self._any_ira_for_bundle('css')
+            self.assertNotEquals(ltr_last_modified0, ltr_last_modified1)
+            self.assertNotEquals(ltr_version0, ltr_version1)
 
-        rtl_bundle1 = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'lang': 'ar_SY'}))
+            rtl_bundle1 = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'lang': 'ar_SY'}))
 
-        rtl_bundle1.css()
-        rtl_last_modified1 = rtl_bundle1.last_modified
-        rtl_version1 = rtl_bundle1.version
-        rtl_ira1 = self._any_ira_for_bundle('css', lang='ar_SY')
-        self.assertNotEquals(rtl_last_modified0, rtl_last_modified1)
-        self.assertNotEquals(rtl_version0, rtl_version1)
+            rtl_bundle1.css()
+            rtl_last_modified1 = rtl_bundle1.last_modified
+            rtl_version1 = rtl_bundle1.version
+            rtl_ira1 = self._any_ira_for_bundle('css', lang='ar_SY')
+            self.assertNotEquals(rtl_last_modified0, rtl_last_modified1)
+            self.assertNotEquals(rtl_version0, rtl_version1)
 
-        # Checks rtl and ltr bundles are different
-        self.assertNotEquals(ltr_ira1.id, rtl_ira1.id)
+            # Checks rtl and ltr bundles are different
+            self.assertNotEquals(ltr_ira1.id, rtl_ira1.id)
 
-        # check if the previous attachment is correctly cleaned
-        css_bundles = self.env['ir.attachment'].search([
-            ('url', '=like', '/web/content/%-%/{0}%.{1}'.format(self.cssbundle_xmlid, 'css'))
-        ])
-        self.assertEquals(len(css_bundles), 2)
+            # check if the previous attachment is correctly cleaned
+            css_bundles = self.env['ir.attachment'].search([
+                ('url', '=like', '/web/content/%-%/{0}%.{1}'.format(self.cssbundle_xmlid, 'css'))
+            ])
+            self.assertEquals(len(css_bundles), 2)
 
     def test_18_css_bundle_content_invalidation(self):
         """ Checks that a bundle is invalidated when its content is modified by adding a file to
@@ -503,7 +458,8 @@ class TestJavascriptAssetsBundle(TransactionCase):
         """ Checks that a bundle rendered in debug mode(assets) with right to left language direction stores css files in assets bundle.
         """
         debug_bundle = self._get_asset(self.cssbundle_xmlid, env=self.env(context={'lang': 'ar_SY'}))
-        content = debug_bundle.to_html(debug='assets')
+        nodes = debug_bundle.to_node(debug='assets')
+        content = self._node_to_list(nodes)
 
         # css file should be available in assets bundle as user's lang direction is rtl
         self.assertIn('/test_assetsbundle/static/src/css/test_cssfile1/rtl/{0}.css'.format(self.cssbundle_xmlid), content)
@@ -562,7 +518,7 @@ class TestAssetsBundleInBrowser(HttpCase):
         """
         self.phantom_js(
             "/test_assetsbundle/js",
-            "a + b + c === 6 ? console.log('ok') : console.log('error')",
+            "a + b + c === 6 ? console.log('test successful') : console.log('error')",
             login="admin"
         )
 
@@ -584,44 +540,44 @@ class TestAssetsBundleInBrowser(HttpCase):
             'arch': view_arch,
             'inherit_id': self.browse_ref('test_assetsbundle.bundle1').id,
         })
+        self.env.user.flush()
 
         self.phantom_js(
             "/test_assetsbundle/js",
-            "a + b + c + d === 10 ? console.log('ok') : console.log('error')",
+            "a + b + c + d === 10 ? console.log('test successful') : console.log('error')",
             login="admin",
         )
 
 
-class TestAssetsBundleWithIRAMock(TransactionCase):
+class TestAssetsBundleWithIRAMock(FileTouchable):
     def setUp(self):
         super(TestAssetsBundleWithIRAMock, self).setUp()
         self.stylebundle_xmlid = 'test_assetsbundle.bundle3'
         self.counter = counter = Counter()
 
         # patch methods 'create' and 'unlink' of model 'ir.attachment'
+        origin_create = IrAttachment.create
+        origin_unlink = IrAttachment.unlink
+
         @api.model
         def create(self, vals):
             counter.update(['create'])
-            return create.origin(self, vals)
+            return origin_create(self, vals)
 
-        @api.multi
         def unlink(self):
             counter.update(['unlink'])
-            return unlink.origin(self)
+            return origin_unlink(self)
 
-        self.env['ir.attachment']._patch_method('create', create)
-        self.addCleanup(self.env['ir.attachment']._revert_method, 'create')
-
-        self.env['ir.attachment']._patch_method('unlink', unlink)
-        self.addCleanup(self.env['ir.attachment']._revert_method, 'unlink')
+        self.patch(IrAttachment, 'create', create)
+        self.patch(IrAttachment, 'unlink', unlink)
 
     def _get_asset(self):
         files, remains = self.env['ir.qweb']._get_asset_content(self.stylebundle_xmlid, {})
-        return AssetsBundle(self.stylebundle_xmlid, files, remains, env=self.env)
+        return AssetsBundle(self.stylebundle_xmlid, files, env=self.env)
 
     def _bundle(self, asset, should_create, should_unlink):
         self.counter.clear()
-        asset.to_html(debug='assets')
+        asset.to_node(debug='assets')
         self.assertEquals(self.counter['create'], int(should_create))
         self.assertEquals(self.counter['unlink'], int(should_unlink))
 
@@ -639,15 +595,16 @@ class TestAssetsBundleWithIRAMock(TransactionCase):
         path = get_resource_path('test_assetsbundle', 'static', 'src', 'scss', 'test_file1.scss')
         t = time.time() + 5
         asset = self._get_asset()
-        _touch(path, asset, t=t)
-        self._bundle(asset, True, True)
+        with self._touch(path, t):
+            self._bundle(asset, True, True)
 
-        # Because we are in the same transaction since the beginning of the test, the first asset
-        # created and the second one have the same write_date, but the file's last modified date
-        # has really been modified. If we do not update the write_date to a posterior date, we are
-        # not able to reproduce the case where we compile this bundle again without changing
-        # anything.
-        self.cr.execute("update ir_attachment set write_date=clock_timestamp() + interval '10 seconds' where id = (select max(id) from ir_attachment)")
+            # Because we are in the same transaction since the beginning of the test, the first asset
+            # created and the second one have the same write_date, but the file's last modified date
+            # has really been modified. If we do not update the write_date to a posterior date, we are
+            # not able to reproduce the case where we compile this bundle again without changing
+            # anything.
+            self.env['ir.attachment'].flush(['checksum'])
+            self.cr.execute("update ir_attachment set write_date=clock_timestamp() + interval '10 seconds' where id = (select max(id) from ir_attachment)")
 
-        # Compile a fourth time, without changes
-        self._bundle(self._get_asset(), False, False)
+            # Compile a fourth time, without changes
+            self._bundle(self._get_asset(), False, False)
