@@ -5,8 +5,8 @@ import itertools
 import pstats
 from cProfile import Profile
 
+from odoo import fields
 from odoo.tests import common
-from odoo.tools import pycompat
 
 
 class CreatorCase(common.TransactionCase):
@@ -43,7 +43,7 @@ class test_xids(CreatorCase):
         record.invalidate_cache()
         self.assertEqual(
             record._export_rows([['id'], ['value']]),
-            [[u'x', u'True']]
+            [[u'x', True]]
         )
 
 class test_boolean_field(CreatorCase):
@@ -52,7 +52,7 @@ class test_boolean_field(CreatorCase):
     def test_true(self):
         self.assertEqual(
             self.export(True),
-            [[u'True']])
+            [[True]])
 
     def test_false(self):
         """ ``False`` value to boolean fields is unique in being exported as a
@@ -60,7 +60,7 @@ class test_boolean_field(CreatorCase):
         """
         self.assertEqual(
             self.export(False),
-            [[u'False']])
+            [[False]])
 
 
 class test_integer_field(CreatorCase):
@@ -73,22 +73,22 @@ class test_integer_field(CreatorCase):
     def test_0(self):
         self.assertEqual(
             self.export(0),
-            [[u'0']])
+            [[0]])
 
     def test_basic_value(self):
         self.assertEqual(
             self.export(42),
-            [[u'42']])
+            [[42]])
 
     def test_negative(self):
         self.assertEqual(
             self.export(-32),
-            [[u'-32']])
+            [[-32]])
 
     def test_huge(self):
         self.assertEqual(
             self.export(2**31-1),
-            [[pycompat.text_type(2**31-1)]])
+            [[2147483647]])
 
 
 class test_float_field(CreatorCase):
@@ -97,27 +97,27 @@ class test_float_field(CreatorCase):
     def test_0(self):
         self.assertEqual(
             self.export(0.0),
-            [[u'0.0']])
+            [[0.0]])
 
     def test_epsilon(self):
         self.assertEqual(
             self.export(0.000000000027),
-            [[u'2.7e-11']])
+            [[0.000000000027]])
 
     def test_negative(self):
         self.assertEqual(
             self.export(-2.42),
-            [[u'-2.42']])
+            [[-2.42]])
 
     def test_positive(self):
         self.assertEqual(
             self.export(47.36),
-            [[u'47.36']])
+            [[47.36]])
 
     def test_big(self):
         self.assertEqual(
             self.export(87654321.4678),
-            [[u'87654321.4678']])
+            [[87654321.4678]])
 
 
 class test_decimal_field(CreatorCase):
@@ -126,28 +126,28 @@ class test_decimal_field(CreatorCase):
     def test_0(self):
         self.assertEqual(
             self.export(0.0),
-            [[u'0.0']])
+            [[0.0]])
 
     def test_epsilon(self):
         """ epsilon gets sliced to 0 due to precision
         """
         self.assertEqual(
             self.export(0.000000000027),
-            [[u'0.0']])
+            [[0.0]])
 
     def test_negative(self):
         self.assertEqual(
             self.export(-2.42),
-            [[u'-2.42']])
+            [[-2.42]])
 
     def test_positive(self):
         self.assertEqual(
             self.export(47.36),
-            [[u'47.36']])
+            [[47.36]])
 
     def test_big(self):
         self.assertEqual(
-            self.export(87654321.4678), [[u'87654321.468']])
+            self.export(87654321.4678), [[87654321.468]])
 
 
 class test_string_field(CreatorCase):
@@ -219,6 +219,10 @@ class test_text(CreatorCase):
               u" programming in the A-normal form. That is indeed all there"
               u" is to monads"]])
 
+    def test_numeric(self):
+        self.assertEqual(
+            self.export(42),
+            [["42"]])
 
 class test_date(CreatorCase):
     model_name = 'export.date'
@@ -231,7 +235,7 @@ class test_date(CreatorCase):
     def test_basic(self):
         self.assertEqual(
             self.export('2011-11-07'),
-            [[u'2011-11-07']])
+            [[fields.Date.from_string('2011-11-07')]])
 
 
 class test_datetime(CreatorCase):
@@ -248,7 +252,7 @@ class test_datetime(CreatorCase):
         self.env.user.write({'tz': False})
         self.assertEqual(
             self.export('2011-11-07 21:05:48'),
-            [[u'2011-11-07 21:05:48']])
+            [[fields.Datetime.from_string('2011-11-07 21:05:48')]])
 
     def test_tz(self):
         """ Export converts the value in the user's TZ
@@ -257,7 +261,7 @@ class test_datetime(CreatorCase):
         """
         self.assertEqual(
             self.export('2011-11-07 21:05:48', context={'tz': 'Pacific/Norfolk'}),
-            [[u'2011-11-08 08:35:48']])
+            [[fields.Datetime.from_string('2011-11-08 08:35:48')]])
 
 
 class test_selection(CreatorCase):
@@ -271,28 +275,24 @@ class test_selection(CreatorCase):
     def test_empty(self):
         self.assertEqual(
             self.export(False),
-            [[False]])
+            [['']])
 
     def test_value(self):
         """ selections export the *label* for their value
         """
         self.assertEqual(
-            self.export(2),
+            self.export('2'),
             [[u"Bar"]])
 
     def test_localized_export(self):
         self.env['res.lang'].load_lang('fr_FR')
-        Translations = self.env['ir.translation']
-        for source, value in self.translations_fr:
-            Translations.create({
-                'name': 'export.selection,value',
-                'lang': 'fr_FR',
-                'type': 'selection',
-                'src': source,
-                'value': value
-            })
+        ir_field = self.env['ir.model.fields']._get('export.selection', 'value')
+        selection = ir_field.selection_ids
+        translations = dict(self.translations_fr)
+        for sel_fr, sel in zip(selection.with_context(lang='fr_FR'), selection):
+            sel_fr.name = translations.get(sel.name, sel_fr.name)
         self.assertEqual(
-            self.export(2, context={'lang': 'fr_FR'}),
+            self.export('2', context={'lang': 'fr_FR'}),
             [[u'titi']])
 
 
@@ -340,7 +340,7 @@ class test_m2o(CreatorCase):
         record = self.env['export.integer'].create({'value': 42})
         self.assertEqual(
             self.export(record.id, fields=['value/.id', 'value/value']),
-            [[pycompat.text_type(record.id), u'42']])
+            [[str(record.id), 42]])
 
     def test_external_id(self):
         record = self.env['export.integer'].create({'value': 42})
@@ -396,30 +396,30 @@ class test_o2m(CreatorCase):
         self.assertEqual(
             self.export([(0, False, {'value': 42})],
                         fields=['value', 'value/value']),
-            [[u'export.one2many.child:42', u'42']])
+            [[u'export.one2many.child:42', 42]])
 
     def test_integrate_one_in_parent(self):
         self.assertEqual(
             self.export([(0, False, {'value': 42})],
                         fields=['const', 'value/value']),
-            [[u'4', u'42']])
+            [[4, 42]])
 
     def test_multiple_records(self):
         self.assertEqual(
             self.export(self.commands, fields=['const', 'value/value']),
             [
-                [u'4', u'4'],
-                [u'', u'42'],
-                [u'', u'36'],
-                [u'', u'4'],
-                [u'', u'13'],
+                [4, 4],
+                [u'', 42],
+                [u'', 36],
+                [u'', 4],
+                [u'', 13],
             ])
 
     def test_multiple_records_name(self):
         self.assertEqual(
             self.export(self.commands, fields=['const', 'value']),
             [
-                [u'4', u'export.one2many.child:4'],
+                [4, u'export.one2many.child:4'],
                 [u'', u'export.one2many.child:42'],
                 [u'', u'export.one2many.child:36'],
                 [u'', u'export.one2many.child:4'],
@@ -432,7 +432,7 @@ class test_o2m(CreatorCase):
         self.assertEqual(
             export,
             [
-                ['4', str(records[0].id)],
+                [4, str(records[0].id)],
                 ['', str(records[1].id)],
                 ['', str(records[2].id)],
                 ['', str(records[3].id)],
@@ -443,44 +443,44 @@ class test_o2m(CreatorCase):
         self.assertEqual(
             self.export(self.commands, fields=['const', 'value', 'value/value']),
             [
-                [u'4', u'export.one2many.child:4', u'4'],
-                ['', u'export.one2many.child:42', u'42'],
-                ['', u'export.one2many.child:36', u'36'],
-                ['', u'export.one2many.child:4', u'4'],
-                ['', u'export.one2many.child:13', u'13'],
+                [4, u'export.one2many.child:4', 4],
+                ['', u'export.one2many.child:42', 42],
+                ['', u'export.one2many.child:36', 36],
+                ['', u'export.one2many.child:4', 4],
+                ['', u'export.one2many.child:13', 13],
             ])
 
     def test_multiple_records_with_name_after(self):
         self.assertEqual(
             self.export(self.commands, fields=['const', 'value/value', 'value']),
             [
-                [u'4', u'4', u'export.one2many.child:4'],
-                ['', u'42', u'export.one2many.child:42'],
-                ['', u'36', u'export.one2many.child:36'],
-                ['', u'4', u'export.one2many.child:4'],
-                ['', u'13', u'export.one2many.child:13'],
+                [4, 4, u'export.one2many.child:4'],
+                ['', 42, u'export.one2many.child:42'],
+                ['', 36, u'export.one2many.child:36'],
+                ['', 4, u'export.one2many.child:4'],
+                ['', 13, u'export.one2many.child:13'],
             ])
 
     def test_multiple_subfields_neighbour(self):
         self.assertEqual(
             self.export(self.commands, fields=['const', 'value/str','value/value']),
             [
-                [u'4', u'record1', u'4'],
-                ['', u'record2', u'42'],
-                ['', u'record3', u'36'],
-                ['', u'record4', u'4'],
-                ['', u'record5', u'13'],
+                [4, u'record1', 4],
+                ['', u'record2', 42],
+                ['', u'record3', 36],
+                ['', u'record4', 4],
+                ['', u'record5', 13],
             ])
 
     def test_multiple_subfields_separated(self):
         self.assertEqual(
             self.export(self.commands, fields=['value/str', 'const', 'value/value']),
             [
-                [u'record1', u'4', u'4'],
-                [u'record2', '', u'42'],
-                [u'record3', '', u'36'],
-                [u'record4', '', u'4'],
-                [u'record5', '', u'13'],
+                [u'record1', 4, 4],
+                [u'record2', '', 42],
+                [u'record3', '', 36],
+                [u'record4', '', 4],
+                [u'record5', '', 13],
             ])
 
 
@@ -520,18 +520,18 @@ class test_o2m_multiple(CreatorCase):
         self.assertEqual(
             self.export(child1=False, child2=[(0, False, {'value': 42})],
                         fields=fields),
-            [[u'36', False, u'42']])
+            [[36, False, 42]])
 
         self.assertEqual(
             self.export(child1=[(0, False, {'value': 43})], child2=False,
                         fields=fields),
-            [[u'36', u'43', False]])
+            [[36, 43, False]])
 
         self.assertEqual(
             self.export(child1=[(0, False, {'value': 43})],
                         child2=[(0, False, {'value': 42})],
                         fields=fields),
-            [[u'36', u'43', u'42']])
+            [[36, 43, 42]])
 
     def test_multiple(self):
         """ With two "concurrent" o2ms, exports the first line combined, then
@@ -539,42 +539,42 @@ class test_o2m_multiple(CreatorCase):
         """
         fields = ['const', 'child1/value', 'child2/value']
         child1 = [(0, False, {'value': v, 'str': 'record%.02d' % index})
-                  for index, v in pycompat.izip(itertools.count(), [4, 42, 36, 4, 13])]
+                  for index, v in zip(itertools.count(), [4, 42, 36, 4, 13])]
         child2 = [(0, False, {'value': v, 'str': 'record%.02d' % index})
-                  for index, v in pycompat.izip(itertools.count(10), [8, 12, 8, 55, 33, 13])]
+                  for index, v in zip(itertools.count(10), [8, 12, 8, 55, 33, 13])]
 
         self.assertEqual(
             self.export(child1=child1, child2=False, fields=fields),
             [
-                [u'36', u'4', False],
-                ['', u'42', ''],
-                ['', u'36', ''],
-                ['', u'4', ''],
-                ['', u'13', ''],
+                [36, 4, False],
+                ['', 42, ''],
+                ['', 36, ''],
+                ['', 4, ''],
+                ['', 13, ''],
             ])
         self.assertEqual(
             self.export(child1=False, child2=child2, fields=fields),
             [
-                [u'36', False, u'8'],
-                ['', '', u'12'],
-                ['', '', u'8'],
-                ['', '', u'55'],
-                ['', '', u'33'],
-                ['', '', u'13'],
+                [36, False, 8],
+                ['', '', 12],
+                ['', '', 8],
+                ['', '', 55],
+                ['', '', 33],
+                ['', '', 13],
             ])
         self.assertEqual(
             self.export(child1=child1, child2=child2, fields=fields),
             [
-                [u'36', u'4', u'8'],
-                ['', u'42', ''],
-                ['', u'36', ''],
-                ['', u'4', ''],
-                ['', u'13', ''],
-                ['', '', u'12'],
-                ['', '', u'8'],
-                ['', '', u'55'],
-                ['', '', u'33'],
-                ['', '', u'13'],
+                [36, 4, 8],
+                ['', 42, ''],
+                ['', 36, ''],
+                ['', 4, ''],
+                ['', 13, ''],
+                ['', '', 12],
+                ['', '', 8],
+                ['', '', 55],
+                ['', '', 33],
+                ['', '', 13],
             ])
 
 
@@ -607,30 +607,30 @@ class test_m2m(CreatorCase):
         self.assertEqual(
             self.export([(0, False, {'value': 42})],
                         fields=['value', 'value/value']),
-            [[u'export.many2many.other:42', u'42']])
+            [[u'export.many2many.other:42', 42]])
 
     def test_integrate_one_in_parent(self):
         self.assertEqual(
             self.export([(0, False, {'value': 42})],
                         fields=['const', 'value/value']),
-            [[u'4', u'42']])
+            [[4, 42]])
 
     def test_multiple_records(self):
         self.assertEqual(
             self.export(self.commands, fields=['const', 'value/value']),
             [
-                [u'4', u'4'],
-                [u'', u'42'],
-                [u'', u'36'],
-                [u'', u'4'],
-                [u'', u'13'],
+                [4, 4],
+                [u'', 42],
+                [u'', 36],
+                [u'', 4],
+                [u'', 13],
             ])
 
     def test_multiple_records_name(self):
         self.assertEqual(
             self.export(self.commands, fields=['const', 'value']),
             [
-                [u'4', u'export.many2many.other:4'],
+                [4, u'export.many2many.other:4'],
                 ['', u'export.many2many.other:42'],
                 ['', u'export.many2many.other:36'],
                 ['', u'export.many2many.other:4'],
@@ -668,21 +668,21 @@ class test_m2m(CreatorCase):
         self.assertEqual(
             r.with_context(import_compat=False)._export_rows([['id'], ['value', 'id'], ['value', 'value']]),
             [
-                [xid, u'__t__.record000', u'4'],
-                [u'', u'__t__.record001', u'42'],
-                [u'', u'__t__.record010', u'36'],
-                [u'', u'__t__.record011', u'4'],
-                [u'', u'__t__.record100', u'13']
+                [xid, u'__t__.record000', 4],
+                [u'', u'__t__.record001', 42],
+                [u'', u'__t__.record010', 36],
+                [u'', u'__t__.record011', 4],
+                [u'', u'__t__.record100', 13]
             ]
         )
         self.assertEqual(
             r.with_context(import_compat=False)._export_rows([['id'], ['value', 'value'], ['value', 'id']]),
             [
-                [xid, u'4', u'__t__.record000'],
-                [u'', u'42', u'__t__.record001'],
-                [u'', u'36', u'__t__.record010'],
-                [u'', u'4', u'__t__.record011'],
-                [u'', u'13', u'__t__.record100']
+                [xid, 4, u'__t__.record000'],
+                [u'', 42, u'__t__.record001'],
+                [u'', 36, u'__t__.record010'],
+                [u'', 4, u'__t__.record011'],
+                [u'', 13, u'__t__.record100']
             ]
         )
 
@@ -695,7 +695,7 @@ class test_function(CreatorCase):
         """
         self.assertEqual(
             self.export(42),
-            [[u'3']])
+            [[3]])
 
 
 @common.tagged('-standard', 'bench')
