@@ -1,20 +1,21 @@
 # -*- coding: utf-8 -*-
 
+from . import test_invite
+from . import test_ir_actions
+from . import test_discuss
 from . import test_mail_activity
 from . import test_mail_followers
 from . import test_mail_message
 from . import test_mail_mail
-from . import test_mail_race
-from . import test_mail_resend
 from . import test_mail_channel
+from . import test_mail_channel_partner
 from . import test_mail_gateway
+from . import test_mail_redirect
 from . import test_mail_template
-from . import test_message_compose
+from . import test_message_composer
+from . import test_message_management
+from . import test_message_post
 from . import test_message_track
-from . import test_invite
-from . import test_ir_actions
-from . import test_update_notification
-from . import test_discuss
-from . import test_performance
-from . import test_res_users
 from . import test_odoobot
+from . import test_performance
+from . import test_update_notification
