@@ -2,10 +2,12 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 import base64
 
+from unittest.mock import patch
 from odoo.addons.test_mail.tests import common
 from odoo.addons.test_mail.tests.common import mail_new_test_user
+from odoo.addons.test_mail.models.test_mail_models import MailTestSimple
 from odoo.exceptions import AccessError, except_orm
-from odoo.tools import mute_logger
+from odoo.tools import mute_logger, formataddr
 from odoo.tests import tagged
 
 
@@ -15,17 +17,16 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
     def setUpClass(cls):
         super(TestMessageValues, cls).setUpClass()
 
-        cls.alias_record = cls.env['mail.test'].with_context(common.BaseFunctionalTest._test_context).create({
+        cls._init_mail_gateway()
+        cls.alias_record = cls.env['mail.test'].with_context(cls._test_context).create({
             'name': 'Pigs',
             'alias_name': 'pigs',
             'alias_contact': 'followers',
         })
+        cls.Message = cls.env['mail.message'].with_user(cls.user_employee)
 
-        cls.Message = cls.env['mail.message'].sudo(cls.user_employee)
-
-    def test_mail_message_values_basic(self):
-        self.env['ir.config_parameter'].search([('key', '=', 'mail.catchall.domain')]).unlink()
-
+    @mute_logger('odoo.models.unlink')
+    def test_mail_message_values_no_document_values(self):
         msg = self.Message.create({
             'reply_to': 'test.reply@example.com',
             'email_from': 'test.from@example.com',
@@ -34,51 +35,57 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
         self.assertEqual(msg.reply_to, 'test.reply@example.com')
         self.assertEqual(msg.email_from, 'test.from@example.com')
 
-    def test_mail_message_values_default(self):
+    @mute_logger('odoo.models.unlink')
+    def test_mail_message_values_no_document(self):
+        msg = self.Message.create({})
+        self.assertIn('-private', msg.message_id.split('@')[0], 'mail_message: message_id for a void message should be a "private" one')
+        reply_to_name = self.env.user.company_id.name
+        reply_to_email = '%s@%s' % (self.alias_catchall, self.alias_domain)
+        self.assertEqual(msg.reply_to, formataddr((reply_to_name, reply_to_email)))
+        self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
+
+        # no alias domain -> author
         self.env['ir.config_parameter'].search([('key', '=', 'mail.catchall.domain')]).unlink()
 
         msg = self.Message.create({})
         self.assertIn('-private', msg.message_id.split('@')[0], 'mail_message: message_id for a void message should be a "private" one')
-        self.assertEqual(msg.reply_to, '%s <%s>' % (self.user_employee.name, self.user_employee.email))
-        self.assertEqual(msg.email_from, '%s <%s>' % (self.user_employee.name, self.user_employee.email))
+        self.assertEqual(msg.reply_to, formataddr((self.user_employee.name, self.user_employee.email)))
+        self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
-    @mute_logger('odoo.models.unlink')
-    def test_mail_message_values_alias(self):
-        alias_domain = 'example.com'
-        self.env['ir.config_parameter'].set_param('mail.catchall.domain', alias_domain)
+        # no alias catchall, no alias -> author
+        self.env['ir.config_parameter'].set_param('mail.catchall.domain', self.alias_domain)
         self.env['ir.config_parameter'].search([('key', '=', 'mail.catchall.alias')]).unlink()
 
         msg = self.Message.create({})
         self.assertIn('-private', msg.message_id.split('@')[0], 'mail_message: message_id for a void message should be a "private" one')
-        self.assertEqual(msg.reply_to, '%s <%s>' % (self.user_employee.name, self.user_employee.email))
-        self.assertEqual(msg.email_from, '%s <%s>' % (self.user_employee.name, self.user_employee.email))
-
-    def test_mail_message_values_alias_catchall(self):
-        alias_domain = 'example.com'
-        alias_catchall = 'pokemon'
-        self.env['ir.config_parameter'].set_param('mail.catchall.domain', alias_domain)
-        self.env['ir.config_parameter'].set_param('mail.catchall.alias', alias_catchall)
-
-        msg = self.Message.create({})
-        self.assertIn('-private', msg.message_id.split('@')[0], 'mail_message: message_id for a void message should be a "private" one')
-        self.assertEqual(msg.reply_to, '%s <%s@%s>' % (self.env.user.company_id.name, alias_catchall, alias_domain))
-        self.assertEqual(msg.email_from, '%s <%s>' % (self.user_employee.name, self.user_employee.email))
-
-    def test_mail_message_values_document_no_alias(self):
-        self.env['ir.config_parameter'].search([('key', '=', 'mail.catchall.domain')]).unlink()
-
-        msg = self.Message.create({
-            'model': 'mail.test',
-            'res_id': self.alias_record.id
-        })
-        self.assertIn('-openerp-%d-mail.test' % self.alias_record.id, msg.message_id.split('@')[0])
-        self.assertEqual(msg.reply_to, '%s <%s>' % (self.user_employee.name, self.user_employee.email))
-        self.assertEqual(msg.email_from, '%s <%s>' % (self.user_employee.name, self.user_employee.email))
+        self.assertEqual(msg.reply_to, formataddr((self.user_employee.name, self.user_employee.email)))
+        self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
     @mute_logger('odoo.models.unlink')
     def test_mail_message_values_document_alias(self):
-        alias_domain = 'example.com'
-        self.env['ir.config_parameter'].set_param('mail.catchall.domain', alias_domain)
+        msg = self.Message.create({
+            'model': 'mail.test',
+            'res_id': self.alias_record.id
+        })
+        self.assertIn('-openerp-%d-mail.test' % self.alias_record.id, msg.message_id.split('@')[0])
+        reply_to_name = '%s %s' % (self.env.user.company_id.name, self.alias_record.name)
+        reply_to_email = '%s@%s' % (self.alias_record.alias_name, self.alias_domain)
+        self.assertEqual(msg.reply_to, formataddr((reply_to_name, reply_to_email)))
+        self.assertEqual(msg.email_from, '"%s" <%s>' % (self.user_employee.name, self.user_employee.email))
+
+        # no alias domain -> author
+        self.env['ir.config_parameter'].search([('key', '=', 'mail.catchall.domain')]).unlink()
+
+        msg = self.Message.create({
+            'model': 'mail.test',
+            'res_id': self.alias_record.id
+        })
+        self.assertIn('-openerp-%d-mail.test' % self.alias_record.id, msg.message_id.split('@')[0])
+        self.assertEqual(msg.reply_to, formataddr((self.user_employee.name, self.user_employee.email)))
+        self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
+
+        # no catchall -> don't care, alias
+        self.env['ir.config_parameter'].set_param('mail.catchall.domain', self.alias_domain)
         self.env['ir.config_parameter'].search([('key', '=', 'mail.catchall.alias')]).unlink()
 
         msg = self.Message.create({
@@ -86,22 +93,46 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
             'res_id': self.alias_record.id
         })
         self.assertIn('-openerp-%d-mail.test' % self.alias_record.id, msg.message_id.split('@')[0])
-        self.assertEqual(msg.reply_to, '%s %s <%s@%s>' % (self.env.user.company_id.name, self.alias_record.name, self.alias_record.alias_name, alias_domain))
-        self.assertEqual(msg.email_from, '%s <%s>' % (self.user_employee.name, self.user_employee.email))
+        reply_to_name = '%s %s' % (self.env.company.name, self.alias_record.name)
+        reply_to_email = '%s@%s' % (self.alias_record.alias_name, self.alias_domain)
+        self.assertEqual(msg.reply_to, formataddr((reply_to_name, reply_to_email)))
+        self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
-    def test_mail_message_values_document_alias_catchall(self):
-        alias_domain = 'example.com'
-        alias_catchall = 'pokemon'
-        self.env['ir.config_parameter'].set_param('mail.catchall.domain', alias_domain)
-        self.env['ir.config_parameter'].set_param('mail.catchall.alias', alias_catchall)
+    @mute_logger('odoo.models.unlink')
+    def test_mail_message_values_document_no_alias(self):
+        test_record = self.env['mail.test.simple'].create({'name': 'Test', 'email_from': 'ignasse@example.com'})
 
         msg = self.Message.create({
-            'model': 'mail.test',
-            'res_id': self.alias_record.id
+            'model': 'mail.test.simple',
+            'res_id': test_record.id
         })
-        self.assertIn('-openerp-%d-mail.test' % self.alias_record.id, msg.message_id.split('@')[0])
-        self.assertEqual(msg.reply_to, '%s %s <%s@%s>' % (self.env.user.company_id.name, self.alias_record.name, self.alias_record.alias_name, alias_domain))
-        self.assertEqual(msg.email_from, '%s <%s>' % (self.user_employee.name, self.user_employee.email))
+        self.assertIn('-openerp-%d-mail.test.simple' % test_record.id, msg.message_id.split('@')[0])
+        reply_to_name = '%s %s' % (self.env.user.company_id.name, test_record.name)
+        reply_to_email = '%s@%s' % (self.alias_catchall, self.alias_domain)
+        self.assertEqual(msg.reply_to, formataddr((reply_to_name, reply_to_email)))
+        self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
+
+    @mute_logger('odoo.models.unlink')
+    def test_mail_message_values_document_manual_alias(self):
+        test_record = self.env['mail.test.simple'].create({'name': 'Test', 'email_from': 'ignasse@example.com'})
+        alias = self.env['mail.alias'].create({
+            'alias_name': 'MegaLias',
+            'alias_user_id': False,
+            'alias_model_id': self.env['ir.model']._get('mail.test.simple').id,
+            'alias_parent_model_id': self.env['ir.model']._get('mail.test.simple').id,
+            'alias_parent_thread_id': test_record.id,
+        })
+
+        msg = self.Message.create({
+            'model': 'mail.test.simple',
+            'res_id': test_record.id
+        })
+
+        self.assertIn('-openerp-%d-mail.test.simple' % test_record.id, msg.message_id.split('@')[0])
+        reply_to_name = '%s %s' % (self.env.user.company_id.name, test_record.name)
+        reply_to_email = '%s@%s' % (alias.alias_name, self.alias_domain)
+        self.assertEqual(msg.reply_to, formataddr((reply_to_name, reply_to_email)))
+        self.assertEqual(msg.email_from, formataddr((self.user_employee.name, self.user_employee.email)))
 
     def test_mail_message_values_no_auto_thread(self):
         msg = self.Message.create({
@@ -114,7 +145,7 @@ class TestMessageValues(common.BaseFunctionalTest, common.MockEmails):
         self.assertNotIn('-%d-' % self.alias_record.id, msg.message_id.split('@')[0])
 
     def test_mail_message_base64_image(self):
-        msg = self.env['mail.message'].sudo(self.user_employee).create({
+        msg = self.env['mail.message'].with_user(self.user_employee).create({
             'body': 'taratata <img src="data:image/png;base64,iV/+OkI=" width="2"> <img src="data:image/png;base64,iV/+OkI=" width="2">',
         })
         self.assertEqual(len(msg.attachment_ids), 1)
@@ -132,7 +163,7 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
         cls.user_public = mail_new_test_user(cls.env, login='bert', groups='base.group_public', name='Bert Tartignole')
         cls.user_portal = mail_new_test_user(cls.env, login='chell', groups='base.group_portal', name='Chell Gladys')
 
-        Channel = cls.env['mail.channel'].with_context(common.BaseFunctionalTest._test_context)
+        Channel = cls.env['mail.channel'].with_context(cls._test_context)
         # Pigs: base group for tests
         cls.group_pigs = Channel.create({
             'name': 'Pigs',
@@ -175,22 +206,22 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
         msg6 = self.env['mail.message'].create({
             'subject': '_ZTest', 'body': 'A Birds', 'subtype_id': self.ref('mail.mt_comment'),
             'model': 'mail.channel', 'res_id': self.group_private.id})
-        msg7 = self.env['mail.message'].sudo(self.user_employee).create({
+        msg7 = self.env['mail.message'].with_user(self.user_employee).create({
             'subject': '_ZTest', 'body': 'B', 'subtype_id': self.ref('mail.mt_comment')})
-        msg8 = self.env['mail.message'].sudo(self.user_employee).create({
+        msg8 = self.env['mail.message'].with_user(self.user_employee).create({
             'subject': '_ZTest', 'body': 'B+E', 'subtype_id': self.ref('mail.mt_comment'),
             'partner_ids': [(6, 0, [self.user_employee.partner_id.id])]})
 
         # Test: Public: 2 messages (recipient)
-        messages = self.env['mail.message'].sudo(self.user_public).search([('subject', 'like', '_ZTest')])
+        messages = self.env['mail.message'].with_user(self.user_public).search([('subject', 'like', '_ZTest')])
         self.assertEqual(messages, msg2 | msg4)
 
         # Test: Employee: 3 messages on Pigs Raoul can read (employee can read group with default values)
-        messages = self.env['mail.message'].sudo(self.user_employee).search([('subject', 'like', '_ZTest'), ('body', 'ilike', 'A')])
+        messages = self.env['mail.message'].with_user(self.user_employee).search([('subject', 'like', '_ZTest'), ('body', 'ilike', 'A')])
         self.assertEqual(messages, msg3 | msg4 | msg5)
 
         # Test: Raoul: 3 messages on Pigs Raoul can read (employee can read group with default values), 0 on Birds (private group) + 2 messages as author
-        messages = self.env['mail.message'].sudo(self.user_employee).search([('subject', 'like', '_ZTest')])
+        messages = self.env['mail.message'].with_user(self.user_employee).search([('subject', 'like', '_ZTest')])
         self.assertEqual(messages, msg3 | msg4 | msg5 | msg7 | msg8)
 
         # Test: Admin: all messages
@@ -198,12 +229,12 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
         self.assertEqual(messages, msg1 | msg2 | msg3 | msg4 | msg5 | msg6 | msg7 | msg8)
 
         # Test: Portal: 0 (no access to groups, not recipient)
-        messages = self.env['mail.message'].sudo(self.user_portal).search([('subject', 'like', '_ZTest')])
+        messages = self.env['mail.message'].with_user(self.user_portal).search([('subject', 'like', '_ZTest')])
         self.assertFalse(messages)
 
         # Test: Portal: 2 messages (public group with a subtype)
         self.group_pigs.write({'public': 'public'})
-        messages = self.env['mail.message'].sudo(self.user_portal).search([('subject', 'like', '_ZTest')])
+        messages = self.env['mail.message'].with_user(self.user_portal).search([('subject', 'like', '_ZTest')])
         self.assertEqual(messages, msg4 | msg5)
 
     # --------------------------------------------------
@@ -216,43 +247,42 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
         # which still generate exception in except_orm.So we need to change all
         # except_orm to warning in mail module.)
         with self.assertRaises(except_orm):
-            self.message.sudo(self.user_employee).read()
+            self.message.with_user(self.user_employee).read()
 
     @mute_logger('odoo.models')
     def test_mail_message_access_read_crash_portal(self):
         with self.assertRaises(except_orm):
-            self.message.sudo(self.user_portal).read(['body', 'message_type', 'subtype_id'])
+            self.message.with_user(self.user_portal).read(['body', 'message_type', 'subtype_id'])
 
     def test_mail_message_access_read_ok_portal(self):
         self.message.write({'subtype_id': self.ref('mail.mt_comment'), 'res_id': self.group_public.id})
-        self.message.sudo(self.user_portal).read(['body', 'message_type', 'subtype_id'])
+        self.message.with_user(self.user_portal).read(['body', 'message_type', 'subtype_id'])
 
     def test_mail_message_access_read_notification(self):
         attachment = self.env['ir.attachment'].create({
             'datas': base64.b64encode(b'My attachment'),
-            'name': 'doc.txt',
-            'datas_fname': 'doc.txt'})
+            'name': 'doc.txt'})
         # attach the attachment to the message
         self.message.write({'attachment_ids': [(4, attachment.id)]})
         self.message.write({'partner_ids': [(4, self.user_employee.partner_id.id)]})
-        self.message.sudo(self.user_employee).read()
+        self.message.with_user(self.user_employee).read()
         # Test: Bert has access to attachment, ok because he can read message
-        attachment.sudo(self.user_employee).read(['name', 'datas'])
+        attachment.with_user(self.user_employee).read(['name', 'datas'])
 
     def test_mail_message_access_read_author(self):
         self.message.write({'author_id': self.user_employee.partner_id.id})
-        self.message.sudo(self.user_employee).read()
+        self.message.with_user(self.user_employee).read()
 
     def test_mail_message_access_read_doc(self):
         self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id})
         # Test: Bert reads the message, ok because linked to a doc he is allowed to read
-        self.message.sudo(self.user_employee).read()
+        self.message.with_user(self.user_employee).read()
 
     def test_mail_message_access_read_crash_moderation(self):
         # with self.assertRaises(AccessError):
         self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id, 'moderation_status': 'pending_moderation'})
         # Test: Bert reads the message, ok because linked to a doc he is allowed to read
-        self.message.sudo(self.user_employee).read()
+        self.message.with_user(self.user_employee).read()
 
     # --------------------------------------------------
     # CREATE
@@ -262,22 +292,22 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
     def test_mail_message_access_create_crash_public(self):
         # Do: Bert creates a message on Pigs -> ko, no creation rights
         with self.assertRaises(AccessError):
-            self.env['mail.message'].sudo(self.user_public).create({'model': 'mail.channel', 'res_id': self.group_pigs.id, 'body': 'Test'})
+            self.env['mail.message'].with_user(self.user_public).create({'model': 'mail.channel', 'res_id': self.group_pigs.id, 'body': 'Test'})
 
         # Do: Bert create a message on Jobs -> ko, no creation rights
         with self.assertRaises(AccessError):
-            self.env['mail.message'].sudo(self.user_public).create({'model': 'mail.channel', 'res_id': self.group_public.id, 'body': 'Test'})
+            self.env['mail.message'].with_user(self.user_public).create({'model': 'mail.channel', 'res_id': self.group_public.id, 'body': 'Test'})
 
     @mute_logger('odoo.models')
     def test_mail_message_access_create_crash(self):
         # Do: Bert create a private message -> ko, no creation rights
         with self.assertRaises(except_orm):
-            self.env['mail.message'].sudo(self.user_employee).create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test'})
+            self.env['mail.message'].with_user(self.user_employee).create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test'})
 
     @mute_logger('odoo.models')
     def test_mail_message_access_create_doc(self):
         # TODO Change the except_orm to Warning
-        Message = self.env['mail.message'].sudo(self.user_employee)
+        Message = self.env['mail.message'].with_user(self.user_employee)
         # Do: Raoul creates a message on Jobs -> ok, write access to the related document
         Message.create({'model': 'mail.channel', 'res_id': self.group_public.id, 'body': 'Test'})
         # Do: Raoul creates a message on Priv -> ko, no write access to the related document
@@ -285,11 +315,45 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
             Message.create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test'})
 
     def test_mail_message_access_create_private(self):
-        self.env['mail.message'].sudo(self.user_employee).create({'body': 'Test'})
+        self.env['mail.message'].with_user(self.user_employee).create({'body': 'Test'})
 
     def test_mail_message_access_create_reply(self):
         self.message.write({'partner_ids': [(4, self.user_employee.partner_id.id)]})
-        self.env['mail.message'].sudo(self.user_employee).create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test', 'parent_id': self.message.id})
+        self.env['mail.message'].with_user(self.user_employee).create({'model': 'mail.channel', 'res_id': self.group_private.id, 'body': 'Test', 'parent_id': self.message.id})
+
+    def test_mail_message_access_create_wo_parent_access(self):
+        """ Purpose is to test posting a message on a record whose first message / parent
+        is not accessible by current user. """
+        test_record = self.env['mail.test.simple'].with_context(self._test_context).create({'name': 'Test', 'email_from': 'ignasse@example.com'})
+        message = test_record.message_post(
+            body='<p>This is First Message</p>', subject='Subject',
+            message_type='comment', subtype='mail.mt_note')
+        # portal user have no rights to read the message
+        with self.assertRaises(except_orm):
+            message.with_user(self.user_portal).read(['subject, body'])
+
+        with patch.object(MailTestSimple, 'check_access_rights', return_value=True):
+            with self.assertRaises(except_orm):
+                message.with_user(self.user_portal).read(['subject, body'])
+
+            # parent message is accessible to references notification mail values
+            # for _notify method and portal user have no rights to send the message for this model
+            new_msg = test_record.with_user(self.user_portal).message_post(
+                body='<p>This is Second Message</p>',
+                subject='Subject',
+                parent_id=message.id,
+                partner_ids=[self.user_admin.partner_id.id],
+                message_type='comment',
+                subtype='mail.mt_comment',
+                mail_auto_delete=False)
+
+        new_mail = self.env['mail.mail'].search([
+            ('mail_message_id', '=', new_msg.id),
+            ('references', '=', message.message_id),
+        ])
+
+        self.assertTrue(new_mail)
+        self.assertEqual(new_msg.parent_id, message)
 
     # --------------------------------------------------
     # WRITE
@@ -304,17 +368,17 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
             'moderator_ids': [(4, self.user_employee.id)],
         })
         self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id, 'moderation_status': 'pending_moderation'})
-        self.message.sudo(self.user_employee).write({'moderation_status': 'accepted'})
+        self.message.with_user(self.user_employee).write({'moderation_status': 'accepted'})
 
     def test_mail_message_access_write_crash_moderation(self):
         self.message.write({'model': 'mail.channel', 'res_id': self.group_public.id, 'moderation_status': 'pending_moderation'})
         with self.assertRaises(AccessError):
-            self.message.sudo(self.user_employee).write({'moderation_status': 'accepted'})
+            self.message.with_user(self.user_employee).write({'moderation_status': 'accepted'})
 
     @mute_logger('openerp.addons.mail.models.mail_mail')
     def test_mark_all_as_read(self):
         self.user_employee.notification_type = 'inbox'
-        emp_partner = self.user_employee.partner_id.sudo(self.user_employee.id)
+        emp_partner = self.user_employee.partner_id.with_user(self.user_employee)
 
         group_private = self.env['mail.channel'].with_context({
             'mail_create_nolog': True,
@@ -329,17 +393,19 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
         ).with_context({'mail_create_nosubscribe': False})
 
         # mark all as read clear needactions
-        group_private.message_post(body='Test', message_type='comment', subtype='mail.mt_comment', partner_ids=[emp_partner.id])
-        emp_partner.env['mail.message'].mark_all_as_read(channel_ids=[], domain=[])
+        msg1 = group_private.message_post(body='Test', message_type='comment', subtype='mail.mt_comment', partner_ids=[emp_partner.id])
+        self._clear_bus()
+        emp_partner.env['mail.message'].mark_all_as_read(domain=[])
+        self.assertBusNotification([(self.cr.dbname, 'res.partner', emp_partner.id)], [{ 'type': 'mark_as_read', 'message_ids': [msg1.id] }])
         na_count = emp_partner.get_needaction_count()
         self.assertEqual(na_count, 0, "mark all as read should conclude all needactions")
 
         # mark all as read also clear inaccessible needactions
-        new_msg = group_private.message_post(body='Zest', message_type='comment', subtype='mail.mt_comment', partner_ids=[emp_partner.id])
+        msg2 = group_private.message_post(body='Zest', message_type='comment', subtype='mail.mt_comment', partner_ids=[emp_partner.id])
         needaction_accessible = len(emp_partner.env['mail.message'].search([['needaction', '=', True]]))
         self.assertEqual(needaction_accessible, 1, "a new message to a partner is readable to that partner")
 
-        new_msg.sudo().partner_ids = self.env['res.partner']
+        msg2.sudo().partner_ids = self.env['res.partner']
         emp_partner.env['mail.message'].search([['needaction', '=', True]])
         needaction_length = len(emp_partner.env['mail.message'].search([['needaction', '=', True]]))
         self.assertEqual(needaction_length, 1, "message should still be readable when notified")
@@ -347,18 +413,20 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
         na_count = emp_partner.get_needaction_count()
         self.assertEqual(na_count, 1, "message not accessible is currently still counted")
 
-        emp_partner.env['mail.message'].mark_all_as_read(channel_ids=[], domain=[])
+        self._clear_bus()
+        emp_partner.env['mail.message'].mark_all_as_read(domain=[])
+        self.assertBusNotification([(self.cr.dbname, 'res.partner', emp_partner.id)], [{ 'type': 'mark_as_read', 'message_ids': [msg2.id] }])
         na_count = emp_partner.get_needaction_count()
         self.assertEqual(na_count, 0, "mark all read should conclude all needactions even inacessible ones")
 
     @mute_logger('openerp.addons.mail.models.mail_mail')
     def test_mark_all_as_read_share(self):
         self.user_portal.notification_type = 'inbox'
-        portal_partner = self.user_portal.partner_id.sudo(self.user_portal.id)
+        portal_partner = self.user_portal.partner_id.with_user(self.user_portal)
 
         # mark all as read clear needactions
         self.group_pigs.message_post(body='Test', message_type='comment', subtype='mail.mt_comment', partner_ids=[portal_partner.id])
-        portal_partner.env['mail.message'].mark_all_as_read(channel_ids=[], domain=[])
+        portal_partner.env['mail.message'].mark_all_as_read(domain=[])
         na_count = portal_partner.get_needaction_count()
         self.assertEqual(na_count, 0, "mark all as read should conclude all needactions")
 
@@ -374,7 +442,7 @@ class TestMessageAccess(common.BaseFunctionalTest, common.MockEmails):
         na_count = portal_partner.get_needaction_count()
         self.assertEqual(na_count, 1, "message not accessible is currently still counted")
 
-        portal_partner.env['mail.message'].mark_all_as_read(channel_ids=[], domain=[])
+        portal_partner.env['mail.message'].mark_all_as_read(domain=[])
         na_count = portal_partner.get_needaction_count()
         self.assertEqual(na_count, 0, "mark all read should conclude all needactions even inacessible ones")
 
@@ -396,7 +464,7 @@ class TestMessageModeration(common.Moderation):
         # A pending moderation message needs to have field channel_ids empty. Moderators
         # need to be able to notify a pending moderation message (in a channel they moderate).
         self.assertFalse(self.msg_admin_pending_c1.channel_ids)
-        self.msg_admin_pending_c1.sudo(self.user_employee)._moderate('accept')
+        self.msg_admin_pending_c1.with_user(self.user_employee)._moderate('accept')
         self.assertEqual(self.msg_admin_pending_c1.channel_ids, self.channel_1)
         self.assertEqual(self.msg_admin_pending_c1.moderation_status, 'accepted')
         self.assertEqual(self.msg_admin_pending_c1_2.moderation_status, 'pending_moderation')
@@ -409,7 +477,7 @@ class TestMessageModeration(common.Moderation):
         # need to be able to notify a pending moderation message (in a channel they moderate).
         self.assertFalse(self.msg_admin_pending_c1.channel_ids)
         self.assertFalse(self.msg_admin_pending_c1_2.channel_ids)
-        self.msg_admin_pending_c1.sudo(self.user_employee)._moderate('allow')
+        self.msg_admin_pending_c1.with_user(self.user_employee)._moderate('allow')
         self.assertEqual(self.msg_admin_pending_c1.channel_ids, self.channel_1)
         self.assertEqual(self.msg_admin_pending_c1_2.channel_ids, self.channel_1)
         self.assertEqual(self.msg_admin_pending_c1.moderation_status, 'accepted')
@@ -421,14 +489,14 @@ class TestMessageModeration(common.Moderation):
     @mute_logger('odoo.models.unlink')
     def test_moderate_reject(self):
         self._init_mock_build_email()
-        (self.msg_admin_pending_c1 | self.msg_emp2_pending_c1).sudo(self.user_employee)._moderate_send_reject_email('Title', 'Message to author')
+        (self.msg_admin_pending_c1 | self.msg_emp2_pending_c1).with_user(self.user_employee)._moderate_send_reject_email('Title', 'Message to author')
         self.env['mail.mail'].process_email_queue()
         self.assertEmails(self.partner_employee, self.partner_employee_2 | self.partner_admin, subject='Title', body_content='Message to author')
 
     def test_moderate_discard(self):
         self._clear_bus()
         id1, id2 = self.msg_admin_pending_c1.id, self.msg_emp2_pending_c1.id  # save ids because unlink will discard them
-        (self.msg_admin_pending_c1 | self.msg_emp2_pending_c1).sudo(self.user_employee)._moderate_discard()
+        (self.msg_admin_pending_c1 | self.msg_emp2_pending_c1).with_user(self.user_employee)._moderate_discard()
 
         self.assertBusNotification(
             [(self.cr.dbname, 'res.partner', self.partner_admin.id),
@@ -445,4 +513,4 @@ class TestMessageModeration(common.Moderation):
         msg_emp_pending_c2 = self._create_new_message(self.channel_2.id, status='pending_moderation', author=self.partner_employee)
 
         self.env['mail.message']._notify_moderators()
-        self.assertEmails(False, self.partner_employee | self.partner_employee_2, subject='Message are pending moderation', email_from=self.env.user.company_id.catchall or self.env.user.company_id.email)
+        self.assertEmails(False, self.partner_employee | self.partner_employee_2, subject='Message are pending moderation', email_from=self.env.company.catchall or self.env.company.email)
