@@ -5,6 +5,6 @@ import odoo.tests
 
 @odoo.tests.tagged('post_install', '-at_install')
 class TestUi(odoo.tests.HttpCase):
-    
+
     def test_01_main_flow_tour(self):
-        self.phantom_js("/web", "odoo.__DEBUG__.services['web_tour.tour'].run('main_flow_tour')", "odoo.__DEBUG__.services['web_tour.tour'].tours.main_flow_tour.ready", login="admin", timeout=180)
+        self.start_tour("/web", 'main_flow_tour', login="admin", timeout=180)
