@@ -9,7 +9,8 @@ class MassMailTest(models.Model):
     mailing features and base performances. """
     _description = 'Simple Mass Mailing Model'
     _name = 'mass.mail.test'
-    _inherit = ['mail.thread']
+    _inherit = ['mail.thread', 'mail.address.mixin']
+    _primary_email = 'email_from'
 
     name = fields.Char()
     email_from = fields.Char()
@@ -19,15 +20,15 @@ class MassMailTestBlacklist(models.Model):
     """ Model using blacklist mechanism for mass mailing. """
     _description = 'Mass Mailing Model w Blacklist'
     _name = 'mass.mail.test.bl'
-    _inherit = ['mail.thread', 'mail.blacklist.mixin']
+    _inherit = ['mail.thread.blacklist']
 
-    _primary_email = ['email_from']  # blacklist field to check
+    _primary_email = 'email_from'  # blacklist field to check
 
     name = fields.Char()
     email_from = fields.Char()
     user_id = fields.Many2one(
         'res.users', 'Responsible',
-        track_visibility='onchange')
+        tracking=True)
     umbrella_id = fields.Many2one(
         'mail.test', 'Meta Umbrella Record',
-        track_visibility='onchange')
+        tracking=True)
