@@ -4,7 +4,9 @@
 from . import test_blacklist
 from . import test_blacklist_mixin
 from . import test_composer
+from . import test_link
 from . import test_mail_auto_blacklist
 from . import test_mail_channel
+from . import test_mailing_internals
 from . import test_mass_mailing
 from . import test_performance
