@@ -44,6 +44,18 @@ class TestReflection(common.TransactionCase):
                     relation = self.env['ir.model.relation'].search([('name', '=', field.relation)])
                     self.assertTrue(relation)
                     self.assertIn(relation.model.model, [field.model_name, field.comodel_name])
+                if field.type == 'selection':
+                    selection = [(sel.value, sel.name) for sel in ir_field.selection_ids]
+                    if isinstance(field.selection, list):
+                        self.assertEqual(selection, field.selection)
+                    else:
+                        self.assertEqual(selection, [])
+
+                field_description = field.get_description(self.env)
+                if field.type in ('many2many', 'one2many'):
+                    self.assertFalse(field_description['sortable'])
+                elif field.store and field.column_type:
+                    self.assertTrue(field_description['sortable'])
 
 
 class TestSchema(common.TransactionCase):
