@@ -4,12 +4,11 @@
 {
     'name': 'Bootswatch Theme',
     'description': 'Bootswatch themes',
-    'category': 'Theme/Corporate',
+    'category': 'Theme',
     'sequence': 900,
     'version': '1.0',
     'depends': ['website', 'website_theme_install'],
     'data': [
-        'data/theme_bootswatch_data.xml',
         'views/theme_bootswatch_templates.xml',
     ],
     'images': [
