@@ -15,19 +15,19 @@ var ActionManager = require('web.ActionManager');
 var concurrency = require('web.concurrency');
 var core = require('web.core');
 var config = require('web.config');
-var crash_manager = require('web.crash_manager');
+var WarningDialog = require('web.CrashManager').WarningDialog;
 var data_manager = require('web.data_manager');
-var Dialog = require('web.Dialog');
 var dom = require('web.dom');
 var KeyboardNavigationMixin = require('web.KeyboardNavigationMixin');
 var Loading = require('web.Loading');
+var local_storage = require('web.local_storage');
 var RainbowMan = require('web.RainbowMan');
 var ServiceProviderMixin = require('web.ServiceProviderMixin');
 var session = require('web.session');
+var utils = require('web.utils');
 var Widget = require('web.Widget');
 
 var _t = core._t;
-var qweb = core.qweb;
 
 var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMixin, {
     dependencies: ['notification'],
@@ -62,10 +62,11 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
         },
         load_filters: function (event) {
             return data_manager
-                .load_filters(event.data.dataset, event.data.action_id)
+                .load_filters(event.data)
                 .then(event.data.on_success);
         },
         create_filter: '_onCreateFilter',
+        delete_filter: '_onDeleteFilter',
         push_state: '_onPushState',
         show_effect: '_onShowEffect',
         // session
@@ -79,7 +80,7 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
                 if (event.data.on_success) {
                     event.data.on_success(result);
                 }
-            }).fail(function (result) {
+            }).guardedCatch(function (result) {
                 if (event.data.on_fail) {
                     event.data.on_fail(result);
                 }
@@ -98,7 +99,7 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
         KeyboardNavigationMixin.init.call(this);
         this.origin = undefined;
         this._current_state = null;
-        this.menu_dm = new concurrency.DropMisordered();
+        this.menu_dp = new concurrency.DropPrevious();
         this.action_mutex = new concurrency.Mutex();
         this.set('title_part', {"zopenerp": "Odoo"});
     },
@@ -113,21 +114,41 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
         this.on("change:title_part", this, this._title_changed);
         this._title_changed();
 
+        var state = $.bbq.getState();
+        // If not set on the url, retrieve cids from the local storage
+        // of from the default company on the user
+        var current_company_id = session.user_companies.current_company[0]
+        if (!state.cids) {
+            state.cids = utils.get_cookie('cids') !== null ? utils.get_cookie('cids') : String(current_company_id);
+        }
+        var stateCompanyIDS = _.map(state.cids.split(','), function (cid) { return parseInt(cid) });
+        var userCompanyIDS = _.map(session.user_companies.allowed_companies, function(company) {return company[0]});
+        // Check that the user has access to all the companies
+        if (!_.isEmpty(_.difference(stateCompanyIDS, userCompanyIDS))) {
+            state.cids = String(current_company_id);
+            stateCompanyIDS = [current_company_id]
+        }
+        // Update the user context with this configuration
+        session.user_context.allowed_company_ids = stateCompanyIDS;
+        $.bbq.pushState(state);
+        // Update favicon
+        $("link[type='image/x-icon']").attr('href', '/web/image/res.company/' + String(stateCompanyIDS[0]) + '/favicon/')
+
         return session.is_bound
             .then(function () {
                 self.$el.toggleClass('o_rtl', _t.database.parameters.direction === "rtl");
                 self.bind_events();
-                return $.when(
+                return Promise.all([
                     self.set_action_manager(),
                     self.set_loading()
-                );
+                ]);
             }).then(function () {
                 if (session.session_is_valid()) {
                     return self.show_application();
                 } else {
                     // database manager needs the webclient to keep going even
                     // though it has no valid session
-                    return $.when();
+                    return Promise.resolve();
                 }
             }).then(function () {
                 // Listen to 'scroll' event and propagate it on main bus
@@ -146,12 +167,13 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
         this.$el.on('mouseenter', '.oe_systray > div:not([data-toggle=tooltip])', function () {
             $(this).attr('data-toggle', 'tooltip').tooltip().trigger('mouseenter');
         });
+        // TODO: this handler seems useless since 11.0, should be removed
         this.$el.on('click', '.oe_dropdown_toggle', function (ev) {
             ev.preventDefault();
             var $toggle = $(this);
             var doc_width = $(document).width();
             var $menu = $toggle.siblings('.oe_dropdown_menu');
-            $menu = $menu.size() >= 1 ? $menu : $toggle.find('.oe_dropdown_menu');
+            $menu = $menu.length >= 1 ? $menu : $toggle.find('.oe_dropdown_menu');
             var state = $menu.is('.oe_opened');
             setTimeout(function () {
                 // Do not alter propagation
@@ -167,7 +189,7 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
                 }
             }, 0);
         });
-        window.addEventListener('blur', function (e) { self._hideAccessKeyOverlay(); });
+        window.addEventListener('blur', function (e) {self._hideAccessKeyOverlay(); });
         core.bus.on('click', this, function (ev) {
             $('.tooltip').remove();
             if (!$(ev.target).is('input[type=file]')) {
@@ -178,53 +200,13 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
         });
         core.bus.on('connection_lost', this, this._onConnectionLost);
         core.bus.on('connection_restored', this, this._onConnectionRestored);
-
-        // crash manager integration
-        core.bus.on('rpc_error', crash_manager, crash_manager.rpc_error);
-        window.onerror = function (message, file, line, col, error) {
-            // Scripts injected in DOM (eg: google API's js files) won't return a clean error on window.onerror.
-            // The browser will just give you a 'Script error.' as message and nothing else for security issue.
-            // To enable onerror to work properly with CORS file, you should:
-            //   1. add crossorigin="anonymous" to your <script> tag loading the file
-            //   2. enabling 'Access-Control-Allow-Origin' on the server serving the file.
-            // Since in some case it wont be possible to to this, this handle should have the possibility to be
-            // handled by the script manipulating the injected file. For this, you will use window.onOriginError
-            // If it is not handled, we should display something clearer than the common crash_manager error dialog
-            // since it won't show anything except "Script error."
-            // This link will probably explain it better: https://blog.sentry.io/2016/05/17/what-is-script-error.html
-            if (!file && !line && !col) {
-                // Chrome and Opera set "Script error." on the `message` and hide the `error`
-                // Firefox handles the "Script error." directly. It sets the error thrown by the CORS file into `error`
-                if (window.onOriginError) {
-                    window.onOriginError();
-                    delete window.onOriginError;
-                } else {
-                    crash_manager.show_error({
-                        type: _t("Odoo Client Error"),
-                        message: _t("Unknown CORS error"),
-                        data: {debug: _t("An unknown CORS error occured. The error probably originates from a JavaScript file served from a different origin. (Opening your browser console might give you a hint on the error.)")},
-                    });
-                }
-            } else {
-                // ignore Chrome video internal error: https://crbug.com/809574
-                if (!error && message === 'ResizeObserver loop limit exceeded') {
-                    return;
-                }
-                var traceback = error ? error.stack : '';
-                crash_manager.show_error({
-                    type: _t("Odoo Client Error"),
-                    message: message,
-                    data: {debug: file + ':' + line + "\n" + _t('Traceback:') + "\n" + traceback},
-                });
-            }
-        };
     },
     set_action_manager: function () {
         var self = this;
         this.action_manager = new ActionManager(this, session.user_context);
         var fragment = document.createDocumentFragment();
         return this.action_manager.appendTo(fragment).then(function () {
-            dom.append(self.$('.o_main_content'), fragment, {
+            dom.append(self.$el, fragment, {
                 in_DOM: true,
                 callbacks: [{widget: self.action_manager}],
             });
@@ -329,6 +311,24 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
         this.$el.toggleClass('o_fullscreen', fullscreen);
     },
 
+    //--------------------------------------------------------------------------
+    // Public
+    //--------------------------------------------------------------------------
+
+    /**
+     * Returns the left and top scroll positions of the main scrolling area
+     * (i.e. the '.o_content' div in desktop).
+     *
+     * @returns {Object} with keys left and top
+     */
+    getScrollPosition: function () {
+        var scrollingEl = this.action_manager.el.getElementsByClassName('o_content')[0];
+        return {
+            left: scrollingEl ? scrollingEl.scrollLeft : 0,
+            top: scrollingEl ? scrollingEl.scrollTop : 0,
+        };
+    },
+
     //--------------------------------------------------------------------------
     // Handlers
     //--------------------------------------------------------------------------
@@ -349,7 +349,7 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
      * @private
      */
     _onConnectionLost: function () {
-        this.connectionNotificationID = this.call('notification', 'notify', {
+        this.connectionNotificationID = this.displayNotification({
             title: _t('Connection lost'),
             message: _t('Trying to reconnect...'),
             sticky: true
@@ -363,7 +363,8 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
     _onConnectionRestored: function () {
         if (this.connectionNotificationID) {
             this.call('notification', 'close', this.connectionNotificationID);
-            this.call('notification', 'notify', {
+            this.displayNotification({
+                type: 'info',
                 title: _t('Connection restored'),
                 message: _t('You are back online'),
                 sticky: false
@@ -383,6 +384,18 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
             .create_filter(e.data.filter)
             .then(e.data.on_success);
     },
+    /**
+     * @private
+     * @param {OdooEvent} e
+     * @param {Object} e.data.filter the filter description
+     * @param {function} e.data.on_success called when the RPC succeeds with its
+     *   returned value as argument
+     */
+    _onDeleteFilter: function (e) {
+        data_manager
+            .delete_filter(e.data.filterId)
+            .then(e.data.on_success);
+    },
     /**
      * Displays a warning in a dialog or with the notification service
      *
@@ -397,24 +410,24 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
     _onDisplayWarning: function (e) {
         var data = e.data;
         if (data.type === 'dialog') {
-            new Dialog(this, {
-                size: 'medium',
+            new WarningDialog(this, {
                 title: data.title,
-                $content: qweb.render("CrashManager.warning", data),
-            }).open({shouldFocusButtons: true});
+            }, data).open();
         } else {
-            this.call('notification', 'notify', e.data);
+            data.type = 'warning';
+            this.call('notification', 'notify', data);
         }
     },
     /**
-     * This function must be implemented to provide to the caller the current
-     * scroll position (left and top) of the webclient.
+     * Provides to the caller the current scroll position (left and top) of the
+     * main scrolling area of the webclient.
      *
-     * @abstract
+     * @private
      * @param {OdooEvent} ev
      * @param {function} ev.data.callback
      */
     _onGetScrollPosition: function (ev) {
+        ev.data.callback(this.getScrollPosition());
     },
     /**
      * Loads an action from the database given its ID.
@@ -435,14 +448,13 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
      * @param {OdooEvent} e
      */
     _onPushState: function (e) {
-        this.do_push_state(e.data.state);
+        this.do_push_state(_.extend(e.data.state, {'cids': $.bbq.getState().cids}));
     },
     /**
-     * This function must be implemented by actual webclient to scroll either to
-     * a given offset or to a target element (given a selector).
+     * Scrolls either to a given offset or to a target element (given a selector).
      * It must be called with: trigger_up('scrollTo', options).
      *
-     * @abstract
+     * @private
      * @param {OdooEvent} ev
      * @param {integer} [ev.data.top] the number of pixels to scroll from top
      * @param {integer} [ev.data.left] the number of pixels to scroll from left
@@ -450,6 +462,19 @@ var AbstractWebClient = Widget.extend(ServiceProviderMixin, KeyboardNavigationMi
      *   scroll to
      */
     _onScrollTo: function (ev) {
+        var scrollingEl = this.action_manager.el.getElementsByClassName('o_content')[0];
+        if (!scrollingEl) {
+            return;
+        }
+        var offset = {top: ev.data.top, left: ev.data.left || 0};
+        if (!offset.top) {
+            offset = dom.getPosition(document.querySelector(ev.data.selector));
+            // Substract the position of the scrolling element
+            offset.top -= dom.getPosition(scrollingEl).top;
+        }
+
+        scrollingEl.scrollTop = offset.top;
+        scrollingEl.scrollLeft = offset.left;
     },
     /**
      * @private
