@@ -30,13 +30,10 @@ odoo.define('web.AbstractField', function (require) {
  * @module web.AbstractField
  */
 
-var ajax = require('web.ajax');
 var field_utils = require('web.field_utils');
 var Widget = require('web.Widget');
 
 var AbstractField = Widget.extend({
-    cssLibs: [],
-    jsLibs: [],
     events: {
         'keydown': '_onKeydown',
     },
@@ -73,6 +70,17 @@ var AbstractField = Widget.extend({
      */
     supportedFieldTypes: [],
 
+    /**
+     * To override to give a user friendly name to the widget.
+     *
+     * @type <string>
+     */
+    description: "",
+    /**
+     * Currently only used in list view.
+     * If this flag is set to true, the list column name will be empty.
+     */
+    noLabel: false,
     /**
      * Abstract field class
      *
@@ -167,7 +175,7 @@ var AbstractField = Widget.extend({
         // to send it to the server. These functions are chosen according to
         // the 'widget' attrs if is is given, and if it is a valid key, with a
         // fallback on the field type, ensuring that the value is formatted and
-        // displayed according to the choosen widget, if any.
+        // displayed according to the chosen widget, if any.
         this.formatType = this.attrs.widget in field_utils.format ?
                             this.attrs.widget :
                             this.field.type;
@@ -182,19 +190,11 @@ var AbstractField = Widget.extend({
             this.resetOnAnyFieldChange = true;
         }
     },
-    /**
-     * Loads the libraries listed in this.jsLibs and this.cssLibs
-     *
-     * @override
-     */
-    willStart: function () {
-        return $.when(ajax.loadLibs(this), this._super.apply(this, arguments));
-    },
     /**
      * When a field widget is appended to the DOM, its start method is called,
      * and will automatically call render. Most widgets should not override this.
      *
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     start: function () {
         var self = this;
@@ -246,7 +246,7 @@ var AbstractField = Widget.extend({
      * if the value changed but was not notified.
      *
      * @abstract
-     * @returns {Deferred|undefined}
+     * @returns {Promise|undefined}
      */
     commitChanges: function () {},
     /**
@@ -300,12 +300,12 @@ var AbstractField = Widget.extend({
      *   is optional, and may be used by a widget to share information from the
      *   moment a field change event is triggered to the moment a reset
      *   operation is applied.
-     * @returns {Deferred} A Deferred, which resolves when the widget rendering
+     * @returns {Promise} A promise, which resolves when the widget rendering
      *   is complete
      */
     reset: function (record, event) {
         this._reset(record, event);
-        return this._render() || $.when();
+        return this._render() || Promise.resolve();
     },
     /**
      * Remove the invalid class on a field
@@ -406,7 +406,7 @@ var AbstractField = Widget.extend({
      * synchronous.
      *
      * @private
-     * @returns {Deferred|undefined}
+     * @returns {Promise|undefined}
      */
     _render: function () {
         if (this.attrs.decorations) {
@@ -423,7 +423,7 @@ var AbstractField = Widget.extend({
      * concrete widget.
      *
      * @private
-     * @returns {Deferred|undefined}
+     * @returns {Promise|undefined}
      */
     _renderEdit: function () {
     },
@@ -432,7 +432,7 @@ var AbstractField = Widget.extend({
      * the concrete widget.
      *
      * @private
-     * @returns {Deferred|undefined}
+     * @returns {Promise|undefined}
      */
     _renderReadonly: function () {
     },
@@ -467,13 +467,13 @@ var AbstractField = Widget.extend({
      *   will not notify and not trigger the onchange, even though it was changed.
      * @param {boolean} [options.forceChange=false] if true, the change event will be
      *   triggered even if the new value is the same as the old one
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _setValue: function (value, options) {
         // we try to avoid doing useless work, if the value given has not
         // changed.  Note that we compare the unparsed values.
         if (this.lastSetValue === value || (this.value === false && value === '')) {
-            return $.when();
+            return Promise.resolve();
         }
         this.lastSetValue = value;
         try {
@@ -482,25 +482,26 @@ var AbstractField = Widget.extend({
         } catch (e) {
             this._isValid = false;
             this.trigger_up('set_dirty', {dataPointID: this.dataPointID});
-            return $.Deferred().reject();
+            return Promise.reject({message: "Value set is not valid"});
         }
         if (!(options && options.forceChange) && this._isSameValue(value)) {
-            return $.when();
+            return Promise.resolve();
         }
-        var def = $.Deferred();
-        var changes = {};
-        changes[this.name] = value;
-        this.trigger_up('field_changed', {
-            dataPointID: this.dataPointID,
-            changes: changes,
-            viewType: this.viewType,
-            doNotSetDirty: options && options.doNotSetDirty,
-            notifyChange: !options || options.notifyChange !== false,
-            allowWarning: options && options.allowWarning,
-            onSuccess: def.resolve.bind(def),
-            onFailure: def.reject.bind(def),
+        var self = this;
+        return new Promise(function (resolve, reject) {
+            var changes = {};
+            changes[self.name] = value;
+            self.trigger_up('field_changed', {
+                dataPointID: self.dataPointID,
+                changes: changes,
+                viewType: self.viewType,
+                doNotSetDirty: options && options.doNotSetDirty,
+                notifyChange: !options || options.notifyChange !== false,
+                allowWarning: options && options.allowWarning,
+                onSuccess: resolve,
+                onFailure: reject,
+            });
         });
-        return def;
     },
 
     //--------------------------------------------------------------------------
@@ -532,6 +533,12 @@ var AbstractField = Widget.extend({
                 }
                 break;
             case $.ui.keyCode.ENTER:
+                // We preventDefault the ENTER key because of two coexisting behaviours:
+                // - In HTML5, pressing ENTER on a <button> triggers two events: a 'keydown' AND a 'click'
+                // - When creating and opening a dialog, the focus is automatically given to the primary button
+                // The end result caused some issues where a modal opened by an ENTER keypress (e.g. saving
+                // changes in multiple edition) confirmed the modal without any intentionnal user input.
+                ev.preventDefault();
                 ev.stopPropagation();
                 this.trigger_up('navigation_move', {direction: 'next_line'});
                 break;
