@@ -1,10 +1,10 @@
 odoo.define('web.clickEverywhere', function (require) {
     "use strict";
     var ajax = require('web.ajax');
-    function startClickEverywhere() {
+    function startClickEverywhere(menu_id) {
         ajax.loadJS('web/static/src/js/tools/test_menus.js').then(
             function() {
-                clickEverywhere();
+                clickEverywhere(menu_id);
             }
         );
     }
