@@ -2,9 +2,9 @@ odoo.define('web.FormController', function (require) {
 "use strict";
 
 var BasicController = require('web.BasicController');
-var dialogs = require('web.view_dialogs');
 var core = require('web.core');
 var Dialog = require('web.Dialog');
+var dialogs = require('web.view_dialogs');
 var Sidebar = require('web.Sidebar');
 
 var _t = core._t;
@@ -14,15 +14,12 @@ var FormController = BasicController.extend({
     custom_events: _.extend({}, BasicController.prototype.custom_events, {
         bounce_edit: '_onBounceEdit',
         button_clicked: '_onButtonClicked',
-        do_action: '_onDoAction',
         edited_list: '_onEditedList',
         open_one2many_record: '_onOpenOne2ManyRecord',
         open_record: '_onOpenRecord',
         toggle_column_order: '_onToggleColumnOrder',
         focus_control_button: '_onFocusControlButton',
         form_dialog_discarded: '_onFormDialogDiscarded',
-        swipe_left: '_onSwipeLeft',
-        swipe_right: '_onSwipeRight',
     }),
     /**
      * @override
@@ -41,20 +38,33 @@ var FormController = BasicController.extend({
         this.toolbarActions = params.toolbarActions || {};
     },
     /**
-     * Force mode back to readonly. Whenever we leave a form view, it is saved,
-     * and should no longer be in edit mode.
+     * Called each time the form view is attached into the DOM
+     *
+     * @todo convert to new style
+     */
+    on_attach_callback: function () {
+        this._super.apply(this, arguments);
+        this.autofocus();
+    },
+    /**
+     * This hook is called when a form view is restored (by clicking on the
+     * breadcrumbs). In general, we force mode back to readonly, because
+     * whenever we leave a form view by stacking another action on the top of
+     * it, it is saved, and should no longer be in edit mode. However, there is
+     * a special case for new records for which we still want to be in 'edit'
+     * as no record has been created (changes have been discarded before
+     * leaving).
      *
      * @override
      */
     willRestore: function () {
-        this.mode = 'readonly';
+        this.mode = this.model.isNew(this.handle) ? 'edit' : 'readonly';
     },
 
     //--------------------------------------------------------------------------
     // Public
     //--------------------------------------------------------------------------
 
-
     /**
      * Calls autofocus on the renderer
      */
@@ -76,7 +86,7 @@ var FormController = BasicController.extend({
      * @todo make record creation a basic controller feature
      * @param {string} [parentID] if given, the parentID will be used as parent
      *                            for the new record.
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     createRecord: function (parentID) {
         var self = this;
@@ -115,15 +125,6 @@ var FormController = BasicController.extend({
     getTitle: function () {
         return this.model.getName(this.handle);
     },
-    /**
-     * Called each time the form view is attached into the DOM
-     *
-     * @todo convert to new style
-     */
-    on_attach_callback: function () {
-        this._super.apply(this, arguments);
-        this.autofocus();
-    },
     /**
      * Render buttons for the control panel.  The form view can be rendered in
      * a dialog, and in that case, if we have buttons defined in the footer, we
@@ -133,7 +134,7 @@ var FormController = BasicController.extend({
      * @param {jQueryElement} $node
      */
     renderButtons: function ($node) {
-        var $footer = this.footerToButtons ? this.$('footer') : null;
+        var $footer = this.footerToButtons ? this.renderer.$('footer') : null;
         var mustRenderFooterButtons = $footer && $footer.length;
         if (!this.defaultButtons && !mustRenderFooterButtons) {
             return;
@@ -166,22 +167,43 @@ var FormController = BasicController.extend({
      * @override method from BasicController
      * @param {jQueryElement} $node
      * @param {Object} options
+     * @returns {Promise}
      */
     renderPager: function ($node, options) {
         options = _.extend({}, options, {
             validate: this.canBeDiscarded.bind(this),
         });
-        this._super($node, options);
+        return this._super($node, options);
     },
     /**
      * Instantiate and render the sidebar if a sidebar is requested
      * Sets this.sidebar
      * @param {jQuery} [$node] a jQuery node where the sidebar should be
      *   inserted
+     * @return {Promise}
      **/
     renderSidebar: function ($node) {
+        var self = this;
         if (this.hasSidebar) {
             var otherItems = [];
+            if (this.archiveEnabled && this.initialState.data.active !== undefined) {
+                var classname = "o_sidebar_item_archive" + (this.initialState.data.active ? "" : " o_hidden")
+                otherItems.push({
+                    label: _t("Archive"),
+                    callback: function () {
+                        Dialog.confirm(self, _t("Are you sure that you want to archive this record?"), {
+                            confirm_callback: self._toggleArchiveState.bind(self, true),
+                        });
+                    },
+                    classname: classname,
+                });
+                classname = "o_sidebar_item_unarchive" + (this.initialState.data.active ? " o_hidden" : "")
+                otherItems.push({
+                    label: _t("Unarchive"),
+                    callback: this._toggleArchiveState.bind(this, false),
+                    classname: classname,
+                });
+            }
             if (this.is_action_enabled('delete')) {
                 otherItems.push({
                     label: _t('Delete'),
@@ -204,11 +226,12 @@ var FormController = BasicController.extend({
                 },
                 actions: _.extend(this.toolbarActions, {other: otherItems}),
             });
-            this.sidebar.appendTo($node);
-
-            // Show or hide the sidebar according to the view mode
-            this._updateSidebar();
+            return this.sidebar.appendTo($node).then(function() {
+                 // Show or hide the sidebar according to the view mode
+                self._updateSidebar();
+            });
         }
+        return Promise.resolve();
     },
     /**
      * Show a warning message if the user modified a translated field.  For each
@@ -220,7 +243,7 @@ var FormController = BasicController.extend({
         var self = this;
         return this._super.apply(this, arguments).then(function (changedFields) {
             // the title could have been changed
-            self.set('title', self.getTitle());
+            self._setTitle(self.getTitle());
             self._updateEnv();
 
             if (_t.database.multi_lang && changedFields.length) {
@@ -232,7 +255,9 @@ var FormController = BasicController.extend({
                 for (var k = 0; k < changedFields.length; k++) {
                     var field = fields[changedFields[k]];
                     var fieldData = data[changedFields[k]];
-                    if (field.translate && fieldData) {
+                    // An empty HTML field will always contain at least '<p><br></p>'. Do not
+                    // suggest a translation in this case.
+                    if (field.translate && fieldData && fieldData !== '<p><br></p>') {
                         alertFields[changedFields[k]] = field;
                     }
                 }
@@ -250,6 +275,9 @@ var FormController = BasicController.extend({
      * @override
      */
     update: function (params, options) {
+        if ('currentId' in params && !params.currentId) {
+            this.mode = 'edit'; // if there is no record, we are in 'edit' mode
+        }
         params = _.extend({viewType: 'form', mode: this.mode}, params);
         return this._super(params, options);
     },
@@ -257,6 +285,39 @@ var FormController = BasicController.extend({
     //--------------------------------------------------------------------------
     // Private
     //--------------------------------------------------------------------------
+
+    /**
+     * @override
+     */
+    _applyChanges: async function () {
+        const result = await this._super.apply(this, arguments);
+        core.bus.trigger('DOM_updated');
+        return result;
+    },
+
+    /**
+     * Archive the current selection
+     *
+     * @private
+     * @param {string[]} ids
+     * @param {boolean} archive
+     * @returns {Promise}
+     */
+    _archive: function (ids, archive) {
+        if (ids.length === 0) {
+            return Promise.resolve();
+        }
+        if (archive) {
+            return  this.model
+            .actionArchive(ids, this.handle)
+            .then(this.update.bind(this, {}, {reload: false}));
+        } else {
+            return this.model
+            .actionUnarchive(ids, this.handle)
+            .then(this.update.bind(this, {}, {reload: false}));
+        }
+    },
+
     /**
      * Assign on the buttons save and discard additionnal behavior to facilitate
      * the work of the users doing input only using the keyboard
@@ -267,18 +328,18 @@ var FormController = BasicController.extend({
      */
     _assignSaveCancelKeyboardBehavior: function ($saveCancelButtonContainer) {
         var self = this;
-        $saveCancelButtonContainer.children().on('keydown', function(e) {
+        $saveCancelButtonContainer.children().on('keydown', function (e) {
             switch(e.which) {
                 case $.ui.keyCode.ENTER:
                     e.preventDefault();
-                    self.saveRecord.apply(self);
+                    self.saveRecord();
                     break;
                 case $.ui.keyCode.ESCAPE:
                     e.preventDefault();
-                    self._discardChanges.apply(self);
+                    self._discardChanges();
                     break;
                 case $.ui.keyCode.TAB:
-                    if (!e.shiftKey && e.target.classList.contains("btn-primary")) {
+                    if (!e.shiftKey && e.target.classList.contains('btn-primary')) {
                         $saveCancelButtonContainer.tooltip('show');
                         e.preventDefault();
                     }
@@ -293,7 +354,7 @@ var FormController = BasicController.extend({
      * @private
      * @override method from field manager mixin
      * @param {string} id - id of the previously changed record
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _confirmSave: function (id) {
         if (id === this.handle) {
@@ -340,6 +401,15 @@ var FormController = BasicController.extend({
         this._super.apply(this, arguments);
         this.renderer.enableButtons();
     },
+    /**
+     * Only display the pager if we are not on a new record.
+     *
+     * @override
+     * @private
+     */
+    _isPagerVisible: function () {
+        return !this.model.isNew(this.handle);
+    },
     /**
      * Hook method, called when record(s) has been deleted.
      *
@@ -403,14 +473,18 @@ var FormController = BasicController.extend({
      * @override
      * @private
      * @param {Object} state
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _update: function () {
-        var title = this.getTitle();
-        this.set('title', title);
-        this._updateButtons();
-        this._updateSidebar();
-        return this._super.apply(this, arguments).then(this.autofocus.bind(this));
+        var self = this;
+
+        return this._super.apply(this, arguments).then(function() {
+            var title = self.getTitle();
+            self._setTitle(title);
+            self._updateButtons();
+            self._updateSidebar();
+            self.autofocus();
+        });
     },
     /**
      * @private
@@ -418,7 +492,7 @@ var FormController = BasicController.extend({
     _updateButtons: function () {
         if (this.$buttons) {
             if (this.footerToButtons) {
-                var $footer = this.$('footer');
+                var $footer = this.renderer.$('footer');
                 if ($footer.length) {
                     this.$buttons.empty().append($footer);
                 }
@@ -437,6 +511,25 @@ var FormController = BasicController.extend({
     _updateSidebar: function () {
         if (this.sidebar) {
             this.sidebar.do_toggle(this.mode === 'readonly');
+            // Hide/Show Archive/Unarchive dropdown items
+            // We could have toggled the o_hidden class on the
+            // item theirselves, but the items are redrawed
+            // at each update, based on the initial definition
+            var archive_item = _.find(this.sidebar.items.other, function(item) {
+                return item.classname && item.classname.includes('o_sidebar_item_archive')
+            })
+            var unarchive_item = _.find(this.sidebar.items.other, function(item) {
+                return item.classname && item.classname.includes('o_sidebar_item_unarchive')
+            })
+            if (archive_item && unarchive_item) {
+                if (this.renderer.state.data.active) {
+                    archive_item.classname = 'o_sidebar_item_archive';
+                    unarchive_item.classname = 'o_sidebar_item_unarchive o_hidden';
+                } else {
+                    archive_item.classname = 'o_sidebar_item_archive o_hidden';
+                    unarchive_item.classname = 'o_sidebar_item_unarchive';
+                }
+            }
         }
     },
 
@@ -456,12 +549,12 @@ var FormController = BasicController.extend({
     },
     /**
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onButtonClicked: function (event) {
+    _onButtonClicked: function (ev) {
         // stop the event's propagation as a form controller might have other
         // form controllers in its descendants (e.g. in a FormViewDialog)
-        event.stopPropagation();
+        ev.stopPropagation();
         var self = this;
         var def;
 
@@ -474,27 +567,35 @@ var FormController = BasicController.extend({
                 // we need to reget the record to make sure we have changes made
                 // by the basic model, such as the new res_id, if the record is
                 // new.
-                var record = self.model.get(event.data.record.id);
+                var record = self.model.get(ev.data.record.id);
                 return self._callButtonAction(attrs, record);
             });
         }
-        var attrs = event.data.attrs;
+        var attrs = ev.data.attrs;
         if (attrs.confirm) {
-            var d = $.Deferred();
-            Dialog.confirm(this, attrs.confirm, {
-                confirm_callback: saveAndExecuteAction,
-            }).on("closed", null, function () {
-                d.resolve();
+            def = new Promise(function (resolve, reject) {
+                Dialog.confirm(this, attrs.confirm, {
+                    confirm_callback: saveAndExecuteAction,
+                }).on("closed", null, resolve);
             });
-            def = d.promise();
         } else if (attrs.special === 'cancel') {
-            def = this._callButtonAction(attrs, event.data.record);
+            def = this._callButtonAction(attrs, ev.data.record);
         } else if (!attrs.special || attrs.special === 'save') {
             // save the record but don't switch to readonly mode
             def = saveAndExecuteAction();
+        } else {
+            console.warn('Unhandled button event', ev);
+            return;
         }
 
-        def.always(this._enableButtons.bind(this));
+        // Kind of hack for FormViewDialog: button on footer should trigger the dialog closing
+        // if the `close` attribute is set
+        def.then(function () {
+            self._enableButtons();
+            if (attrs.close) {
+                self.trigger_up('close_dialog');
+            }
+        }).guardedCatch(this._enableButtons.bind(this));
     },
     /**
      * Called when the user wants to create a new record -> @see createRecord
@@ -521,26 +622,6 @@ var FormController = BasicController.extend({
     _onDiscard: function () {
         this._discardChanges();
     },
-    /**
-     * Destroy subdialog widgets after an action is finished.
-     *
-     * @param {OdooEvent} event
-     * @private
-     */
-    _onDoAction: function (event) {
-        var self=this;
-        // A priori, different widgets could write on the "on_success" key.
-        // Below we ensure that all the actions required by those widgets
-        // are executed in a suitable order before every cycle of destruction.
-        var callback = event.data.on_success || function () {};
-        event.data.on_success = function () {
-            callback();
-            function isDialog (widget) {
-                return (widget instanceof Dialog);
-            }
-            _.invoke(self.getChildren().filter(isDialog), 'destroy');
-        };
-    },
     /**
      * Called when the user clicks on 'Duplicate Record' in the sidebar
      *
@@ -561,7 +642,9 @@ var FormController = BasicController.extend({
      * @private
      */
     _onEdit: function () {
-        this._setMode('edit');
+        // wait for potential pending changes to be saved (done with widgets
+        // allowing to edit in readonly)
+        this.mutex.getUnlockedDef().then(this._setMode.bind(this, 'edit'));
     },
     /**
      * This method is called when someone tries to freeze the order, most likely
@@ -596,9 +679,12 @@ var FormController = BasicController.extend({
      * @private
      * @param {OdooEvent} event
      */
-    _onFormDialogDiscarded: function(e) {
-        e.stopPropagation();
-        this.renderer.focusLastActivatedWidget();
+    _onFormDialogDiscarded: function(ev) {
+        ev.stopPropagation();
+        var isFocused = this.renderer.focusLastActivatedWidget();
+        if (ev.data.callback) {
+            ev.data.callback(_.str.toBool(isFocused));
+        }
     },
     /**
      * Opens a one2many record (potentially new) in a dialog. This handler is
@@ -610,16 +696,19 @@ var FormController = BasicController.extend({
      * with the one of the form view.
      *
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onOpenOne2ManyRecord: function (event) {
-        event.stopPropagation();
-        var data = event.data;
+    _onOpenOne2ManyRecord: async function (ev) {
+        ev.stopPropagation();
+        var data = ev.data;
         var record;
         if (data.id) {
             record = this.model.get(data.id, {raw: true});
         }
 
+        // Sync with the mutex to wait for potential onchanges
+        await this.model.mutex.getUnlockedDef();
+
         new dialogs.FormViewDialog(this, {
             context: data.context,
             domain: data.domain,
@@ -629,34 +718,34 @@ var FormController = BasicController.extend({
             on_remove: data.on_remove,
             parentID: data.parentID,
             readonly: data.readonly,
-            deletable: data.deletable,
+            deletable: record ? data.deletable : false,
             recordID: record && record.id,
             res_id: record && record.res_id,
             res_model: data.field.relation,
             shouldSaveLocally: true,
-            title: (record ? _t("Open: ") : _t("Create ")) + (event.target.string || data.field.string),
+            title: (record ? _t("Open: ") : _t("Create ")) + (ev.target.string || data.field.string),
         }).open();
     },
     /**
      * Open an existing record in a form view dialog
      *
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onOpenRecord: function (event) {
-        event.stopPropagation();
+    _onOpenRecord: function (ev) {
+        ev.stopPropagation();
         var self = this;
-        var record = this.model.get(event.data.id, {raw: true});
+        var record = this.model.get(ev.data.id, {raw: true});
         new dialogs.FormViewDialog(self, {
-            context: event.data.context,
-            fields_view: event.data.fields_view,
-            on_saved: event.data.on_saved,
-            on_remove: event.data.on_remove,
-            readonly: event.data.readonly,
-            deletable: event.data.deletable,
+            context: ev.data.context,
+            fields_view: ev.data.fields_view,
+            on_saved: ev.data.on_saved,
+            on_remove: ev.data.on_remove,
+            readonly: ev.data.readonly,
+            deletable: ev.data.deletable,
             res_id: record.res_id,
             res_model: record.model,
-            title: _t("Open: ") + event.data.string,
+            title: _t("Open: ") + ev.data.string,
         }).open();
     },
     /**
@@ -669,50 +758,33 @@ var FormController = BasicController.extend({
         ev.stopPropagation(); // Prevent x2m lines to be auto-saved
         var self = this;
         this._disableButtons();
-        this.saveRecord().always(function () {
-            self._enableButtons();
-        });
-    },
-    /**
-     * Called when user swipes left. Move to next record.
-     *
-     * @private
-     * @param {OdooEvent} ev
-     */
-    _onSwipeLeft: function (ev) {
-        ev.stopPropagation();
-        if (this.pager) {
-            this.pager.next();
-        }
-    },
-    /**
-     * Called when user swipes right. Move to previous record.
-     *
-     * @private
-     * @param {OdooEvent} ev
-     */
-    _onSwipeRight: function (ev) {
-        ev.stopPropagation();
-        if (this.pager) {
-            this.pager.previous();
-        }
+        this.saveRecord().then(this._enableButtons.bind(this)).guardedCatch(this._enableButtons.bind(this));
     },
     /**
      * This method is called when someone tries to sort a column, most likely
      * in a x2many list view
      *
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onToggleColumnOrder: function (event) {
-        event.stopPropagation();
+    _onToggleColumnOrder: function (ev) {
+        ev.stopPropagation();
         var self = this;
-        this.model.setSort(event.data.id, event.data.name).then(function () {
-            var field = event.data.field;
+        this.model.setSort(ev.data.id, ev.data.name).then(function () {
+            var field = ev.data.field;
             var state = self.model.get(self.handle);
             self.renderer.confirmChange(state, state.id, [field]);
         });
     },
+    /**
+     * Called when clicking on 'Archive' or 'Unarchive' in the sidebar.
+     *
+     * @private
+     * @param {boolean} archive
+     */
+    _toggleArchiveState: function (archive) {
+        this._archive([this.handle], archive);
+    },
 });
 
 return FormController;
