@@ -10,6 +10,7 @@ odoo.define('web.KanbanController', function (require) {
 var BasicController = require('web.BasicController');
 var Context = require('web.Context');
 var core = require('web.core');
+var Dialog = require('web.Dialog');
 var Domain = require('web.Domain');
 var view_dialogs = require('web.view_dialogs');
 var viewUtils = require('web.viewUtils');
@@ -18,6 +19,7 @@ var _t = core._t;
 var qweb = core.qweb;
 
 var KanbanController = BasicController.extend({
+    buttons_template: 'KanbanView.buttons',
     custom_events: _.extend({}, BasicController.prototype.custom_events, {
         quick_create_add_column: '_onAddColumn',
         quick_create_record: '_onQuickCreateRecord',
@@ -33,15 +35,20 @@ var KanbanController = BasicController.extend({
         column_toggle_fold: '_onToggleColumn',
         kanban_column_records_toggle_active: '_onToggleActiveRecords',
     }),
+    events: _.extend({}, BasicController.prototype.events, {
+        click: '_onClick',
+    }),
     /**
      * @override
      * @param {Object} params
      * @param {boolean} params.quickCreateEnabled set to false to disable the
      *   quick create feature
+     * @param {SearchPanel} [params.searchPanel]
+     * @param {Array[]} [params.controlPanelDomain=[]] initial domain coming
+     *   from the controlPanel
      */
     init: function (parent, model, renderer, params) {
         this._super.apply(this, arguments);
-
         this.on_create = params.on_create;
         this.hasButtons = params.hasButtons;
         this.quickCreateEnabled = params.quickCreateEnabled;
@@ -53,18 +60,20 @@ var KanbanController = BasicController.extend({
 
     /**
      * @param {jQueryElement} $node
+     * @returns {Promise}
      */
     renderButtons: function ($node) {
         if (this.hasButtons && this.is_action_enabled('create')) {
-            this.$buttons = $(qweb.render('KanbanView.buttons', {
+            this.$buttons = $(qweb.render(this.buttons_template, {
                 btnClass: 'btn-primary',
                 widget: this,
             }));
             this.$buttons.on('click', 'button.o-kanban-button-new', this._onButtonNew.bind(this));
-            this.$buttons.on('keydown',this._onButtonsKeyDown.bind(this));
+            this.$buttons.on('keydown', this._onButtonsKeyDown.bind(this));
             this._updateButtons();
-            this.$buttons.appendTo($node);
+            return Promise.resolve(this.$buttons.appendTo($node));
         }
+        return Promise.resolve();
     },
 
     //--------------------------------------------------------------------------
@@ -75,7 +84,7 @@ var KanbanController = BasicController.extend({
      * @override method comes from field manager mixin
      * @private
      * @param {string} id local id from the basic record data
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _confirmSave: function (id) {
         var data = this.model.get(this.handle, {raw: true});
@@ -86,6 +95,16 @@ var KanbanController = BasicController.extend({
         }
         return this.renderer.updateRecord(this.model.get(id));
     },
+    /**
+     * Only display the pager in the ungrouped case, with data.
+     *
+     * @override
+     * @private
+     */
+    _isPagerVisible: function () {
+        var state = this.model.get(this.handle, {raw: true});
+        return !!(state.count && !state.groupedBy.length);
+    },
     /**
      * @private
      * @param {Widget} kanbanRecord
@@ -148,7 +167,7 @@ var KanbanController = BasicController.extend({
     /**
      * @param {number[]} ids
      * @private
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _resequenceColumns: function (ids) {
         var state = this.model.get(this.handle, {raw: true});
@@ -163,7 +182,7 @@ var KanbanController = BasicController.extend({
      * @private
      * @param {string} column_id
      * @param {string[]} ids
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _resequenceRecords: function (column_id, ids) {
         var self = this;
@@ -206,12 +225,12 @@ var KanbanController = BasicController.extend({
      * a group and to update the renderer
      *
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onAddColumn: function (event) {
+    _onAddColumn: function (ev) {
         var self = this;
         this.mutex.exec(function () {
-            return self.model.createGroup(event.data.value, self.handle).then(function () {
+            return self.model.createGroup(ev.data.value, self.handle).then(function () {
                 var state = self.model.get(self.handle, {raw: true});
                 var ids = _.pluck(state.data, 'res_id').filter(_.isNumber);
                 return self._resequenceColumns(ids);
@@ -220,36 +239,39 @@ var KanbanController = BasicController.extend({
             }).then(function () {
                 self._updateButtons();
                 self.renderer.quickCreateToggleFold();
+                self.renderer.trigger_up("quick_create_column_created");
             });
         });
     },
     /**
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onAddRecordToColumn: function (event) {
+    _onAddRecordToColumn: function (ev) {
         var self = this;
-        var record = event.data.record;
-        var column = event.target;
+        var record = ev.data.record;
+        var column = ev.target;
         this.alive(this.model.moveRecord(record.db_id, column.db_id, this.handle))
             .then(function (column_db_ids) {
-                return self._resequenceRecords(column.db_id, event.data.ids)
+                return self._resequenceRecords(column.db_id, ev.data.ids)
                     .then(function () {
                         _.each(column_db_ids, function (db_id) {
                             var data = self.model.get(db_id);
                             self.renderer.updateColumn(db_id, data);
                         });
                     });
-            }).fail(this.reload.bind(this));
+            }).guardedCatch(this.reload.bind(this));
     },
     /**
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onButtonClicked: function (event) {
-        event.stopPropagation();
-        var attrs = event.data.attrs;
-        var record = event.data.record;
+    _onButtonClicked: function (ev) {
+        var self = this;
+        ev.stopPropagation();
+        var attrs = ev.data.attrs;
+        var record = ev.data.record;
+        var def = Promise.resolve();
         if (attrs.context) {
             attrs.context = new Context(attrs.context)
                 .set_eval_context({
@@ -258,15 +280,25 @@ var KanbanController = BasicController.extend({
                     active_model: record.model,
                 });
         }
-        this.trigger_up('execute_action', {
-            action_data: attrs,
-            env: {
-                context: record.getContext(),
-                currentID: record.res_id,
-                model: record.model,
-                resIDs: record.res_ids,
-            },
-            on_closed: this._reloadAfterButtonClick.bind(this, event.target, event.data),
+        if (attrs.confirm) {
+            def = new Promise(function (resolve, reject) {
+                Dialog.confirm(this, attrs.confirm, {
+                    confirm_callback: resolve,
+                    cancel_callback: reject,
+                }).on("closed", null, reject);
+            });
+        }
+        def.then(function () {
+            self.trigger_up('execute_action', {
+                action_data: attrs,
+                env: {
+                    context: record.getContext(),
+                    currentID: record.res_id,
+                    model: record.model,
+                    resIDs: record.res_ids,
+                },
+                on_closed: self._reloadAfterButtonClick.bind(self, ev.target, ev.data),
+            });
         });
     },
     /**
@@ -286,7 +318,7 @@ var KanbanController = BasicController.extend({
         } else if (this.on_create && this.on_create !== 'quick_create') {
             // Execute the given action
             this.do_action(this.on_create, {
-                on_close: this.reload.bind(this),
+                on_close: this.reload.bind(this, {}),
                 additional_context: state.context,
             });
         } else {
@@ -301,39 +333,51 @@ var KanbanController = BasicController.extend({
      * Moves the focus from the controller buttons to the first kanban record
      *
      * @private
-     * @param {jQueryEvent} e
+     * @param {jQueryEvent} ev
      */
-    _onButtonsKeyDown: function (e) {
-        switch(e.keyCode) {
+    _onButtonsKeyDown: function (ev) {
+        switch(ev.keyCode) {
             case $.ui.keyCode.DOWN:
                 this.$('.o_kanban_record:first').focus();
         }
     },
     /**
+     * Bounce the 'Create' button.
+     *
      * @private
-     * @param {OdooEvent} event
-     */
-    _onColumnResequence: function (event) {
-        this._resequenceRecords(event.target.db_id, event.data.ids);
+     * @param {MouseEvent} ev
+     **/
+    _onClick: function (ev) {
+        var state = this.model.get(this.handle, {raw: true});
+        if (!state.count && this.buttons) {
+            var classesList = ['o_kanban_view', 'o_kanban_group', 'o_column_quick_create', 'o_view_nocontent_smiling_face'];
+            var $target = $(ev.target);
+            var hasClassList = _.map(classesList, function(klass){ return $target.hasClass(klass) });
+            if (_.some(hasClassList)) {
+                this.$buttons.find('.o-kanban-button-new').odooBounce();
+            }
+        }
     },
     /**
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onDeleteColumn: function (event) {
+    _onColumnResequence: function (ev) {
+        this._resequenceRecords(ev.target.db_id, ev.data.ids);
+    },
+    /**
+     * @private
+     * @param {OdooEvent} ev
+     */
+    _onDeleteColumn: function (ev) {
         var self = this;
-        var column = event.target;
+        var column = ev.target;
         var state = this.model.get(this.handle, {raw: true});
         var relatedModelName = state.fields[state.groupedBy[0]].relation;
         this.model
             .deleteRecords([column.db_id], relatedModelName)
-            .done(function () {
-                if (column.isEmpty()) {
-                    self.renderer.removeWidget(column);
-                    self._updateButtons();
-                } else {
-                    self.reload();
-                }
+            .then(function () {
+                self.update({}, {reload: !column.isEmpty()});
             });
     },
     /**
@@ -341,26 +385,27 @@ var KanbanController = BasicController.extend({
      * lazy loaded)
      *
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onLoadColumnRecords: function (event) {
+    _onLoadColumnRecords: function (ev) {
         var self = this;
-        this.model.loadColumnRecords(event.data.columnID).then(function (dbID) {
+        this.model.loadColumnRecords(ev.data.columnID).then(function (dbID) {
             var data = self.model.get(dbID);
-            self.renderer.updateColumn(dbID, data);
-            self._updateEnv();
-            if (event.data.onSuccess) {
-                event.data.onSuccess();
-            }
+            return self.renderer.updateColumn(dbID, data).then(function() {
+                self._updateEnv();
+                if (ev.data.onSuccess) {
+                    ev.data.onSuccess();
+                }
+            });
         });
     },
     /**
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onLoadMore: function (event) {
+    _onLoadMore: function (ev) {
         var self = this;
-        var column = event.target;
+        var column = ev.target;
         this.model.loadMore(column.db_id).then(function (db_id) {
             var data = self.model.get(db_id);
             self.renderer.updateColumn(db_id, data);
@@ -402,8 +447,8 @@ var KanbanController = BasicController.extend({
 
         this.model.createRecordInGroup(column.db_id, values)
             .then(update)
-            .fail(function (error, ev) {
-                ev.preventDefault();
+            .guardedCatch(function (reason) {
+                reason.event.preventDefault();
                 var columnState = self.model.get(column.db_id, {raw: true});
                 var context = columnState.getContext();
                 var state = self.model.get(self.handle, {raw: true});
@@ -423,34 +468,34 @@ var KanbanController = BasicController.extend({
     },
     /**
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onRecordDelete: function (event) {
-        this._deleteRecords([event.data.id]);
+    _onRecordDelete: function (ev) {
+        this._deleteRecords([ev.data.id]);
     },
     /**
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onResequenceColumn: function (event) {
+    _onResequenceColumn: function (ev) {
         var self = this;
-        this._resequenceColumns(event.data.ids).then(function () {
+        this._resequenceColumns(ev.data.ids).then(function () {
             self._updateEnv();
         });
     },
     /**
      * @private
-     * @param {OdooEvent} event
-     * @param {boolean} [event.data.openQuickCreate=false] if true, opens the
+     * @param {OdooEvent} ev
+     * @param {boolean} [ev.data.openQuickCreate=false] if true, opens the
      *   QuickCreate in the toggled column (it assumes that we are opening it)
      */
-    _onToggleColumn: function (event) {
+    _onToggleColumn: function (ev) {
         var self = this;
-        var column = event.target;
+        var column = ev.target;
         this.model.toggleGroup(column.db_id).then(function (db_id) {
             var data = self.model.get(db_id);
             var options = {
-                openQuickCreate: !!event.data.openQuickCreate,
+                openQuickCreate: !!ev.data.openQuickCreate,
             };
             self.renderer.updateColumn(db_id, data, options);
             self._updateEnv();
@@ -461,31 +506,38 @@ var KanbanController = BasicController.extend({
      *
      * @private
      * @param {OdooEvent} ev
+     * @param {function} [ev.data.onSuccess] callback to execute after applying
+     *   changes
      */
     _onUpdateRecord: function (ev) {
+        var onSuccess = ev.data.onSuccess;
+        delete ev.data.onSuccess;
         var changes = _.clone(ev.data);
         ev.data.force_save = true;
-        this._applyChanges(ev.target.db_id, changes, ev);
+        this._applyChanges(ev.target.db_id, changes, ev).then(onSuccess);
     },
     /**
      * Allow the user to archive/restore all the records of a column.
      *
      * @private
-     * @param {OdooEvent} event
+     * @param {OdooEvent} ev
      */
-    _onToggleActiveRecords: function (event) {
+    _onToggleActiveRecords: function (ev) {
         var self = this;
-        var active = !event.data.archive;
-        var column = event.target;
+        var archive = ev.data.archive;
+        var column = ev.target;
         var recordIds = _.pluck(column.records, 'db_id');
         if (recordIds.length) {
-            this.model
-                .toggleActive(recordIds, active, column.db_id)
-                .then(function (dbID) {
+            var prom = archive ?
+              this.model.actionArchive(recordIds, column.db_id) :
+              this.model.actionUnarchive(recordIds, column.db_id);
+            prom.then(function (dbID) {
+                if (_.isString(dbID)) {
                     var data = self.model.get(dbID);
                     self.renderer.updateColumn(dbID, data);
                     self._updateEnv();
-                });
+                }
+            });
         }
     },
 });
