@@ -8,6 +8,7 @@ odoo.define('web.KanbanRecord', function (require) {
 var config = require('web.config');
 var core = require('web.core');
 var Domain = require('web.Domain');
+var Dialog = require('web.Dialog');
 var field_utils = require('web.field_utils');
 var utils = require('web.utils');
 var Widget = require('web.Widget');
@@ -51,6 +52,7 @@ var KanbanRecord = Widget.extend({
         this.editable = options.editable;
         this.deletable = options.deletable;
         this.read_only_mode = options.read_only_mode;
+        this.selectionMode = options.selectionMode;
         this.qweb = options.qweb;
         this.subWidgets = {};
 
@@ -62,18 +64,20 @@ var KanbanRecord = Widget.extend({
      * @override
      */
     start: function () {
-        return $.when(this._super.apply(this, arguments), this._render());
+        return Promise.all([this._super.apply(this, arguments), this._render()]);
     },
     /**
      * Called each time the record is attached to the DOM.
      */
     on_attach_callback: function () {
+        this.isInDOM = true;
         _.invoke(this.subWidgets, 'on_attach_callback');
     },
     /**
      * Called each time the record is detached from the DOM.
      */
     on_detach_callback: function () {
+        this.isInDOM = false;
         _.invoke(this.subWidgets, 'on_detach_callback');
     },
 
@@ -85,7 +89,7 @@ var KanbanRecord = Widget.extend({
      * Re-renders the record with a new state
      *
      * @param {Object} state
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     update: function (state) {
         // detach the widgets because the record will empty its $el, which will
@@ -147,7 +151,7 @@ var KanbanRecord = Widget.extend({
         }
         if (typeof variable === 'string') {
             var index = 0;
-            for (var i = 0 ; i < variable.length ; i++) {
+            for (var i = 0; i < variable.length; i++) {
                 index += variable.charCodeAt(i);
             }
             return index % NB_KANBAN_RECORD_COLORS;
@@ -176,30 +180,30 @@ var KanbanRecord = Widget.extend({
      * @param {string} model the name of the model
      * @param {string} field the name of the field
      * @param {integer} id the id of the resource
-     * @param {integer} cache the cache duration, in seconds
-     * @param {Object} options
+     * @param {string} placeholder
      * @returns {string} the url of the image
      */
-    _getImageURL: function (model, field, id, cache, options) {
-        options = options || {};
+    _getImageURL: function (model, field, id, placeholder) {
+        id = (_.isArray(id) ? id[0] : id) || false;
+        placeholder = placeholder || "/web/static/src/img/placeholder.png";
+        var isCurrentRecord = this.modelName === model && this.recordData.id === id;
         var url;
-        if (this.record[field] && this.record[field].value && !utils.is_bin_size(this.record[field].value)) {
+        if (isCurrentRecord && this.record[field] && this.record[field].raw_value && !utils.is_bin_size(this.record[field].raw_value)) {
             // Use magic-word technique for detecting image type
-            url = 'data:image/' + this.file_type_magic_word[this.record[field].value[0]] + ';base64,' + this.record[field].value;
-        } else if (this.record[field] && ! this.record[field].value) {
-            url = "/web/static/src/img/placeholder.png";
+            url = 'data:image/' + this.file_type_magic_word[this.record[field].raw_value[0]] + ';base64,' + this.record[field].raw_value;
+        } else if (!model || !field || !id || (isCurrentRecord && this.record[field] && !this.record[field].raw_value)) {
+            url = placeholder;
         } else {
-            if (_.isArray(id)) { id = id[0]; }
-            if (!id) { id = undefined; }
-            if (options.preview_image)
-                field = options.preview_image;
-            var unique = this.record.__last_update && this.record.__last_update.value.replace(/[^0-9]/g, '');
             var session = this.getSession();
-            url = session.url('/web/image', {model: model, field: field, id: id, unique: unique});
-            if (cache !== undefined) {
-                // Set the cache duration in seconds.
-                url += '&cache=' + parseInt(cache, 10);
+            var params = {
+                model: model,
+                field: field,
+                id: id
+            };
+            if (isCurrentRecord) {
+                params.unique = this.record.__last_update && this.record.__last_update.value.replace(/[^0-9]/g, '');
             }
+            url = session.url('/web/image', params);
         }
         return url;
     },
@@ -244,7 +248,7 @@ var KanbanRecord = Widget.extend({
                     if (Widget) {
                         widget = self._processWidget($field, field_name, Widget);
                         self.subWidgets[field_name] = widget;
-                    } else if (config.debug) {
+                    } else if (config.isDebug()) {
                         // the widget is not implemented
                         $field.replaceWith($('<span>', {
                             text: _.str.sprintf(_t('[No widget %s]'), field_widget),
@@ -254,6 +258,9 @@ var KanbanRecord = Widget.extend({
                     // a widget already exists for that field, so reset it with the new state
                     widget.reset(self.state);
                     $field.replaceWith(widget.$el);
+                    if (self.isInDOM && widget.on_attach_callback) {
+                        widget.on_attach_callback();
+                    }
                 }
             } else {
                 self._processField($field, field_name);
@@ -309,14 +316,13 @@ var KanbanRecord = Widget.extend({
             }
             attrs[key] = value;
         });
-        var options = _.extend({}, this.options, {attrs: attrs});
+        var options = _.extend({}, this.options, { attrs: attrs });
         var widget = new Widget(this, field_name, this.state, options);
-        var def = widget.replace($field).then(function () {
+        var def = widget.replace($field);
+        this.defs.push(def);
+        def.then(function () {
             self._setFieldDisplay(widget.$el, field_name);
         });
-        if (def.state() === 'pending') {
-            this.defs.push(def);
-        }
         return widget;
     },
     _processWidgets: function () {
@@ -326,26 +332,32 @@ var KanbanRecord = Widget.extend({
             var Widget = widgetRegistry.get($field.attr('name'));
             var widget = new Widget(self, self.state);
 
-            var def = widget._widgetRenderAndInsert(function () {}).then(function () {
-                widget.$el.addClass('o_widget');
+            var def = widget._widgetRenderAndInsert(function () { });
+            self.defs.push(def);
+            def.then(function () {
                 $field.replaceWith(widget.$el);
+                widget.$el.addClass('o_widget');
             });
-            if (def.state() === 'pending') {
-                self.defs.push(def);
-            }
         });
     },
     /**
      * Renders the record
      *
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _render: function () {
         this.defs = [];
+        // call 'on_detach_callback' on each subwidget as they will be removed
+        // from the DOM at the next line
+        _.invoke(this.subWidgets, 'on_detach_callback');
         this._replaceElement(this.qweb.render('kanban-box', this.qweb_context));
-        this.$el.addClass('o_kanban_record').attr("tabindex",0);
+        this.$el.addClass('o_kanban_record').attr("tabindex", 0);
         this.$el.attr('role', 'article');
         this.$el.data('record', this);
+        // forcefully add class oe_kanban_global_click to have clickable record always to select it
+        if (this.selectionMode) {
+            this.$el.addClass('oe_kanban_global_click');
+        }
         if (this.$el.hasClass('oe_kanban_global_click') ||
             this.$el.hasClass('oe_kanban_global_click_edit')) {
             this.$el.on('click', this._onGlobalClick.bind(this));
@@ -360,9 +372,108 @@ var KanbanRecord = Widget.extend({
         this._attachTooltip();
 
         // We use boostrap tooltips for better and faster display
-        this.$('span.o_tag').tooltip({delay: {'show': 50}});
+        this.$('span.o_tag').tooltip({ delay: { 'show': 50 } });
 
-        return $.when.apply(this, this.defs);
+        return Promise.all(this.defs);
+    },
+    /**
+     * Sets cover image on a kanban card through an attachment dialog.
+     *
+     * @private
+     * @param {string} fieldName field used to set cover image
+     */
+    _setCoverImage: function (fieldName) {
+        var self = this;
+        this._rpc({
+            model: 'ir.attachment',
+            method: 'search_read',
+            domain: [
+                ['res_model', '=', this.modelName],
+                ['res_id', '=', this.id],
+                ['mimetype', 'ilike', 'image']
+            ],
+            fields: ['id', 'name'],
+        }).then(function (attachmentIds) {
+            self.imageUploadID = _.uniqueId('o_cover_image_upload');
+            self.image_only = true;  // prevent uploading of other file types
+            var coverId = self.record[fieldName] && self.record[fieldName].raw_value;
+            var $content = $(QWeb.render('KanbanView.SetCoverModal', {
+                coverId: coverId,
+                attachmentIds: attachmentIds,
+                widget: self,
+            }));
+            var $imgs = $content.find('.o_kanban_cover_image');
+            var dialog = new Dialog(self, {
+                title: _t("Set a Cover Image"),
+                $content: $content,
+                buttons: [{
+                    text: _t("Select"),
+                    classes: attachmentIds.length ? 'btn-primary' : 'd-none',
+                    close: true,
+                    disabled: !coverId,
+                    click: function () {
+                        var $img = $imgs.filter('.o_selected').find('img');
+                        var data = {};
+                        data[fieldName] = {
+                            id: $img.data('id'),
+                            display_name: $img.data('name')
+                        };
+                        self.trigger_up('kanban_record_update', data);
+                    },
+                }, {
+                    text: _t('Upload and Set'),
+                    classes: attachmentIds.length ? '' : 'btn-primary',
+                    close: false,
+                    click: function () {
+                        $content.find('input.o_input_file').click();
+                    },
+                }, {
+                    text: _t("Remove Cover Image"),
+                    classes: coverId ? '' : 'd-none',
+                    close: true,
+                    click: function () {
+                        var data = {};
+                        data[fieldName] = false;
+                        self.trigger_up('kanban_record_update', data);
+                    },
+                }, {
+                    text: _t("Discard"),
+                    close: true,
+                }],
+            });
+            dialog.opened().then(function () {
+                var $selectBtn = dialog.$footer.find('.btn-primary');
+                $content.on('click', '.o_kanban_cover_image', function (ev) {
+                    $imgs.not(ev.currentTarget).removeClass('o_selected');
+                    $selectBtn.prop('disabled', !$(ev.currentTarget).toggleClass('o_selected').hasClass('o_selected'));
+                });
+
+                $content.on('dblclick', '.o_kanban_cover_image', function (ev) {
+                    var $img  = $(ev.currentTarget).find('img');
+                    var data = {};
+                    data[fieldName] = {
+                        id: $img.data('id'),
+                        display_name: $img.data('name')
+                    };
+                    self.trigger_up('kanban_record_update', data);
+                    dialog.close();
+                });
+                $content.on('change', 'input.o_input_file', function () {
+                    $content.find('form.o_form_binary_form').submit();
+                });
+                $(window).on(self.imageUploadID, function () {
+                    var images = Array.prototype.slice.call(arguments, 1);
+                    var data = {};
+                    data[fieldName] = {
+                        id: images[0].id,
+                        display_name: images[0].filename
+                    };
+                    self.trigger_up('kanban_record_update', data);
+                    dialog.close();
+                });
+            });
+            dialog.open();
+        });
     },
     /**
      * Sets particular classnames on a field $el according to the
@@ -398,11 +509,13 @@ var KanbanRecord = Widget.extend({
         this.recordData = recordState.data;
         this.record = this._transformRecord(recordState.data);
         this.qweb_context = {
+            context: this.state.getContext(),
             kanban_image: this._getImageURL.bind(this),
             kanban_color: this._getColorClassname.bind(this),
             kanban_getcolor: this._getColorID.bind(this),
             kanban_getcolorname: this._getColorname.bind(this),
             kanban_compute_domain: this._computeDomain.bind(this),
+            selection_mode: this.selectionMode,
             read_only_mode: this.read_only_mode,
             record: this.record,
             user_context: this.getSession().user_context,
@@ -421,7 +534,7 @@ var KanbanRecord = Widget.extend({
             var colorHelp = _.str.sprintf(_t("Card color: %s"), this._getColorname(this.recordData[color_field]));
             var colorClass = this._getColorClassname(this.recordData[color_field]);
             this.$el.addClass(colorClass);
-            this.$el.prepend('<span title="' + colorHelp + '" aria-label="' + colorHelp +'" role="img" class="oe_kanban_color_help"/>');
+            this.$el.prepend('<span title="' + colorHelp + '" aria-label="' + colorHelp + '" role="img" class="oe_kanban_color_help"/>');
         }
     },
     /**
@@ -456,7 +569,7 @@ var KanbanRecord = Widget.extend({
                 r.raw_value = value.toDate();
             } else if (r.type === 'one2many' || r.type === 'many2many') {
                 r.raw_value = value.count ? value.res_ids : [];
-            } else if (r.type === 'many2one' ) {
+            } else if (r.type === 'many2one') {
                 r.raw_value = value && value.res_id || false;
             } else {
                 r.raw_value = value;
@@ -556,21 +669,34 @@ var KanbanRecord = Widget.extend({
 
         switch (type) {
             case 'edit':
-                this.trigger_up('open_record', {id: this.db_id, mode: 'edit'});
+                this.trigger_up('open_record', { id: this.db_id, mode: 'edit' });
                 break;
             case 'open':
-                this.trigger_up('open_record', {id: this.db_id});
+                this.trigger_up('open_record', { id: this.db_id });
                 break;
             case 'delete':
-                this.trigger_up('kanban_record_delete', {id: this.db_id, record: this});
+                this.trigger_up('kanban_record_delete', { id: this.db_id, record: this });
                 break;
             case 'action':
             case 'object':
+                var attrs = $action.data();
+                attrs.confirm = $action.attr('confirm');
                 this.trigger_up('button_clicked', {
-                    attrs: $action.data(),
+                    attrs: attrs,
                     record: this.state,
                 });
                 break;
+            case 'set_cover':
+                var fieldName = $action.data('field');
+                if (this.fields[fieldName].type === 'many2one' &&
+                    this.fields[fieldName].relation === 'ir.attachment' &&
+                    this.fieldsInfo[fieldName].widget === 'attachment_image') {
+                    this._setCoverImage(fieldName);
+                } else {
+                    var warning = _.str.sprintf(_t('Could not set the cover image: incorrect field ("%s") is provided in the view.'), fieldName);
+                    this.do_warn(warning);
+                }
+                break;
             default:
                 this.do_warn("Kanban: no action for type : " + type);
         }
@@ -585,9 +711,11 @@ var KanbanRecord = Widget.extend({
     _onKeyDownCard: function (event) {
         switch (event.keyCode) {
             case $.ui.keyCode.ENTER:
-                event.preventDefault();
-                this._onGlobalClick(event);
-                break;
+                if ($(event.target).hasClass('oe_kanban_global_click')) {
+                    event.preventDefault();
+                    this._onGlobalClick(event);
+                    break;
+                }
         }
     },
     /**
