@@ -3,14 +3,10 @@ odoo.define('web.view_dialogs', function (require) {
 
 var config = require('web.config');
 var core = require('web.core');
-var data = require('web.data');
 var Dialog = require('web.Dialog');
 var dom = require('web.dom');
-var ListController = require('web.ListController');
-var ListView = require('web.ListView');
-var pyUtils = require('web.py_utils');
-var SearchView = require('web.SearchView');
 var view_registry = require('web.view_registry');
+var select_create_controllers_registry = require('web.select_create_controllers_registry');
 
 var _t = core._t;
 
@@ -21,9 +17,6 @@ var _t = core._t;
 var ViewDialog = Dialog.extend({
     custom_events: _.extend({}, Dialog.prototype.custom_events, {
         push_state: '_onPushState',
-        env_updated: function (event) {
-            event.stopPropagation();
-        },
     }),
     /**
      * @constructor
@@ -45,10 +38,6 @@ var ViewDialog = Dialog.extend({
         this.domain = options.domain || [];
         this.context = options.context || {};
         this.options = _.extend(this.options || {}, options || {});
-
-        // FIXME: remove this once a dataset won't be necessary anymore to interact
-        // with data_manager and instantiate views
-        this.dataset = new data.DataSet(this, this.res_model, this.context);
     },
 
     //--------------------------------------------------------------------------
@@ -114,6 +103,7 @@ var FormViewDialog = ViewDialog.extend({
         this.readonly = options.readonly;
         this.deletable = options.deletable;
         this.disable_multiple_selection = options.disable_multiple_selection;
+        var oBtnRemove = 'o_btn_remove';
 
         var multi_select = !_.isNumber(options.res_id) && !options.disable_multiple_selection;
         var readonly = _.isNumber(options.res_id) && options.readonly;
@@ -137,7 +127,7 @@ var FormViewDialog = ViewDialog.extend({
                     text: (multi_select ? _t("Save & Close") : _t("Save")),
                     classes: "btn-primary",
                     click: function () {
-                        this._save().then(self.close.bind(self));
+                        self._save().then(self.close.bind(self));
                     }
                 });
 
@@ -146,18 +136,26 @@ var FormViewDialog = ViewDialog.extend({
                         text: _t("Save & New"),
                         classes: "btn-primary",
                         click: function () {
-                            this._save().then(self.form_view.createRecord.bind(self.form_view, self.parentID));
+                            self._save()
+                                .then(self.form_view.createRecord.bind(self.form_view, self.parentID))
+                                .then(function () {
+                                    if (!self.deletable) {
+                                        return;
+                                    }
+                                    self.deletable = false;
+                                    self.buttons = self.buttons.filter(function (button) {
+                                        return button.classes.split(' ').indexOf(oBtnRemove) < 0;
+                                    });
+                                    self.set_buttons(self.buttons);
+                                    self.set_title(_t("Create ") + _.str.strRight(self.title, _t("Open: ")));
+                                });
                         },
                     });
                 }
 
                 var multi = options.disable_multiple_selection;
                 if (!multi && this.deletable) {
-                    options.buttons.push({
-                        text: _t("Remove"),
-                        classes: 'btn-secondary o_btn_remove',
-                        click: this._remove.bind(this),
-                    });
+                    this._setRemoveButtonOption(options, oBtnRemove);
                 }
             }
         }
@@ -180,9 +178,9 @@ var FormViewDialog = ViewDialog.extend({
         var FormView = view_registry.get('form');
         var fields_view_def;
         if (this.options.fields_view) {
-            fields_view_def = $.when(this.options.fields_view);
+            fields_view_def = Promise.resolve(this.options.fields_view);
         } else {
-            fields_view_def = this.loadFieldView(this.dataset, this.options.view_id, 'form');
+            fields_view_def = this.loadFieldView(this.res_model, this.context, this.options.view_id, 'form');
         }
 
         fields_view_def.then(function (viewInfo) {
@@ -210,9 +208,9 @@ var FormViewDialog = ViewDialog.extend({
             if (self.recordID && self.shouldSaveLocally) {
                 self.model.save(self.recordID, {savePoint: true});
             }
-            self.form_view.appendTo(fragment)
+            return self.form_view.appendTo(fragment)
                 .then(function () {
-                    self.opened().always(function () {
+                    self.opened().then(function () {
                         var $buttons = $('<div>');
                         self.form_view.renderButtons($buttons);
                         if ($buttons.children().length) {
@@ -223,7 +221,7 @@ var FormViewDialog = ViewDialog.extend({
                             in_DOM: true,
                         });
                     });
-                    _super();
+                    return _super();
                 });
         });
 
@@ -238,21 +236,25 @@ var FormViewDialog = ViewDialog.extend({
      * @override
      */
     _focusOnClose: function() {
-        this.trigger_up('form_dialog_discarded');
-        return true;
+        var isFocusSet = false;
+        this.trigger_up('form_dialog_discarded', {
+            callback: function (isFocused) {
+                isFocusSet = isFocused;
+            },
+        });
+        return isFocusSet;
     },
 
     /**
      * @private
      */
     _remove: function () {
-        this.on_remove();
-        this.close();
+        return Promise.resolve(this.on_remove());
     },
 
     /**
      * @private
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _save: function () {
         var self = this;
@@ -265,23 +267,27 @@ var FormViewDialog = ViewDialog.extend({
             // record might have been changed by the save (e.g. if this was a new record, it has an
             // id now), so don't re-use the copy obtained before the save
             var record = self.form_view.model.get(self.form_view.handle);
-            self.on_saved(record, !!changedFields.length);
+            return self.on_saved(record, !!changedFields.length);
         });
     },
-});
 
-var SelectCreateListController = ListController.extend({
-    // Override the ListView to handle the custom events 'open_record' (triggered when clicking on a
-    // row of the list) such that it triggers up 'select_record' with its res_id.
-    custom_events: _.extend({}, ListController.prototype.custom_events, {
-        open_record: function (event) {
-            var selectedRecord = this.model.get(event.data.id);
-            this.trigger_up('select_record', {
-                id: selectedRecord.res_id,
-                display_name: selectedRecord.data.display_name,
-            });
-        },
-    }),
+    /**
+     * Set the "remove" button into the options' buttons list
+     *
+     * @private
+     * @param {Object} options The options object to modify
+     * @param {string} btnClasses The classes for the remove button
+     */
+    _setRemoveButtonOption(options, btnClasses) {
+        const self = this;
+        options.buttons.push({
+            text: _t("Remove"),
+            classes: 'btn-secondary ' + btnClasses,
+            click: function() {
+                self._remove().then(self.close.bind(self));
+            }
+        });
+    },
 });
 
 /**
@@ -299,13 +305,6 @@ var SelectCreateDialog = ViewDialog.extend({
             event.stopPropagation();
             this.$footer.find(".o_select_button").prop('disabled', !event.data.selection.length);
         },
-        search: function (event) {
-            event.stopPropagation(); // prevent this event from bubbling up to the action manager
-            var d = event.data;
-            var searchData = this._process_search_data(d.domains, d.contexts, d.groupbys);
-            this.list_controller.reload(_.extend({offset: 0}, searchData));
-        },
-        get_controller_context: '_onGetControllerContext',
     }),
 
     /**
@@ -315,12 +314,15 @@ var SelectCreateDialog = ViewDialog.extend({
      * - list_view_options: dict of options to pass to the List View
      * - on_selected: optional callback to execute when records are selected
      * - disable_multiple_selection: true to allow create/select multiple records
+     * - dynamicFilters: filters to add to the searchview
      */
     init: function () {
         this._super.apply(this, arguments);
         _.defaults(this.options, { initial_view: 'search' });
         this.on_selected = this.options.on_selected || (function () {});
-        this.initial_ids = this.options.initial_ids;
+        this.on_closed = this.options.on_closed || (function () {});
+        this.initialIDs = this.options.initial_ids;
+        this.viewType = 'list';
     },
 
     open: function () {
@@ -328,125 +330,75 @@ var SelectCreateDialog = ViewDialog.extend({
             return this.create_edit_record();
         }
         var self = this;
-        var user_context = this.getSession().user_context;
-
         var _super = this._super.bind(this);
-        var context = pyUtils.eval_domains_and_contexts({
-            domains: [],
-            contexts: [user_context, this.context]
-        }).context;
-        var search_defaults = {};
-        _.each(context, function (value_, key) {
-            var match = /^search_default_(.*)$/.exec(key);
-            if (match) {
-                search_defaults[match[1]] = value_;
-            }
-        });
-        this.loadViews(this.dataset.model, this.dataset.get_context().eval(), [[false, 'list'], [false, 'search']], {})
-            .then(this.setup.bind(this, search_defaults))
+        var viewRefID = this.viewType === 'kanban' ?
+            (this.options.kanban_view_ref && JSON.parse(this.options.kanban_view_ref) || false) : false;
+        return this.loadViews(this.res_model, this.context, [[viewRefID, this.viewType], [false, 'search']], {})
+            .then(this.setup.bind(this))
             .then(function (fragment) {
                 self.opened().then(function () {
                     dom.append(self.$el, fragment, {
-                        callbacks: [{widget: self.list_controller}],
+                        callbacks: [{widget: self.viewController}],
                         in_DOM: true,
                     });
                     self.set_buttons(self.__buttons);
                 });
-                _super();
+                return _super();
             });
-        return this;
     },
 
-    setup: function (search_defaults, fields_views) {
+    setup: function (fieldsViews) {
         var self = this;
         var fragment = document.createDocumentFragment();
 
-        var searchDef = $.Deferred();
-
-        // Set the dialog's header and its search view
-        var $header = $('<div/>').addClass('o_modal_header').appendTo(fragment);
-        var $pager = $('<div/>').addClass('o_pager').appendTo($header);
-        var options = {
-            $buttons: $('<div/>').addClass('o_search_options').appendTo($header),
-            search_defaults: search_defaults,
-        };
-        var searchview = new SearchView(this, this.dataset, fields_views.search, options);
-        searchview.prependTo($header).done(function () {
-            var d = searchview.build_search_data();
-            if (self.initial_ids) {
-                d.domains.push([["id", "in", self.initial_ids]]);
-                self.initial_ids = undefined;
-            }
-            var searchData = self._process_search_data(d.domains, d.contexts, d.groupbys);
-            searchDef.resolve(searchData);
-        });
-
-        return $.when(searchDef).then(function (searchResult) {
-            // Set the list view
-            var listView = new ListView(fields_views.list, _.extend({
-                context: searchResult.context,
-                domain: searchResult.domain,
-                groupBy: searchResult.groupBy,
-                modelName: self.dataset.model,
-                hasSelectors: !self.options.disable_multiple_selection,
+        var domain = this.domain;
+        if (this.initialIDs) {
+            domain = domain.concat([['id', 'in', this.initialIDs]]);
+        }
+        var ViewClass = view_registry.get(this.viewType);
+        var viewOptions = {};
+        var selectCreateController;
+        if (this.viewType === 'list') { // add listview specific options
+            _.extend(viewOptions, {
+                hasSelectors: !this.options.disable_multiple_selection,
                 readonly: true,
-            }, self.options.list_view_options));
-            listView.setController(SelectCreateListController);
-            return listView.getController(self);
-        }).then(function (controller) {
-            self.list_controller = controller;
-            // Set the dialog's buttons
-            self.__buttons = [{
-                text: _t("Cancel"),
-                classes: "btn-secondary o_form_button_cancel",
-                close: true,
-            }];
-            if (!self.options.no_create) {
-                self.__buttons.unshift({
-                    text: _t("Create"),
-                    classes: "btn-primary",
-                    click: self.create_edit_record.bind(self)
-                });
-            }
-            if (!self.options.disable_multiple_selection) {
-                self.__buttons.unshift({
-                    text: _t("Select"),
-                    classes: "btn-primary o_select_button",
-                    disabled: true,
-                    close: true,
-                    click: function () {
-                        var records = self.list_controller.getSelectedRecords();
-                        var values = _.map(records, function (record) {
-                            return {
-                                id: record.res_id,
-                                display_name: record.data.display_name,
-                            };
-                        });
-                        self.on_selected(values);
-                    },
-                });
-            }
-            return self.list_controller.appendTo(fragment);
+
+            }, this.options.list_view_options);
+            selectCreateController = select_create_controllers_registry.SelectCreateListController;
+        }
+        if (this.viewType === 'kanban') {
+            _.extend(viewOptions, {
+                noDefaultGroupby: true,
+                selectionMode: this.options.selectionMode || false,
+            });
+            selectCreateController = select_create_controllers_registry.SelectCreateKanbanController;
+        }
+        var view = new ViewClass(fieldsViews[this.viewType], _.extend(viewOptions, {
+            action: {
+                controlPanelFieldsView: fieldsViews.search,
+                help: _.str.sprintf("<p>%s</p>", _t("No records found!")),
+            },
+            action_buttons: false,
+            dynamicFilters: this.options.dynamicFilters,
+            context: this.context,
+            domain: domain,
+            modelName: this.res_model,
+            withBreadcrumbs: false,
+            withSearchPanel: false,
+        }));
+        view.setController(selectCreateController);
+        return view.getController(this).then(function (controller) {
+            self.viewController = controller;
+            // render the footer buttons
+            self._prepareButtons();
+            return self.viewController.appendTo(fragment);
         }).then(function () {
-            searchview.toggle_visibility(true);
-            self.list_controller.do_show();
-            self.list_controller.renderPager($pager);
             return fragment;
         });
     },
-    _process_search_data: function (domains, contexts, groupbys) {
-        var results = pyUtils.eval_domains_and_contexts({
-            domains: [this.domain].concat(domains),
-            contexts: [this.context].concat(contexts),
-            group_by_seq: groupbys || [],
-            eval_context: this.getSession().user_context,
-        });
-        var context = _.omit(results.context, function (value, key) { return key.indexOf('search_default_') === 0; });
-        return {
-            context: context,
-            domain: results.domain,
-            groupBy: results.group_by,
-        };
+    close: function () {
+        this._super.apply(this, arguments);
+        this.on_closed();
     },
     create_edit_record: function () {
         var self = this;
@@ -466,26 +418,51 @@ var SelectCreateDialog = ViewDialog.extend({
      * @override
      */
     _focusOnClose: function() {
-        this.trigger_up('form_dialog_discarded');
-        return true;
+        var isFocusSet = false;
+        this.trigger_up('form_dialog_discarded', {
+            callback: function (isFocused) {
+                isFocusSet = isFocused;
+            },
+        });
+        return isFocusSet;
     },
-    //--------------------------------------------------------------------------
-    // Handlers
-    //--------------------------------------------------------------------------
-
     /**
-     * Handles a context request: provides to the caller the context of the
-     * list controller.
+     * prepare buttons for dialog footer based on options
      *
      * @private
-     * @param {OdooEvent} ev
-     * @param {function} ev.data.callback used to send the requested context
      */
-    _onGetControllerContext: function (ev) {
-        ev.stopPropagation();
-        var context = this.list_controller && this.list_controller.getContext();
-        ev.data.callback(context || {});
-    }
+    _prepareButtons: function () {
+        this.__buttons = [{
+            text: _t("Cancel"),
+            classes: 'btn-secondary o_form_button_cancel',
+            close: true,
+        }];
+        if (!this.options.no_create) {
+            this.__buttons.unshift({
+                text: _t("Create"),
+                classes: 'btn-primary',
+                click: this.create_edit_record.bind(this)
+            });
+        }
+        if (!this.options.disable_multiple_selection) {
+            this.__buttons.unshift({
+                text: _t("Select"),
+                classes: 'btn-primary o_select_button',
+                disabled: true,
+                close: true,
+                click: function () {
+                    var records = this.viewController.getSelectedRecords();
+                    var values = _.map(records, function (record) {
+                        return {
+                            id: record.res_id,
+                            display_name: record.data.display_name,
+                        };
+                    });
+                    this.on_selected(values);
+                },
+            });
+        }
+    },
 });
 
 return {
