@@ -40,7 +40,10 @@ var ModelFieldSelector = Widget.extend({
         "click li.o_field_selector_select_button": "_onLastFieldClick",
 
         // Handle a direct change in the debug input
-        "change input": "_onInputChange",
+        "change input.o_field_selector_debug": "_onDebugInputChange",
+
+        // Handle a change in the search input
+        "keyup .o_field_selector_search > input": "_onSearchInputChange",
 
         // Handle keyboard and mouse navigation to build the field chain
         "mouseover li.o_field_selector_item": "_onItemHover",
@@ -68,6 +71,8 @@ var ModelFieldSelector = Widget.extend({
      *                   the fields itself)
      * @param {boolean|function} [options.followRelations=true]
      *                  true if can follow relation when building the chain
+     * @param {boolean} [options.showSearchInput=true]
+     *                  false to hide a search input to filter displayed fields
      * @param {boolean} [options.debugMode=false]
      *                  true if the widget is in debug mode, false otherwise
      */
@@ -84,6 +89,7 @@ var ModelFieldSelector = Widget.extend({
             filter: function () {return true;},
             followRelations: true,
             debugMode: false,
+            showSearchInput: true,
         }, options || {});
         this.options.filters = _.extend({
             searchable: true,
@@ -101,25 +107,28 @@ var ModelFieldSelector = Widget.extend({
         if (!this.options.readonly) {
             _.extend(this.events, this.editionEvents);
         }
+
+        this.searchValue = '';
     },
     /**
      * @see Widget.willStart()
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     willStart: function () {
-        return $.when(
+        return Promise.all([
             this._super.apply(this, arguments),
             this._prefill()
-        );
+        ]);
     },
     /**
      * @see Widget.start
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     start: function () {
         this.$value = this.$(".o_field_selector_value");
         this.$popover = this.$(".o_field_selector_popover");
-        this.$input = this.$popover.find("input");
+        this.$input = this.$popover.find(".o_field_selector_popover_footer > input");
+        this.$searchInput = this.$popover.find(".o_field_selector_search > input");
         this.$valid = this.$(".o_field_selector_warning");
 
         this._render();
@@ -153,11 +162,11 @@ var ModelFieldSelector = Widget.extend({
      * Saves a new field chain (array) and re-render.
      *
      * @param {string[]} chain - the new field chain
-     * @returns {Deferred} resolved once the re-rendering is finished
+     * @returns {Promise} resolved once the re-rendering is finished
      */
     setChain: function (chain) {
         if (_.isEqual(chain, this.chain)) {
-            return $.when();
+            return Promise.resolve();
         }
 
         this.chain = chain;
@@ -179,6 +188,9 @@ var ModelFieldSelector = Widget.extend({
         this.dirty = true;
         this.chain = this.chain.slice(0, this.pages.length-1);
         this.chain.push(fieldName);
+
+        this.searchValue = '';
+        this.$searchInput.val('');
     },
     /**
      * Searches a field in the last page by its name.
@@ -278,26 +290,35 @@ var ModelFieldSelector = Widget.extend({
      * chain.
      *
      * @private
-     * @returns {Deferred} resolved once the whole field chain has been
+     * @returns {Promise} resolved once the whole field chain has been
      *                     processed
      */
     _prefill: function () {
         this.pages = [];
         return this._pushPageData(this.model).then((function () {
             this._validate(true);
-            return (this.chain.length ? processChain.call(this, this.chain.slice().reverse()) : $.when());
+            return (this.chain.length ? processChain.call(this, this.chain.slice().reverse()) : Promise.resolve());
         }).bind(this));
 
         function processChain(chain) {
-            var field = this._getLastPageField(chain.pop());
-            if (field && field.relation && chain.length > 0) { // Fetch next chain node if any and possible
-                return this._pushPageData(field.relation).then(processChain.bind(this, chain));
+            var fieldName = chain.pop();
+            var field = this._getLastPageField(fieldName);
+            if (field && field.relation) {
+                if (chain.length) { // Fetch next chain node if any and possible
+                    return this._pushPageData(field.relation).then(processChain.bind(this, chain));
+                } else { // Simply update the last popover page
+                    return this._pushPageData(field.relation);
+                }
             } else if (field && chain.length === 0) { // Last node fetched
-                return $.when();
+                return Promise.resolve();
+            } else if (!field && fieldName === "1") { // TRUE_LEAF
+                this._validate(true);
+            } else if (!field && fieldName === "0") { // FALSE_LEAF
+                this._validate(true);
             } else { // Wrong node chain
                 this._validate(false);
             }
-            return $.when();
+            return Promise.resolve();
         }
     },
     /**
@@ -306,12 +327,12 @@ var ModelFieldSelector = Widget.extend({
      *
      * @private
      * @param {string} model - the model name whose fields have to be fetched
-     * @returns {Deferred} resolved once the fields have been added
+     * @returns {Promise} resolved once the fields have been added
      */
     _pushPageData: function (model) {
         var def;
         if (this.model === model && this.options.fields) {
-            def = $.when(sortFields(this.options.fields, model, this.options.order));
+            def = Promise.resolve(sortFields(this.options.fields, model, this.options.order));
         } else {
             def = this._getModelFieldsFromCache(model, this.options.filters);
         }
@@ -337,6 +358,7 @@ var ModelFieldSelector = Widget.extend({
      * @private
      */
     _render: function () {
+
         // Render the chain value
         this.$value.html(core.qweb.render(this.template + ".value", {
             chain: this.chain,
@@ -356,8 +378,17 @@ var ModelFieldSelector = Widget.extend({
             if (prevField) title = prevField.string;
         }
         this.$(".o_field_selector_popover_header .o_field_selector_title").text(title);
+
+        var lines = _.filter(page, this.options.filter);
+        if (this.searchValue) {
+            var matches = fuzzy.filter(this.searchValue, _.pluck(lines, 'string'));
+            lines = _.map(_.pluck(matches, 'index'), function (i) {
+                return lines[i];
+            });
+        }
+
         this.$(".o_field_selector_page").replaceWith(core.qweb.render(this.template + ".page", {
-            lines: _.filter(page, this.options.filter),
+            lines: lines,
             followRelations: this.options.followRelations,
             debug: this.options.debugMode,
         }));
@@ -460,7 +491,7 @@ var ModelFieldSelector = Widget.extend({
     /**
      * Called when the debug input value is changed -> adapts the chain
      */
-    _onInputChange: function () {
+    _onDebugInputChange: function () {
         var userChainStr = this.$input.val();
         var userChain = userChainStr.split(".");
         if (!this.options.followRelations && userChain.length > 1) {
@@ -471,6 +502,13 @@ var ModelFieldSelector = Widget.extend({
             this.trigger_up("field_chain_changed", {chain: this.chain});
         }).bind(this));
     },
+    /**
+     * Called when the search input value is changed -> adapts the popover
+     */
+    _onSearchInputChange: function () {
+        this.searchValue = this.$searchInput.val();
+        this._render();
+    },
     /**
      * Called when a popover field button item is hovered -> toggles its
      * "active" status
@@ -490,6 +528,7 @@ var ModelFieldSelector = Widget.extend({
     _onKeydown: function (e) {
         if (!this.$popover.is(":visible")) return;
         var inputHasFocus = this.$input.is(":focus");
+        var searchInputHasFocus = this.$searchInput.is(":focus");
 
         switch (e.which) {
             case $.ui.keyCode.UP:
@@ -535,7 +574,7 @@ var ModelFieldSelector = Widget.extend({
                 this._hidePopover();
                 break;
             case $.ui.keyCode.ENTER:
-                if (inputHasFocus) break;
+                if (inputHasFocus || searchInputHasFocus) break;
                 e.preventDefault();
                 this._selectField(this._getLastPageField(this.$("li.o_field_selector_item.active").data("name")));
                 break;
