@@ -31,12 +31,14 @@ QUnit.module('core', function () {
     });
 
     QUnit.test('simple arithmetic', function(assert) {
-        assert.expect(2);
+        assert.expect(3);
 
         var result = pyUtils.py_eval("1 + 2");
         assert.strictEqual(result, 3, "should properly evaluate sum");
         result = pyUtils.py_eval("42 % 5");
         assert.strictEqual(result, 2, "should properly evaluate modulo operator");
+        result = pyUtils.py_eval("2 ** 3");
+        assert.strictEqual(result, 8, "should properly evaluate power operator");
     });
 
 
@@ -901,20 +903,20 @@ QUnit.module('core', function () {
                                     "number": false, "type": "out_invoice",
                                     "currency_id": 1, "partner_id": 4,
                                     "fiscal_position_id": false,
-                                    "date_invoice": false, "date": false,
+                                    "invoice_date": false, "date": false,
                                     "payment_term_id": false,
                                     "reference": false, "account_id": 440,
                                     "name": false, "invoice_line_ids": [],
                                     "tax_line_ids": [], "amount_untaxed": 0,
                                     "amount_tax": 0, "reconciled": false,
                                     "amount_total": 0, "state": "draft",
-                                    "residual": 0, "company_id": 1,
+                                    "amount_residual": 0, "company_id": 1,
                                     "date_due": false, "user_id": 1,
                                     "partner_bank_id": false, "origin": false,
                                     "move_id": false, "comment": false,
                                     "payment_ids": [[6, false, []]],
                                     "active_id": false, "active_ids": [],
-                                    "active_model": "account.invoice",
+                                    "active_model": "account.move",
                                     "parent": {}}
                     ], "__eval_context": null}
                 }, {
@@ -932,17 +934,17 @@ QUnit.module('core', function () {
                     "invoice_line_tax_ids": [[6, false, [1]]],
                     "active_id": false,
                     "active_ids": [],
-                    "active_model": "account.invoice.line",
+                    "active_model": "account.move.line",
                     "parent": {
                         "id": false, "journal_id": 10, "number": false,
                         "type": "out_invoice", "currency_id": 1,
                         "partner_id": 4, "fiscal_position_id": false,
-                        "date_invoice": false, "date": false,
+                        "invoice_date": false, "date": false,
                         "payment_term_id": false,
                         "reference": false, "account_id": 440, "name": false,
                         "tax_line_ids": [], "amount_untaxed": 0, "amount_tax": 0,
                         "reconciled": false, "amount_total": 0,
-                        "state": "draft", "residual": 0, "company_id": 1,
+                        "state": "draft", "amount_residual": 0, "company_id": 1,
                         "date_due": false, "user_id": 1,
                         "partner_bank_id": false, "origin": false,
                         "move_id": false, "comment": false,
