@@ -52,7 +52,6 @@ QUnit.module('Views', {
             },
         };
     },
-
 }, function () {
 
     QUnit.module('view_dialogs');
@@ -60,11 +59,11 @@ QUnit.module('Views', {
     function createParent(params) {
         var widget = new Widget();
 
-        testUtils.addMockEnvironment(widget, params);
+        testUtils.mock.addMockEnvironment(widget, params);
         return widget;
     }
 
-    QUnit.test('formviewdialog buttons in footer are positioned properly', function (assert) {
+    QUnit.test('formviewdialog buttons in footer are positioned properly', async function (assert) {
         assert.expect(2);
 
         var parent = createParent({
@@ -80,15 +79,11 @@ QUnit.module('Views', {
             },
         });
 
-        testUtils.intercept(parent, 'env_updated', function () {
-            throw new Error("The environment should not be propagated to the action manager");
-        });
-
-
         new dialogs.FormViewDialog(parent, {
             res_model: 'partner',
             res_id: 1,
         }).open();
+        await testUtils.nextTick();
 
         assert.notOk($('.modal-body button').length,
             "should not have any button in body");
@@ -97,7 +92,7 @@ QUnit.module('Views', {
         parent.destroy();
     });
 
-    QUnit.test('formviewdialog buttons in footer are not duplicated', function (assert) {
+    QUnit.test('formviewdialog buttons in footer are not duplicated', async function (assert) {
         assert.expect(2);
         this.data.partner.fields.poney_ids = {string: "Poneys", type: "one2many", relation: 'partner'};
         this.data.partner.records[0].poney_ids = [];
@@ -117,22 +112,20 @@ QUnit.module('Views', {
             res_model: 'partner',
             res_id: 1,
         }).open();
+        await testUtils.nextTick();
 
         assert.strictEqual($('.modal button.btn-primary').length, 1,
             "should have 1 buttons in modal");
 
-        $('.o_field_x2many_list_row_add a').click();
-        $('input.o_input').trigger($.Event('keydown', {
-            which: $.ui.keyCode.ESCAPE,
-            keyCode: $.ui.keyCode.ESCAPE,
-        }));
+        await testUtils.dom.click($('.o_field_x2many_list_row_add a'));
+        await testUtils.fields.triggerKeydown($('input.o_input'), 'escape');
 
         assert.strictEqual($('.modal button.btn-primary').length, 1,
             "should still have 1 buttons in modal");
         parent.destroy();
     });
 
-    QUnit.test('SelectCreateDialog use domain, group_by and search default', function (assert) {
+    QUnit.test('SelectCreateDialog use domain, group_by and search default', async function (assert) {
         assert.expect(3);
 
         var search = 0;
@@ -153,31 +146,32 @@ QUnit.module('Views', {
                     '</search>',
             },
             mockRPC: function (route, args) {
-                if (args.method === 'read_group') {
+                if (args.method === 'web_read_group') {
                     assert.deepEqual(args.kwargs, {
-                        context: {group_by: "bar"},
-                        domain: [["display_name","like","a"], ["display_name","ilike","piou"], ["foo","ilike","piou"]],
-                        fields:["display_name","foo","bar"],
-                        groupby:["bar"],
+                        context: {},
+                        domain: [["display_name", "like", "a"], "&", ["display_name", "ilike", "piou"], ["foo", "ilike", "piou"]],
+                        fields: ["display_name", "foo", "bar"],
+                        groupby: ["bar"],
                         orderby: '',
-                        lazy: true
+                        lazy: true,
+                        limit: 80,
                     }, "should search with the complete domain (domain + search), and group by 'bar'");
                 }
                 if (search === 0 && route === '/web/dataset/search_read') {
                     search++;
                     assert.deepEqual(args, {
-                        context: {},
-                        domain: [["display_name","like","a"], ["display_name","ilike","piou"], ["foo","ilike","piou"]],
-                        fields:["display_name","foo"],
+                        context: {'bin_size': true},  // not part of the test, may change
+                        domain: [["display_name", "like", "a"], "&", ["display_name", "ilike", "piou"], ["foo", "ilike", "piou"]],
+                        fields: ["display_name", "foo"],
                         model: "partner",
                         limit: 80,
                         sort: ""
                     }, "should search with the complete domain (domain + search)");
                 } else if (search === 1 && route === '/web/dataset/search_read') {
                     assert.deepEqual(args, {
-                        context: {},
-                        domain: [["display_name","like","a"]],
-                        fields:["display_name","foo"],
+                        context: {'bin_size': true},  // not part of the test, may change
+                        domain: [["display_name", "like", "a"]],
+                        fields: ["display_name", "foo"],
                         model: "partner",
                         limit: 80,
                         sort: ""
@@ -188,7 +182,8 @@ QUnit.module('Views', {
             },
         });
 
-        var dialog = new dialogs.SelectCreateDialog(parent, {
+        var dialog;
+        new dialogs.SelectCreateDialog(parent, {
             no_create: true,
             readonly: true,
             res_model: 'partner',
@@ -197,15 +192,18 @@ QUnit.module('Views', {
                 search_default_groupby_bar: true,
                 search_default_foo: 'piou',
             },
-        }).open();
+        }).open().then(function (result) {
+            dialog = result;
+        });
+        await testUtils.nextTick();
 
-        dialog.$('.o_searchview_facet:contains(groupby_bar) .o_facet_remove').click();
-        dialog.$('.o_searchview_facet .o_facet_remove').click();
+        await testUtils.dom.click(dialog.$('.o_searchview_facet:contains(groupby_bar) .o_facet_remove'));
+        await testUtils.dom.click(dialog.$('.o_searchview_facet .o_facet_remove'));
 
         parent.destroy();
     });
 
-    QUnit.test('SelectCreateDialog correctly evaluates domains', function (assert) {
+    QUnit.test('SelectCreateDialog correctly evaluates domains', async function (assert) {
         assert.expect(1);
 
         var parent = createParent({
@@ -239,11 +237,12 @@ QUnit.module('Views', {
             res_model: 'partner',
             domain: "[['id', '=', uid]]",
         }).open();
+        await testUtils.nextTick();
 
         parent.destroy();
     });
 
-    QUnit.test('SelectCreateDialog list view in readonly', function (assert) {
+    QUnit.test('SelectCreateDialog list view in readonly', async function (assert) {
         assert.expect(1);
 
         var parent = createParent({
@@ -259,12 +258,16 @@ QUnit.module('Views', {
             },
         });
 
-        var dialog = new dialogs.SelectCreateDialog(parent, {
+        var dialog;
+        new dialogs.SelectCreateDialog(parent, {
             res_model: 'partner',
-        }).open();
+        }).open().then(function (result) {
+            dialog = result;
+        });
+        await testUtils.nextTick();
 
         // click on the first row to see if the list is editable
-        dialog.$('.o_list_view tbody tr:first td:not(.o_list_record_selector):first').click();
+        await testUtils.dom.click(dialog.$('.o_list_view tbody tr:first td:not(.o_list_record_selector):first'));
 
         assert.equal(dialog.$('.o_list_view tbody tr:first td:not(.o_list_record_selector):first input').length, 0,
             "list view should not be editable in a SelectCreateDialog");
@@ -272,16 +275,16 @@ QUnit.module('Views', {
         parent.destroy();
     });
 
-    QUnit.test('SelectCreateDialog cascade x2many in create mode', function (assert) {
+    QUnit.test('SelectCreateDialog cascade x2many in create mode', async function (assert) {
         assert.expect(5);
 
-        var form = createView({
+        var form = await createView({
             View: FormView,
             model: 'product',
             data: this.data,
             arch: '<form>' +
                      '<field name="name"/>' +
-                     '<field name="partner" widget="one2many_list" >' +
+                     '<field name="partner" widget="one2many" >' +
                         '<tree editable="top">' +
                             '<field name="display_name"/>' +
                             '<field name="instrument"/>' +
@@ -292,7 +295,7 @@ QUnit.module('Views', {
             archs: {
                 'partner,false,form': '<form>' +
                                            '<field name="name"/>' +
-                                           '<field name="instrument" widget="one2many_list" mode="tree"/>' +
+                                           '<field name="instrument" widget="one2many" mode="tree"/>' +
                                         '</form>',
 
                 'instrument,false,form': '<form>'+
@@ -315,31 +318,31 @@ QUnit.module('Views', {
 
             mockRPC: function(route, args) {
                 if (route === '/web/dataset/call_kw/partner/get_formview_id') {
-                    return $.when(false);
+                    return Promise.resolve(false);
                 }
                 if (route === '/web/dataset/call_kw/instrument/get_formview_id') {
-                    return $.when(false);
+                    return Promise.resolve(false);
                 }
                 if (route === '/web/dataset/call_kw/instrument/create') {
                     assert.deepEqual(args.args, [{badassery: [[6, false, [1]]], name: false}],
                         'The method create should have been called with the right arguments');
-                    return $.when(false);
+                    return Promise.resolve(false);
                 }
                 return this._super(route, args);
             },
         });
 
-        form.$buttons.find('.o_form_button_edit').click();
-        form.$('.o_field_x2many_list_row_add a').click();
-        form.$('.o_field_widget .o_field_many2one[name=instrument] input').click();
-        $('ul.ui-autocomplete.ui-front.ui-menu.ui-widget.ui-widget-content li.o_m2o_dropdown_option').first().click();
+        await testUtils.form.clickEdit(form);
+        await testUtils.dom.click(form.$('.o_field_x2many_list_row_add a'));
+        await testUtils.dom.click(form.$('.o_field_widget .o_field_many2one[name=instrument] input'));
+        await testUtils.dom.click($('ul.ui-autocomplete.ui-front.ui-menu.ui-widget.ui-widget-content li.o_m2o_dropdown_option').first());
 
         var $modal = $('.modal-lg');
 
         assert.equal($modal.length, 1,
             'There should be one modal');
 
-        $modal.find('.o_field_x2many_list_row_add a').click();
+        await testUtils.dom.click($modal.find('.o_field_x2many_list_row_add a'));
 
         var $modals = $('.modal-lg');
 
@@ -347,9 +350,9 @@ QUnit.module('Views', {
             'There should be two modals');
 
         var $second_modal = $modals.not($modal);
-        $second_modal.find('.o_list_view.table.table-sm.table-striped.o_list_view_ungrouped .o_data_row input[type=checkbox]').click();
+        await testUtils.dom.click($second_modal.find('.o_list_table.table.table-sm.table-striped.o_list_table_ungrouped .o_data_row input[type=checkbox]'));
 
-        $second_modal.find('.o_select_button').click();
+        await testUtils.dom.click($second_modal.find('.o_select_button'));
 
         $modal = $('.modal-lg');
 
@@ -359,18 +362,18 @@ QUnit.module('Views', {
         assert.equal($modal.find('.o_data_cell').text(), 'Awsome',
             'There should be one item in the list of the modal');
 
-        $modal.find('.btn.btn-primary').click();
+        await testUtils.dom.click($modal.find('.btn.btn-primary'));
 
         form.destroy();
     });
 
-    QUnit.test('Form dialog and subview with _view_ref contexts', function (assert) {
+    QUnit.test('Form dialog and subview with _view_ref contexts', async function (assert) {
         assert.expect(2);
 
         this.data.instrument.records = [{id: 1, name: 'Tromblon', badassery: [1]}];
         this.data.partner.records[0].instrument = 1;
 
-        var form = createView({
+        var form = await createView({
             View: FormView,
             model: 'partner',
             data: this.data,
@@ -395,7 +398,7 @@ QUnit.module('Views', {
 
             mockRPC: function(route, args) {
                 if (args.method === 'get_formview_id') {
-                    return $.when(false);
+                    return Promise.resolve(false);
                 }
                 return this._super(route, args);
             },
@@ -408,24 +411,28 @@ QUnit.module('Views', {
                             'The correct _view_ref should have been sent to the server, first time');
                     }
                     if (ev.data.modelName === 'badassery') {
-                        assert.deepEqual(evaluatedContext, {tree_view_ref: 'some_other_tree_view'},
-                            'The correct _view_ref should have been sent to the server for the subview');
+                        assert.deepEqual(evaluatedContext, {
+                            base_model_name: 'instrument',
+                            tree_view_ref: 'some_other_tree_view',
+                        }, 'The correct _view_ref should have been sent to the server for the subview');
                     }
                 },
             },
         });
 
-        form.$('.o_field_widget[name="instrument"] button.o_external_button').click();
+        await testUtils.dom.click(form.$('.o_field_widget[name="instrument"] button.o_external_button'));
         form.destroy();
     });
 
-    QUnit.test('SelectCreateDialog: save current search', function (assert) {
+    QUnit.test('SelectCreateDialog: save current search', async function (assert) {
         assert.expect(4);
 
-        testUtils.patch(ListController, {
-            getContext: function () {
+        testUtils.mock.patch(ListController, {
+            getOwnedQueryParams: function () {
                 return {
-                    shouldBeInFilterContext: true,
+                    context: {
+                        shouldBeInFilterContext: true,
+                    },
                 };
             },
         });
@@ -454,30 +461,34 @@ QUnit.module('Views', {
             },
         });
 
-        var dialog = new dialogs.SelectCreateDialog(parent, {
+        var dialog;
+        new dialogs.SelectCreateDialog(parent, {
             context: {shouldNotBeInFilterContext: false},
             res_model: 'partner',
-        }).open();
+        }).open().then(function (result) {
+            dialog = result;
+        });
+        await testUtils.nextTick();
 
-        assert.strictEqual(dialog.$('.o_data_row').length, 3,
-            "should contain 3 records");
+        assert.containsN(dialog, '.o_data_row', 3, "should contain 3 records");
 
         // filter on bar
-        dialog.$('.o_filters_menu a:contains(Bar)').click();
+        await testUtils.dom.click(dialog.$('.o_dropdown_toggler_btn:contains(Filters)'));
+        await testUtils.dom.click(dialog.$('.o_filters_menu a:contains(Bar)'));
 
-        assert.strictEqual(dialog.$('.o_data_row').length, 2,
-            "should contain 2 records");
+        assert.containsN(dialog, '.o_data_row', 2, "should contain 2 records");
 
         // save filter
-        dialog.$('.o_save_search a').click(); // toggle 'Save current search'
-        dialog.$('.o_save_name input[type=text]').val('some name'); // name the filter
-        dialog.$('.o_save_name button').click(); // click on 'Save'
+        await testUtils.dom.click(dialog.$('.o_dropdown_toggler_btn:contains(Favorites)'));
+        await testUtils.dom.click(dialog.$('.o_add_favorite'));
+        await testUtils.fields.editInput(dialog.$('.o_favorite_name input[type=text]'), 'some name'); // name the filter
+        await testUtils.dom.click(dialog.$('.o_save_favorite button'));
 
-        testUtils.unpatch(ListController);
+        testUtils.mock.unpatch(ListController);
         parent.destroy();
     });
 
-    QUnit.test('propagate can_create onto the search popup o2m', function (assert) {
+    QUnit.test('propagate can_create onto the search popup o2m', async function (assert) {
         assert.expect(3);
 
         this.data.instrument.records = [
@@ -491,7 +502,7 @@ QUnit.module('Views', {
             {id: 8, name: 'Tromblon8'},
         ];
 
-        var form = createView({
+        var form = await createView({
             View: FormView,
             model: 'partner',
             data: this.data,
@@ -514,18 +525,19 @@ QUnit.module('Views', {
 
             mockRPC: function(route, args) {
                 if (args.method === 'get_formview_id') {
-                    return $.when(false);
+                    return Promise.resolve(false);
                 }
                 return this._super(route, args);
             },
         });
 
-        form.$('.o_field_widget[name="instrument"] .o_input').click();
+        await testUtils.dom.click(form.$('.o_field_widget[name="instrument"] .o_input'));
 
         assert.notOk($('.ui-autocomplete a:contains(Create and Edit)').length,
             'Create and edit not present in dropdown');
 
-        $('.ui-autocomplete a:contains(Search More)').trigger('mouseenter').trigger('click');
+        await testUtils.dom.triggerEvents($('.ui-autocomplete a:contains(Search More)'),
+            ['mouseenter', 'click']);
 
         var $modal = $('.modal-dialog.modal-lg');
 
