@@ -4,22 +4,32 @@ odoo.define('web_editor.rte.summernote', function (require) {
 var ajax = require('web.ajax');
 var Class = require('web.Class');
 var core = require('web.core');
-var ColorpickerDialog = require('web.colorpicker');
+var ColorpickerDialog = require('web.ColorpickerDialog');
 var mixins = require('web.mixins');
-var base = require('web_editor.base');
+const session = require('web.session');
+var fonts = require('wysiwyg.fonts');
 var weContext = require('web_editor.context');
 var rte = require('web_editor.rte');
-var weWidgets = require('web_editor.widget');
+var ServicesMixin = require('web.ServicesMixin');
+var weWidgets = require('wysiwyg.widgets');
 
 var QWeb = core.qweb;
 var _t = core._t;
 
-ajax.jsonRpc('/web/dataset/call', 'call', {
-    'model': 'ir.ui.view',
-    'method': 'read_template',
-    'args': ['web_editor.colorpicker', weContext.get()]
-}).done(function (data) {
-    QWeb.add_template(data);
+session.is_bound.then(() => {
+    if (session.is_website_user) {
+        return;
+    }
+    ajax.jsonRpc('/web/dataset/call_kw', 'call', {
+        'model': 'ir.ui.view',
+        'method': 'read_template',
+        'args': ['web_editor.colorpicker'],
+        'kwargs': {
+            'context': weContext.get(),
+        }
+    }).then(function (data) {
+        QWeb.add_template('<templates>' + data + '</templates>');
+    });
 });
 
 // Summernote Lib (neek change to make accessible: method and object)
@@ -40,7 +50,7 @@ function _rgbToHex(cssColor) {
     if (rgba[4]) {
         return cssColor;
     }
-    var hex = ColorpickerDialog.prototype.convertRgbToHex(
+    var hex = ColorpickerDialog.convertRgbToHex(
         parseInt(rgba[1]),
         parseInt(rgba[2]),
         parseInt(rgba[3])
@@ -56,11 +66,9 @@ var fn_createPalette = renderer.createPalette;
 renderer.createPalette = function ($container, options) {
     fn_createPalette.call(this, $container, options);
 
-    if (!QWeb.has_template('web_editor.colorpicker')) {
-        return;
-    }
-
-    var $clpicker = $(QWeb.render('web_editor.colorpicker'));
+    const $clpicker = QWeb.has_template('web_editor.colorpicker')
+        ? $(QWeb.render('web_editor.colorpicker'))
+        : $(document.createElement('colorpicker'));
 
     var groups;
     if ($clpicker.is("colorpicker")) {
@@ -71,17 +79,20 @@ renderer.createPalette = function ($container, options) {
         groups = [$clpicker.find("button").empty()];
     }
 
-    var html = "<h6 class='mt-2'>" + _t("Theme colors") + "</h6>" + _.map(groups, function ($group) {
-        var $row = $("<div/>", {"class": "note-color-row mb8"}).append($group);
-        var $after_breaks = $row.find(".o_small + :not(.o_small)");
-        if ($after_breaks.length === 0) {
-            $after_breaks = $row.find(":nth-child(8n+9)");
-        }
-        $after_breaks.addClass("o_clear");
-        return $row[0].outerHTML;
-    }).join("") + "<h6 class='mt-2'>" + _t("Common colors") + "</h6>";
     var $palettes = $container.find(".note-color .note-color-palette");
-    $palettes.prepend(html);
+
+    if (groups.length) {
+        var html = "<h6 class='mt-2'>" + _t("Theme colors") + "</h6>" + _.map(groups, function ($group) {
+            var $row = $("<div/>", {"class": "note-color-row mb8"}).append($group);
+            var $after_breaks = $row.find(".o_small + :not(.o_small)");
+            if ($after_breaks.length === 0) {
+                $after_breaks = $row.find(":nth-child(8n+9)");
+            }
+            $after_breaks.addClass("o_clear");
+            return $row[0].outerHTML;
+        }).join("") + "<h6 class='mt-2'>" + _t("Common colors") + "</h6>";
+        $palettes.prepend(html);
+    }
 
     // Find the custom colors which are used in the page and add them to the color palette
     var colors = [];
@@ -107,13 +118,27 @@ renderer.createPalette = function ($container, options) {
 
     $palettes.push.apply($palettes, $customColorPalettes);
 
-    var $fore = $palettes.filter(":even").find("button:not(.note-color-btn)").addClass("note-color-btn");
-    var $bg = $palettes.filter(":odd").find("button:not(.note-color-btn)").addClass("note-color-btn");
+    var $forePalette = $palettes.filter(":even").closest('li').addClass('note-palette');
+    var $bgPalette = $palettes.filter(":odd").closest('li').addClass('note-palette');
+
+    $forePalette.add($bgPalette).find('.note-color-reset').remove();
+    $forePalette.children().each(function () {
+        var $reset = $('<div class="note-color-reset" data-event="foreColor" data-value="inherit"></div>').text(_t('Reset to default'));
+        $(this).prepend($reset);
+    })
+    $bgPalette.children().each(function () {
+        var $reset = $('<div class="note-color-reset" data-event="backColor" data-value="inherit"></div>').text(_t('Reset to default'));
+        $(this).prepend($reset);
+    })
+
+    var $fore = $forePalette.find("button:not(.note-color-btn)").addClass("note-color-btn");
     $fore.each(function () {
         var $el = $(this);
         var className = $el.hasClass('o_custom_color') ? $el.data('color') : 'text-' + $el.data('color');
         $el.attr('data-event', 'foreColor').attr('data-value', className).addClass($el.hasClass('o_custom_color') ? '' : 'bg-' + $el.data('color'));
     });
+
+    var $bg = $bgPalette.find("button:not(.note-color-btn)").addClass("note-color-btn");
     $bg.each(function () {
         var $el = $(this);
         var className = $el.hasClass('o_custom_color') ? $el.data('color') : 'bg-' + $el.data('color');
@@ -325,6 +350,15 @@ eventHandler.modules.popover.button.update = function ($container, oStyle) {
     }
 };
 
+var fn_toolbar_boutton_update = eventHandler.modules.toolbar.button.update;
+eventHandler.modules.toolbar.button.update = function ($container, oStyle) {
+    fn_toolbar_boutton_update.call(this, $container, oStyle);
+
+    $container.find('button[data-event="insertUnorderedList"]').toggleClass("active", $(oStyle.ancestors).is('ul:not(.o_checklist)'));
+    $container.find('button[data-event="insertOrderedList"]').toggleClass("active", $(oStyle.ancestors).is('ol'));
+    $container.find('button[data-event="insertCheckList"]').toggleClass("active", $(oStyle.ancestors).is('ul.o_checklist'));
+};
+
 var fn_popover_update = eventHandler.modules.popover.update;
 eventHandler.modules.popover.update = function ($popover, oStyle, isAirMode) {
     var $imagePopover = $popover.find('.note-image-popover');
@@ -426,6 +460,12 @@ eventHandler.modules.linkDialog.showLinkDialog = function ($editable, $dialog, l
     $editable.data('range').select();
     $editable.data('NoteHistory').recordUndo();
 
+    var commonAncestor = linkInfo.range.commonAncestor();
+    if (commonAncestor && commonAncestor.closest) {
+        var link = commonAncestor.closest('a');
+        linkInfo.className = link && link.className;
+    }
+
     var def = new $.Deferred();
     core.bus.trigger('link_dialog_demand', {
         $editable: $editable,
@@ -449,13 +489,18 @@ eventHandler.modules.imageDialog.showImageDialog = function ($editable) {
     var media = $(r.sc).parents().addBack().filter(function (i, el) {
         return dom.isImg(el);
     })[0];
+    var options = $editable.closest('.o_editable, .note-editor').data('options');
     core.bus.trigger('media_dialog_demand', {
         $editable: $editable,
         media: media,
         options: {
-            lastFilters: ['background'],
             onUpload: $editable.data('callbacks').onUpload,
-            noVideos: $editable.data('oe-model') === "mail.compose.message",
+            noVideos: options && options.noVideos,
+        },
+        onSave: function (media) {
+            if(media && !document.body.contains(media)) {
+            r.insertNode(media);
+            };
         },
     });
     return new $.Deferred().reject();
@@ -520,8 +565,8 @@ dom.isImgFont = function (node) {
     var className = (node && node.className || "");
     if (node && (nodeName === "SPAN" || nodeName === "I") && className.length) {
         var classNames = className.split(/\s+/);
-        for (var k=0; k<base.fontIcons.length; k++) {
-            if (_.intersection(base.fontIcons[k].alias, classNames).length) {
+        for (var k=0; k<fonts.fontIcons.length; k++) {
+            if (_.intersection(fonts.fontIcons[k].alias, classNames).length) {
                 return true;
             }
         }
@@ -621,7 +666,7 @@ function prettify_html(html) {
  *   disable (false) or enable (true) the code view mode.
  */
 $.summernote.pluginEvents.codeview = function (event, editor, layoutInfo, enable) {
-    if (layoutInfo === undefined) {
+    if (!layoutInfo) {
         return;
     }
     if (layoutInfo.toolbar) {
@@ -1096,7 +1141,7 @@ $.summernote.lang.odoo = {
  * instantiate media, link and alt dialogs outside the main editor: in the
  * simple HTML fields and forum textarea.
  */
-var SummernoteManager = Class.extend(mixins.EventDispatcherMixin, {
+var SummernoteManager = Class.extend(mixins.EventDispatcherMixin, ServicesMixin, {
     /**
      * @constructor
      */
@@ -1123,6 +1168,56 @@ var SummernoteManager = Class.extend(mixins.EventDispatcherMixin, {
         core.bus.off('media_dialog_demand', this, this._onMediaDialogDemand);
     },
 
+    /**
+     * Create/Update cropped attachments.
+     *
+     * @param {jQuery} $editable
+     * @returns {Promise}
+     */
+    saveCroppedImages: function ($editable) {
+        var defs = _.map($editable.find('.o_cropped_img_to_save'), async croppedImg => {
+            var $croppedImg = $(croppedImg);
+            $croppedImg.removeClass('o_cropped_img_to_save');
+
+            var resModel = $croppedImg.data('crop:resModel');
+            var resID = $croppedImg.data('crop:resID');
+            var cropID = $croppedImg.data('crop:id');
+            var mimetype = $croppedImg.data('crop:mimetype');
+            var originalSrc = $croppedImg.data('crop:originalSrc');
+
+            var datas = $croppedImg.attr('src').split(',')[1];
+            let attachmentID = cropID;
+            if (!cropID) {
+                var name = originalSrc + '.crop';
+                attachmentID = await this._rpc({
+                    model: 'ir.attachment',
+                    method: 'create',
+                    args: [{
+                        res_model: resModel,
+                        res_id: resID,
+                        name: name,
+                        datas: datas,
+                        mimetype: mimetype,
+                        url: originalSrc, // To save the original image that was cropped
+                    }],
+                });
+            } else {
+                await this._rpc({
+                    model: 'ir.attachment',
+                    method: 'write',
+                    args: [[cropID], {datas: datas}],
+                });
+            }
+            const access_token = await this._rpc({
+                model: 'ir.attachment',
+                method: 'generate_access_token',
+                args: [[attachmentID]],
+            });
+            $croppedImg.attr('src', '/web/image/' + attachmentID + '?access_token=' + access_token[0]);
+        });
+        return Promise.all(defs);
+    },
+
     //--------------------------------------------------------------------------
     // Handlers
     //--------------------------------------------------------------------------
@@ -1140,7 +1235,6 @@ var SummernoteManager = Class.extend(mixins.EventDispatcherMixin, {
         data.__alreadyDone = true;
         var altDialog = new weWidgets.AltDialog(this,
             data.options || {},
-            data.$editable,
             data.media
         );
         if (data.onSave) {
@@ -1188,7 +1282,6 @@ var SummernoteManager = Class.extend(mixins.EventDispatcherMixin, {
                 res_model: data.$editable.data('oe-model'),
                 res_id: data.$editable.data('oe-id'),
             }, data.options || {}),
-            data.$editable,
             data.media
         );
         if (data.onSave) {
@@ -1241,7 +1334,6 @@ var SummernoteManager = Class.extend(mixins.EventDispatcherMixin, {
                 res_id: data.$editable.data('oe-id'),
                 domain: data.$editable.data('oe-media-domain'),
             }, data.options),
-            data.$editable,
             data.media
         );
         if (data.onSave) {
@@ -1272,6 +1364,20 @@ rte.Class.include({
         this._super.apply(this, arguments);
         this._summernoteManager.destroy();
     },
+
+    //--------------------------------------------------------------------------
+    // Public
+    //--------------------------------------------------------------------------
+
+    /**
+     * Create/Update cropped attachments.
+     *
+     * @param {jQuery} $editable
+     * @returns {Deferred}
+     */
+    saveCroppedImages: function ($editable) {
+        return this._summernoteManager.saveCroppedImages($editable);
+    },
 });
 return SummernoteManager;
 });
