@@ -1,15 +1,23 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import assets
 from . import ir_actions
 from . import ir_attachment
 from . import ir_http
 from . import ir_qweb
 from . import ir_qweb_fields
+from . import mixins
 from . import website
+from . import website_menu
+from . import website_page
+from . import website_rewrite
 from . import ir_rule
+from . import ir_translation
 from . import ir_ui_view
 from . import res_company
 from . import res_partner
 from . import res_users
 from . import res_config_settings
+from . import res_lang
+from . import website_visitor
