@@ -5,10 +5,9 @@ var Class = require('web.Class');
 var core = require('web.core');
 var Dialog = require('web.Dialog');
 var time = require('web.time');
-var weContext = require('web_editor.context');
-var weWidgets = require('web_editor.widget');
+var weWidgets = require('wysiwyg.widgets');
 var websiteNavbarData = require('website.navbar');
-var websiteRootData = require('website.WebsiteRoot');
+var websiteRootData = require('website.root');
 var Widget = require('web.Widget');
 
 var _t = core._t;
@@ -87,13 +86,13 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
         }));
 
         defs.push(this._rpc({
-            model: 'website.redirect',
+            model: 'website.rewrite',
             method: 'fields_get',
         }).then(function (fields) {
             self.fields = fields;
         }));
 
-        return $.when.apply($, defs);
+        return Promise.all(defs);
     },
     /**
      * @override
@@ -176,7 +175,7 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
         }
         this.$('#date_publish_container').datetimepicker(datepickersOptions);
 
-        return $.when.apply($, defs);
+        return Promise.all(defs);
     },
     /**
      * @override
@@ -195,7 +194,12 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
      */
     save: function (data) {
         var self = this;
-        var context = weContext.get();
+        var context;
+        this.trigger_up('context_get', {
+            callback: function (ctx) {
+                context = ctx;
+            },
+        });
         var url = this.$('#page_url').val();
 
         var $date_publish = this.$("#date_publish");
@@ -251,7 +255,7 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
      *
      * @private
      * @param {integer} moID
-     * @returns {Deferred<Array>}
+     * @returns {Promise<Array>}
      */
     _getPageDependencies: function (moID) {
         return this._rpc({
@@ -265,7 +269,7 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
      *
      * @private
      * @param {integer} moID
-     * @returns {Deferred<Array>}
+     * @returns {Promise<Array>}
      */
     _getPageKeyDependencies: function (moID) {
         return this._rpc({
@@ -278,7 +282,7 @@ var PagePropertiesDialog = weWidgets.Dialog.extend({
      * Retrieves supported mimtype
      *
      * @private
-     * @returns {Deferred<Array>}
+     * @returns {Promise<Array>}
      */
     _getSupportedMimetype: function () {
         return this._rpc({
@@ -364,13 +368,16 @@ var MenuEntryDialog = weWidgets.LinkDialog.extend({
     /**
      * @constructor
      */
-    init: function (parent, options, editor, data) {
-        data.text = data.name || '';
-        data.isNewWindow = data.new_window;
-        this.data = data;
-        this._super(parent, _.extend({}, {
-            title: _t("Create Menu"),
-        }, options || {}), editor, data);
+    init: function (parent, options, editable, data) {
+        this._super(parent, _.extend({
+            title: _t("Add a menu item"),
+        }, options || {}), editable, _.extend({
+            needLabel: true,
+            text: data.name || '',
+            isNewWindow: data.new_window,
+        }, data || {}));
+
+        this.menuType = data.menuType;
     },
     /**
      * @override
@@ -385,7 +392,12 @@ var MenuEntryDialog = weWidgets.LinkDialog.extend({
         // Adapt URL label
         this.$('label[for="o_link_dialog_label_input"]').text(_t("Menu Label"));
 
-        this.$('#o_link_dialog_url_input').after(qweb.render('website.contentMenu.dialog.edit.link_menu_hint'));
+        // Auto add '#' URL and hide the input if for mega menu
+        if (this.menuType === 'mega') {
+            var $url = this.$('input[name="url"]');
+            $url.val('#').trigger('change');
+            $url.closest('.form-group').addClass('d-none');
+        }
 
         return this._super.apply(this, arguments);
     },
@@ -400,13 +412,12 @@ var MenuEntryDialog = weWidgets.LinkDialog.extend({
     save: function () {
         var $e = this.$('#o_link_dialog_label_input');
         if (!$e.val() || !$e[0].checkValidity()) {
-            $e.closest('.form-group').addClass('o_has_error').find('.form-control, .custom-select').addClass('is-invalid')
+            $e.closest('.form-group').addClass('o_has_error').find('.form-control, .custom-select').addClass('is-invalid');
             $e.focus();
             return;
         }
         return this._super.apply(this, arguments);
     },
-
 });
 
 var SelectEditMenuDialog = weWidgets.Dialog.extend({
@@ -423,7 +434,8 @@ var SelectEditMenuDialog = weWidgets.Dialog.extend({
         var self = this;
         self.roots = [{id: null, name: _t("Top Menu")}];
         $('[data-content_menu_id]').each(function () {
-            self.roots.push({id: $(this).data('content_menu_id'), name: $(this).attr('name')});
+            // Remove name fallback in master
+            self.roots.push({id: $(this).data('content_menu_id'), name: $(this).attr('name') || $(this).data('menu_name')});
         });
         this._super(parent, _.extend({}, {
             title: _t("Select a Menu"),
@@ -466,19 +478,23 @@ var EditMenuDialog = weWidgets.Dialog.extend({
      */
     willStart: function () {
         var defs = [this._super.apply(this, arguments)];
-        var self = this;
-        var context = weContext.get();
+        var context;
+        this.trigger_up('context_get', {
+            callback: function (ctx) {
+                context = ctx;
+            },
+        });
         defs.push(this._rpc({
             model: 'website.menu',
             method: 'get_tree',
             args: [context.website_id, this.rootID],
-        }).then(function (menu) {
-            self.menu = menu;
-            self.root_menu_id = menu.id;
-            self.flat = self._flatenize(menu);
-            self.to_delete = [];
+        }).then(menu => {
+            this.menu = menu;
+            this.rootMenuID = menu.fields['id'];
+            this.flat = this._flatenize(menu);
+            this.toDelete = [];
         }));
-        return $.when.apply($, defs);
+        return Promise.all(defs);
     },
     /**
      * @override
@@ -497,6 +513,10 @@ var EditMenuDialog = weWidgets.Dialog.extend({
             tolerance: 'pointer',
             attribute: 'data-menu-id',
             expression: '()(.+)', // nestedSortable takes the second match of an expression (*sigh*)
+            isAllowed: (placeholder, placeholderParent, currentItem) => {
+                return !placeholderParent
+                    || !currentItem[0].dataset.megaMenu && !placeholderParent[0].dataset.megaMenu;
+            },
         });
         return r;
     },
@@ -510,26 +530,35 @@ var EditMenuDialog = weWidgets.Dialog.extend({
      */
     save: function () {
         var _super = this._super.bind(this);
-        var self = this;
-        var new_menu = this.$('.oe_menu_editor').nestedSortable('toArray', {startDepthCount: 0});
+        var newMenus = this.$('.oe_menu_editor').nestedSortable('toArray', {startDepthCount: 0});
         var levels = [];
         var data = [];
-        var context = weContext.get();
+        var context;
+        this.trigger_up('context_get', {
+            callback: function (ctx) {
+                context = ctx;
+            },
+        });
         // Resequence, re-tree and remove useless data
-        new_menu.forEach(function (menu) {
+        newMenus.forEach(menu => {
             if (menu.id) {
                 levels[menu.depth] = (levels[menu.depth] || 0) + 1;
-                var mobj = self.flat[menu.id];
-                mobj.sequence = levels[menu.depth];
-                mobj.parent_id = (menu.parent_id|0) || menu.parent_id || self.root_menu_id;
-                delete(mobj.children);
-                data.push(mobj);
+                var menuFields = this.flat[menu.id].fields;
+                menuFields['sequence'] = levels[menu.depth];
+                menuFields['parent_id'] = menu['parent_id'] || this.rootMenuID;
+                data.push(menuFields);
             }
         });
-        this._rpc({
+        return this._rpc({
             model: 'website.menu',
             method: 'save',
-            args: [context.website_id, { data: data, to_delete: self.to_delete }],
+            args: [
+                context.website_id,
+                {
+                    'data': data,
+                    'to_delete': this.toDelete,
+                }
+            ],
         }).then(function () {
             return _super();
         });
@@ -550,10 +579,9 @@ var EditMenuDialog = weWidgets.Dialog.extend({
      */
     _flatenize: function (node, _dict) {
         _dict = _dict || {};
-        var self = this;
-        _dict[node.id] = node;
-        node.children.forEach(function (child) {
-            self._flatenize(child, _dict);
+        _dict[node.fields['id']] = node;
+        node.children.forEach(child => {
+            this._flatenize(child, _dict);
         });
         return _dict;
     },
@@ -567,23 +595,31 @@ var EditMenuDialog = weWidgets.Dialog.extend({
      * dialog to edit this new menu.
      *
      * @private
+     * @param {Event} ev
      */
-    _onAddMenuButtonClick: function () {
-        var self = this;
-        var dialog = new MenuEntryDialog(this, {}, undefined, {});
-        dialog.on('save', this, function (link) {
-            var new_menu = {
-                id: _.uniqueId('new-'),
-                name: link.text,
-                url: link.url,
-                new_window: link.isNewWindow,
-                parent_id: false,
-                sequence: 0,
-                children: [],
+    _onAddMenuButtonClick: function (ev) {
+        var menuType = ev.currentTarget.dataset.type;
+        var dialog = new MenuEntryDialog(this, {}, null, {
+            menuType: menuType,
+        });
+        dialog.on('save', this, link => {
+            var newMenu = {
+                'fields': {
+                    'id': _.uniqueId('new-'),
+                    'name': link.text,
+                    'url': link.url,
+                    'new_window': link.isNewWindow,
+                    'is_mega_menu': menuType === 'mega',
+                    'sequence': 0,
+                    'parent_id': false,
+                },
+                'children': [],
+                'is_homepage': false,
             };
-            self.flat[new_menu.id] = new_menu;
-            self.$('.oe_menu_editor').append(
-                qweb.render('website.contentMenu.dialog.submenu', { submenu: new_menu }));
+            this.flat[newMenu.fields['id']] = newMenu;
+            this.$('.oe_menu_editor').append(
+                qweb.render('website.contentMenu.dialog.submenu', {submenu: newMenu})
+            );
         });
         dialog.open();
     },
@@ -594,9 +630,9 @@ var EditMenuDialog = weWidgets.Dialog.extend({
      */
     _onDeleteMenuButtonClick: function (ev) {
         var $menu = $(ev.currentTarget).closest('[data-menu-id]');
-        var menuID = $menu.data('menu-id')|0;
+        var menuID = parseInt($menu.data('menu-id'));
         if (menuID) {
-            this.to_delete.push(menuID);
+            this.toDelete.push(menuID);
         }
         $menu.remove();
     },
@@ -607,21 +643,20 @@ var EditMenuDialog = weWidgets.Dialog.extend({
      * @private
      */
     _onEditMenuButtonClick: function (ev) {
-        var self = this;
-        var menu_id = $(ev.currentTarget).closest('[data-menu-id]').data('menu-id');
-        var menu = self.flat[menu_id];
+        var $menu = $(ev.currentTarget).closest('[data-menu-id]');
+        var menuID = $menu.data('menu-id');
+        var menu = this.flat[menuID];
         if (menu) {
-            var dialog = new MenuEntryDialog(this, {}, undefined, menu);
-            dialog.on('save', this, function (link) {
-                var id = link.id;
-                var menu_obj = self.flat[id];
-                _.extend(menu_obj, {
+            var dialog = new MenuEntryDialog(this, {}, null, _.extend({
+                menuType: menu.fields['is_mega_menu'] ? 'mega' : undefined,
+            }, menu.fields));
+            dialog.on('save', this, link => {
+                _.extend(menu.fields, {
                     'name': link.text,
                     'url': link.url,
                     'new_window': link.isNewWindow,
                 });
-                var $menu = self.$('[data-menu-id="' + id + '"]');
-                $menu.find('.js_menu_label').first().text(menu_obj.name);
+                $menu.find('.js_menu_label').first().text(menu.fields['name']);
             });
             dialog.open();
         } else {
@@ -720,63 +755,73 @@ var ContentMenu = websiteNavbarData.WebsiteNavbarActionWidget.extend({
      *
      * @private
      * @param {function} [beforeReloadCallback]
-     * @returns {Deferred}
+     * @returns {Promise}
      *          Unresolved if the menu is edited and saved as the page will be
      *          reloaded.
      *          Resolved otherwise.
      */
     _editMenu: function (beforeReloadCallback) {
         var self = this;
-        var def = $.Deferred();
-
-        // If there is multiple menu on the page, ask the user which one he
-        // wants to edit
-        var selectDef = $.Deferred();
-        if ($('[data-content_menu_id]').length) {
-            var select = new SelectEditMenuDialog(this);
-            select.on('save', selectDef, selectDef.resolve);
-            select.on('cancel', def, def.resolve);
-            select.open();
-        } else {
-            selectDef.resolve(null);
-        }
-        selectDef.then(function (rootID) {
-            // Open the dialog to show the menu structure and allow its edition
-            var editDef = $.Deferred();
-            var dialog = new EditMenuDialog(self, {}, rootID).open();
-            dialog.on('save', editDef, editDef.resolve);
-            dialog.on('cancel', def, def.resolve);
-            return editDef;
-        }).then(function () {
-            // Before reloading the page after menu modification, does the
-            // given action to do.
-            return beforeReloadCallback && beforeReloadCallback();
-        }).then(function () {
-            // Reload the page so that the menu modification are shown
-            window.location.reload(true);
+        return new Promise(function (resolve) {
+            function resolveWhenEditMenuDialogIsCancelled(rootID) {
+                return self._openEditMenuDialog(rootID, beforeReloadCallback).then(resolve);
+            }
+            if ($('[data-content_menu_id]').length) {
+                var select = new SelectEditMenuDialog(self);
+                select.on('save', self, resolveWhenEditMenuDialogIsCancelled);
+                select.on('cancel', self, resolve);
+                select.open();
+            } else {
+                resolveWhenEditMenuDialogIsCancelled(null);
+            }
         });
-
-        return def;
     },
+    /**
+     *
+     * @param {*} rootID
+     * @param {function|undefied} beforeReloadCallback function that returns a promise
+     * @returns {Promise}
+     */
+    _openEditMenuDialog: function (rootID, beforeReloadCallback) {
+        var self = this;
+        return new Promise(function (resolve) {
+            var dialog = new EditMenuDialog(self, {}, rootID);
+            dialog.on('save', self, function () {
+                // Before reloading the page after menu modification, does the
+                // given action to do.
+                if (beforeReloadCallback) {
+                    // Reload the page so that the menu modification are shown
+                    beforeReloadCallback().then(function () {
+                        window.location.reload(true);
+                    });
+                } else {
+                    window.location.reload(true);
+                }
+            });
+            dialog.on('cancel', self, resolve);
+            dialog.open();
+        });
+    },
+
     /**
      * Retrieves the value of a page option.
      *
      * @private
      * @param {string} name
-     * @returns {Deferred<*>}
+     * @returns {Promise<*>}
      */
     _getPageOption: function (name) {
         var option = this.pageOptions[name];
         if (!option) {
-            return $.Deferred().reject();
+            return Promise.reject();
         }
-        return $.when(option.value);
+        return Promise.resolve(option.value);
     },
     /**
      * On save, simulated page options have to be server-saved.
      *
      * @private
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _onSave: function () {
         var self = this;
@@ -788,13 +833,13 @@ var ContentMenu = websiteNavbarData.WebsiteNavbarActionWidget.extend({
                 }, true, true);
             }
         });
-        return $.when.apply($, defs);
+        return Promise.all(defs);
     },
     /**
      * Opens the page properties dialog.
      *
      * @private
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _pageProperties: function () {
         var mo;
@@ -815,7 +860,7 @@ var ContentMenu = websiteNavbarData.WebsiteNavbarActionWidget.extend({
      * @param {*} [params.value] (change value by default true -> false -> true)
      * @param {boolean} [forceSave=false]
      * @param {boolean} [noReload=false]
-     * @returns {Deferred}
+     * @returns {Promise}
      */
     _togglePageOption: function (params, forceSave, noReload) {
         // First check it is a website page
@@ -826,13 +871,13 @@ var ContentMenu = websiteNavbarData.WebsiteNavbarActionWidget.extend({
             },
         });
         if (mo.model !== 'website.page') {
-            return $.Deferred().reject();
+            return Promise.reject();
         }
 
         // Check if this is a valid option
         var option = this.pageOptions[params.name];
         if (!option) {
-            return $.Deferred().reject();
+            return Promise.reject();
         }
 
         // Toggle the value
@@ -841,23 +886,23 @@ var ContentMenu = websiteNavbarData.WebsiteNavbarActionWidget.extend({
         // If simulate is true, it means we want the option to be toggled but
         // not saved on the server yet
         if (!forceSave) {
-            return $.when();
+            return Promise.resolve();
         }
 
         // If not, write on the server page and reload the current location
         var vals = {};
         vals[params.name] = option.value;
-        var def = this._rpc({
+        var prom = this._rpc({
             model: 'website.page',
             method: 'write',
             args: [[mo.id], vals],
         });
         if (noReload) {
-            return def;
+            return prom;
         }
-        return def.then(function () {
+        return prom.then(function () {
             window.location.reload();
-            return $.Deferred();
+            return new Promise(function () {});
         });
     },
 });
@@ -879,7 +924,7 @@ var PageManagement = Widget.extend({
      *
      * @private
      * @param {integer} moID
-     * @returns {Deferred<Array>}
+     * @returns {Promise<Array>}
      */
     _getPageDependencies: function (moID) {
         return this._rpc({
@@ -926,35 +971,34 @@ var PageManagement = Widget.extend({
 // TODO: This function should be integrated in a widget in the future
 function _deletePage(pageId, fromPageManagement) {
     var self = this;
-    var def = $.Deferred();
-
-    // Search the page dependencies
-    this._getPageDependencies(pageId)
-    .then(function (dependencies) {
-    // Inform the user about those dependencies and ask him confirmation
-        var confirmDef = $.Deferred();
-        Dialog.safeConfirm(self, "", {
-            title: _t("Delete Page"),
-            $content: $(qweb.render('website.delete_page', {dependencies: dependencies})),
-            confirm_callback: confirmDef.resolve.bind(confirmDef),
-            cancel_callback: def.resolve.bind(self),
-        });
-        return confirmDef;
-    }).then(function () {
-    // Delete the page if the user confirmed
-        return self._rpc({
-            model: 'website.page',
-            method: 'unlink',
-            args: [pageId],
-        });
-    }).then(function () {
-        if (fromPageManagement) {
-            window.location.reload(true);
-        }
-        else {
-            window.location.href = '/';
-        }
-    }, def.reject.bind(def));
+    new Promise(function (resolve, reject) {
+        // Search the page dependencies
+        self._getPageDependencies(pageId)
+        .then(function (dependencies) {
+            // Inform the user about those dependencies and ask him confirmation
+            return new Promise(function (confirmResolve, confirmReject) {
+                Dialog.safeConfirm(self, "", {
+                    title: _t("Delete Page"),
+                    $content: $(qweb.render('website.delete_page', {dependencies: dependencies})),
+                    confirm_callback: confirmResolve,
+                    cancel_callback: resolve,
+                });
+            });
+        }).then(function () {
+            // Delete the page if the user confirmed
+            return self._rpc({
+                model: 'website.page',
+                method: 'unlink',
+                args: [pageId],
+            });
+        }).then(function () {
+            if (fromPageManagement) {
+                window.location.reload(true);
+            } else {
+                window.location.href = '/';
+            }
+        }, reject);
+    });
 }
 
 websiteNavbarData.websiteNavbarRegistry.add(ContentMenu, '#content-menu');
