@@ -7,8 +7,7 @@ var Dialog = require('web.Dialog');
 var mixins = require('web.mixins');
 var rpc = require('web.rpc');
 var Widget = require('web.Widget');
-var weContext = require('web_editor.context');
-var weWidgets = require('web_editor.widget');
+var weWidgets = require('wysiwyg.widgets');
 var websiteNavbarData = require('website.navbar');
 
 var _t = core._t;
@@ -21,7 +20,7 @@ var Suggestion = Widget.extend({
     template: 'website.seo_suggestion',
     xmlDependencies: ['/website/static/src/xml/website.seo.xml'],
     events: {
-        'click .js_seo_suggestion': 'select',
+        'click .o_seo_suggestion': 'select',
     },
 
     init: function (parent, options) {
@@ -49,7 +48,13 @@ var SuggestionList = Widget.extend({
     refresh: function () {
         var self = this;
         self.$el.append(_t("Loading..."));
-        var language = self.language || weContext.get().lang.toLowerCase();
+        var context;
+        this.trigger_up('context_get', {
+            callback: function (ctx) {
+                context = ctx;
+            },
+        });
+        var language = self.language || context.lang.toLowerCase();
         this._rpc({
             route: '/website/seo_suggest',
             params: {
@@ -100,7 +105,8 @@ var Keyword = Widget.extend({
         this._super(parent);
     },
     start: function () {
-        this.$('.js_seo_keyword_suggestion').empty();
+        var self = this;
+        this.$('.o_seo_keyword_suggestion').empty();
         this.suggestionList = new SuggestionList(this, {
             root: this.keyword,
             language: this.language,
@@ -109,12 +115,12 @@ var Keyword = Widget.extend({
         this.suggestionList.on('selected', this, function (word, language) {
             this.trigger('selected', word, language);
         });
-        this.suggestionList.appendTo(this.$('.js_seo_keyword_suggestion'));
-
-        this.htmlPage.on('title-changed', this, this._updateTitle);
-        this.htmlPage.on('description-changed', this, this._updateDescription);
-        this._updateTitle();
-        this._updateDescription();
+        return this.suggestionList.appendTo(this.$('.o_seo_keyword_suggestion')).then(function() {
+            self.htmlPage.on('title-changed', self, self._updateTitle);
+            self.htmlPage.on('description-changed', self, self._updateDescription);
+            self._updateTitle();
+            self._updateDescription();
+        });
     },
     destroy: function () {
         this.trigger('removed');
@@ -123,17 +129,17 @@ var Keyword = Widget.extend({
     _updateTitle: function () {
         var $title = this.$('.js_seo_keyword_title');
         if (this.htmlPage.isInTitle(this.keyword)) {
-            $title.css('visibility','visible');
+            $title.css('visibility', 'visible');
         } else {
-            $title.css('visibility','hidden');
+            $title.css('visibility', 'hidden');
         }
     },
     _updateDescription: function () {
         var $description = this.$('.js_seo_keyword_description');
         if (this.htmlPage.isInDescription(this.keyword)) {
-            $description.css('visibility','visible');
+            $description.css('visibility', 'visible');
         } else {
-            $description.css('visibility','hidden');
+            $description.css('visibility', 'hidden');
         }
     },
 });
@@ -169,7 +175,7 @@ var KeywordList = Widget.extend({
     exists: function (word) {
         return _.contains(this.keywords(), word);
     },
-    add: function (candidate, language) {
+    add: async function (candidate, language) {
         var self = this;
         // TODO Refine
         var word = candidate ? candidate.replace(/[,;.:<>]+/g, ' ').replace(/ +/g, ' ').trim().toLowerCase() : '';
@@ -186,7 +192,7 @@ var KeywordList = Widget.extend({
             keyword.on('selected', self, function (word, language) {
                 self.trigger('selected', word, language);
             });
-            keyword.appendTo(self.$el);
+            await keyword.appendTo(self.$el);
         }
         if (self.isFull()) {
             self.trigger('list-full');
@@ -202,9 +208,9 @@ var Preview = Widget.extend({
     init: function (parent, options) {
         this.title = options.title;
         this.url = options.url;
-        this.description = options.description || _t("The description will be generated by search engines based on page content unless you specify one.");
+        this.description = options.description;
         if (this.description.length > 160) {
-            this.description = this.description.substring(0,159) + '…';
+            this.description = this.description.substring(0, 159) + '…';
         }
         this._super(parent);
     },
@@ -214,6 +220,7 @@ var HtmlPage = Class.extend(mixins.PropertiesMixin, {
     init: function () {
         mixins.PropertiesMixin.init.call(this);
         this.initTitle = this.title();
+        this.defaultTitle = $('meta[name="default_title"]').attr('content');
         this.initDescription = this.description();
     },
     url: function () {
@@ -224,7 +231,7 @@ var HtmlPage = Class.extend(mixins.PropertiesMixin, {
     },
     changeTitle: function (title) {
         // TODO create tag if missing
-        $('title').text(title.trim() || this.initTitle);
+        $('title').text(title.trim() || this.defaultTitle);
         this.trigger('title-changed', title);
     },
     description: function () {
@@ -281,19 +288,19 @@ var HtmlPage = Class.extend(mixins.PropertiesMixin, {
         return $('body').children().not('.oe_seo_configuration').find('h2').text();
     },
     isInBody: function (text) {
-        return new RegExp(WORD_SEPARATORS_REGEX+text+WORD_SEPARATORS_REGEX, 'gi').test(this.bodyText());
+        return new RegExp(WORD_SEPARATORS_REGEX + text + WORD_SEPARATORS_REGEX, 'gi').test(this.bodyText());
     },
     isInTitle: function (text) {
-        return new RegExp(WORD_SEPARATORS_REGEX+text+WORD_SEPARATORS_REGEX, 'gi').test(this.title());
+        return new RegExp(WORD_SEPARATORS_REGEX + text + WORD_SEPARATORS_REGEX, 'gi').test(this.title());
     },
     isInDescription: function (text) {
-        return new RegExp(WORD_SEPARATORS_REGEX+text+WORD_SEPARATORS_REGEX, 'gi').test(this.description());
+        return new RegExp(WORD_SEPARATORS_REGEX + text + WORD_SEPARATORS_REGEX, 'gi').test(this.description());
     },
     isInHeading1: function (text) {
-        return new RegExp(WORD_SEPARATORS_REGEX+text+WORD_SEPARATORS_REGEX, 'gi').test(this.heading1());
+        return new RegExp(WORD_SEPARATORS_REGEX + text + WORD_SEPARATORS_REGEX, 'gi').test(this.heading1());
     },
     isInHeading2: function (text) {
-        return new RegExp(WORD_SEPARATORS_REGEX+text+WORD_SEPARATORS_REGEX, 'gi').test(this.heading2());
+        return new RegExp(WORD_SEPARATORS_REGEX + text + WORD_SEPARATORS_REGEX, 'gi').test(this.heading2());
     },
 });
 
@@ -325,6 +332,7 @@ var MetaTitleDescription = Widget.extend({
         this.canEditTitle = !!options.canEditTitle;
         this.canEditDescription = !!options.canEditDescription;
         this.isIndexed = !!options.isIndexed;
+        this.previewDescription = options.previewDescription;
         this._super(parent, options);
     },
     /**
@@ -336,17 +344,18 @@ var MetaTitleDescription = Widget.extend({
         this.$warning = this.$('div#website_meta_description_warning');
         this.$preview = this.$('.js_seo_preview');
 
-        this._renderPreview();
-
         if (!this.canEditTitle) {
             this.$title.attr('disabled', true);
         }
         if (!this.canEditDescription) {
             this.$description.attr('disabled', true);
         }
-
-        this.$title.val(this.htmlPage.title());
-        this.$description.val(this.htmlPage.description());
+        if (this.htmlPage.title().trim() !== this.htmlPage.defaultTitle.trim()) {
+            this.$title.val(this.htmlPage.title());
+        }
+        if (this.htmlPage.description().trim() !== this.previewDescription) {
+            this.$description.val(this.htmlPage.description());
+        }
 
         this._descriptionOnChange();
     },
@@ -354,13 +363,19 @@ var MetaTitleDescription = Widget.extend({
      * Get the current title
      */
     getTitle: function () {
-        return this.$title.val().trim() || this.htmlPage.initTitle;
+        return this.$title.val().trim() || this.htmlPage.defaultTitle;
     },
     /**
      * Get the current description
      */
     getDescription: function () {
-        return this.$description.val();
+        return this.getRealDescription() || this.previewDescription;
+    },
+    /**
+     * Get the current description chosen by the user
+     */
+    getRealDescription: function () {
+        return this.$description.val() || '';
     },
     /**
      * @private
@@ -406,7 +421,7 @@ var MetaTitleDescription = Widget.extend({
     _renderPreview: function () {
         var indexed = this.isIndexed;
         var preview = "";
-        if (indexed){
+        if (indexed) {
             preview = new Preview(this, {
                 title: this.getTitle(),
                 description: this.getDescription(),
@@ -438,7 +453,7 @@ var MetaKeywords = Widget.extend({
     start: function () {
         var self = this;
         this.$input = this.$('input[name=website_meta_keywords]');
-        this.keywordList = new KeywordList(this, { htmlPage: this.htmlPage });
+        this.keywordList = new KeywordList(this, {htmlPage: this.htmlPage});
         this.keywordList.on('list-full', this, function () {
             self.$input.attr({
                 readonly: 'readonly',
@@ -456,10 +471,10 @@ var MetaKeywords = Widget.extend({
         this.keywordList.on('content-updated', this, function (removed) {
             self._updateTable(removed);
         });
-        this.keywordList.insertAfter(this.$('.table thead'));
-
-        this._getLanguages();
-        this._updateTable();
+        return this.keywordList.insertAfter(this.$('.table thead')).then(function() {
+            self._getLanguages();
+            self._updateTable();
+        });
     },
     _addKeyword: function () {
         var $language = this.$('select[name=seo_page_language]');
@@ -475,14 +490,18 @@ var MetaKeywords = Widget.extend({
     },
     _getLanguages: function () {
         var self = this;
+        var context;
+        this.trigger_up('context_get', {
+            callback: function (ctx) {
+                context = ctx;
+            },
+        });
         this._rpc({
-            model: 'website',
-            method: 'get_languages',
-            args: [[weContext.get().website_id]],
-        }).then( function (data) {
+            route: '/website/get_languages',
+        }).then(function (data) {
             self.$('#language-box').html(core.qweb.render('Configurator.language_promote', {
                 'language': data,
-                'def_lang': weContext.get().lang
+                'def_lang': context.lang
             }));
         });
     },
@@ -493,7 +512,7 @@ var MetaKeywords = Widget.extend({
      * @param {boolean} removed: a keyword is about to be removed,
      *   we need to exclude it from the count
      */
-    _updateTable : function (removed) {
+    _updateTable: function (removed) {
         var min = removed ? 1 : 0;
         if (this.keywordList.keywords().length > min) {
             this.$('table').show();
@@ -517,13 +536,14 @@ var MetaImageSelector = Widget.extend({
      */
     init: function (parent, data) {
         this.metaTitle = data.title || '';
-        this._setDescription(data.description);
         this.activeMetaImg = data.metaImg;
         this.serverUrl = data.htmlpage.url();
-        data.pageImages.unshift(_.str.sprintf('/web/image/res.company/%s/logo', odoo.session_info.website_company_id));
+        data.pageImages.unshift(_.str.sprintf('/web/image/website/%s/logo', odoo.session_info.website_id));
         data.pageImages.unshift(_.str.sprintf('/web/image/website/%s/social_default_image', odoo.session_info.website_id));
         this.images = _.uniq(data.pageImages);
         this.customImgUrl = _.contains(data.pageImages, data.metaImg) ? false : data.metaImg;
+        this.previewDescription = data.previewDescription;
+        this._setDescription(this.previewDescription);
         this._super(parent);
     },
     setTitle: function (title) {
@@ -545,9 +565,9 @@ var MetaImageSelector = Widget.extend({
      * @private
     */
     _setDescription: function (description) {
-        this.metaDescription = description || _t("The description will be generated by social media based on page content unless you specify one.");
+        this.metaDescription = description || this.previewDescription;
         if (this.metaDescription.length > 160) {
-            this.metaDescription = this.metaDescription.substring(0,159) + '…';
+            this.metaDescription = this.metaDescription.substring(0, 159) + '…';
         }
     },
 
@@ -589,12 +609,11 @@ var MetaImageSelector = Widget.extend({
         var mediaDialog = new weWidgets.MediaDialog(this, {
             onlyImages: true,
             res_model: 'ir.ui.view',
-        }, null, $image);
+        }, $image[0]);
         mediaDialog.open();
         mediaDialog.on('save', this, function (image) {
-            var src = image.attr('src');
-            self.activeMetaImg = src;
-            self.customImgUrl = src;
+            self.activeMetaImg = image.src;
+            self.customImgUrl = image.src;
             self._updateTemplateBody();
         });
     },
@@ -634,9 +653,9 @@ var SeoConfigurator = Dialog.extend({
             self.metaImageSelector = new MetaImageSelector(self, {
                 htmlpage: self.htmlPage,
                 title: self.htmlPage.getOgMeta().metaTitle,
-                description: self.htmlPage.getOgMeta().metaDescription,
                 metaImg : self.metaImg || self.htmlPage.getOgMeta().ogImageUrl,
                 pageImages : _.pluck(self.htmlPage.images().get(), 'src'),
+                previewDescription: _t('The description will be generated by social media based on page content unless you specify one.'),
             });
             self.metaImageSelector.appendTo(self.$('.js_seo_image'));
 
@@ -646,6 +665,7 @@ var SeoConfigurator = Dialog.extend({
                 canEditTitle: self.canEditTitle,
                 canEditDescription: self.canEditDescription,
                 isIndexed: self.isIndexed,
+                previewDescription: _t('The description will be generated by search engines based on page content unless you specify one.'),
             });
             self.metaTitleDescription.on('title-changed', self, self.titleChanged);
             self.metaTitleDescription.on('description-changed', self, self.descriptionChanged);
@@ -694,10 +714,10 @@ var SeoConfigurator = Dialog.extend({
         var self = this;
         var data = {};
         if (this.canEditTitle) {
-            data.website_meta_title = this.htmlPage.title();
+            data.website_meta_title = this.metaTitleDescription.$title.val();
         }
         if (this.canEditDescription) {
-            data.website_meta_description = this.htmlPage.description();
+            data.website_meta_description = this.metaTitleDescription.$description.val();
         }
         if (this.canEditKeywords) {
             data.website_meta_keywords = this.metaKeywords.keywordList.keywords().join(', ');
@@ -717,52 +737,56 @@ var SeoConfigurator = Dialog.extend({
         });
     },
     getMainObject: function () {
-        var repr = $('html').data('main-object');
-        var m = repr.match(/(.+)\((\d+),(.*)\)/);
-        if (!m) {
-            return null;
-        } else {
-            return {
-                model: m[1],
-                id: m[2]|0
-            };
-        }
+        var mainObject;
+        this.trigger_up('main_object_request', {
+            callback: function (value) {
+                mainObject = value;
+            },
+        });
+        return mainObject;
+    },
+    getSeoObject: function () {
+        var seoObject;
+        this.trigger_up('seo_object_request', {
+            callback: function (value) {
+                seoObject = value;
+            },
+        });
+        return seoObject;
     },
     loadMetaData: function () {
-        var obj = this.getMainObject();
-        var def = $.Deferred();
-        if (!obj) {
-            // return $.Deferred().reject(new Error("No main_object was found."));
-            def.resolve(null);
-        } else {
-            var fields = ['website_meta_title', 'website_meta_description', 'website_meta_keywords'
-                            ,'website_meta_og_img'];
-            if (obj.model === 'website.page'){
-                fields.push('website_indexed');
-                fields.push('website_id');
-            }
-            rpc.query({
-                model: obj.model,
-                method: 'read',
-                args: [[obj.id], fields],
-            }).then(function (data) {
-                if (data.length) {
-                    var meta = data[0];
-                    meta.model = obj.model;
-                    def.resolve(meta);
-                } else {
-                    def.resolve(null);
+        var obj = this.getSeoObject() || this.getMainObject();
+        return new Promise(function (resolve, reject) {
+            if (!obj) {
+                // return Promise.reject(new Error("No main_object was found."));
+                resolve(null);
+            } else {
+                var fields = ['website_meta_title', 'website_meta_description', 'website_meta_keywords'
+                                ,'website_meta_og_img'];
+                if (obj.model === 'website.page') {
+                    fields.push('website_indexed');
+                    fields.push('website_id');
                 }
-            }).fail(function () {
-                def.reject();
-            });
-        }
-        return def;
+                rpc.query({
+                    model: obj.model,
+                    method: 'read',
+                    args: [[obj.id], fields],
+                }).then(function (data) {
+                    if (data.length) {
+                        var meta = data[0];
+                        meta.model = obj.model;
+                        resolve(meta);
+                    } else {
+                        resolve(null);
+                    }
+                }).guardedCatch(reject);
+            }
+        });
     },
     saveMetaData: function (data) {
-        var obj = this.getMainObject();
+        var obj = this.getSeoObject() || this.getMainObject();
         if (!obj) {
-            return $.Deferred().reject();
+            return Promise.reject();
         } else {
             return this._rpc({
                 model: obj.model,
@@ -782,7 +806,7 @@ var SeoConfigurator = Dialog.extend({
     descriptionChanged: function () {
         var self = this;
         _.defer(function () {
-            var description = self.metaTitleDescription.getDescription();
+            var description = self.metaTitleDescription.getRealDescription();
             self.htmlPage.changeDescription(description);
             self.metaImageSelector.setDescription(description);
         });
