@@ -221,6 +221,21 @@ class TestViewSaving(common.TransactionCase):
             'text node characters wrongly unescaped when rendering'
         )
 
+    def test_save_oe_structure_with_attr(self):
+        """ Test saving oe_structure with attributes """
+        view = self.env['ir.ui.view'].create({
+            'arch': u'<t t-name="dummy"><div class="oe_structure" t-att-test="1" data-test="1" id="oe_structure_test"/></t>',
+            'type': 'qweb'
+        }).with_context(website_id=1, load_all_views=True)
+        replacement = u'<div class="oe_structure" data-test="1" id="oe_structure_test" data-oe-id="55" test="2">hello</div>'
+        view.save(replacement, xpath='/t/div')
+        # branding data-oe-* should be stripped
+        self.assertIn(
+            '<div class="oe_structure" data-test="1" id="oe_structure_test" test="2">hello</div>',
+            view.read_combined(['arch'])['arch'],
+            'saved element attributes are saved excluding branding ones'
+        )
+
     def test_save_only_embedded(self):
         Company = self.env['res.company']
         company_id = 1
@@ -610,6 +625,20 @@ class TestCowViewSaving(common.TransactionCase):
             'arch': '<div position="inside">, sub ext</div>',
             'key': 'II',
         })
+
+        #  B
+        #  |
+        #  I
+        #  |
+        #  II
+
+        # First, test that children of inactive children are not returned (not multiwebsite related)
+        self.inherit_view.active = False
+        views = View.get_related_views('B')
+        self.assertEqual(views.mapped('key'), ['B', 'I'], "As 'I' is inactive, 'II' (its own child) should not be returned.")
+        self.inherit_view.active = True
+
+        # Second, test multi-website
         self.inherit_view.with_context(website_id=1).write({'name': 'Extension'})  # Trigger cow on hierarchy
         View.create({
             'name': 'II2',
@@ -742,6 +771,10 @@ class TestCowViewSaving(common.TransactionCase):
             'arch': '<div position="replace"><p>COMPARE</p></div>',
             'key': '_website_sale_comparison.product_add_to_compare',
         })])
+        Website.with_context(load_all_views=True).viewref('_website_sale_comparison.product_add_to_compare').invalidate_cache()
+
+        # Simulate end of installation/update
+        View._create_all_specific_views(['_website_sale_comparison'])
 
         specific_view = Website.with_context(load_all_views=True, website_id=1).viewref('_website_sale.product')
         specific_view_arch = specific_view.read_combined(['arch'])['arch']
@@ -752,7 +785,6 @@ class TestCowViewSaving(common.TransactionCase):
         View._load_records([dict(xml_id='_website_sale_comparison.product_add_to_compare', values={
             'arch': '<div position="replace"><p>COMPARE EDITED</p></div>',
         })])
-
         specific_view_arch = Website.with_context(load_all_views=True, website_id=1).viewref('_website_sale.product').read_combined(['arch'])['arch']
         self.assertEqual(specific_view_arch, '<p>COMPARE EDITED</p>', "When a module updates an inherited view (on a generic tree), it should also update the copies of that view (COW).")
 
@@ -791,12 +823,173 @@ class TestCowViewSaving(common.TransactionCase):
             'model': self.base_view._name,
             'res_id': self.base_view.id,
         })
+        self.base_view.invalidate_cache()
         View._load_records([dict(xml_id='_website_sale.product', values={
             'website_meta_title': 'A bug got fixed by updating this field',
         })])
         all_title_updated = specific_view.website_meta_title == self.base_view.website_meta_title == "A bug got fixed by updating this field"
         self.assertEqual(all_title_updated, True, "Update on top level generic views should also be applied on specific views")
 
+    def test_module_new_inherit_view_on_parent_already_forked_xpath_replace(self):
+        """ Deeper, more specific test of above behavior.
+            A module install should add/update the COW view (if allowed fields,
+            eg not modified or prohibited (website_id, inherit_id..)).
+            This test ensure it does not crash if the child view is a primary view.
+        """
+        View = self.env['ir.ui.view']
+
+        # Simulate layout views
+        base_view = View.create({
+            'name': 'Main Frontend Layout',
+            'type': 'qweb',
+            'arch': '<t t-call="web.layout"><t t-set="head_website"/></t>',
+            'key': '_portal.frontend_layout',
+        }).with_context(load_all_views=True)
+
+        inherit_view = View.create({
+            'name': 'Main layout',
+            'mode': 'extension',
+            'inherit_id': base_view.id,
+            'arch': '<xpath expr="//t[@t-set=\'head_website\']" position="replace"><t t-call-assets="web_editor.summernote" t-js="false" groups="website.group_website_publisher"/></xpath>',
+            'key': '_website.layout',
+        })
+
+        # Trigger cow on website_sale hierarchy for website 1
+        base_view.with_context(website_id=1).write({'name': 'Main Frontend Layout (W1)'})
+
+        # Simulate website_sale_comparison install, that's the real test, it
+        # should not crash.
+        View._load_records([dict(xml_id='_website_forum.layout', values={
+            'name': 'Forum Layout',
+            'mode': 'primary',
+            'inherit_id': inherit_view.id,
+            'arch': '<xpath expr="//t[@t-call-assets=\'web_editor.summernote\'][@t-js=\'false\']" position="attributes"><attribute name="groups"/></xpath>',
+            'key': '_website_forum.layout',
+        })])
+
+    def test_multiple_inherit_level(self):
+        """ Test multi-level inheritance:
+            Base
+            |
+            ---> Extension (Website-specific)
+                |
+                ---> Extension 2 (Website-specific)
+        """
+        View = self.env['ir.ui.view']
+
+        self.inherit_view.website_id = 1
+        inherit_view_2 = View.create({
+            'name': 'Extension 2',
+            'mode': 'extension',
+            'inherit_id': self.inherit_view.id,
+            'arch': '<div position="inside">, extended content 2</div>',
+            'key': 'website.extension_view_2',
+            'website_id': 1,
+        })
+
+        total_views = View.search_count([])
+
+        # id | name        | content               | website_id | inherit  | key
+        # --------------------------------------------------------------------------------------------
+        # 1  | Base        |  base content         |     /      |     /    |  website.base_view
+        # 2  | Extension   |  , extended content   |     1      |     1    |  website.extension_view
+        # 3  | Extension 2 |  , extended content 2 |     1      |     2    |  website.extension_view_2
+
+        self.base_view.with_context(website_id=1).write({'arch': '<div>modified content</div>'})
+
+        # 2 views are created, one is deleted
+        self.assertEqual(View.search_count([]), total_views + 1)
+        self.assertFalse(self.inherit_view.exists())
+        self.assertTrue(inherit_view_2.exists())
+
+        # Verify the inheritance
+        base_specific = View.search([('key', '=', self.base_view.key), ('website_id', '=', 1)]).with_context(load_all_views=True)
+        extend_specific = View.search([('key', '=', 'website.extension_view'), ('website_id', '=', 1)])
+        self.assertEqual(extend_specific.inherit_id, base_specific)
+        self.assertEqual(inherit_view_2.inherit_id, extend_specific)
+
+        # id | name        | content               | website_id | inherit  | key
+        # --------------------------------------------------------------------------------------------
+        # 1  | Base        |  base content         |     /      |     /    |  website.base_view
+        # 4  | Base        |  modified content     |     1      |     /    |  website.base_view
+        # 5  | Extension   |  , extended content   |     1      |     4    |  website.extension_view
+        # 3  | Extension 2 |  , extended content 2 |     1      |     5    |  website.extension_view_2
+
+    def test_cow_extension_with_install(self):
+        View = self.env['ir.ui.view']
+        # Base
+        v1 = View.create({
+            'name': 'Base',
+            'type': 'qweb',
+            'arch': '<div>base content</div>',
+            'key': 'website.base_view_v1',
+        }).with_context(load_all_views=True)
+        self.env['ir.model.data'].create({
+            'module': 'website',
+            'name': 'base_view_v1',
+            'model': v1._name,
+            'res_id': v1.id,
+        })
+
+        # Extension
+        v2 = View.create({
+            'name': 'Extension',
+            'mode': 'extension',
+            'inherit_id': v1.id,
+            'arch': '<div position="inside"><ooo>extended content</ooo></div>',
+            'key': 'website.extension_view_v2',
+        })
+        self.env['ir.model.data'].create({
+            'module': 'website',
+            'name': 'extension_view_v2',
+            'model': v2._name,
+            'res_id': v2.id,
+        })
+
+        # multiwebsite specific
+        v1.with_context(website_id=1).write({'name': 'Extension Specific'})
+
+        original_pool_init = View.pool._init
+        View.pool._init = True
+
+        try:
+            # Simulate module install
+            View._load_records([dict(xml_id='website.extension2_view', values={
+                'name': ' ---',
+                'mode': 'extension',
+                'inherit_id': v1.id,
+                'arch': '<ooo position="replace"><p>EXTENSION</p></ooo>',
+                'key': 'website.extension2_view',
+            })])
+        finally:
+            View.pool._init = original_pool_init
+
+    def test_specific_view_translation(self):
+        Translation = self.env['ir.translation']
+
+        Translation.insert_missing(self.base_view._fields['arch_db'],  self.base_view)
+        translation = Translation.search([
+            ('res_id', '=', self.base_view.id), ('name', '=', 'ir.ui.view,arch_db')
+        ])
+        translation.value = 'hello'
+        translation.module = 'website'
+
+        self.base_view.with_context(website_id=1).write({'active': True})
+        specific_view = self.base_view._get_specific_views() - self.base_view
+
+        self.assertEquals(specific_view.with_context(lang='en_US').arch, '<div>hello</div>',
+            "copy on write (COW) also copy existing translations")
+
+        translation.value = 'hi'
+        self.assertEquals(specific_view.with_context(lang='en_US').arch, '<div>hello</div>',
+            "updating translation of base view doesn't update specific view")
+
+        Translation.with_context(overwrite=True)._load_module_terms(['website'], ['en_US'])
+
+        specific_view.invalidate_cache(['arch_db', 'arch'])
+        self.assertEquals(specific_view.with_context(lang='en_US').arch, '<div>hi</div>',
+            "loading module translation copy translation from base to specific view")
+
 
 class Crawler(HttpCase):
     def setUp(self):
@@ -856,6 +1049,8 @@ class Crawler(HttpCase):
         event_child_view.copy({'name': 'Filter by Category', 'inherit_id': event_child_view.id, 'key': '_website_event.event_category'})
         event_child_view.copy({'name': 'Filter by Country', 'inherit_id': event_child_view.id, 'key': '_website_event.event_location'})
 
+        View.flush()
+
         # Customize
         #   | Main Frontend Layout
         #       | Show Sign In
