@@ -26,4 +26,6 @@ class Tags(models.Model):
     partner_ids = fields.Many2many('res.partner', 'res_partner_res_partner_tag_rel', 'tag_id', 'partner_id', string='Partners')
     classname = fields.Selection(get_selection_class, 'Class', default='default', help="Bootstrap class to customize the color", required=True)
     active = fields.Boolean('Active', default=True)
-    website_published = fields.Boolean(default=True)
+
+    def _default_is_published(self):
+        return True
