@@ -2,7 +2,7 @@
 
 {
     'name': 'Events',
-    'category': 'Website',
+    'category': 'Website/Website',
     'sequence': 166,
     'summary': 'Publish events, sell tickets',
     'website': 'https://www.odoo.com/page/events',
@@ -11,6 +11,7 @@
     'data': [
         'data/event_data.xml',
         'views/res_config_settings_views.xml',
+        'views/event_snippets.xml',
         'views/event_templates.xml',
         'views/event_views.xml',
         'security/ir.model.access.csv',
