@@ -11,9 +11,9 @@ class TestUi(odoo.tests.HttpCase):
         # - that this test awaits for hardcoded USDs amount
         # we have to force company currency as USDs only for this test
         self.cr.execute("UPDATE res_company SET currency_id = %s WHERE id = %s", [self.env.ref('base.USD').id, self.env.ref('base.main_company').id])
-        self.phantom_js("/", "odoo.__DEBUG__.services['web_tour.tour'].run('event_buy_tickets')", "odoo.__DEBUG__.services['web_tour.tour'].tours.event_buy_tickets.ready", login="admin")
+        self.start_tour("/", 'event_buy_tickets', login="admin")
 
     def test_demo(self):
-        self.phantom_js("/", "odoo.__DEBUG__.services['web_tour.tour'].run('event_buy_tickets')", "odoo.__DEBUG__.services['web_tour.tour'].tours.event_buy_tickets.ready", login="demo")
+        self.start_tour("/", 'event_buy_tickets', login="demo")
 
     # TO DO - add public test with new address when convert to web.tour format.
