@@ -6,10 +6,10 @@ import odoo.tests
 class TestUi(odoo.tests.HttpCase):
 
     def test_01_admin_forum_tour(self):
-        self.phantom_js("/", "odoo.__DEBUG__.services['web_tour.tour'].run('question')", "odoo.__DEBUG__.services['web_tour.tour'].tours.question.ready", login="admin")
+        self.start_tour("/", 'question', login="admin")
 
     def test_02_demo_question(self):
         forum = self.env.ref('website_forum.forum_help')
         demo = self.env.ref('base.user_demo')
         demo.karma = forum.karma_post + 1
-        self.phantom_js("/", "odoo.__DEBUG__.services['web_tour.tour'].run('forum_question')", "odoo.__DEBUG__.services['web_tour.tour'].tours.forum_question.ready", login="demo")
+        self.start_tour("/", 'forum_question', login="demo")
