@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 {
     'name': 'Website Live Chat',
-    'category': 'Technical Settings',
+    'category': 'Hidden',
     'summary': 'Chat with your website visitors',
     'version': '1.0',
     'description': """
@@ -9,14 +9,18 @@ Allow website visitors to chat with the collaborators. This module also brings a
     """,
     'depends': ['website', 'im_livechat'],
     'installable': True,
-    'application': True,
+    'application': False,
     'auto_install': True,
     'data': [
         'views/website_livechat.xml',
         'views/res_config_settings_views.xml',
         'views/website_livechat_view.xml',
+        'views/website_visitor_views.xml',
         'security/ir.model.access.csv',
         'security/website_livechat.xml',
         'data/website_livechat_data.xml',
     ],
+    'qweb': [
+        'static/src/xml/thread.xml',
+    ],
 }
