@@ -12,13 +12,10 @@ It is responsible of comments moderation for published documents (forum, slides,
 """,
     'depends': ['website', 'mail'],
     'data': [
+        'views/assets.xml',
         'views/website_mail_templates.xml',
-        'data/mail_channel_data.xml',
         'security/website_mail_security.xml',
     ],
-    'qweb': [
-        'static/src/xml/website_mail.xml'
-    ],
     'installable': True,
     'auto_install': True,
 }
