@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import account_invoice
+from . import account_move
 from . import crm_team
 from . import ir_http
 from . import mail_compose_message
+from . import product_image
 from . import product
 from . import res_country
 from . import res_partner
@@ -13,3 +14,5 @@ from . import website
 from . import res_config_settings
 from . import digest
 from . import res_company
+from . import product_attribute
+from . import website_visitor
