@@ -1,9 +1,12 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-from odoo import api, fields, models, tools, _
-from odoo.addons import decimal_precision as dp
+
+from odoo import api, fields, models, _
+from odoo.exceptions import ValidationError, UserError
+from odoo.addons.http_routing.models.ir_http import slug
 from odoo.addons.website.models import ir_http
 from odoo.tools.translate import html_translate
+from odoo.osv import expression
 
 
 class ProductStyle(models.Model):
@@ -19,11 +22,15 @@ class ProductPricelist(models.Model):
 
     def _default_website(self):
         """ Find the first company's website, if there is one. """
-        company_id = self.env.user.company_id.id
+        company_id = self.env.company.id
+
+        if self._context.get('default_company_id'):
+            company_id = self._context.get('default_company_id')
+
         domain = [('company_id', '=', company_id)]
         return self.env['website'].search(domain, limit=1)
 
-    website_id = fields.Many2one('website', string="Website", default=_default_website)
+    website_id = fields.Many2one('website', string="Website", ondelete='restrict', default=_default_website)
     code = fields.Char(string='E-commerce Promotional Code', groups="base.group_user")
     selectable = fields.Boolean(help="Allow the end user to choose this price list")
 
@@ -36,24 +43,30 @@ class ProductPricelist(models.Model):
 
     @api.model
     def create(self, data):
+        if data.get('company_id') and not data.get('website_id'):
+            # l10n modules install will change the company currency, creating a
+            # pricelist for that currency. Do not use user's company in that
+            # case as module install are done with OdooBot (company 1)
+            self = self.with_context(default_company_id=data['company_id'])
         res = super(ProductPricelist, self).create(data)
         self.clear_cache()
         return res
 
-    @api.multi
     def write(self, data):
         res = super(ProductPricelist, self).write(data)
+        if data.keys() & {'code', 'active', 'website_id', 'selectable', 'company_id'}:
+            self._check_website_pricelist()
         self.clear_cache()
         return res
 
-    @api.multi
     def unlink(self):
         res = super(ProductPricelist, self).unlink()
+        self._check_website_pricelist()
         self.clear_cache()
         return res
 
-    def _get_partner_pricelist_multi_search_domain_hook(self):
-        domain = super(ProductPricelist, self)._get_partner_pricelist_multi_search_domain_hook()
+    def _get_partner_pricelist_multi_search_domain_hook(self, company_id):
+        domain = super(ProductPricelist, self)._get_partner_pricelist_multi_search_domain_hook(company_id)
         website = ir_http.get_request_website()
         if website:
             domain += self._get_website_pricelists_domain(website.id)
@@ -66,12 +79,17 @@ class ProductPricelist(models.Model):
             res = res.filtered(lambda pl: pl._is_available_on_website(website.id))
         return res
 
-    @api.multi
+    def _check_website_pricelist(self):
+        for website in self.env['website'].search([]):
+            if not website.pricelist_ids:
+                raise UserError(_("With this action, '%s' website would not have any pricelist available.") % (website.name))
+
     def _is_available_on_website(self, website_id):
         """ To be able to be used on a website, a pricelist should either:
         - Have its `website_id` set to current website (specific pricelist).
         - Have no `website_id` set and should be `selectable` (generic pricelist)
           or should have a `code` (generic promotion).
+        - Have no `company_id` or a `company_id` matching its website one.
 
         Note: A pricelist without a website_id, not selectable and without a
               code is a backend pricelist.
@@ -79,13 +97,17 @@ class ProductPricelist(models.Model):
         Change in this method should be reflected in `_get_website_pricelists_domain`.
         """
         self.ensure_one()
+        if self.company_id and self.company_id != self.env["website"].browse(website_id).company_id:
+            return False
         return self.website_id.id == website_id or (not self.website_id and (self.selectable or self.sudo().code))
 
     def _get_website_pricelists_domain(self, website_id):
         ''' Check above `_is_available_on_website` for explanation.
         Change in this method should be reflected in `_is_available_on_website`.
         '''
+        company_id = self.env["website"].browse(website_id).company_id.id
         return [
+            '&', ('company_id', 'in', [False, company_id]),
             '|', ('website_id', '=', website_id),
             '&', ('website_id', '=', False),
             '|', ('selectable', '=', True), ('code', '!=', False),
@@ -102,98 +124,89 @@ class ProductPricelist(models.Model):
             company_id = website.company_id.id
         return super(ProductPricelist, self)._get_partner_pricelist_multi(partner_ids, company_id)
 
+    @api.onchange('company_id')
+    def _onchange_company_id(self):
+        ''' Show only the company's website '''
+        domain = self.company_id and [('company_id', '=', self.company_id.id)] or []
+        return {'domain': {'website_id': domain}}
+
+    @api.constrains('company_id', 'website_id')
+    def _check_websites_in_company(self):
+        '''Prevent misconfiguration multi-website/multi-companies.
+           If the record has a company, the website should be from that company.
+        '''
+        for record in self.filtered(lambda pl: pl.website_id and pl.company_id):
+            if record.website_id.company_id != record.company_id:
+                raise ValidationError(_("Only the company's websites are allowed. \
+                    Leave the Company field empty or select a website from that company."))
+
 
 class ProductPublicCategory(models.Model):
     _name = "product.public.category"
-    _inherit = ["website.seo.metadata", "website.multi.mixin"]
+    _inherit = ["website.seo.metadata", "website.multi.mixin", 'image.mixin']
     _description = "Website Product Category"
+    _parent_store = True
     _order = "sequence, name"
 
     name = fields.Char(required=True, translate=True)
     parent_id = fields.Many2one('product.public.category', string='Parent Category', index=True)
+    parent_path = fields.Char(index=True)
     child_id = fields.One2many('product.public.category', 'parent_id', string='Children Categories')
-    sequence = fields.Integer(help="Gives the sequence order when displaying a list of product categories.")
-    # NOTE: there is no 'default image', because by default we don't show
-    # thumbnails for categories. However if we have a thumbnail for at least one
-    # category, then we display a default image on the other, so that the
-    # buttons have consistent styling.
-    # In this case, the default image is set by the js code.
-    image = fields.Binary(attachment=True, help="This field holds the image used as image for the category, limited to 1024x1024px.")
-    image_medium = fields.Binary(string='Medium-sized image', attachment=True,
-                                 help="Medium-sized image of the category. It is automatically "
-                                 "resized as a 128x128px image, with aspect ratio preserved. "
-                                 "Use this field in form views or some kanban views.")
-    image_small = fields.Binary(string='Small-sized image', attachment=True,
-                                help="Small-sized image of the category. It is automatically "
-                                "resized as a 64x64px image, with aspect ratio preserved. "
-                                "Use this field anywhere a small image is required.")
-
-    @api.model
-    def create(self, vals):
-        tools.image_resize_images(vals)
-        return super(ProductPublicCategory, self).create(vals)
-
-    @api.multi
-    def write(self, vals):
-        tools.image_resize_images(vals)
-        return super(ProductPublicCategory, self).write(vals)
+    parents_and_self = fields.Many2many('product.public.category', compute='_compute_parents_and_self')
+    sequence = fields.Integer(help="Gives the sequence order when displaying a list of product categories.", index=True)
+    website_description = fields.Html('Category Description', sanitize_attributes=False, translate=html_translate)
+    product_tmpl_ids = fields.Many2many('product.template', relation='product_public_category_product_template_rel')
 
     @api.constrains('parent_id')
     def check_parent_id(self):
         if not self._check_recursion():
             raise ValueError(_('Error ! You cannot create recursive categories.'))
 
-    @api.multi
     def name_get(self):
         res = []
         for category in self:
-            names = [category.name]
-            parent_category = category.parent_id
-            while parent_category:
-                names.append(parent_category.name)
-                parent_category = parent_category.parent_id
-            res.append((category.id, ' / '.join(reversed(names))))
+            res.append((category.id, " / ".join(category.parents_and_self.mapped('name'))))
         return res
 
+    def unlink(self):
+        self.child_id.parent_id = None
+        return super(ProductPublicCategory, self).unlink()
+
+    def _compute_parents_and_self(self):
+        for category in self:
+            if category.parent_path:
+                category.parents_and_self = self.env['product.public.category'].browse([int(p) for p in category.parent_path.split('/')[:-1]])
+            else:
+                category.parents_and_self = category
+
 
 class ProductTemplate(models.Model):
     _inherit = ["product.template", "website.seo.metadata", 'website.published.multi.mixin', 'rating.mixin']
     _name = 'product.template'
     _mail_post_access = 'read'
+    _check_company_auto = True
 
     website_description = fields.Html('Description for the website', sanitize_attributes=False, translate=html_translate)
-    alternative_product_ids = fields.Many2many('product.template', 'product_alternative_rel', 'src_id', 'dest_id',
-                                               string='Alternative Products', help='Suggest alternatives to your customer'
-                                               '(upsell strategy).Those product show up on the product page.')
-    accessory_product_ids = fields.Many2many('product.product', 'product_accessory_rel', 'src_id', 'dest_id',
-                                             string='Accessory Products', help='Accessories show up when the customer'
-                                            'reviews the cart before payment (cross-sell strategy).')
+    alternative_product_ids = fields.Many2many(
+        'product.template', 'product_alternative_rel', 'src_id', 'dest_id', check_company=True,
+        string='Alternative Products', help='Suggest alternatives to your customer (upsell strategy). '
+                                            'Those products show up on the product page.')
+    accessory_product_ids = fields.Many2many(
+        'product.product', 'product_accessory_rel', 'src_id', 'dest_id', string='Accessory Products', check_company=True,
+        help='Accessories show up when the customer reviews the cart before payment (cross-sell strategy).')
     website_size_x = fields.Integer('Size X', default=1)
     website_size_y = fields.Integer('Size Y', default=1)
     website_style_ids = fields.Many2many('product.style', string='Styles')
     website_sequence = fields.Integer('Website Sequence', help="Determine the display order in the Website E-commerce",
-                                      default=lambda self: self._default_website_sequence())
-    public_categ_ids = fields.Many2many('product.public.category', string='Website Product Category',
-                                        help="The product will be available in each mentioned e-commerce category. Go to"
-                                        "Shop > Customize and enable 'E-commerce categories' to view all e-commerce categories.")
-    product_image_ids = fields.One2many('product.image', 'product_tmpl_id', string='Images')
+                                      default=lambda self: self._default_website_sequence(), copy=False)
+    public_categ_ids = fields.Many2many(
+        'product.public.category', relation='product_public_category_product_template_rel',
+        string='Website Product Category',
+        help="The product will be available in each mentioned eCommerce category. Go to Shop > "
+             "Customize and enable 'eCommerce categories' to view all eCommerce categories.")
 
-    # website_price deprecated, directly use _get_combination_info instead
-    website_price = fields.Float('Website price', compute='_website_price', digits=dp.get_precision('Product Price'))
-    # website_public_price deprecated, directly use _get_combination_info instead
-    website_public_price = fields.Float('Website public price', compute='_website_price', digits=dp.get_precision('Product Price'))
-    # website_price_difference deprecated, directly use _get_combination_info instead
-    website_price_difference = fields.Boolean('Website price difference', compute='_website_price')
+    product_template_image_ids = fields.One2many('product.image', 'product_tmpl_id', string="Extra Product Media", copy=True)
 
-    def _website_price(self):
-        current_website = self.env['website'].get_current_website()
-        for template in self.with_context(website_id=current_website.id):
-            res = template._get_combination_info()
-            template.website_price = res.get('price')
-            template.website_public_price = res.get('list_price')
-            template.website_price_difference = res.get('has_discounted_price')
-
-    @api.multi
     def _has_no_variant_attributes(self):
         """Return whether this `product.template` has at least one no_variant
         attribute.
@@ -202,9 +215,8 @@ class ProductTemplate(models.Model):
         :rtype: bool
         """
         self.ensure_one()
-        return any(a.create_variant == 'no_variant' for a in self._get_valid_product_attributes())
+        return any(a.create_variant == 'no_variant' for a in self.valid_product_template_attribute_line_ids.attribute_id)
 
-    @api.multi
     def _has_is_custom_values(self):
         self.ensure_one()
         """Return whether this `product.template` has at least one is_custom
@@ -213,41 +225,8 @@ class ProductTemplate(models.Model):
         :return: True if at least one is_custom attribute value, False otherwise
         :rtype: bool
         """
-        return any(v.is_custom for v in self._get_valid_product_attribute_values())
+        return any(v.is_custom for v in self.valid_product_template_attribute_line_ids.product_template_value_ids._only_active())
 
-    @api.multi
-    def _is_quick_add_to_cart_possible(self, parent_combination=None):
-        """
-        It's possible to quickly add to cart if there's no optional product,
-        there's only one possible combination and no value is set to is_custom.
-
-        Attributes set to dynamic or no_variant don't have to be tested
-        specifically because they will be taken into account when checking for
-        the possible combinations.
-
-        :param parent_combination: combination from which `self` is an
-            optional or accessory product
-        :type parent_combination: recordset `product.template.attribute.value`
-
-        :return: True if it's possible to quickly add to cart, else False
-        :rtype: bool
-        """
-        self.ensure_one()
-
-        if not self._is_add_to_cart_possible(parent_combination):
-            return False
-        gen = self._get_possible_combinations(parent_combination)
-        first_possible_combination = next(gen)
-        if next(gen, False) is not False:
-            # there are at least 2 possible combinations.
-            return False
-        if self._has_is_custom_values():
-            return False
-        if self.optional_product_ids.filtered(lambda p: p._is_add_to_cart_possible(first_possible_combination)):
-            return False
-        return True
-
-    @api.multi
     def _get_possible_variants_sorted(self, parent_combination=None):
         """Return the sorted recordset of variants that are possible.
 
@@ -277,15 +256,14 @@ class ProductTemplate(models.Model):
                     - second level sort: same as "product.attribute.value"._order
             """
             keys = []
-            for attribute in variant.attribute_value_ids.sorted(_sort_key_attribute_value):
+            for attribute in variant.product_template_attribute_value_ids.sorted(_sort_key_attribute_value):
                 # if you change this order, keep it in sync with _order from `product.attribute.value`
-                keys.append(attribute.sequence)
+                keys.append(attribute.product_attribute_value_id.sequence)
                 keys.append(attribute.id)
             return keys
 
         return self._get_possible_variants(parent_combination).sorted(_sort_key_variant)
 
-    @api.multi
     def _get_combination_info(self, combination=False, product_id=False, add_qty=1, pricelist=False, parent_combination=False, only_template=False):
         """Override for website, where we want to:
             - take the website pricelist if no pricelist is set
@@ -332,7 +310,6 @@ class ProductTemplate(models.Model):
 
         return combination_info
 
-    @api.multi
     def _create_first_product_variant(self, log_warning=False):
         """Create if necessary and possible and return the first product
         variant for this template.
@@ -345,7 +322,21 @@ class ProductTemplate(models.Model):
         """
         return self._create_product_variant(self._get_first_possible_combination(), log_warning)
 
-    @api.multi
+    def _get_image_holder(self):
+        """Returns the holder of the image to use as default representation.
+        If the product template has an image it is the product template,
+        otherwise if the product has variants it is the first variant
+
+        :return: this product template or the first product variant
+        :rtype: recordset of 'product.template' or recordset of 'product.product'
+        """
+        self.ensure_one()
+        if self.image_1920:
+            return self
+        variant = self.env['product.product'].browse(self._get_first_possible_variant_id())
+        # if the variant has no image anyway, spare some queries by using template
+        return variant if variant.image_variant_1920 else self
+
     def _get_current_company_fallback(self, **kwargs):
         """Override: if a website is set on the product or given, fallback to
         the company of the website. Otherwise use the one from parent method."""
@@ -354,27 +345,41 @@ class ProductTemplate(models.Model):
         return website and website.company_id or res
 
     def _default_website_sequence(self):
-        self._cr.execute("SELECT MIN(website_sequence) FROM %s" % self._table)
-        min_sequence = self._cr.fetchone()[0]
-        return min_sequence and min_sequence - 1 or 10
+        ''' We want new product to be the last (highest seq).
+        Every product should ideally have an unique sequence.
+        Default sequence (10000) should only be used for DB first product.
+        As we don't resequence the whole tree (as `sequence` does), this field
+        might have negative value.
+        '''
+        self._cr.execute("SELECT MAX(website_sequence) FROM %s" % self._table)
+        max_sequence = self._cr.fetchone()[0]
+        if max_sequence is None:
+            return 10000
+        return max_sequence + 5
 
     def set_sequence_top(self):
-        self.website_sequence = self.sudo().search([], order='website_sequence desc', limit=1).website_sequence + 1
+        min_sequence = self.sudo().search([], order='website_sequence ASC', limit=1)
+        self.website_sequence = min_sequence.website_sequence - 5
 
     def set_sequence_bottom(self):
-        self.website_sequence = self.sudo().search([], order='website_sequence', limit=1).website_sequence - 1
+        max_sequence = self.sudo().search([], order='website_sequence DESC', limit=1)
+        self.website_sequence = max_sequence.website_sequence + 5
 
     def set_sequence_up(self):
-        previous_product_tmpl = self.sudo().search(
-            [('website_sequence', '>', self.website_sequence), ('website_published', '=', self.website_published)],
-            order='website_sequence', limit=1)
+        previous_product_tmpl = self.sudo().search([
+            ('website_sequence', '<', self.website_sequence),
+            ('website_published', '=', self.website_published),
+        ], order='website_sequence DESC', limit=1)
         if previous_product_tmpl:
             previous_product_tmpl.website_sequence, self.website_sequence = self.website_sequence, previous_product_tmpl.website_sequence
         else:
             self.set_sequence_top()
 
     def set_sequence_down(self):
-        next_prodcut_tmpl = self.search([('website_sequence', '<', self.website_sequence), ('website_published', '=', self.website_published)], order='website_sequence desc', limit=1)
+        next_prodcut_tmpl = self.search([
+            ('website_sequence', '>', self.website_sequence),
+            ('website_published', '=', self.website_published),
+        ], order='website_sequence ASC', limit=1)
         if next_prodcut_tmpl:
             next_prodcut_tmpl.website_sequence, self.website_sequence = self.website_sequence, next_prodcut_tmpl.website_sequence
         else:
@@ -384,14 +389,36 @@ class ProductTemplate(models.Model):
         res = super(ProductTemplate, self)._default_website_meta()
         res['default_opengraph']['og:description'] = res['default_twitter']['twitter:description'] = self.description_sale
         res['default_opengraph']['og:title'] = res['default_twitter']['twitter:title'] = self.name
-        res['default_opengraph']['og:image'] = res['default_twitter']['twitter:image'] = "/web/image/product.template/%s/image" % (self.id)
+        res['default_opengraph']['og:image'] = res['default_twitter']['twitter:image'] = self.env['website'].image_url(self, 'image_1024')
+        res['default_meta_description'] = self.description_sale
         return res
 
-    @api.multi
     def _compute_website_url(self):
         super(ProductTemplate, self)._compute_website_url()
         for product in self:
-            product.website_url = "/shop/product/%s" % (product.id,)
+            product.website_url = "/shop/product/%s" % slug(product)
+
+    # ---------------------------------------------------------
+    # Rating Mixin API
+    # ---------------------------------------------------------
+
+    def _rating_domain(self):
+        """ Only take the published rating into account to compute avg and count """
+        domain = super(ProductTemplate, self)._rating_domain()
+        return expression.AND([domain, [('website_published', '=', True)]])
+
+    def _get_images(self):
+        """Return a list of records implementing `image.mixin` to
+        display on the carousel on the website for this template.
+
+        This returns a list and not a recordset because the records might be
+        from different models (template and image).
+
+        It contains in this order: the main image of the template and the
+        Template Extra Images.
+        """
+        self.ensure_one()
+        return [self] + list(self.product_template_image_ids)
 
 
 class Product(models.Model):
@@ -399,30 +426,47 @@ class Product(models.Model):
 
     website_id = fields.Many2one(related='product_tmpl_id.website_id', readonly=False)
 
-    # website_price deprecated, directly use _get_combination_info instead
-    website_price = fields.Float('Website price', compute='_website_price', digits=dp.get_precision('Product Price'))
-    # website_public_price deprecated, directly use _get_combination_info instead
-    website_public_price = fields.Float('Website public price', compute='_website_price', digits=dp.get_precision('Product Price'))
-    # website_price_difference deprecated, directly use _get_combination_info instead
-    website_price_difference = fields.Boolean('Website price difference', compute='_website_price')
+    product_variant_image_ids = fields.One2many('product.image', 'product_variant_id', string="Extra Variant Images")
 
-    def _website_price(self):
+    website_url = fields.Char('Website URL', compute='_compute_product_website_url', help='The full URL to access the document through the website.')
+
+    @api.depends_context('lang')
+    @api.depends('product_tmpl_id.website_url', 'product_template_attribute_value_ids')
+    def _compute_product_website_url(self):
         for product in self:
-            res = product._get_combination_info_variant()
-            product.website_price = res.get('price')
-            product.website_public_price = res.get('list_price')
-            product.website_price_difference = res.get('has_discounted_price')
+            attributes = ','.join(str(x) for x in product.product_template_attribute_value_ids.ids)
+            product.website_url = "%s#attr=%s" % (product.product_tmpl_id.website_url, attributes)
 
-    @api.multi
     def website_publish_button(self):
         self.ensure_one()
         return self.product_tmpl_id.website_publish_button()
 
+    def open_website_url(self):
+        self.ensure_one()
+        res = self.product_tmpl_id.open_website_url()
+        res['url'] = self.website_url
+        return res
 
-class ProductImage(models.Model):
-    _name = 'product.image'
-    _description = 'Product Image'
+    def _get_images(self):
+        """Return a list of records implementing `image.mixin` to
+        display on the carousel on the website for this variant.
 
-    name = fields.Char('Name')
-    image = fields.Binary('Image', attachment=True)
-    product_tmpl_id = fields.Many2one('product.template', 'Related Product', copy=True)
+        This returns a list and not a recordset because the records might be
+        from different models (template, variant and image).
+
+        It contains in this order: the main image of the variant (if set), the
+        Variant Extra Images, and the Template Extra Images.
+        """
+        self.ensure_one()
+        variant_images = list(self.product_variant_image_ids)
+        if self.image_variant_1920:
+            # if the main variant image is set, display it first
+            variant_images = [self] + variant_images
+        else:
+            # If the main variant image is empty, it will fallback to template
+            # image, in this case insert it after the other variant images, so
+            # that all variant images are first and all template images last.
+            variant_images = variant_images + [self]
+        # [1:] to remove the main image from the template, we only display
+        # the template extra images here
+        return variant_images + self.product_tmpl_id._get_images()[1:]
