@@ -1,7 +1,10 @@
 from . import test_customize
 from . import test_sale_process
+from . import test_sitemap
 from . import test_website_sale_cart_recovery
 from . import test_website_sale_mail
 from . import test_website_sale_pricelist
 from . import test_website_sale_product_attribute_value_config
-from . import test_website_sale_zoom
+from . import test_website_sale_image
+from . import test_website_sequence
+from . import test_website_sale_visitor
