@@ -80,7 +80,7 @@ def load_demo(cr, package, idref, mode, report=None):
 
     try:
         _logger.info("Module %s: loading demo", package.name)
-        with cr.savepoint():
+        with cr.savepoint(flush=False):
             load_data(cr, idref, mode, kind='demo', package=package, report=report)
         return True
     except Exception as e:
@@ -175,6 +175,9 @@ def load_module_graph(cr, graph, status=None, perform_checks=True,
             if package.name != 'base':
                 registry.setup_models(cr)
             migrations.migrate_module(package, 'pre')
+            if package.name != 'base':
+                env = api.Environment(cr, SUPERUSER_ID, {})
+                env['base'].flush()
 
         load_openerp_module(package.name)
 
@@ -187,12 +190,16 @@ def load_module_graph(cr, graph, status=None, perform_checks=True,
 
         model_names = registry.load(cr, package)
 
+        mode = 'update'
+        if hasattr(package, 'init') or package.state == 'to install':
+            mode = 'init'
+
         loaded_modules.append(package.name)
         if needs_update:
             models_updated |= set(model_names)
             models_to_check -= set(model_names)
             registry.setup_models(cr)
-            registry.init_models(cr, model_names, {'module': package.name})
+            registry.init_models(cr, model_names, {'module': package.name}, new_install)
         elif package.state != 'to remove':
             # The current module has simply been loaded. The models extended by this module
             # and for which we updated the schema, must have their schema checked again.
@@ -203,10 +210,6 @@ def load_module_graph(cr, graph, status=None, perform_checks=True,
 
         idref = {}
 
-        mode = 'update'
-        if hasattr(package, 'init') or package.state == 'to install':
-            mode = 'init'
-
         if needs_update:
             env = api.Environment(cr, SUPERUSER_ID, {})
             # Can't put this line out of the loop: ir.module.module will be
@@ -246,17 +249,15 @@ def load_module_graph(cr, graph, status=None, perform_checks=True,
             # update made to the schema or data so the tests can run
             # (separately in their own transaction)
             cr.commit()
-            if demo_loaded:
-                # launch tests only in demo mode, allowing tests to use demo data.
-                if tools.config.options['test_enable']:
-                    # Yamel test
-                    report.record_result(load_test(idref, mode))
-                    # Python tests
-                    env['ir.http']._clear_routing_map()     # force routing map to be rebuilt
-                    report.record_result(odoo.modules.module.run_unit_tests(module_name, cr.dbname))
-                    # tests may have reset the environment
-                    env = api.Environment(cr, SUPERUSER_ID, {})
-                    module = env['ir.module.module'].browse(module_id)
+
+            if tools.config.options['test_enable']:
+                report.record_result(load_test(idref, mode))
+                # Python tests
+                env['ir.http']._clear_routing_map()     # force routing map to be rebuilt
+                report.record_result(odoo.modules.module.run_unit_tests(module_name))
+                # tests may have reset the environment
+                env = api.Environment(cr, SUPERUSER_ID, {})
+                module = env['ir.module.module'].browse(module_id)
 
             processed_modules.append(package.name)
 
@@ -270,6 +271,7 @@ def load_module_graph(cr, graph, status=None, perform_checks=True,
             for kind in ('init', 'demo', 'update'):
                 if hasattr(package, kind):
                     delattr(package, kind)
+            module.flush()
 
         if package.name is not None:
             registry._init_modules.add(package.name)
@@ -393,7 +395,7 @@ def load_modules(db, force_demo=False, status=None, update_module=False):
 
             cr.execute("update ir_module_module set state=%s where name=%s", ('installed', 'base'))
             Module.invalidate_cache(['state'])
-
+            Module.flush()
 
         # STEP 3: Load marked modules (skipping base which was done in STEP 1)
         # IMPORTANT: this is done in two parts, first loading all installed or
@@ -420,6 +422,18 @@ def load_modules(db, force_demo=False, status=None, update_module=False):
                     ['to install'], force, status, report,
                     loaded_modules, update_module, models_to_check)
 
+        # check that new module dependencies have been properly installed after a migration/upgrade
+        cr.execute("SELECT name from ir_module_module WHERE state IN ('to install', 'to upgrade')")
+        module_list = [name for (name,) in cr.fetchall()]
+        if module_list:
+            _logger.error("Some modules have inconsistent states, some dependencies may be missing: %s", sorted(module_list))
+
+        # check that all installed modules have been loaded by the registry after a migration/upgrade
+        cr.execute("SELECT name from ir_module_module WHERE state = 'installed' and name != 'studio_customization'")
+        module_list = [name for (name,) in cr.fetchall() if name not in graph]
+        if module_list:
+            _logger.error("Some modules are not loaded, some dependencies or manifest may be missing: %s", sorted(module_list))
+
         registry.loaded = True
         registry.setup_models(cr)
 
@@ -428,6 +442,9 @@ def load_modules(db, force_demo=False, status=None, update_module=False):
         for package in graph:
             migrations.migrate_module(package, 'end')
 
+        # STEP 3.6: apply remaining constraints in case of an upgrade
+        registry.finalize_constraints()
+
         # STEP 4: Finish and cleanup installations
         if processed_modules:
             env = api.Environment(cr, SUPERUSER_ID, {})
@@ -449,10 +466,11 @@ def load_modules(db, force_demo=False, status=None, update_module=False):
                 if model in registry:
                     env[model]._check_removed_columns(log=True)
                 elif _logger.isEnabledFor(logging.INFO):    # more an info that a warning...
-                    _logger.warning("Model %s is declared but cannot be loaded! (Perhaps a module was partially removed or renamed)", model)
+                    _logger.log(25, "Model %s is declared but cannot be loaded! (Perhaps a module was partially removed or renamed)", model)
 
             # Cleanup orphan records
             env['ir.model.data']._process_end(processed_modules)
+            env['base'].flush()
 
         for kind in ('init', 'demo', 'update'):
             tools.config[kind] = {}
@@ -512,9 +530,14 @@ def load_modules(db, force_demo=False, status=None, update_module=False):
             _logger.info('Modules loaded.')
 
         # STEP 8: call _register_hook on every model
+        # This is done *exactly once* when the registry is being loaded. See the
+        # management of those hooks in `Registry.setup_models`: all the calls to
+        # setup_models() done here do not mess up with hooks, as registry.ready
+        # is False.
         env = api.Environment(cr, SUPERUSER_ID, {})
         for model in env.values():
             model._register_hook()
+        env['base'].flush()
 
         # STEP 9: save installed/updated modules for post-install tests
         registry.updated_modules += processed_modules
