@@ -2,7 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import ast
-import functools
+import collections
 import imp
 import importlib
 import inspect
@@ -16,6 +16,7 @@ import time
 import types
 import unittest
 import threading
+import warnings
 from operator import itemgetter
 from os.path import join as opj
 
@@ -24,6 +25,7 @@ import odoo.tools as tools
 import odoo.release as release
 from odoo import SUPERUSER_ID, api
 from odoo.tools import pycompat
+from odoo.tools.misc import mute_logger
 
 MANIFEST_NAMES = ('__manifest__.py', '__openerp__.py')
 README = ['README.rst', 'README.md', 'README.txt']
@@ -31,60 +33,40 @@ README = ['README.rst', 'README.md', 'README.txt']
 _logger = logging.getLogger(__name__)
 
 # addons path as a list
-ad_paths = []
-hooked = False
+# ad_paths is a deprecated alias, please use odoo.addons.__path__
+@tools.lazy
+def ad_paths():
+    warnings.warn(
+        '"odoo.modules.module.ad_paths" is a deprecated proxy to '
+        '"odoo.addons.__path__".', DeprecationWarning, stacklevel=2)
+    return odoo.addons.__path__
 
 # Modules already loaded
 loaded = []
 
 class AddonsHook(object):
-    """ Makes modules accessible through openerp.addons.* and odoo.addons.* """
+    """ Makes modules accessible through openerp.addons.* """
 
     def find_module(self, name, path=None):
-        if name.startswith(('odoo.addons.', 'openerp.addons.'))\
-                and name.count('.') == 2:
+        if name.startswith('openerp.addons.') and name.count('.') == 2:
+            warnings.warn(
+                '"openerp.addons" is a deprecated alias to "odoo.addons".',
+                DeprecationWarning, stacklevel=2)
             return self
 
     def load_module(self, name):
         assert name not in sys.modules
 
-        # get canonical names
         odoo_name = re.sub(r'^openerp.addons.(\w+)$', r'odoo.addons.\g<1>', name)
-        openerp_name = re.sub(r'^odoo.addons.(\w+)$', r'openerp.addons.\g<1>', odoo_name)
 
-        assert odoo_name not in sys.modules
-        assert openerp_name not in sys.modules
+        odoo_module = sys.modules.get(odoo_name)
+        if not odoo_module:
+            odoo_module = importlib.import_module(odoo_name)
 
-        # get module name in addons paths
-        _1, _2, addon_name = name.split('.')
-        # load module
-        f, path, (_suffix, _mode, type_) = imp.find_module(addon_name, ad_paths)
-        if f: f.close()
+        sys.modules[name] = odoo_module
 
-        # TODO: fetch existing module from sys.modules if reloads permitted
-        # create empty odoo.addons.* module, set name
-        new_mod = types.ModuleType(odoo_name)
-        new_mod.__loader__ = self
+        return odoo_module
 
-        # module top-level can only be a package
-        assert type_ == imp.PKG_DIRECTORY, "Odoo addon top-level must be a package"
-        modfile = opj(path, '__init__.py')
-        new_mod.__file__ = modfile
-        new_mod.__path__ = [path]
-        new_mod.__package__ = odoo_name
-
-        # both base and alias should be in sys.modules to handle recursive and
-        # corecursive situations
-        sys.modules[odoo_name] = sys.modules[openerp_name] = new_mod
-
-        # execute source in context of module *after* putting everything in
-        # sys.modules, so recursive import works
-        exec(open(modfile, 'rb').read(), new_mod.__dict__)
-
-        # people import openerp.addons and expect openerp.addons.<module> to work
-        setattr(odoo.addons, addon_name, new_mod)
-
-        return sys.modules[name]
 # need to register loader with setuptools as Jinja relies on it when using
 # PackageLoader
 pkg_resources.register_loader_type(AddonsHook, pkg_resources.DefaultProvider)
@@ -96,6 +78,9 @@ class OdooHook(object):
         # openerp.addons.<identifier> should already be matched by AddonsHook,
         # only framework and subdirectories of modules should match
         if re.match(r'^openerp\b', name):
+            warnings.warn(
+                'openerp is a deprecated alias to odoo.',
+                DeprecationWarning, stacklevel=2)
             return self
 
     def load_module(self, name):
@@ -125,33 +110,42 @@ def initialize_sys_path():
     ``import odoo.addons.crm``) works even if the addons are not in the
     PYTHONPATH.
     """
-    global ad_paths
-    global hooked
-
+    # hook odoo.addons on data dir
     dd = os.path.normcase(tools.config.addons_data_dir)
-    if os.access(dd, os.R_OK) and dd not in ad_paths:
-        ad_paths.append(dd)
+    if os.access(dd, os.R_OK) and dd not in odoo.addons.__path__:
+        odoo.addons.__path__.append(dd)
 
+    # hook odoo.addons on addons paths
     for ad in tools.config['addons_path'].split(','):
         ad = os.path.normcase(os.path.abspath(tools.ustr(ad.strip())))
-        if ad not in ad_paths:
-            ad_paths.append(ad)
+        if ad not in odoo.addons.__path__:
+            odoo.addons.__path__.append(ad)
 
-    # add base module path
+    # hook odoo.addons on base module path
     base_path = os.path.normcase(os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)), 'addons')))
-    if base_path not in ad_paths and os.path.isdir(base_path):
-        ad_paths.append(base_path)
+    if base_path not in odoo.addons.__path__ and os.path.isdir(base_path):
+        odoo.addons.__path__.append(base_path)
 
-    # add odoo.addons.__path__
-    for ad in __import__('odoo.addons').addons.__path__:
-        ad = os.path.abspath(ad)
-        if ad not in ad_paths and os.path.isdir(ad):
-            ad_paths.append(ad)
+    # hook odoo.upgrade on upgrade-path
+    from odoo import upgrade
+    legacy_upgrade_path = os.path.join(base_path, 'base', 'maintenance', 'migrations')
+    for up in (tools.config['upgrade_path'] or legacy_upgrade_path).split(','):
+        up = os.path.normcase(os.path.abspath(tools.ustr(up.strip())))
+        if up not in upgrade.__path__:
+            upgrade.__path__.append(up)
 
-    if not hooked:
+    # create decrecated module alias from odoo.addons.base.maintenance.migrations to odoo.upgrade
+    spec = importlib.machinery.ModuleSpec("odoo.addons.base.maintenance", None, is_package=True)
+    maintenance_pkg = importlib.util.module_from_spec(spec)
+    maintenance_pkg.migrations = upgrade
+    sys.modules["odoo.addons.base.maintenance"] = maintenance_pkg
+    sys.modules["odoo.addons.base.maintenance.migrations"] = upgrade
+
+    if not getattr(initialize_sys_path, 'called', False): # only initialize once
         sys.meta_path.insert(0, OdooHook())
         sys.meta_path.insert(0, AddonsHook())
-        hooked = True
+        initialize_sys_path.called = True
+
 
 def get_module_path(module, downloaded=False, display_warning=True):
     """Return the path of the given module.
@@ -162,7 +156,7 @@ def get_module_path(module, downloaded=False, display_warning=True):
 
     """
     initialize_sys_path()
-    for adp in ad_paths:
+    for adp in odoo.addons.__path__:
         files = [opj(adp, module, manifest) for manifest in MANIFEST_NAMES] +\
                 [opj(adp, module + '.zip')]
         if any(os.path.exists(f) for f in files):
@@ -241,7 +235,7 @@ def get_resource_from_path(path):
     :return: tuple(module_name, relative_path, os_relative_path) if possible, else None
     """
     resource = False
-    for adpath in ad_paths:
+    for adpath in odoo.addons.__path__:
         # force trailing separator
         adpath = os.path.join(adpath, "")
         if os.path.commonprefix([adpath, path]) == adpath:
@@ -272,7 +266,7 @@ def module_manifest(path):
 
 def get_module_root(path):
     """
-    Get closest module's root begining from path
+    Get closest module's root beginning from path
 
         # Given:
         # /foo/bar/module_dir/static/src/...
@@ -324,13 +318,13 @@ def load_information_from_description_file(module, mod_path=None):
             'summary': '',
             'website': '',
         }
-        info.update(pycompat.izip(
+        info.update(zip(
             'depends data demo test init_xml update_xml demo_xml'.split(),
             iter(list, None)))
 
         f = tools.file_open(manifest_file, mode='rb')
         try:
-            info.update(ast.literal_eval(pycompat.to_native(f.read())))
+            info.update(ast.literal_eval(pycompat.to_text(f.read())))
         finally:
             f.close()
 
@@ -338,12 +332,26 @@ def load_information_from_description_file(module, mod_path=None):
             readme_path = [opj(mod_path, x) for x in README
                            if os.path.isfile(opj(mod_path, x))]
             if readme_path:
-                readme_text = tools.file_open(readme_path[0]).read()
-                info['description'] = readme_text
+                with tools.file_open(readme_path[0]) as fd:
+                    info['description'] = fd.read()
 
-        if 'active' in info:
-            # 'active' has been renamed 'auto_install'
-            info['auto_install'] = info['active']
+        # auto_install is set to `False` if disabled, and a set of
+        # auto_install dependencies otherwise. That way, we can set
+        # auto_install: [] to always auto_install a module regardless of its
+        # dependencies
+        auto_install = info.get('auto_install', info.get('active', False))
+        if isinstance(auto_install, collections.Iterable):
+            info['auto_install'] = set(auto_install)
+            non_dependencies = info['auto_install'].difference(info['depends'])
+            assert not non_dependencies,\
+                "auto_install triggers must be dependencies, found " \
+                "non-dependencies [%s] for module %s" % (
+                    ', '.join(non_dependencies), module
+                )
+        elif auto_install:
+            info['auto_install'] = set(info['depends'])
+        else:
+            info['auto_install'] = False
 
         info['version'] = adapt_version(info['version'])
         return info
@@ -404,7 +412,7 @@ def get_modules():
 
     plist = []
     initialize_sys_path()
-    for ad in ad_paths:
+    for ad in odoo.addons.__path__:
         plist.extend(listdir(ad))
     return list(set(plist))
 
@@ -429,17 +437,25 @@ def get_test_modules(module):
     """ Return a list of module for the addons potentially containing tests to
     feed unittest.TestLoader.loadTestsFromModule() """
     # Try to import the module
-    modpath = 'odoo.addons.' + module
+    results = _get_tests_modules('odoo.addons', module)
+
+    try:
+        importlib.import_module('odoo.upgrade.%s' % module)
+    except ImportError:
+        pass
+    else:
+        results += _get_tests_modules('odoo.upgrade', module)
+
+    return results
+
+def _get_tests_modules(path, module):
+    modpath = '%s.%s' % (path, module)
     try:
         mod = importlib.import_module('.tests', modpath)
     except ImportError as e:  # will also catch subclass ModuleNotFoundError of P3.6
         # Hide ImportErrors on `tests` sub-module, but display other exceptions
-        if pycompat.PY2:
-            if e.message.startswith('No module named') and e.message.endswith("tests"):
-                return []
-        else:
-            if e.name == modpath + '.tests' and e.msg.startswith('No module named'):
-                return []
+        if e.name == modpath + '.tests' and e.msg.startswith('No module named'):
+            return []
         _logger.exception('Can not `import %s`.', module)
         return []
     except Exception as e:
@@ -455,27 +471,122 @@ def get_test_modules(module):
               if name.startswith('test_')]
     return result
 
-# Use a custom stream object to log the test executions.
-class TestStream(object):
-    def __init__(self, logger_name='odoo.tests'):
-        self.logger = logging.getLogger(logger_name)
-        self.r = re.compile(r'^-*$|^ *... *$|^ok$')
-    def flush(self):
-        pass
-    def write(self, s):
-        if self.r.match(s):
+
+class OdooTestResult(unittest.result.TestResult):
+    """
+    This class in inspired from TextTestResult (https://github.com/python/cpython/blob/master/Lib/unittest/runner.py)
+    Instead of using a stream, we are using the logger,
+    but replacing the "findCaller" in order to give the information we
+    have based on the test object that is running.
+    """
+
+    def log(self, level, msg, *args, test=None, exc_info=None, extra=None, stack_info=False, caller_infos=None):
+        """
+        ``test`` is the running test case, ``caller_infos`` is
+        (fn, lno, func, sinfo) (logger.findCaller format), see logger.log for
+        the other parameters.
+        """
+        logger = logging.getLogger((test or self).__module__)  # test should be always set
+        try:
+            caller_infos = caller_infos or logger.findCaller(stack_info)
+        except ValueError:
+            caller_infos = "(unknown file)", 0, "(unknown function)", None
+        (fn, lno, func, sinfo) = caller_infos
+        # using logger.log makes it difficult to spot-replace findCaller in
+        # order to provide useful location information (the problematic spot
+        # inside the test function), so use lower-level functions instead
+        if logger.isEnabledFor(level):
+            record = logger.makeRecord(logger.name, level, fn, lno, msg, args, exc_info, func, extra, sinfo)
+            logger.handle(record)
+
+    def getDescription(self, test):
+        if isinstance(test, unittest.TestCase):
+            # since we have the module name in the logger, this will avoid to duplicate module info in log line
+            # we only apply this for TestCase since we can receive error handler or other special case
+            return "%s.%s" % (test.__class__.__qualname__, test._testMethodName)
+        return str(test)
+
+    def startTest(self, test):
+        super().startTest(test)
+        self.log(logging.INFO, 'Starting %s ...', self.getDescription(test), test=test)
+
+    def addError(self, test, err):
+        super().addError(test, err)
+        self.logError("ERROR", test, err)
+
+    def addFailure(self, test, err):
+        super().addFailure(test, err)
+        self.logError("FAIL", test, err)
+
+    def addSubTest(self, test, subtest, err):
+        # since addSubTest is not making a call to addFailure or addError we need to manage it too
+        # https://github.com/python/cpython/blob/3.7/Lib/unittest/result.py#L136
+        if err is not None:
+            if issubclass(err[0], test.failureException):
+                flavour = "FAIL"
+            else:
+                flavour = "ERROR"
+            self.logError(flavour, subtest, err)
+        super().addSubTest(test, subtest, err)
+
+    def addSkip(self, test, reason):
+        super().addSkip(test, reason)
+        self.log(logging.INFO, 'skipped %s', self.getDescription(test), test=test)
+
+    def addUnexpectedSuccess(self, test):
+        super().addUnexpectedSuccess(test)
+        self.log(logging.ERROR, 'unexpected success for %s', self.getDescription(test), test=test)
+
+    def logError(self, flavour, test, error):
+        err = self._exc_info_to_string(error, test)
+        caller_infos = self.getErrorCallerInfo(error, test)
+        self.log(logging.INFO, '=' * 70, test=test, caller_infos=caller_infos)  # keep this as info !!!!!!
+        self.log(logging.ERROR, "%s: %s\n%s", flavour, self.getDescription(test), err, test=test, caller_infos=caller_infos)
+
+    def getErrorCallerInfo(self, error, test):
+        """
+        :param error: A tuple (exctype, value, tb) as returned by sys.exc_info().
+        :param test: A TestCase that created this error.
+        :returns: a tuple (fn, lno, func, sinfo) matching the logger findCaller format or None
+        """
+
+        # only test case should be executed in odoo, this is only a safe guard
+        if isinstance(test, unittest.suite._ErrorHolder):
             return
-        first = True
-        level = logging.ERROR if s.startswith(('ERROR', 'FAIL', 'Traceback')) else logging.INFO
-        for c in s.splitlines():
-            if not first:
-                c = '` ' + c
-            first = False
-            self.logger.log(level, c)
+        if not isinstance(test, unittest.TestCase):
+            _logger.warning('%r is not a TestCase' % test)
+            return
+        _, _, error_traceback = error
+
+        while error_traceback:
+            code = error_traceback.tb_frame.f_code
+            if code.co_name == test._testMethodName:
+                lineno = error_traceback.tb_lineno
+                filename = code.co_filename
+                method = test._testMethodName
+                infos = (filename, lineno, method, None)
+                return infos
+            error_traceback = error_traceback.tb_next
+
+
+class OdooTestRunner(object):
+    """A test runner class that displays results in in logger.
+    Simplified verison of TextTestRunner(
+    """
+
+    def run(self, test):
+        result = OdooTestResult()
+
+        start_time = time.perf_counter()
+        test(result)
+        time_taken = time.perf_counter() - start_time
+        run = result.testsRun
+        _logger.info("Ran %d test%s in %.3fs", run, run != 1 and "s" or "", time_taken)
+        return result
 
 current_test = None
 
-def run_unit_tests(module_name, dbname, position='at_install'):
+def run_unit_tests(module_name, position='at_install'):
     """
     :returns: ``True`` if all of ``module_name``'s tests succeeded, ``False``
               if any of them failed.
@@ -497,7 +608,7 @@ def run_unit_tests(module_name, dbname, position='at_install'):
             t0 = time.time()
             t0_sql = odoo.sql_db.sql_counter
             _logger.info('%s running tests.', m.__name__)
-            result = unittest.TextTestRunner(verbosity=2, stream=TestStream(m.__name__)).run(suite)
+            result = OdooTestRunner().run(suite)
             if time.time() - t0 > 5:
                 _logger.log(25, "%s tested in %.2fs, %s queries", m.__name__, time.time() - t0, odoo.sql_db.sql_counter - t0_sql)
             if not result.wasSuccessful():
