@@ -10,6 +10,7 @@ import traceback
 from xml.etree import ElementTree as ET
 import zipfile
 
+from pytz import country_timezones
 from functools import wraps
 from contextlib import closing
 from decorator import decorator
@@ -24,7 +25,6 @@ import odoo.sql_db
 import odoo.tools
 from odoo.sql_db import db_connect
 from odoo.release import version_info
-from odoo.tools import pycompat
 
 _logger = logging.getLogger(__name__)
 
@@ -71,6 +71,9 @@ def _initialize_db(id, db_name, demo, lang, user_password, login='admin', countr
             if country_code:
                 country = env['res.country'].search([('code', 'ilike', country_code)])[0]
                 env['res.company'].browse(1).write({'country_id': country_code and country.id, 'currency_id': country_code and country.currency_id.id})
+                if len(country_timezones.get(country_code, [])) == 1:
+                    users = env['res.users'].search([])
+                    users.write({'tz': country_timezones[country_code][0]})
             if phone:
                 env['res.company'].browse(1).write({'phone': phone})
             if '@' in login:
@@ -255,7 +258,7 @@ def exp_restore(db_name, data, copy=False):
 
 @check_db_management_enabled
 def restore_db(db, dump_file, copy=False):
-    assert isinstance(db, pycompat.string_types)
+    assert isinstance(db, str)
     if exp_db_exist(db):
         _logger.info('RESTORE DB: %s already exists', db)
         raise Exception("Database already exists")
@@ -301,7 +304,7 @@ def restore_db(db, dump_file, copy=False):
 
             if odoo.tools.config['unaccent']:
                 try:
-                    with cr.savepoint():
+                    with cr.savepoint(flush=False):
                         cr.execute("CREATE EXTENSION unaccent")
                 except psycopg2.Error:
                     pass
