@@ -22,6 +22,8 @@ from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT, ISOLATION_LEVEL_READ
 from psycopg2.pool import PoolError
 from werkzeug import urls
 
+from odoo.api import Environment
+
 psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
 
 _logger = logging.getLogger(__name__)
@@ -41,7 +43,6 @@ psycopg2.extensions.register_type(psycopg2.extensions.new_type((700, 701, 1700,)
 
 from . import tools
 from .tools.func import frame_codeinfo
-from .tools import pycompat
 
 from .tools import parse_version as pv
 if pv(psycopg2.__version__) < pv('2.7'):
@@ -52,13 +53,37 @@ if pv(psycopg2.__version__) < pv('2.7'):
             raise ValueError("A string literal cannot contain NUL (0x00) characters.")
         return QuotedString(adapted)
 
-    for type_ in pycompat.string_types:
-        psycopg2.extensions.register_adapter(type_, adapt_string)
+    psycopg2.extensions.register_adapter(str, adapt_string)
 
 from datetime import timedelta
 import threading
 from inspect import currentframe
 
+
+def flush_env(cr, *, clear=True):
+    """ Retrieve and flush an environment corresponding to the given cursor.
+        Also clear the environment if ``clear`` is true.
+    """
+    env_to_flush = None
+    for env in list(Environment.envs):
+        # don't flush() on another cursor or with a RequestUID
+        if env.cr is cr and (isinstance(env.uid, int) or env.uid is None):
+            env_to_flush = env
+            if env.uid is not None:
+                break               # prefer an environment with a real uid
+
+    if env_to_flush is not None:
+        env_to_flush['base'].flush()
+        if clear:
+            env_to_flush.clear()    # clear remaining new records to compute
+
+def clear_env(cr):
+    """ Retrieve and clear an environment corresponding to the given cursor """
+    for env in list(Environment.envs):
+        if env.cr is cr:
+            env.clear()
+            break
+
 import re
 re_from = re.compile('.* from "?([a-zA-Z_0-9]+)"? .*$')
 re_into = re.compile('.* into "?([a-zA-Z_0-9]+)"? .*$')
@@ -225,7 +250,7 @@ class Cursor(object):
             res = self._obj.execute(query, params)
         except Exception as e:
             if self._default_log_exceptions if log_exceptions is None else log_exceptions:
-                _logger.error("bad query: %s\nERROR: %s", self._obj.query or query, e)
+                _logger.error("bad query: %s\nERROR: %s", tools.ustr(self._obj.query or query), e)
             raise
 
         # simple query count is always computed
@@ -239,12 +264,13 @@ class Cursor(object):
         if self.sql_log:
             delay *= 1E6
 
-            res_from = re_from.match(query.lower())
+            query_lower = self._obj.query.decode().lower()
+            res_from = re_from.match(query_lower)
             if res_from:
                 self.sql_from_log.setdefault(res_from.group(1), [0, 0])
                 self.sql_from_log[res_from.group(1)][0] += 1
                 self.sql_from_log[res_from.group(1)][1] += delay
-            res_into = re_into.match(query.lower())
+            res_into = re_into.match(query_lower)
             if res_into:
                 self.sql_into_log.setdefault(res_into.group(1), [0, 0])
                 self.sql_into_log[res_into.group(1)][0] += 1
@@ -314,6 +340,11 @@ class Cursor(object):
         # Clean the underlying connection.
         self._cnx.rollback()
 
+        # Call rollback hooks. This list is empty if commit()
+        # has been called just before closing the cursor.
+        for func in self._pop_event_handlers()['rollback']:
+            func()
+
         if leak:
             self._cnx.leaked = True
         else:
@@ -369,6 +400,7 @@ class Cursor(object):
     def commit(self):
         """ Perform an SQL `COMMIT`
         """
+        flush_env(self)
         result = self._cnx.commit()
         for func in self._pop_event_handlers()['commit']:
             func()
@@ -378,6 +410,7 @@ class Cursor(object):
     def rollback(self):
         """ Perform an SQL `ROLLBACK`
         """
+        clear_env(self)
         result = self._cnx.rollback()
         for func in self._pop_event_handlers()['rollback']:
             func()
@@ -402,13 +435,19 @@ class Cursor(object):
 
     @contextmanager
     @check
-    def savepoint(self):
+    def savepoint(self, flush=True):
         """context manager entering in a new savepoint"""
         name = uuid.uuid1().hex
+        if flush:
+            flush_env(self, clear=False)
         self.execute('SAVEPOINT "%s"' % name)
         try:
             yield
+            if flush:
+                flush_env(self, clear=False)
         except Exception:
+            if flush:
+                clear_env(self)
             self.execute('ROLLBACK TO SAVEPOINT "%s"' % name)
             raise
         else:
@@ -465,9 +504,11 @@ class TestCursor(object):
         _logger.debug("TestCursor.autocommit(%r) does nothing", on)
 
     def commit(self):
+        flush_env(self)
         self._cursor.execute('SAVEPOINT "%s"' % self._savepoint)
 
     def rollback(self):
+        clear_env(self)
         self._cursor.execute('ROLLBACK TO SAVEPOINT "%s"' % self._savepoint)
 
     def __enter__(self):
@@ -485,40 +526,6 @@ class TestCursor(object):
         return value
 
 
-class LazyCursor(object):
-    """ A proxy object to a cursor. The cursor itself is allocated only if it is
-        needed. This class is useful for cached methods, that use the cursor
-        only in the case of a cache miss.
-    """
-    def __init__(self, dbname=None):
-        self._dbname = dbname
-        self._cursor = None
-        self._depth = 0
-
-    @property
-    def dbname(self):
-        return self._dbname or threading.currentThread().dbname
-
-    def __getattr__(self, name):
-        cr = self._cursor
-        if cr is None:
-            from odoo import registry
-            cr = self._cursor = registry(self.dbname).cursor()
-            for _ in range(self._depth):
-                cr.__enter__()
-        return getattr(cr, name)
-
-    def __enter__(self):
-        self._depth += 1
-        if self._cursor is not None:
-            self._cursor.__enter__()
-        return self
-
-    def __exit__(self, exc_type, exc_value, traceback):
-        self._depth -= 1
-        if self._cursor is not None:
-            self._cursor.__exit__(exc_type, exc_value, traceback)
-
 class PsycoConnection(psycopg2.extensions.connection):
     pass
 
