@@ -5,9 +5,9 @@ from . import pycompat
 from . import win32
 from . import appdirs
 from . import pdf
+from . import cloc
 from .config import config
 from .misc import *
-from .convert import *
 from .translate import *
 from .graph import graph
 from .image import *
@@ -18,3 +18,5 @@ from .func import *
 from .debugger import *
 from .xml_utils import *
 from .date_utils import *
+from .convert import *
+from .template_inheritance import *
\ No newline at end of file
